package se.su.dsv.scipro.activityplan;

import com.google.common.eventbus.EventBus;
import com.google.inject.persist.Transactional;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.checklist.*;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.FileSource;
import se.su.dsv.scipro.file.ProjectFile;
import se.su.dsv.scipro.file.ProjectFileUpload;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.*;

import static com.mysema.query.types.ExpressionUtils.allOf;

public class ActivityPlanFacadeImpl implements ActivityPlanFacade {

    private static final Logger LOGGER = LoggerFactory.getLogger(ActivityPlanFacadeImpl.class);

    private final EventBus eventBus;
    private final FileDescriptionService fileDescriptionService;
    private final ChecklistTemplateService checklistTemplateService;
    private final DaysService daysService;

    @Inject
    ActivityService activityService; // yuck
    @Inject
    ActivityPlanService activityPlanService; //yuck
    @Inject
    ActivityPlanTemplateService activityPlanTemplateService;
    @Inject
    ChecklistService checklistService;
    @Inject
    ChecklistQuestionRepo checklistQuestionRepo;
    @Inject
    ProjectService projectService;

    @Inject
    public ActivityPlanFacadeImpl(final EventBus eventBus, final FileDescriptionService fileDescriptionService, ChecklistTemplateService checklistTemplateService, final DaysService daysService) {
        this.eventBus = eventBus;
        this.fileDescriptionService = fileDescriptionService;
        this.checklistTemplateService = checklistTemplateService;
        this.daysService = daysService;
    }

    /**
     * Creates a new schedule event and makes the needed object-state linking.
     * If either parameter is null, a wrapped UnsupportedOperationException is thrown.
     *
     * @param ps   .
     * @param date .
     * @return The newly created event.
     */
    @Transactional
    private Activity createNewActivity(final ActivityPlan ps,
                                       String name,
                                       String description,
                                       Action action,
                                       final Date date,
                                       final Checklist checklist,
                                       final boolean editable) {
        Activity pse = new Activity();
        pse.setActivityPlan(ps);
        if (date == null) {
            pse.setDate(new Date());
        } else {
            pse.setDate(date);
        }

        pse.setTitle(name);
        pse.setDescription(description);
        pse.setAction(action);
        pse.setChecklist(checklist);
        pse.setEditable(editable);
        return activityService.save(pse);
    }

    /**
     * Saves already existing (and possibly detached) schedule event.
     */
    @Override
    @Transactional
    public Activity saveActivity(final Activity event) {
        if (event.getTitle() == null) {
            event.setTitle("");
        }
        if (event.getDescription() == null) {
            event.setDescription("");
        }
        if (event.getId() == null)//Attempt to create a new one if this is a transient entity
        {
            return createNewActivity(
                    event.getActivityPlan(),
                    event.getTitle(),
                    event.getDescription(),
                    event.getAction(),
                    event.getDate(),
                    event.getChecklist(),
                    event.isEditable());
        }
        return activityService.save(event);
    }

    /**
     * Retrieves a project schedule for the given project, if none exists it will be silently created and returned.
     * If the project parameter is null, a wrapped UnsupportedOperationException is thrown.
     *
     * @param p .
     * @return The newly created event.
     */
    @Override
    @Transactional
    public ActivityPlan retrieveActivityPlan(final Project p) {
        if (p == null) {
            throw new RuntimeException(new UnsupportedOperationException("Sorry, can't create a schedule without a valid project"));
        }

        ActivityPlan schedule = findByProject(p);

        if (schedule == null) {
            LOGGER.debug("No existing schedule for project " + p + ", creating a new one");
            schedule = new ActivityPlan();
            schedule.updateTimeStamps();
            schedule.setStartDate(new Date());
            schedule.setProject(p);
            schedule = activityPlanService.save(schedule);
        }
        return schedule;
    }

    @Override
    @Transactional
    public void storeActivityUpload(ProjectFileUpload fileUpload, final Activity event) {
        ProjectFile fd = fileDescriptionService.storeFile(fileUpload, FileSource.ACTIVITY_PLAN);
        event.setFileUpload(fd);
        activityService.save(event);
        eventBus.post(new ActivityFileUploadedEvent(event, fd));
    }

    /**
     * Attempts to delete a linked file upload from the given event, if the file is referenced from other places: only the linking is removed.
     *
     * @param event .
     */
    @Override
    @Transactional
    public void deleteActivityUpload(final Activity event) {
        if (event != null) {
            deleteUpload(event);
        }
    }

    @Override
    @Transactional
    public void addActivitiesFromTemplate(final Project project, final ActivityPlanTemplate template, final Date startDate) {
        addActivitiesFromTemplate(retrieveActivityPlan(project), template, startDate);
    }

    private void addActivitiesFromTemplate(final ActivityPlan schedule,
                                           ActivityPlanTemplate template,
                                           final Date startDate) {
        Date startDateForEvent = (startDate != null ? startDate : new DateTime().withTimeAtStartOfDay().toDate());
        ActivityPlanTemplate reloadedTemplate = activityPlanTemplateService.findOne(template.getId());//Reload lazily linked entities
        int accumulatedOffset = 0;
        for (final ActivityTemplate eventTemplate : reloadedTemplate.getActivityTemplates()) {
            accumulatedOffset += eventTemplate.getDaysOffset();
            final Date dateForEvent = daysService.workDaysAfter(startDateForEvent, accumulatedOffset);
            final String title = eventTemplate.getTitle() != null ? eventTemplate.getTitle() : "no title";
            final String desc = eventTemplate.getDescription() != null ? eventTemplate.getDescription() : "";
            final ChecklistTemplate checklistTemplate = eventTemplate.getChecklistTemplate() != null ? eventTemplate.getChecklistTemplate() : null;
            final Action action = eventTemplate.getAction();
            if (checklistTemplate != null) {
                createNewActivity(
                        schedule,
                        title,
                        desc,
                        action,
                        dateForEvent,
                        createChecklist(schedule.getProject(), checklistTemplate),
                        true);
            } else {
                createNewActivity(schedule, title, desc, action, dateForEvent, null, true);
            }
        }
    }

    /**
     * Generates a new template from the given ActivityPlan.
     */
    @Override
    @Transactional
    public ActivityPlanTemplate createTemplateFromSchedule(
            final ActivityPlan schedule,
            final User user,
            final String name,
            final String description) {
        Objects.requireNonNull(schedule, "Schedule may not be null");
        Objects.requireNonNull(user, "User may not be null");

        final ActivityPlanTemplate template = new ActivityPlanTemplate();
        template.setCreator(user);
        template.setTitle(name);
        template.setDescription(description);
        template.setIsSysAdminTemplate(false);

        Date previousActivityDate = null;
        // Fucking Hibernate that replaces the original TreeSet with an unsorted implementation because it can.
        SortedSet<Activity> activities = new TreeSet<>(new Activity.ByDateComparator());
        activities.addAll(schedule.getActivities());
        for (final Activity activity : activities) {
            final ActivityTemplate activityTemplate = createTemplateFromActivity(previousActivityDate, activity);
            template.addActivity(activityTemplate);
            previousActivityDate = activity.getDate();
        }

        return activityPlanTemplateService.save(template);
    }

    private ActivityTemplate createTemplateFromActivity(Date previousEventDate, Activity event) {
        ActivityTemplate eventTemplate = new ActivityTemplate();
        eventTemplate.setTitle(event.getTitle());
        eventTemplate.setDaysOffset(daysBetween(previousEventDate, event.getDate()));
        eventTemplate.setDescription(event.getDescription());
        eventTemplate.setAction(event.getAction());
        if (event.getChecklist() != null) {
            eventTemplate.setChecklistTemplate(checklistTemplateService.findByName(event.getChecklist().getName()));
        }
        return eventTemplate;
    }

    private int daysBetween(Date previousEventDate, Date currentEventDate) {
        return previousEventDate == null ? 0
                : daysService.workDaysBetween(previousEventDate, currentEventDate);
    }

    @Override
    @Transactional
    public void deleteActivity(Activity event) {
        deleteActivityUpload(event);
        if (event != null && event.getId() != null) {
            Activity reloadedEvent = activityService.findOne(event.getId());
            activityService.delete(reloadedEvent.getId());
        }
    }

    @Override
    public ActivityPlan deleteAllEvents(ActivityPlan activityPlan) {
        for (Activity pse : activityPlan.getActivities()) {
            deleteActivity(pse);
        }
        activityPlan.getActivities().clear();
        return activityPlanService.save(activityPlan);
    }

    //moving from activityservice
    @Override
    public List<Activity> findEventsByProject(Project project, Pageable pageable) {
        return findEventsByProject(project, null, null, pageable);
    }

    @Override
    public List<Activity> findEventsByProject(Project project, Date from, Date to, Pageable pageable) {
        final Date dateFrom = from != null ? from : new Date(0);
        final Date dateTo = to != null ? to : new Date(Long.MAX_VALUE / 4);
        QActivity event = QActivity.activity;
        return activityService.findAll(allOf(event.activityPlan.project.eq(project), event.date.after(dateFrom), event.date.before(dateTo)), pageable);
    }

    @Override
    public long countEventsByProject(Project project, Date from, Date to) {
        final Date dateFrom = from != null ? from : new Date(0);
        final Date dateTo = to != null ? to : new Date(Long.MAX_VALUE / 4);
        QActivity event = QActivity.activity;
        return activityService.count(allOf(event.activityPlan.project.eq(project), event.date.after(dateFrom), event.date.before(dateTo)));
    }

    private void deleteUpload(Activity activity) {
        activity.setFileUpload(null);
        activityService.save(activity);
    }

    //moving from activityplanservice
    private ActivityPlan findByProject(Project project) {
        return activityPlanService.findOne(QActivityPlan.activityPlan.project.eq(project));
    }

    @Override
    public Checklist createChecklist(Project project, ChecklistTemplate template) {
        List<ChecklistCategory> categories = new ArrayList<>(template.getCategories());
        Checklist checklist = Checklist.builder().name(template.getName()).project(project).description(template.getDescription()).build();
        checklist.setCategories(categories);
        for (String question : template.getQuestions()) {
            ChecklistQuestion clQuestion = new ChecklistQuestion(question, checklist.getNumberOfQuestions());
            clQuestion = checklistQuestionRepo.save(clQuestion);
            checklist.addQuestion(clQuestion);
        }
        return checklistService.save(checklist);
    }

    @Override
    @Transactional
    public Checklist updateUserLastOpenDate(Checklist checklist, User user) {
        checklist.getUserLastOpenDate().put(user, new Date());
        return checklistService.save(checklist);
    }

    @Override
    public boolean isCompletedByUser(User user, Checklist checklist) {
        for (ChecklistQuestion question : checklist.getQuestions()) {
            if (question.getAnswerFromUser(user).getAnswer().equals(ChecklistAnswerEnum.NO_ANSWER)) {
                return false;
            }
        }
        return true;
    }

}
