package se.su.dsv.scipro.file;

import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.SneakyThrows;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import javax.validation.constraints.Size;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.NoSuchFileException;
import java.nio.file.Path;
import java.text.DecimalFormat;
import java.text.NumberFormat;

@Data
@EqualsAndHashCode(of = "id", callSuper = false)
@Entity
@Table(name = "file_description")
@Cacheable(true)
@Inheritance(strategy = InheritanceType.SINGLE_TABLE)
@DiscriminatorColumn(name = "type", length = 255)
@SuppressWarnings("PMD.UnusedPrivateField")
public class FileDescription extends DomainObject {

    private static final Logger LOGGER = LoggerFactory.getLogger(FileDescription.class);

    public static final int STRING_MAX_LENGTH = 255;
    public static final int FILES_PER_SUBDIRECTORY = 1000;
    public static final String FILE_ROOT = "/scipro-files";
    public static final int GIGA_BYTE = 1073741824;
    public static final int MEGA_BYTE = 1048576;
    public static final int KILO_BYTE = 1024;

    @Id
    @GeneratedValue
    private Long id;

    @Column(length = STRING_MAX_LENGTH)
    private String name;

    @Column(length = STRING_MAX_LENGTH)
    private String mimeType;

    @Column(length = STRING_MAX_LENGTH)
    private String identifier;

    @Column(length = STRING_MAX_LENGTH)
    @Size(max = STRING_MAX_LENGTH)
    private String path;

    @ManyToOne(optional = true)
    @JoinColumn(name = "userId")
    private User uploader;

    private long size;

    @Basic
    private boolean deletable = false;

    @Basic
    private boolean exposed = false;

    @PostRemove
    void removeActualData() {
        try {
            Files.delete(getPath0());
        } catch (NoSuchFileException ignored) {
            LOGGER.debug("File is already gone from the filesystem: {}", this, ignored);
        } catch (IOException e) {
            LOGGER.warn("Failed to delete file data: {}", this, e);
        }
    }

    @SneakyThrows(IOException.class)
    public void storeActualData(InputStream is) {
        if (id == null) {
            throw new IllegalStateException("Metadata must have an id");
        }

        Path path0 = getPath0();
        if (path0.getParent() != null) {
            createDirectory(path0.getParent());
        }

        Files.copy(is, path0);
    }

    public Path getPath0() {
        return FileSystems.getDefault().getPath(FILE_ROOT, getSubdirectory(), Long.toString(id));
    }

    private String getSubdirectory() {
        return Long.toString((id / FILES_PER_SUBDIRECTORY) * FILES_PER_SUBDIRECTORY + FILES_PER_SUBDIRECTORY);
    }

    private void createDirectory(Path directory) {
        File file = directory.toFile();
        if (!file.exists() && !file.mkdirs()) {
            throw new RuntimeException(new IOException("Could not create parent directory"));
        }
    }

    public boolean isDataAvailable() {
        return Files.exists(getPath0());
    }

    public InputStream getData() throws IOException {
        return Files.newInputStream(getPath0());
    }

    @SuppressWarnings("UnusedDeclaration")
    public String getFormattedSize() {
        NumberFormat nf = new DecimalFormat("#.00");
        if (size >= GIGA_BYTE) {
            return nf.format(size / GIGA_BYTE) + " Gb";
        } else if (size >= MEGA_BYTE) {
            return nf.format(size / MEGA_BYTE) + " Mb";
        } else if (size >= KILO_BYTE) {
            return nf.format(size / KILO_BYTE) + " KB";
        } else {
            return nf.format(size) + " bytes";
        }
    }

    public String getName() {
        return name;
    }

    // Todo
    @Override
    public String toString() {
        if (name != null) {
            return name;
        } else {
            return super.toString();
        }
    }
}
