package se.su.dsv.scipro.file;

import com.google.common.base.Function;
import com.google.inject.persist.Transactional;
import com.mysema.query.types.PathMetadataFactory;
import com.mysema.query.types.path.StringPath;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.AbstractServiceImpl;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;

public class FileDescriptionServiceImpl extends AbstractServiceImpl<FileDescription, Long> implements FileDescriptionService {

    public static final String FILE_ROOT = "/scipro-files";
    public static final int FILES_PER_SUBDIRECTORY = 1000;

    @Inject
    public FileDescriptionServiceImpl(Provider<EntityManager> em) {
        super(em, FileDescription.class, QFileDescription.fileDescription);
    }

    @Override
    public List<ProjectFile> latestUpload(Project project, int amount) {
        List<ProjectFile> rootFiles = from(QProjectFile.projectFile)
                .where(QProjectFile.projectFile.project.eq(project))
                .orderBy(QProjectFile.projectFile.dateCreated.desc())
                .list(QProjectFile.projectFile);
        return rootFiles.subList(0, rootFiles.size() >= amount ? amount : rootFiles.size());
    }

    @Override
    @Transactional
    public FileDescription storeFile(FileUpload fileUpload) {
        return storeFile(fileUpload, new FileDescription());
    }

    @Override
    @Transactional
    public ProjectFile storeFile(ProjectFileUpload projectFileUpload, FileSource fileSource) {
        ProjectFile projectFile = new ProjectFile();
        projectFile.setProject(projectFileUpload.getProject());
        projectFile.setDeletable(projectFileUpload.isDeletable());
        projectFile.setFileSource(fileSource);
        return storeFile(projectFileUpload, projectFile);
    }

    private <T extends FileDescription> T storeFile(FileUpload fileUpload, T projectFile) {
        projectFile.setSize(fileUpload.getSize());
        projectFile.setMimeType(fileUpload.getContentType());
        projectFile.setName(fileUpload.getFileName());
        projectFile.setUploader(fileUpload.getUploader());
        T metaData = save(projectFile);
        storeData(fileUpload, metaData);
        return metaData;
    }

    protected <T extends FileDescription> void storeData(final FileUpload fileUpload, final T metaData) {
        fileUpload.handleData(new Function<InputStream, T>() {
            @Override
            public T apply(InputStream input) {
                metaData.storeActualData(input);
                return metaData;
            }
        });
    }

    @Override
    public Iterable<ProjectFile> getProjectFiles(Project project, Pageable pageable) {
        StringPath sort = new StringPath(PathMetadataFactory.forProperty(QProjectFile.projectFile, pageable.getSort().getProperty()));
        return from(QProjectFile.projectFile)
                .where(QProjectFile.projectFile.project.eq(project))
                .offset(pageable.getOffset())
                .limit(pageable.getLimit())
                .orderBy(pageable.getSort().isAscending() ? sort.asc() : sort.desc())
                .list(QProjectFile.projectFile);
    }

    @Override
    public Long countProjectFiles(Project project) {
        return from(QProjectFile.projectFile).where(QProjectFile.projectFile.project.eq(project)).count();
    }

    @Override
    public InputStream getData(FileDescription attachment) throws IOException {
        return Files.newInputStream(getPathFor(attachment));
    }

    private Path getPathFor(FileDescription fileDescription) {
        return FileSystems.getDefault().getPath(FILE_ROOT, getSubdirectory(fileDescription), Long.toString(fileDescription.getId()));
    }

    private String getSubdirectory(FileDescription fileDescription) {
        return Long.toString((fileDescription.getId() / FILES_PER_SUBDIRECTORY) * FILES_PER_SUBDIRECTORY + FILES_PER_SUBDIRECTORY);
    }

}
