package se.su.dsv.scipro.forum;

import com.google.common.eventbus.EventBus;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.forum.dataobjects.ForumPostReadState;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

class BasicForumServiceImpl implements BasicForumService {

    private final AbstractThreadRepository threadRepository;
    private final ForumPostRepository postRepository;
    private final ForumPostReadStateRepository readStateRepository;
    private final EventBus eventBus;

    @Inject
    BasicForumServiceImpl(
            final ForumPostRepository postRepository,
            final ForumPostReadStateRepository readStateRepository,
            AbstractThreadRepository threadRepository,
            final EventBus eventBus) {
        this.postRepository = postRepository;
        this.readStateRepository = readStateRepository;
        this.threadRepository = threadRepository;
        this.eventBus = eventBus;
    }


    @Override
    public boolean setRead(User user, ForumPost post, boolean read) {
        ForumPostReadState state = getReadState(user, post);
        state.setRead(read);
        state = readStateRepository.save(state);
        if (read) {
            eventBus.post(new ForumPostReadEvent(post, user));
        }
        return state.isRead();
    }

    @Override
    public boolean setThreadRead(User user, ForumThread forumThread, boolean read) {
        for (ForumPost post : forumThread.getPosts()) {
            setRead(user, post, read);
        }
        return read;
    }

    @Override
    public boolean isRead(User user, ForumPost post) {
        return getReadState(user, post).isRead();
    }

    @Override
    public boolean isThreadRead(User user, ForumThread forumThread) {
        for (ForumPost post : forumThread.getPosts()) {
            if (!getReadState(user, post).isRead()) {
                return false;
            }
        }
        return true;
    }

    @Override
    public List<ForumPost> getPosts(ForumThread forumThread) {
        return postRepository.findByThread(forumThread);
    }

    @Override
    public ForumThread createThread(final String subject) {
        ForumThread forumThread = new ForumThread();
        forumThread.setSubject(subject);
        return threadRepository.save(forumThread);
    }

    private ForumPostReadState getReadState(final User user, final ForumPost post) {
        ForumPostReadState state = readStateRepository.find(user, post);
        if (state == null) {
            state = new ForumPostReadState(user, post);
        }
        return state;
    }

    private ForumPost createForumPost(User user, String content, ForumThread forumThread, Set<FileDescription> fileUploads) {
        ForumPost post = new ForumPost();
        post.setPostedBy(user);
        post.setContent(content);
        post.setForumThread(forumThread);
        post.setAttachments(new HashSet<>(fileUploads)); // Need to copy since Hibernate likes to mutate the collection
        forumThread.addPost(post);
        return post;
    }

    @Override
    public ForumPost createReply(final ForumThread forumThread, final User user, final String content, final Set<FileDescription> attachments) {
        ForumPost post = createForumPost(user, content, forumThread, attachments);
        post = postRepository.save(post);

        threadRepository.save(forumThread);

        setRead(user, post, true);

        return post;
    }
}
