package se.su.dsv.scipro.forummail;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.mail.*;
import javax.mail.internet.MimeMessage;
import javax.mail.search.FlagTerm;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Properties;

public class IMAPReader implements ForumMailReader {

    private static final Logger LOGGER = LoggerFactory.getLogger(IMAPReader.class);

    private final ForumMailSettingsService forumMailSettingsService;
    private final MailContentParser contentParser;

    @Inject
    public IMAPReader(ForumMailSettingsService forumMailSettingsService,
                      MailContentParser contentParser) {
        this.forumMailSettingsService = forumMailSettingsService;
        this.contentParser = contentParser;
    }

    @Override
    public void accept(MailHandler mailHandler) {
        try {
            Folder folder = null;
            Store store = null;
            try {
                store = connectToIMAP();
                folder = openFolder(store);
                List<ForumMailMessage> forumMailMessages = messagesAsList(folder.search(unreadOnly()));
                for (ForumMailMessage forumMailMessage : forumMailMessages) {
                    mailHandler.accept(forumMailMessage);
                }
            } finally {
                if (folder != null) {
                    folder.close(true);
                }
                if (store != null) {
                    store.close();
                }
            }
        } catch (MessagingException | IOException e) {
            LOGGER.info("Something went wrong while reading e-mail: ", e);
        }
    }

    private FlagTerm unreadOnly() {
        return new FlagTerm(new Flags(Flags.Flag.SEEN), false);
    }

    private Store connectToIMAP() throws MessagingException, IOException {
        Properties props = System.getProperties();
        props.setProperty("mail.store.protocol", "imaps");
        Session session = Session.getInstance(props);
        Store store = session.getStore("imaps");
        ForumMailSettings settings = forumMailSettingsService.getSettings();
        store.connect(settings.getHost(), settings.getPort(),
                settings.getAccount(), settings.getPassword());
        return store;
    }

    private Folder openFolder(Store store) throws MessagingException {
        Folder folder = store.getFolder("inbox");
        if (!folder.isOpen()) {
            folder.open(Folder.READ_WRITE);
        }
        return folder;
    }

    private List<ForumMailMessage> messagesAsList(Message[] messages) throws MessagingException, IOException {
        List<ForumMailMessage> messageList = new ArrayList<>();
        for (Message message : messages) {
            MimeMessage mime = (MimeMessage) message;
            String inReplyTo = mime.getHeader("In-Reply-To", null);
            if (inReplyTo != null) {
                Collection<MailAttachment> attachments = getAttachments(message);
                messageList.add(new ForumMailMessage(inReplyTo, contentParser.format(message), attachments));
            }
        }
        return messageList;
    }

    private Collection<MailAttachment> getAttachments(Part part) throws MessagingException, IOException {
        Collection<MailAttachment> attachments = new ArrayList<>();
        if (Part.ATTACHMENT.equalsIgnoreCase(part.getDisposition())) {
            attachments.add(new MailAttachment(part));
        }
        if (part.isMimeType("multipart/*")) {
            Multipart multipart = (Multipart) part.getContent();
            for (int i = 0; i < multipart.getCount(); i++) {
                attachments.addAll(getAttachments(multipart.getBodyPart(i)));
            }
        }
        return attachments;
    }
}
