package se.su.dsv.scipro.mail;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.workerthreads.AbstractWorker;

import javax.inject.Inject;
import javax.mail.MessagingException;
import java.util.HashSet;
import java.util.Set;

public class MailEventWorker extends AbstractWorker {

    private static final Logger LOGGER = LoggerFactory.getLogger(MailEventWorker.class);

    private final MailEventService mailEventService;
    private final GeneralSystemSettingsService systemSettingsService;
    private final Mailer mailer;

    @Inject
    public MailEventWorker(MailEventService mailEventService,
                           GeneralSystemSettingsService systemSettingsService,
                           Mailer mailer) {
        this.mailEventService = mailEventService;
        this.systemSettingsService = systemSettingsService;
        this.mailer = mailer;
    }

    @Override
    protected void doWork() {
        if (mailTurnedOff()) {
            return;
        }

        GeneralSystemSettings systemSettings = systemSettingsService.getGeneralSystemSettingsInstance();

        this.beginTransaction();
        Iterable<MailEvent> unsent = mailEventService.findUnsent();

        this.commitTransaction();

        for (MailEvent mailEvent : unsent) {
            try {
                this.beginTransaction();

                mailEvent = mailEventService.findOne(mailEvent.getId());
                String subject = mailEvent.getSubject();

                String fromName = StringUtils.defaultIfBlank(mailEvent.getFromName(), systemSettings.getMailFromName());

                String fromEmail = StringUtils.defaultIfBlank(mailEvent.getFromEmail(), systemSettings.getSystemFromMail());

                final String[] recipientsEmails = generateRecipients(mailEvent);
                final String messageBody = mailEvent.getMessageBody();

                if (recipientsEmails.length > 0) { //Vanish if no notifications or empty
                    MailResult mailResult = mailer.mail(fromName, fromEmail, recipientsEmails, subject, messageBody, mailEvent.getAttachment());
                    mailResult.log(LOGGER);
                    mailEvent.markSent(mailResult.getMessageID());
                    mailEventService.save(mailEvent);
                }
                else {
                    mailEventService.delete(mailEvent);
                }
                this.commitTransaction();

            } catch (MessagingException e) {
                this.rollbackTransaction();
                LOGGER.error("Messaging error", e);
            }
        }
    }

    private boolean mailTurnedOff() {
        return !systemSettingsService.getGeneralSystemSettingsInstance().isMailNotifications();
    }

    private String[] generateRecipients(final MailEvent mailEvent) {
        final Set<User> recipients = filteredRecipients(mailEvent.getRecipients());
        final String[] recipientsEmails = new String[recipients.size() + mailEvent.getNonUserRecipients().size()];
        int i = 0;
        for (User u : recipients) {
            recipientsEmails[i] = u.getEmailAddress();
            i++;
        }
        for (String s : mailEvent.getNonUserRecipients()) {
            recipientsEmails[i] = s;
            i++;
        }
        return recipientsEmails;
    }

    private Set<User> filteredRecipients(Set<User> input){
        Set<User> output = new HashSet<>();
        for (User u : input){
            if (!StringUtils.isEmpty(u.getEmailAddress())){
                output.add(u);
            }
        }
        return output;
    }
}
