package se.su.dsv.scipro.match;

import com.google.common.base.Function;
import com.google.common.collect.Iterables;
import com.google.inject.persist.Transactional;
import com.mysema.query.BooleanBuilder;
import com.mysema.query.jpa.JPASubQuery;
import com.mysema.query.types.Predicate;
import com.mysema.query.types.expr.BooleanExpression;
import com.mysema.query.types.expr.DateExpression;
import com.mysema.query.types.expr.DateTimeExpression;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.date.DateService;
import se.su.dsv.scipro.match.Idea.Status;
import se.su.dsv.scipro.match.Idea.Type;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.IdeaEvent;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.system.*;
import se.su.dsv.scipro.util.Pair;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.*;

import static com.mysema.query.support.Expressions.allOf;
import static com.mysema.query.support.Expressions.anyOf;

class IdeaServiceImpl extends AbstractServiceImpl<Idea, Long> implements IdeaService {

    private static final Logger LOGGER = LoggerFactory.getLogger(IdeaServiceImpl.class);

    public static final String VALIDATION_PASSED = "Validation passed";
    public static final String ALREADY_MATCHED_ERROR = "Idea has already been matched";
    public static final String TARGET_NUMBERS_REACHED_ERROR = "You have reached your current target numbers, you may not accept another idea";
    public static final String SUPERVISING_CONFIRM = "You are now supervising idea: ";
    public static final String NO_LONGER_AVAILABLE_ERROR = "Idea is no longer available";
    public static final String ALREADY_PARTICIPATING_ERROR = "You are already participating in another idea";
    public static final String PARTNER_ALREADY_PARTICIPATING_ERROR = "Your partner is already participating in another idea";
    public static final String SELECTED_IDEA = "You selected idea: ";
    public static final String BACHELOR_NEED_PARTNER_ERROR = "You need to select a partner when the idea is on bachelor level";
    public static final String ADD_SELF_AS_PARTNER_ERROR = "You may not add yourself as project partner";
    public static final String NO_AUTHORS_ERROR = "The idea is submitted by a student, number of students is not allowed";
    public static final int TITLE_LENGTH = 3;
    public static final long EMPTY = 0;

    private final ApplicationPeriodService applicationPeriodService;
    private final FirstMeetingRepository firstMeetingRepository;
    private final NotificationController notificationController;
    private final DateService dateService;

    @Inject
    IdeaServiceImpl(Provider<EntityManager> em, final ApplicationPeriodService applicationPeriodService, final FirstMeetingRepository firstMeetingRepository, final NotificationController notificationController, final DateService dateService) {
        super(em, Idea.class, QIdea.idea);
        this.applicationPeriodService = applicationPeriodService;
        this.firstMeetingRepository = firstMeetingRepository;
        this.notificationController = notificationController;
        this.dateService = dateService;
    }

    @Override
    public Iterable<Idea> findAll(Filter params) {
        return findAll(fromParams(params));
    }

    @Override
    public Iterable<Idea> findAll(Filter params, Pageable pageable) {
        return findAll(fromParams(params), pageable);
    }

    @Override
    public long count(Filter params) {
        return count(fromParams(params));
    }

    @Override
    public Iterable<Idea> findAuthorIdeas(User author, Pageable pageable) {
        return findAll(QIdea.idea.ideaParticipations.any().user.eq(author), pageable);
    }

    @Override
    public Iterable<Idea> findAllInList(List<Idea> list, Pageable pageable) {
        return findAll(inList(list), pageable);
    }

    @Override
    public Iterable<Idea> findReadyToStart() {
        DateExpression<Date> currentDate = DateExpression.currentDate();
        DateTimeExpression<Date> courseStartDate = QIdea.idea.applicationPeriod.courseStartDate;
        return findAll(
                allOf(
                        QIdea.idea.cachedStatus.eq(Status.MATCHED),
                        anyOf(
                                courseStartDate.before(currentDate),
                                courseStartDate.dayOfYear().eq(currentDate.dayOfYear()).and(courseStartDate.year().eq(currentDate.year()))
                        )
                )
        );
    }

    @Override
    public Long countIdeas(Status status, ProjectType pc) {
        if (pc == null) {
            return count(byStatus(status));
        } else {
            return count(byStatus(status).and(byLevel(pc)));
        }
    }

    @Override
    public Long countAuthorIdeas(User author) {
        return count(QIdea.idea.ideaParticipations.any().user.eq(author));
    }

    @Override
    public Long countAuthorsByApplicationPeriod(ApplicationPeriod ap, Filter params) {
        if (params.getStatuses().isEmpty()) {
            return EMPTY;
        }
        QIdeaParticipation participation = QIdeaParticipation.ideaParticipation;
        return from(participation).where(allOf(participation.idea.applicationPeriod.eq(ap), participation.idea.cachedStatus.in(params.getStatuses()))).count();
    }

    @Override
    public Pair<Boolean, String> validateSupervisorAcceptance(Idea idea, boolean hasTargetsLeft) {
        if (!hasTargetsLeft) {
            return new Pair<>(false, TARGET_NUMBERS_REACHED_ERROR);
        }
        if (!idea.getMatchStatus().equals(Idea.Status.UNMATCHED)) {
            return new Pair<>(false, ALREADY_MATCHED_ERROR);
        }
        return new Pair<>(true, SUPERVISING_CONFIRM + idea.getTitle());
    }

    @Override
    public Pair<Boolean, String> validateSupervisorAcceptance(Idea idea, User supervisor) {
        Target target = getTarget(idea.getApplicationPeriod(), supervisor, idea.getProjectType());
        boolean targetsLeft = target.getTarget() > countMatched(idea.getApplicationPeriod(), supervisor, idea.getProjectType());
        return validateSupervisorAcceptance(idea, targetsLeft);
    }

    public Target getTarget(ApplicationPeriod applicationPeriod, User supervisor, ProjectType projectType) {
        QTarget qTargetId = QTarget.target1;
        Target target = from(qTargetId).where(qTargetId.applicationPeriod.eq(applicationPeriod)
                .and(qTargetId.projectType.eq(projectType)
                        .and(qTargetId.user.eq(supervisor)))).uniqueResult(qTargetId);
        if (target == null) {
            target = new Target(supervisor, applicationPeriod, projectType);
        }

        return target;
    }

    // TODO: Replace with a proper ADT describing the result instead of booleans and strings
    // TODO: Merge this validation with the #acceptIdea method so that validation is always done
    @Override
    public Pair<Boolean, String> validateStudentAcceptance(Idea idea, User creator, User coAuthor, final ApplicationPeriod ap) {
        if (!idea.getMatchStatus().equals(Idea.Status.UNMATCHED)) {
            return new Pair<>(false, NO_LONGER_AVAILABLE_ERROR);
        }
        else if (idea.getProjectType().getProjectTypeSettings().getMinAuthors() > 1 && coAuthor == null && !applicationPeriodService.hasExemption(creator, ap)) {
            return new Pair<>(false, BACHELOR_NEED_PARTNER_ERROR);
        }
        else if (coAuthor != null) {
            if (coAuthor.equals(creator)) {
                return new Pair<>(false, ADD_SELF_AS_PARTNER_ERROR);
            }
            else if (authorParticipatingOnActiveIdea(coAuthor, ap)) {
                return new Pair<>(false, PARTNER_ALREADY_PARTICIPATING_ERROR);
            }
        }
        else if (authorParticipatingOnActiveIdea(creator, ap)) {
            return new Pair<>(false, ALREADY_PARTICIPATING_ERROR);
        }
        return new Pair<>(true, SELECTED_IDEA + idea.getTitle());

    }

    // TODO: Merge this validation with the #adminSetAuthors method so that validation is always done
    @Override
    public Pair<Boolean, String> validateAdminAddAuthors(final Idea idea, final Set<User> authors) {
        if (authors.isEmpty()) {
            if (idea.getType().equals(Type.STUDENT)) {
                return new Pair<>(false, NO_AUTHORS_ERROR);
            }
        } else {
            final ApplicationPeriod currentPeriod = applicationPeriodService.getCurrentPeriod(idea.getProjectType());
            final ApplicationPeriod previousPeriod = applicationPeriodService.getPreviousPeriodByProjectType(idea.getProjectType());

            if (currentPeriod == null && previousPeriod == null) {
                return new Pair<>(false, "No application period is open and no previous application period is available. Students may not be added to the idea");
            }

            if ((authors.size() < idea.getMinAuthors() || authors.size() > idea.getMaxAuthors()) &&
                    !Iterables.all(authors, new com.google.common.base.Predicate<User>() {
                        @Override
                        public boolean apply(final User author) {
                            return hasExemption(author, idea.getApplicationPeriod());
                        }}))
            {
                return new Pair<>(false, "Selected number of authors is not allowed");
            }

            ApplicationPeriod ap = currentPeriod != null ? currentPeriod : previousPeriod;
            for (User s : authors) {
                if (authorParticipatingOnOtherActiveIdea(s, ap, idea)) {
                    return new Pair<>(false, "Student already attached to another idea");
                }
            }
        }
        return new Pair<>(true, VALIDATION_PASSED);
    }

    private boolean hasExemption(User student, ApplicationPeriod applicationPeriod) {
        return applicationPeriodService.hasExemption(student, applicationPeriod);
    }

    @Override
    @Transactional
    public Idea saveSupervisorIdea(Idea idea, User supervisor, List<Keyword> keywords, boolean newIdea) {
        Set<Keyword> keywordSet = new HashSet<>(keywords);
        Idea theIdea = idea;
        theIdea.setKeywords(keywordSet);
        theIdea.setCreator(supervisor);
        theIdea.setType(Idea.Type.SUPERVISOR);
        if (newIdea) {
            theIdea = save(theIdea);
        }
        if (newIdea || theIdea.getMatchStatus().equals(Idea.Status.MATCHED)) {
            addMatch(supervisor, supervisor, theIdea, null);
        }
        if (!newIdea) {
            theIdea = save(theIdea);
        }
        return theIdea;
    }

    @Override
    @Transactional
    public Idea saveStudentIdea(Idea idea, User creator, Program creatorProgram, User coAuthor, List<Keyword> keywords, boolean isNewIdea) {
        Idea theIdea = idea;
        boolean newCoAuthor = true;
        for (IdeaParticipation nip : theIdea.getIdeaParticipations()) {
            if (nip.getUser().equals(coAuthor)) {
                newCoAuthor = false;
            }
        }
        theIdea = saveStudentIdea0(theIdea, creator, creatorProgram, coAuthor, keywords, isNewIdea);
        NotificationSource source = new NotificationSource();
        if (coAuthor != null && newCoAuthor) {
            notificationController.notifyIdea(theIdea, IdeaEvent.Event.ADDED_AS_PARTNER, source);
            source.setMessage(coAuthor.getFullName());
        }
        return theIdea;
    }

    @Transactional
    private Idea saveStudentIdea0(Idea idea, User creator, Program creatorProgram, User coAuthor, List<Keyword> keywords, boolean newIdea) {
        Idea theIdea = idea;
        Set<Keyword> keywordSet = new HashSet<>(keywords);
        theIdea.setKeywords(keywordSet);
        theIdea.setCreator(creator);
        theIdea.setType(Idea.Type.STUDENT);
        if (newIdea) {
            theIdea = save(theIdea);
        }
        IdeaParticipation ip = new IdeaParticipation(creator, theIdea);
        ip.setProgram(creatorProgram);
        theIdea.addIdeaParticipation(ip);
        if (coAuthor != null) {
            IdeaParticipation coIp = new IdeaParticipation(coAuthor, theIdea);
            theIdea.addIdeaParticipation(coIp);
        }

        ApplicationPeriod period = applicationPeriodService.getCurrentPeriod(theIdea.getProjectType(), creator);
        if (period == null) {
            LOGGER.error("Trying to save idea without a period: {}", theIdea);
        }
        theIdea.setApplicationPeriod(period);
        addMatch(creator, null, theIdea, null);

        if (!newIdea) {
            theIdea = save(theIdea);
        }
        return theIdea;
    }

    @Override
    @Transactional
    public void confirmSupervision(Idea idea, User supervisor) {
        addMatch(supervisor, supervisor, idea, new NotificationSource());
    }

    @Override
    @Transactional
    public void adminUnmatchIdea(Idea idea, User admin) {
        addMatch(admin, null, idea, new NotificationSource());
    }

    @Override
    @Transactional
    public void deleteUnmatchedIdea(Idea idea) {
        Idea reloadedIdea = findOne(idea.getId());
        if (reloadedIdea.getMatchStatus().equals(Idea.Status.UNMATCHED)) {
            delete(reloadedIdea);
        }
    }

    @Transactional
    private Idea acceptIdea0(Idea idea, User creator, Program accepteeProgram, User coAuthor, final ApplicationPeriod applicationPeriod) {

        Idea localIdea = idea;

        localIdea.setApplicationPeriod(applicationPeriod);

        if (coAuthor != null) {
            IdeaParticipation ip = new IdeaParticipation(coAuthor, localIdea);
            localIdea.addIdeaParticipation(ip);
        }

        IdeaParticipation mainIp = new IdeaParticipation(creator, localIdea);
        mainIp.setProgram(accepteeProgram);

        localIdea.addIdeaParticipation(mainIp);

        localIdea = save(localIdea);
        return addMatch(creator, localIdea.getMatch().getSupervisor(), localIdea, null);
    }

    @Override
    @Transactional
    public Idea acceptIdea(Idea idea, User creator, Program accepteeProgram, User coAuthor, final ApplicationPeriod applicationPeriod) {
        Idea theIdea = acceptIdea0(idea, creator, accepteeProgram, coAuthor, applicationPeriod);

        notificationController.notifyIdea(theIdea, IdeaEvent.Event.PARTNER_ACCEPT, new NotificationSource());

        return theIdea;
    }

    @Override
    @Transactional
    public void changeSupervisor(Idea idea, User newSelection, User creator) {
        Idea reloaded = findOne(idea.getId());
        addMatch(creator, newSelection, reloaded, null);
    }

    @Override
    @Transactional
    public void adminSetAuthors(Idea idea, Set<User> authors, User creator) {
        Idea reloaded = findOne(idea.getId());
        if (!authors.isEmpty()) {
            for (User s : authors) {
                for (Iterator<IdeaParticipation> iterator = reloaded.getIdeaParticipations().iterator(); iterator.hasNext(); ) {
                    IdeaParticipation ideaParticipation = iterator.next();
                    if (!authors.contains(ideaParticipation.getUser())) {
                        iterator.remove();
                    }
                }
                IdeaParticipation ip = new IdeaParticipation(s, reloaded);
                reloaded.addIdeaParticipation(ip);
            }
            switch (reloaded.getType()) {
                case SUPERVISOR:
                    if (!reloaded.getMatchStatus().equals(Idea.Status.MATCHED)) {
                        addMatch(creator, reloaded.getMatch().getSupervisor(), reloaded, null);
                    }
                    break;
                case STUDENT:
                default:
                    break;
            }
            if (reloaded.getApplicationPeriod() == null) {
                if (applicationPeriodService.getCurrentPeriod(reloaded.getProjectType()) != null) {
                    reloaded.setApplicationPeriod(applicationPeriodService.getCurrentPeriod(idea.getProjectType()));
                } else {
                    reloaded.setApplicationPeriod(applicationPeriodService.getPreviousPeriodByProjectType(reloaded.getProjectType()));
                }
            }
            save(reloaded);
        } else {
            LOGGER.info("All authors removed from idea '" + reloaded.getTitle() + "'");
            reloaded.getIdeaParticipations().removeAll(reloaded.getIdeaParticipations());
            if (reloaded.getFirstMeeting() != null) {
                firstMeetingRepository.delete(reloaded.getFirstMeeting().getId());
            }
            reloaded.setFirstMeeting(null);
            reloaded.setWatson(new Watson());
            reloaded.setApplicationPeriod(null);
            addMatch(creator, reloaded.getMatch().getSupervisor(), reloaded, null);
        }
    }

    @Override
    public String getStatusString(Idea idea) {
        if (idea.getMatch() == null) {
            return "-";
        }
        Status status = idea.getMatchStatus();
        String returnValue;
        switch (status) {
            case UNMATCHED:
                returnValue = "Unmatched";
                break;
            case COMPLETED:
                returnValue = "Matched, project started";
                break;
            case MATCHED:
                returnValue = getMatchedStatusString(idea);
                break;
            case INACTIVE:
                returnValue = "Inactive";
                break;
            default:
                returnValue = "-";
                break;
        }
        return returnValue;
    }

    private String getMatchedStatusString(Idea idea) {
        String returnValue;
        if (applicationPeriodService.courseStartHasPassed(idea.getApplicationPeriod())) {
            returnValue = "Matched, course started, awaiting project creation";
            if (idea.isExported()) {
                if (idea.wasExportSuccessful()) {
                    returnValue = "Matched, project started";
                } else {
                    returnValue = "Matched, idea could not be exported";
                }
            }
        } else {
            returnValue = "Matched, course starting " + dateService.format(idea.getApplicationPeriod().getCourseStartDate());
        }
        return returnValue;
    }

    @Transactional
    private Idea addMatch(User creator, User supervisor, Idea idea, NotificationSource notificationSource) {
        Match match = new Match();
        match.setIdea(idea);
        idea.setMatch(match);
        match.setChangedBy(creator);
        match.setSupervisor(supervisor);
        match.setStatus(idea.getMatchStatus());
        if (notificationSource != null) {
            notificationSource.setMessage(String.valueOf(idea.getMatchStatus()));
            notificationController.notifyIdea(idea, IdeaEvent.Event.STATUS_CHANGE, notificationSource);
        }
        return save(idea);
    }

    @Override
    public boolean authorParticipatingOnActiveIdea(User author, ApplicationPeriod ap) {
        return authorParticipatingOnOtherActiveIdea(author, ap, null);
    }

    @Override
    @Transactional
    public void inactivateIdea(Idea idea, boolean state, User creator) {
        idea.setInactive(state);
        addMatch(creator, idea.getMatch().getSupervisor(), idea, new NotificationSource());
    }

    @Override
    public Long countByApplicationPeriodAndType(ApplicationPeriod applicationPeriod, Type type, Filter params) {
        return count(byPeriod(applicationPeriod).and(byType(type).and(fromParams(params))));
    }

    @Override
    public Long countByApplicationPeriodAndProjectType(ApplicationPeriod applicationPeriod, ProjectType projectType, Filter params) {
        return count(byPeriod(applicationPeriod).and(byLevel(projectType).and(fromParams(params))));
    }

    @Override
    public Long countByApplicationPeriod(ApplicationPeriod applicationPeriod, Filter params) {
        return count(byPeriod(applicationPeriod).and(fromParams(params)));
    }

    /**
     * This method intended to only be used with supervisor ideas but there is no such type so it accepts all ideas.
     */
    @Override
    public SelectionState getSelectionState(final Idea idea) {
        if (!idea.isPublished()) {
            return SelectionState.NOT_PUBLISHED;
        }
        switch (idea.getMatchStatus()) {
            case MATCHED:
            case COMPLETED:
                return SelectionState.TAKEN;
            case INACTIVE:
                return SelectionState.INACTIVE;
            case UNMATCHED:
            default: // The match is exhaustive stupid Java
                return isSelectable(idea);
        }
    }

    private SelectionState isSelectable(final Idea idea) {
        return Iterables.tryFind(applicationPeriodService.getCurrentPeriods(), new com.google.common.base.Predicate<ApplicationPeriod>() {
            @Override
            public boolean apply(final ApplicationPeriod applicationPeriod) {
                return applicationPeriod.getProjectTypes().contains(idea.getProjectType());
            }
        }).transform(new Function<ApplicationPeriod, SelectionState>() {
            @Override
            public SelectionState apply(final ApplicationPeriod applicationPeriod) {
                return isSelectableInBasedOnTarget(idea, applicationPeriod);
            }
        }).or(SelectionState.NO_APPLICATION_PERIOD);
    }

    private SelectionState isSelectableInBasedOnTarget(final Idea idea, final ApplicationPeriod applicationPeriod) {
        Target target = getTarget(applicationPeriod, idea.getCreator(), idea.getProjectType());
        long matched = countMatched(applicationPeriod, idea.getCreator(), idea.getProjectType());
        return matched >= target.getTarget() ? SelectionState.NO_TARGETS_LEFT : SelectionState.AVAILABLE;
    }

    private boolean authorParticipatingOnOtherActiveIdea(User author, ApplicationPeriod ap, Idea ideaToSave) {
        for (Idea idea : findAll(QIdea.idea.ideaParticipations.any().user.eq(author))) {
            if (idea.getApplicationPeriod().equals(ap) && !idea.equals(ideaToSave) && !idea.isInactive()) {
                return true;
            }
        }
        return false;
    }

    private BooleanBuilder fromParams(Filter params) {
        BooleanBuilder bb = new BooleanBuilder();
        if (params.getPublished() != null) {
            bb.and(isPublished(params.getPublished()));
        }
        if (params.getStatuses() != null) {
            if (params.getStatuses().isEmpty()) {
                bb.and(QIdea.idea.cachedStatus.isNull());
            } else {
                bb.and(QIdea.idea.cachedStatus.in(params.getStatuses()));
            }
        }
        if (params.getLevels() != null) {
            bb.and(levelFilter(params.getLevels()));
        }
        if (params.getSupervisor() != null) {
            bb.and(bySupervisor(params.getSupervisor()));
        }
        if (params.getSupervisor() == null && params.getNoSupervisor() != null && params.getNoSupervisor()) {
            bb.and(hasNoSupervisor());
        }
        if (params.getTypes() != null) {
            bb.and(typeFilter(params.getTypes()));
        }
        if (params.getResearchArea() != null) {
            bb.and(byArea(params.getResearchArea()));
        }
        if (params.getKeywords() != null && !params.getKeywords().isEmpty()) {
            bb.and(keywordFilter(params.getKeywords()));
        }
        if (params.getUnit() != null) {
            bb.and(byUnit(params.getUnit()));
        }
        if (params.getApplicationPeriod() != null) {
            bb.and(byPeriod(params.getApplicationPeriod()));
        }
        if (params.getTitleContains() != null && params.getTitleContains().length() >= TITLE_LENGTH) {
            bb.and(titleContains(params.getTitleContains()));
        }
        if (params.getCreatedAfter() != null) {
            bb.and(dateFilter(params.getCreatedAfter(), params.getCreatedBefore()));
        }
        if (params.getAuthor() != null) {
            bb.and(authorFilter(params.getAuthor()));
        }
        if (params.getProgram() != null) {
            bb.and(programFilter(params.getProgram()));
        }
        if (params.getNoFirstMeeting()) {
            bb.and(noFirstMeetingFilter());
        }
        return bb;
    }

    private Predicate noFirstMeetingFilter() {
        return new JPASubQuery().from(QFirstMeeting.firstMeeting).where(QFirstMeeting.firstMeeting.idea.eq(QIdea.idea)).notExists();
    }

    private BooleanBuilder levelFilter(Collection<ProjectType> levels) {
        BooleanBuilder bb = new BooleanBuilder();
        if (levels != null && !levels.isEmpty()) {
            for (ProjectType level : levels) {
                bb.or(byLevel(level));
            }
        } else {
            bb.and(QIdea.idea.projectType.isNull());
        }
        return bb;
    }

    private BooleanBuilder typeFilter(Collection<Type> types) {
        BooleanBuilder bb = new BooleanBuilder();
        if (types != null && !types.isEmpty()) {
            for (Type t : types) {
                bb.or(byType(t));
            }
        } else {
            bb.and(QIdea.idea.type.isNull());
        }
        return bb;
    }

    private BooleanExpression dateFilter(Date from, Date to) {
        if (from != null && to != null) {
            return QIdea.idea.dateCreated.between(from, to);
        } else {
            return null;
        }
    }

    private BooleanExpression authorFilter(User author) {
        return QIdea.idea.ideaParticipations.any().user.eq(author);
    }

    private BooleanExpression programFilter(Program program) {
        return QIdea.idea.ideaParticipations.any().program.eq(program);
    }

    private BooleanExpression keywordFilter(Set<Keyword> keywords) {
        return QIdea.idea.keywords.any().in(keywords);
    }

    private BooleanExpression byLevel(ProjectType level) {
        return QIdea.idea.projectType.eq(level);
    }

    private BooleanExpression bySupervisor(User supervisor) {
        return QIdea.idea.match.supervisor.eq(supervisor);
    }

    private BooleanExpression byType(Type type) {
        return QIdea.idea.type.eq(type);
    }

    private BooleanExpression byStatus(Status status) {
        return QIdea.idea.match.status.eq(status);
    }

    private BooleanExpression byArea(ResearchArea area) {
        return QIdea.idea.researchArea.eq(area);
    }

    private BooleanExpression byUnit(Unit unit) {
        return QIdea.idea.match.supervisor.unit.eq(unit);
    }

    private BooleanExpression byPeriod(ApplicationPeriod period) {
        return QIdea.idea.applicationPeriod.eq(period);
    }

    private BooleanExpression hasNoSupervisor() {
        return QIdea.idea.match.supervisor.isNull();
    }

    private BooleanExpression isPublished(boolean isPublished) {
        return QIdea.idea.published.eq(isPublished);
    }

    private BooleanExpression titleContains(String titleContains) {
        if (titleContains != null) {
            return QIdea.idea.title.contains(titleContains);
        } else {
            return null;
        }
    }

    private BooleanExpression inList(List<Idea> ideas) {
        return QIdea.idea.in(ideas);
    }

    @Transactional
    @Override
    public Idea savePartnerProgram(Idea idea, User student, Program program) {
        for (IdeaParticipation npi : idea.getIdeaParticipations()) {
            if (npi.getUser().equals(student)) {
                npi.setProgram(program);
            }
        }
        return save(idea);
    }

    @Override
    public Long countMatched(ApplicationPeriod ap, User supervisor, ProjectType pc) {
        Filter params = new Filter();
        Set<Status> status = new HashSet<>();
        status.add(Idea.Status.MATCHED);
        status.add(Idea.Status.COMPLETED);
        params.setStatuses(status);
        params.setApplicationPeriod(ap);
        params.setSupervisor(supervisor);
        List<ProjectType> level = new ArrayList<>();
        level.add(pc);
        params.setLevels(level);
        return count(params);
    }

}
