package se.su.dsv.scipro.match;

import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.collect.FluentIterable;
import com.google.common.primitives.Ints;
import com.mysema.query.jpa.JPASubQuery;
import com.mysema.query.jpa.impl.JPAQuery;
import com.mysema.query.types.EntityPath;
import com.mysema.query.types.Projections;
import com.mysema.query.types.path.StringPath;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.List;
import java.util.Set;

import static com.mysema.query.support.Expressions.anyOf;

class MatchFollowUpServiceImpl implements MatchFollowUpService {

    private final Provider<EntityManager> entityManagerProvider;

    @Inject
    MatchFollowUpServiceImpl(final Provider<EntityManager> entityManagerProvider) {
        this.entityManagerProvider = entityManagerProvider;
    }

    private JPAQuery from(final EntityPath<?>... entityPath) {
        return new JPAQuery(entityManagerProvider.get()).from(entityPath);
    }

    private JPASubQuery getQuery(ApplicationPeriod applicationPeriod) {
        return new JPASubQuery()
                .from(QIdea.idea)
                .where(QIdea.idea.published.eq(true),
                        QIdea.idea.projectType.id.eq(QTarget.target1.projectType.id))
                .where(QIdea.idea.match.supervisor.id.eq(QTarget.target1.user.id),
                        anyOf(QIdea.idea.applicationPeriod.id.eq(applicationPeriod.getId()),
                              QIdea.idea.cachedStatus.in(Idea.Status.UNMATCHED)
                        )
                );
    }

    private static Predicate<TargetResult> unmet() {
        return new Predicate<TargetResult>() {
            @Override
            public boolean apply(final TargetResult input) {
                return input != null && (input.getTarget().getTarget() > input.getSubmittedAndTaken());
            }
        };
    }

    @Override
    public long countUnmetTargets(final ApplicationPeriod applicationPeriod) {
        List<TargetResult> list = from(QTarget.target1)
                .where(QTarget.target1.applicationPeriod.eq(applicationPeriod),
                        QTarget.target1.target.gt(0))
                .list(Projections.constructor(TargetResult.class,
                        QTarget.target1,
                        getQuery(applicationPeriod)
                                .count()
                ));


        return FluentIterable.from(list)
                .filter(unmet())
                .size();
    }

    @Override
    public Iterable<TargetResult> findUnmetTargets(final ApplicationPeriod applicationPeriod, final Pageable pageable) {
        StringPath sort = new StringPath(QTarget.target1, pageable.getSort().getProperty());
        List<TargetResult> targetResults = from(QTarget.target1)
                .where(QTarget.target1.applicationPeriod.eq(applicationPeriod),
                        QTarget.target1.target.gt(0))
                .orderBy(pageable.getSort().isAscending() ? sort.asc() : sort.desc())
                .list(Projections.constructor(TargetResult.class,
                        QTarget.target1,
                        getQuery(applicationPeriod)
                                .count()
                ));

        return FluentIterable.from(targetResults)
                .filter(unmet())
                .skip(Ints.checkedCast(pageable.getOffset()))
                .limit(Ints.checkedCast(pageable.getLimit()))
                .toList();
    }

    @Override
    public Set<User> findSupervisorsWithUnmetTargets(ApplicationPeriod applicationPeriod) {
        List<TargetResult> targetResults = from(QTarget.target1)
                .where(QTarget.target1.applicationPeriod.eq(applicationPeriod),
                        QTarget.target1.target.gt(0))
                .list(Projections.constructor(TargetResult.class,
                        QTarget.target1,
                        getQuery(applicationPeriod)
                                .count()
                ));

        return FluentIterable.from(targetResults)
                .filter(unmet())
                .transform(new Function<TargetResult, User>() {
                    @Override
                    public User apply(final TargetResult unmet) {
                        return unmet.getTarget().getUser();
                    }
                })
                .toSet();
    }
}
