package se.su.dsv.scipro.match;

import com.mysema.query.BooleanBuilder;
import com.mysema.query.types.expr.BooleanExpression;
import se.su.dsv.scipro.reusable.SciProUtilities;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.*;

class TargetServiceImpl extends AbstractServiceImpl<Target, TargetId> implements TargetService {

    private final IdeaService ideaService;

    @Inject
    TargetServiceImpl(Provider<EntityManager> em, final IdeaService ideaService) {
        super(em, Target.class, QTarget.target1);
        this.ideaService = ideaService;
    }

    @Override
    public Target findOne(ApplicationPeriod applicationPeriod, User supervisor, ProjectType projectType) {
        Target t = findOne(byApplicationPeriod(applicationPeriod).and(bySupervisor(supervisor)).and(byLevel(projectType)));
        if (t == null) {
            t = new Target(supervisor, applicationPeriod, projectType);
        }

        return t;
    }

    private Iterator<Target> findAll(FilterParams params) {
        return findAll(fromParams(params)).iterator();
    }

    @Override
    public boolean hasTargetsLeft(ApplicationPeriod applicationPeriod, User supervisor) {
        TargetService.FilterParams params = new FilterParams();
        params.setApplicationPeriod(applicationPeriod);
        params.setSupervisor(supervisor);

        Iterator<Target> i = findAll(params);
        while (i.hasNext()) {
            Target t = i.next();
            ProjectType pc = t.getProjectType();
            if (t.getTarget() > 0 && t.getTarget() > ideaService.countMatched(applicationPeriod, supervisor, pc)) {
                return true;
            }
        }

        return false;
    }

    @Override
    public boolean hasTargetsLeft(ApplicationPeriod applicationPeriod, User supervisor, ProjectType projectType) {
        Target t = findOne(applicationPeriod, supervisor, projectType);
        return (t.getTarget() > 0 && t.getTarget() > ideaService.countMatched(applicationPeriod, supervisor, projectType));
    }

    @Override
    public Set<ProjectType> projectTypesWithTargetsLeft(ApplicationPeriod applicationPeriod, User supervisor) {
        Set<ProjectType> projectTypes = new HashSet<>();
        TargetService.FilterParams params = new FilterParams();
        params.setApplicationPeriod(applicationPeriod);
        params.setSupervisor(supervisor);

        Iterator<Target> i = findAll(params);
        while (i.hasNext()) {
            Target t = i.next();
            ProjectType pc = t.getProjectType();
            if (t.getTarget() > 0 && t.getTarget() > ideaService.countMatched(applicationPeriod, supervisor, pc)) {
                projectTypes.add(pc);
            }
        }

        return projectTypes;
    }

    @Override
    public int ideasLeftToCreateToMeetTarget(ApplicationPeriod ap, User supervisor, ProjectType pc) {
        IdeaService.Filter countParams = new IdeaService.Filter();
        countParams.setSupervisor(supervisor);

        countParams.setTypes(EnumSet.of(Idea.Type.SUPERVISOR));

        countParams.setStatuses(EnumSet.of(Idea.Status.UNMATCHED));
        countParams.setPublished(true);

        List<ProjectType> projectTypes = new ArrayList<>();
        projectTypes.add(pc);
        countParams.setLevels(projectTypes);

        int availableIdeas = SciProUtilities.safeLongToInt(ideaService.count(countParams));

        int ideasLeftToCreate = findOne(ap, supervisor, pc).getTarget() - availableIdeas;
        ideasLeftToCreate = ideasLeftToCreate - SciProUtilities.safeLongToInt(ideaService.countMatched(ap, supervisor, pc));

        if (ideasLeftToCreate > 0) {
            return ideasLeftToCreate;
        } else {
            return 0;
        }
    }

    private BooleanBuilder fromParams(TargetService.FilterParams params) {
        BooleanBuilder bb = new BooleanBuilder();
        if (params.getProjectTypes() != null && !params.getProjectTypes().isEmpty()) {

            BooleanBuilder b = new BooleanBuilder();
            for (ProjectType pc : params.getProjectTypes()) {
                b.or(byLevel(pc));
            }
            bb.and(b);

        }
        if (params.getSupervisor() != null) {
            bb.and(bySupervisor(params.getSupervisor()));
        }
        if (params.getApplicationPeriod() != null) {
            bb.and(byApplicationPeriod(params.getApplicationPeriod()));
        }
        return bb;
    }

    public static BooleanExpression byApplicationPeriod(ApplicationPeriod ap) {
        return QTarget.target1.targetId.applicationPeriodId.eq(ap.getId());
    }

    public static BooleanExpression bySupervisor(User supervisor) {
        return QTarget.target1.user.eq(supervisor);
    }

    public static BooleanExpression byLevel(ProjectType pc) {
        return QTarget.target1.targetId.projectTypeId.eq(pc.getId());
    }
}
