package se.su.dsv.scipro.notifications.settings.service;

import com.google.inject.persist.Transactional;
import scala.Option;
import scala.runtime.AbstractFunction0;
import scala.runtime.AbstractFunction1;
import se.su.dsv.scipro.notifications.dataobject.Notification;
import se.su.dsv.scipro.notifications.settings.entities.DeliveryConfiguration;
import se.su.dsv.scipro.notifications.settings.entities.DeliveryMethod;
import se.su.dsv.scipro.notifications.settings.entities.QDeliveryConfiguration;
import se.su.dsv.scipro.springdata.services.UserProfileService;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;

import static com.mysema.query.support.Expressions.allOf;

public class DeliveryConfigurationServiceImpl extends AbstractServiceImpl<DeliveryConfiguration, Long> implements DeliveryConfigurationService {

    private final UserProfileService profileService;

    @Inject
    public DeliveryConfigurationServiceImpl(Provider<EntityManager> em, UserProfileService profileService) {
        super(em, DeliveryConfiguration.class, QDeliveryConfiguration.deliveryConfiguration);
        this.profileService = profileService;
    }

    @Override
    @Transactional
    public boolean setDelivery(final Notification.Type type, final Enum event, final DeliveryMethod method, final boolean enabled, final Option<User> user) {
        DeliveryConfiguration configuration = user.map(new AbstractFunction1<User, DeliveryConfiguration>() {
            @Override
            public DeliveryConfiguration apply(final User user) {
                return getUserConfiguration(type, event, method, user).getOrElse(new AbstractFunction0<DeliveryConfiguration>() {
                    @Override
                    public DeliveryConfiguration apply() {
                        return createBasedOnSystemConfiguration(type, event, method, user);
                    }
                });
            }
        }).getOrElse(new AbstractFunction0<DeliveryConfiguration>() {
            @Override
            public DeliveryConfiguration apply() {
                return getSystemConfiguration(type, event, method);
            }
        });
        configuration.setEnabled(enabled);
        configuration = save(configuration);
        return configuration.isEnabled();
    }

    private DeliveryConfiguration createBasedOnSystemConfiguration(final Notification.Type type, final Enum event, final DeliveryMethod method, final User user) {
        final DeliveryConfiguration systemConfiguration = getSystemConfiguration(type, event, method);
        final DeliveryConfiguration userConfiguration = new DeliveryConfiguration(type, event.name(), method, user);
        userConfiguration.setEnabled(systemConfiguration.isEnabled());
        return userConfiguration;
    }

    @Override
    public boolean isDelivery(final Notification.Type type, final Enum event, final DeliveryMethod method, final Option<User> user) {
        return user
                .flatMap(new AbstractFunction1<User, Option<DeliveryConfiguration>>() {
                    @Override
                    public Option<DeliveryConfiguration> apply(final User v1) {
                        return getUserConfiguration(type, event, method, v1);
                    }
                })
                .getOrElse(new AbstractFunction0<DeliveryConfiguration>() {
                    @Override
                    public DeliveryConfiguration apply() {
                        return getSystemConfiguration(type, event, method);
                    }
                })
                .isEnabled();
    }

    @Override
    public boolean isMailCompilation(final User user) {
        return profileService.findByUser(user).isMailCompilation();
    }

    private Option<DeliveryConfiguration> getUserConfiguration(final Notification.Type type, final Enum event, final DeliveryMethod method, final User user) {
        QDeliveryConfiguration q = QDeliveryConfiguration.deliveryConfiguration;
        return Option.apply(findOne(allOf(q.type.eq(type), q.event.eq(event.name()), q.method.eq(method), q.user.eq(user))));
    }

    private DeliveryConfiguration getSystemConfiguration(final Notification.Type type, final Enum event, final DeliveryMethod method) {
        QDeliveryConfiguration q = QDeliveryConfiguration.deliveryConfiguration;
        DeliveryConfiguration configuration =
                findOne(allOf(q.type.eq(type), q.event.eq(event.name()), q.method.eq(method), q.user.isNull()));

        if (configuration == null) {
            configuration = new DeliveryConfiguration(type, event.name(), method, null); // Still null, create new setting
        }

        return configuration;
    }
}
