package se.su.dsv.scipro.plagiarism;

import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.multipart.*;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.mail.MailEvent;
import se.su.dsv.scipro.mail.MailEventService;
import se.su.dsv.scipro.reusable.SciProUtilities;

import javax.inject.Inject;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.TimeZone;


/**
 * INTRODUCTION This class handles our communication with turnitin. We also have a TurnItInCheck class and a
 * turnitin worker which are used to tie up events in scipro with the communication with turnitin. It provides a
 * clean interface consisting of methods that return only the values developers are interested in and hides the
 * rest in this class. Originally the methods in here were used to communicate with turnitins "quicksubmit"
 * feature, but that feature was dropped. Turnitin wanted us to adapt more to their processes. In turnitin,
 * licenses have users and accounts, which have classes, which have assignments and instructors. Students are
 * then supposed to register in a class and upload files to individual assignments, after which the instructor
 * for the class gets access to originality reports.
 * <p/>
 * HOW WE WORK WITH TURNITIN (ABSTRACT LEVEL) The way we decided to work with turnitin was to set up a class in
 * our turnitin account which corresponds to our thesis course. The instructor for the class in turnitin should
 * be a turnitin user which scipro can use to upload files. The class has only one assignment in turnitin, which
 * is used for all file uploads. Since scipro uses a single instructor turnitin user to upload all the files,
 * there is no need for students to register in a class an upload files to the correct assignment etc.
 * <p/>
 * For this to work, some things need to be setup in and acquired from turnitin first:
 * <p/>
 * REQUIREMENTS -A turnitin user which scipro will use to communicate with turnitin. -A turnitin account and its
 * account number. NOTE the account must have a custom integration configured and a "shared key" for the
 * integration. -A turnitin class with class id and unique class name setup in the account. The instructor for
 * the class should be the same turnitin user as scipro uses to communicate with turnitin. -An assignment with a
 * unique name needs to be created in the class. -A turnitin "integration id" (different ids for
 * testing/development and production, a production id is awarded by turnitin after theyve seen and approved the
 * integration.
 * <p/>
 * HOW IT WORKS (LOW LEVEL) Communicating with turnitin is done through a REST-based API. The client makes GET
 * or POST requests containing arguments to an https address at turnitin. There are some default arguments which
 * must be present in every request, and some request-specific arguments. Some of the arguments are only found
 * in this file and some are loaded from GeneralSystemSettings. Some of the arguments are hashed in alphabetical
 * order with md5 before they are sent. In the DropBox folder ProjectThesis/SciPro there is an API manual from
 * turnitin with more detailed instructions. We only use methods for posting a file to turnitin and downloading
 * a file from turnitin.
 */

public class TurnitinComponent implements PlagiarismControl {

    private static final Logger LOGGER = LoggerFactory.getLogger(TurnitinComponent.class);

    @Inject
    private GeneralSystemSettingsService systemSettingsService;
    @Inject
    private MailEventService mailEventService;
    @Inject
    private TurnitinSettingsService turnitinSettingsService;

    private String aid;
    private String diagnostic;
    private String encrypt = "0"; //using https connection, not much need for encryption
    private String gmtime;
    private String uem;
    private String ufn;
    private String uln;
    private String utp;
    private String sharedKey;
    private String apiAddress;

    private String src;
    private TurnitinSettings settings;
    private static final String RETURN_REPORT = "6";
    private static final String API_OPEN_NEW_WINDOW = "1";
    private static final String UPLOADED_FILE_ID = "5";
    private static final String API_REDIRECT = "2";
    private static final String FIRST_NAME = "John";
    private static final String LAST_NAME = "Smith";
    private static final String FILE_TYPE_IS_FILE = "2";

    private void sendReminder(TurnitinSettings settings) {
        Date today = new Date();
        Date lastFailedDate = settings.getLastFailedDate();

        if (lastFailedDate == null || !DateUtils.isSameDay(today, lastFailedDate)) {
            MailEvent mail = buildMail();
            mailEventService.save(mail);
            settings.setLastFailedDate(today);
            turnitinSettingsService.save(settings);
        }
    }

    private MailEvent buildMail() {
        GeneralSystemSettings generalSystemSettings = systemSettingsService.getGeneralSystemSettingsInstance();
        MailEvent mail = new MailEvent();
        mail.setFromEmail(generalSystemSettings.getSystemFromMail());
        mail.setFromName(generalSystemSettings.getMailFromName());
        mail.setSubject("Turnitin worker failed");
        mail.setMessageBody("The Turnitin worker did not get the desired return code when attempting to upload a file.");
        mail.setNonUserRecipients(new HashSet<>(settings.getExpirationMails()));
        return mail;
    }

    private void setupVariables() {
        settings = turnitinSettingsService.getSettings();
        SimpleDateFormat dateFormatGmt = new SimpleDateFormat("yyyyMMddHHmm");
        dateFormatGmt.setTimeZone(TimeZone.getTimeZone("GMT"));
        gmtime = dateFormatGmt.format(new Date());
        gmtime = gmtime.substring(0, gmtime.length() - 1);

        aid = String.valueOf(settings.getAccountNo()); //primary account id
        /**
         * According to turnitin, diagnostic flag should be 1 when testing, 0 when in production. But since
         * the response from turnitin is not valid XML, our return parsing method fails if this flag is not
         * set to 0. Also, since we are just making calls to the turnitin sandbox when the DEV profile is
         * activated, it arguably doesn't matter if this is set to 0 always. Set it to 1 if you wish to get
         * a more detailed response from turnitin and print that to screen though, e.g. you can compare the
         * md5 we generate locally to the one they generated etc.
         */
        diagnostic = "0"; //diagnostic flag should be 1 when testing, 0 when production

        uem = settings.getUsername();
        ufn = settings.getFirstName();
        uln = settings.getLastName();
        utp = "2"; //user type (1 for students, 2 for instructors, and 3 for administrators)

        sharedKey = settings.getSharedKey(); //shared key for integration
        src = String.valueOf(settings.getIntegrationId());
        apiAddress = settings.getIntegrationURL();
    }

    @Override
    public String returnReport(String oid) {

        setupVariables();

        LOGGER.info("Return report function starting for turnitin check with turnitin id " + oid);
        LOGGER.info("Turnitin API address: " + apiAddress);

        PostMethod filePost = new PostMethod(apiAddress + "api.asp");

        /**
         * Prepare md5. These variables must come in alphabetical order, except shared key which should come last.
         */
        String hash = aid +
                diagnostic +
                encrypt +
                API_OPEN_NEW_WINDOW +
                RETURN_REPORT +
                gmtime +
                oid +
                uem +
                ufn +
                uln +
                utp +
                sharedKey;
        String md5 = SciProUtilities.md5(hash);

        filePost.setParameter("aid", aid);
        filePost.setParameter("diagnostic", diagnostic);
        filePost.setParameter("encrypt", encrypt);
        filePost.setParameter("fcmd", API_OPEN_NEW_WINDOW);
        filePost.setParameter("fid", RETURN_REPORT);
        filePost.setParameter("gmtime", gmtime);
        filePost.setParameter("md5", md5);
        filePost.setParameter("oid", oid);
        filePost.setParameter("uem", uem);
        filePost.setParameter("ufn", ufn);
        filePost.setParameter("uln", uln);
        filePost.setParameter("utp", utp);
        filePost.setParameter("sharedKey", sharedKey);
        filePost.setParameter("src", src);

        HttpClient httpClient = new HttpClient();
        try {
            httpClient.executeMethod(filePost);
            String redirectLocation;
            Header locationHeader = filePost.getResponseHeader("location");
            if (locationHeader != null) {
                redirectLocation = locationHeader.getValue();
                if (!redirectLocation.startsWith("new")) {
                    return null;
                }
                return apiAddress + redirectLocation;
            }
        } catch (IOException e) {
            LOGGER.error("Error while posting to turnitin api: " + e.getMessage(), e);
        } finally {
            filePost.releaseConnection();
        }
        return null;
    }

    @Override
    public String submitPaper(final TurnItInCheck turnItInCheck) {

        setupVariables();

        LOGGER.info("Submit paper function starting for turnitincheck id " + turnItInCheck.getId());
        LOGGER.info("Turnitin API address: " + apiAddress);

        final FileDescription fileDescription = turnItInCheck.getFileDescription();

        String paperTitle = SciProUtilities.convertToHex(turnItInCheck.getHash()); //title of paper to submit (required)

//        pdata: paper in original format (required) THIS COMES LATER WHEN WE ACTUALLY MAKE THE REQUEST

        String className = settings.getClassName(); //title of class to submit paper to (required)
        String assignmentName = settings.getAssignmentName(); //title of assignment to submit paper to (unique for that class) (required)

        /**
         * Prepare md5. These variables must come in alphabetical order, except shared key which should come last.
         */
        String hash = createHashString(paperTitle, className, assignmentName);

        String md5;

        LOGGER.info("String to be md5'd: " + hash);
        BufferedReader bufferedReader = null;
        PostMethod postMethod = new PostMethod(apiAddress + "api.asp");

        /**
         * Create md5
         */
        try {
            md5 = SciProUtilities.md5(hash);

            LOGGER.info("Calculated md5: " + md5);

            /**
             * Prepare POST
             */
            Part[] parts = createParts(paperTitle, className, assignmentName, md5, fileDescription);

            postMethod.setRequestEntity(new MultipartRequestEntity(parts, postMethod.getParams()));

            HttpClient httpClient = new HttpClient();

            /**
             * Parse XML response
             */
            StringBuilder xml = new StringBuilder();
            httpClient.executeMethod(postMethod);
            bufferedReader = new BufferedReader(new InputStreamReader(postMethod.getResponseBodyAsStream(), StandardCharsets.UTF_8));
            String readLine;

            while (((readLine = bufferedReader.readLine()) != null)) {
                xml.append(readLine);
                LOGGER.info("Return line from Turnitin: " + readLine);
            }


            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = documentBuilder.parse(new InputSource(new StringReader(xml.toString())));

            /**
             * objectID is the value we are after
             */
            document.getDocumentElement().normalize();
            NodeList nodeList = document.getElementsByTagName("rcode");
            Element studentPaperOverlapElement = (Element) nodeList.item(0);
            NodeList studentPaperOverlap = studentPaperOverlapElement.getChildNodes();
            if (studentPaperOverlap.item(0).getNodeValue().equals("51")) {
                nodeList = document.getElementsByTagName("objectID");
                Element objectIdElement = (Element) nodeList.item(0);
                NodeList objectId = objectIdElement.getChildNodes();
                return objectId.item(0).getNodeValue();
            } else {
                sendReminder(settings);
            }
        } catch (ParserConfigurationException e) {
            LOGGER.error("Error while creating DocumentBuilder: " + e.getMessage(), e);
        } catch (SAXException e) {
            LOGGER.error("Error while parsing the content of InputSource: " + e.getMessage(), e);
        } catch (IOException e) {
            LOGGER.error("Error while posting to turnitin api: " + e.getMessage(), e);
        } finally {
            if (bufferedReader != null) {
                try {
                    bufferedReader.close();
                } catch (IOException e) {
                    LOGGER.error("Error while posting to turnitin api: " + e.getMessage(), e);
                }
            }
            postMethod.releaseConnection();
        }
        return null;
    }

    private Part[] createParts(String paperTitle, String className, String assign, String md5, final FileDescription fileDescription) {
        return new Part[]{new StringPart("aid", aid),
                new StringPart("assign", assign),
                new StringPart("encrypt", encrypt),
                new StringPart("ctl", className),
                new StringPart("diagnostic", diagnostic),
                new StringPart("fcmd", API_REDIRECT),
                new StringPart("fid", UPLOADED_FILE_ID),
                new StringPart("gmtime", gmtime),
                new StringPart("md5", md5),
                new StringPart("pfn", FIRST_NAME),
                new StringPart("pln", LAST_NAME),
                new StringPart("ptl", paperTitle),
                new StringPart("ptype", FILE_TYPE_IS_FILE),
                new StringPart("uem", uem),
                new StringPart("ufn", ufn),
                new StringPart("uln", uln),
                new StringPart("utp", utp),
                new StringPart("sharedKey", sharedKey),
                new StringPart("src", src),
                new FilePart("pdata", new PartSource() {

                    @Override
                    public long getLength() {
                        return fileDescription.getSize();
                    }

                    @Override
                    public String getFileName() {
                        return fileDescription.getName();
                    }

                    @Override
                    public InputStream createInputStream() throws IOException {
                        return fileDescription.getData();
                    }

                })};
    }

    private String createHashString(String paperTitle, String className, String assign) {
        return aid +
                assign +
                className +
                diagnostic +
                encrypt +
                API_REDIRECT +
                UPLOADED_FILE_ID +
                gmtime +
                FIRST_NAME +
                LAST_NAME +
                paperTitle +
                FILE_TYPE_IS_FILE +
                uem +
                ufn +
                uln +
                utp +
                sharedKey;
    }
}
