package se.su.dsv.scipro.project;

import com.google.common.base.Joiner;
import com.mysema.query.annotations.QueryInit;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.reusable.SciProUtilities;
import se.su.dsv.scipro.system.*;

import javax.persistence.*;
import java.util.*;

@Data
@EqualsAndHashCode(callSuper = false, of = "id")
@ToString(of = {"id", "identifier", "title", "projectType", "headSupervisor", "projectParticipants"})
@Entity
@Table(name = "project")
@Cacheable(true)

@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class Project extends DomainObject {
    public static final String NO_REVIEWER = "No reviewer";
    public static final String NO_CO_SUPERVISOR = "No co-supervisor";
    public static final int TITLE_MAX_LENGTH = 255;

    @Id
    @GeneratedValue
    private Long id;

    @Column(unique = true)
    private Long identifier;

    @Column(length = TITLE_MAX_LENGTH)
    @Basic(optional = false)
    private String title;

    @ManyToMany
    @JoinTable(name = "project_user", inverseJoinColumns = @JoinColumn(name = "user_id"))
    private Set<User> projectParticipants = new TreeSet<>(new User.ByNameComparator());

    @ManyToMany
    @JoinTable(name = "project_reviewer", inverseJoinColumns = @JoinColumn(name = "user_id"))
    private Set<User> reviewers = new TreeSet<>(new User.ByNameComparator());

    @ManyToMany
    @JoinTable(name = "project_cosupervisor", inverseJoinColumns = @JoinColumn(name = "user_id"))
    private Set<User> coSupervisors = new TreeSet<>(new User.ByNameComparator());

    @ManyToOne(optional = false)
    @QueryInit({"unit"})
    @JoinColumn(name = "supervisor_id")
    private User headSupervisor;

    @Enumerated(EnumType.STRING)
    private ProjectStatus projectStatus = ProjectStatus.ACTIVE;

    @Enumerated(EnumType.STRING)
    private StateOfMind stateOfMind = StateOfMind.FINE;

    @Basic(optional = true)
    private Date stateOfMindDate;

    @Basic(optional = true)
    private String stateOfMindReason;

    @ManyToOne(optional = false)
    private ProjectType projectType;

    @Basic(optional = false)
    private String statusMessage = "";

    @Embedded
    @AttributeOverride(name = "name", column = @Column(name = "externalOrganization"))
    private ExternalOrganization externalOrganization;

    @Column(name = "fs_rule_exmpt")
    private boolean finalSeminarRuleExempted = false;

    @Basic
    private int credits;

    @ManyToOne(optional = true)
    private ResearchArea researchArea;

    @PrePersist
    @PreUpdate
    void cleanTitle() {
        title = title.replaceAll("\\p{C}", "");
    }

    public boolean isFinalSeminarRuleExempted() {
        return finalSeminarRuleExempted;
    }

    public void setFinalSeminarRuleExempted(boolean finalSeminarRuleExempted) {
        this.finalSeminarRuleExempted = finalSeminarRuleExempted;
    }

    public User getHeadSupervisor() {
        return headSupervisor;
    }

    public ProjectType getProjectType() {
        return projectType;
    }

    public SortedSet<User> getCoSupervisors() {
        TreeSet<User> s = new TreeSet<>(new User.ByNameComparator());
        s.addAll(coSupervisors);
        return Collections.unmodifiableSortedSet(s);
    }

    public void setCoSupervisors(SortedSet<User> coSupervisors) {
        this.coSupervisors.clear();
        this.coSupervisors.addAll(coSupervisors);
    }

    public void addCoSupervisor(User coSupervisor) {
        coSupervisors.add(coSupervisor);
    }

    public SortedSet<User> getReviewers() {
        TreeSet<User> s = new TreeSet<>(new User.ByNameComparator());
        s.addAll(reviewers);
        return Collections.unmodifiableSortedSet(s);
    }

    public void setReviewers(SortedSet<User> reviewers) {
        this.reviewers.clear();
        this.reviewers.addAll(reviewers);
    }

    public void addReviewer(User reviewer) {
        reviewers.add(reviewer);
    }

    public void removeReviewer(User reviewer) {
        reviewers.remove(reviewer);
    }

    public void setProjectStatus(ProjectStatus projectStatus) {
        this.projectStatus = projectStatus;
        if (projectStatus == ProjectStatus.COMPLETED) {
            this.stateOfMind = StateOfMind.FINE;
        }
    }

    public Date getStateOfMindDate() {
        return stateOfMindDate == null ? null
                : new Date(stateOfMindDate.getTime());
    }

    public void setStateOfMindDate(Date stateOfMindDate) {
        this.stateOfMindDate = stateOfMindDate == null ? null
                : new Date(stateOfMindDate.getTime());
    }

    public String getTitle() {
        return SciProUtilities.cleanString(title);
    }

    public String getTitle(final int characters) {
        String result = getTitle();
        if (getTitle().length() > characters) {
            result = getTitle().substring(0, characters) + "..";
        }
        return result;
    }

    public SortedSet<User> getProjectParticipants() {
        TreeSet<User> s = new TreeSet<>(new User.ByNameComparator());
        s.addAll(projectParticipants);
        return Collections.unmodifiableSortedSet(s);
    }

    public void setProjectParticipants(SortedSet<User> projectParticipants) {
        this.projectParticipants.clear();
        this.projectParticipants.addAll(projectParticipants);
    }

    public void addProjectParticipant(User s) {
        projectParticipants.add(s);
    }

    //TODO remove this method
    public User getReviewer() {
        if (reviewers.isEmpty()) {
            return null;
        } else {
            return reviewers.iterator().next();
        }
    }

    public List<Member> getMembers() {
        List<Member> members = new ArrayList<>();

        members = getAuthors(members);

        for (User user : this.getReviewers()) {
            members.add(new Member(user, Member.Type.REVIEWER));
        }

        for (User user : this.getCoSupervisors()) {
            members.add(new Member(user, Member.Type.CO_SUPERVISOR));
        }

        members.add(new Member(getHeadSupervisor(), Member.Type.SUPERVISOR));

        return members;
    }

    private List<Member> getAuthors(List<Member> members) {
        for (User student : this.getProjectParticipants()) {
            members.add(new Member(student, Member.Type.AUTHOR));
        }
        return members;
    }

    public String getAuthorNames() {
        List<String> names = new ArrayList<>();
        for (User student : getProjectParticipants()) {
            names.add(student.getFullName());
        }
        return Joiner.on(", ").join(names);
    }

    public boolean hasExternalOrganization() {
        return externalOrganization != null;
    }

    public boolean isParticipant(User user) {
        for (User s : projectParticipants) {
            if (s.equals(user)) {
                return true;
            }
        }
        return false;
    }

    public String getSupervisorName() {
        return getHeadSupervisor().getFullName();
    }

    public String getReviewerName() {
        return getReviewer() != null ? getReviewer().getFullName() : NO_REVIEWER;
    }

    public String getCoSupervisorName() {
        List<String> names = new ArrayList<>();
        SortedSet<User> coSupervisors = getCoSupervisors();
        if (coSupervisors.size() > 0) {
            for (User coSupervisor : coSupervisors) {
                names.add(coSupervisor.getFullName());
            }
            return Joiner.on(", ").join(names);
        }
        return NO_CO_SUPERVISOR;
    }

    public DegreeLevel getProjectTypeDegreeLevel() {
        return getProjectType().getDegreeLevel();
    }

    @SuppressWarnings("UnusedDeclaration")
    public String getProjectTypeName() {
        return getProjectType().getName();
    }

    public int getProjectTypeMaxOpponents() {
        return getProjectType().getMaxOpponents();
    }

    public int getMinOpponentsOnFinalSeminar() {
        return getProjectType().getProjectTypeSettings().getMinOpponentsOnFinalSeminar();
    }

    public int getProjectTypeMaxActiveParticipants() {
        return getProjectType().getMaxActiveParticipants();
    }

    public int getMinFinalSeminarActiveParticipation() {
        return getProjectType().getProjectTypeSettings().getMinFinalSeminarActiveParticipation();
    }

    public Set<ProjectModule> getAuthorizedModules() {
        return getProjectType().getProjectModules();
    }

    public boolean hasModule(ProjectModule projectModule) {
        return getProjectType().hasModule(projectModule);
    }

    public static ITitle builder() {
        return new Builder();
    }

    public boolean isSupervisor(User user) {
        return headSupervisor != null && headSupervisor.equals(user);
    }

    public boolean isReviewer(User user) {
        for (User reviewer : reviewers) {
            if (user.equals(reviewer)) {
                return true;
            }
        }
        return false;
    }

    public boolean isCoSupervisor(User user) {
        for (User coSupervisor : coSupervisors) {
            if (user.equals(coSupervisor)) {
                return true;
            }
        }
        return false;
    }

    private static class Builder implements ITitle, IProjectType, IBuild {
        private final Project instance = new Project();

        @Override
        public IProjectType title(String title) {
            instance.setTitle(title);
            return this;
        }

        @Override
        public IBuild projectType(ProjectType projectType) {
            instance.setProjectType(projectType);
            return this;
        }

        @Override
        public IBuild headSupervisor(User headSupervisor) {
            instance.setHeadSupervisor(headSupervisor);
            return this;
        }

        @Override
        public IBuild coSupervisors(SortedSet<User> coSupervisors) {
            instance.setCoSupervisors(coSupervisors);
            return this;
        }

        @Override
        public IBuild reviewers(SortedSet<User> reviewers) {
            instance.setReviewers(reviewers);
            return this;
        }

        @Override
        public IBuild identifier(Long identifier) {
            instance.setIdentifier(identifier);
            return this;
        }

        @Override
        public IBuild projectParticipants(SortedSet<User> projectParticipants) {
            instance.setProjectParticipants(projectParticipants);
            return this;
        }

        @Override
        public IBuild projectStatus(ProjectStatus projectStatus) {
            instance.setProjectStatus(projectStatus);
            return this;
        }

        @Override
        public IBuild stateOfMind(StateOfMind stateOfMind) {
            instance.setStateOfMind(stateOfMind);
            return this;
        }

        @Override
        public IBuild dateCreated(Date dateCreated) {
            instance.setDateCreated(dateCreated);
            return this;
        }

        @Override
        public Project build() {
            return instance;
        }
    }

    public interface ITitle {
        IProjectType title(String title);
    }

    public interface IProjectType {
        IBuild projectType(ProjectType projectType);
    }

    public interface IBuild {
        IBuild identifier(Long identifier);

        IBuild projectParticipants(SortedSet<User> projectParticipants);

        IBuild headSupervisor(User headSupervisor);

        IBuild coSupervisors(SortedSet<User> coSupervisors);

        IBuild reviewers(SortedSet<User> reviewers);

        IBuild projectStatus(ProjectStatus projectStatus);

        IBuild stateOfMind(StateOfMind stateOfMind);

        IBuild dateCreated(Date dateCreated);

        Project build();
    }
}
