package se.su.dsv.scipro.report;

import com.google.common.eventbus.EventBus;
import scala.util.Either;
import scala.util.Left;
import scala.util.Right;
import se.su.dsv.scipro.finalseminar.FinalSeminarOpposition;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.ArrayList;
import java.util.List;

@Named
public class GradingReportServiceImpl extends AbstractServiceImpl<GradingReport, Long> implements GradingReportService {

    public static final String OPPOSITION_SWEDISH = "Ö1 Oppositionsrapport";
    public static final String OPPOSITION_ENGLISH = "Ö1 Opposition report";
    private final EventBus eventBus;

    @Inject
    public GradingReportServiceImpl(Provider<EntityManager> em, EventBus eventBus) {
        super(em, GradingReport.class, QGradingReport.gradingReport);
        this.eventBus = eventBus;
    }

    @Override
    public ReviewerGradingReport getReviewerGradingReport(Project project) {
        ReviewerGradingReport report = from(QReviewerGradingReport.reviewerGradingReport)
                .where(QReviewerGradingReport.reviewerGradingReport.project.eq(project))
                .uniqueResult(QReviewerGradingReport.reviewerGradingReport);
        if (report == null) {
            report = save(getTemplate(project).createReviewerReport(project));
        }
        return report;
    }

    @Override
    public boolean updateOppositionCriteria(SupervisorGradingReport report, FinalSeminarOpposition opposition) {
        for (GradingCriterion gradingCriterion : report.getIndividualCriteria()) {
            if ((gradingCriterion.getTitle().equals(OPPOSITION_SWEDISH) || gradingCriterion.getTitle().equals(OPPOSITION_ENGLISH))
                    && gradingCriterion.getFeedback() == null && gradingCriterion.getPoints() == null) {
                gradingCriterion.setFeedback(opposition.getFeedback());
                gradingCriterion.setPoints(opposition.getPoints());
                save(report);
                return true;
            }
        }
        return false;
    }

    @Override
    public List<SupervisorGradingReport> getSupervisorGradingReports(Project project) {
        List<SupervisorGradingReport> gradingReports = new ArrayList<>();
        for (User user : project.getProjectParticipants()) {
            gradingReports.add(getSupervisorGradingReport(project, user));
        }
        return gradingReports;
    }

    @Override
    public SupervisorGradingReport getSupervisorGradingReport(Project project, User user) {
        SupervisorGradingReport supervisorGradingReport = from(QSupervisorGradingReport.supervisorGradingReport)
                .where(QSupervisorGradingReport.supervisorGradingReport.user.eq(user).and(
                        QSupervisorGradingReport.supervisorGradingReport.project.eq(project)))
                .uniqueResult(QSupervisorGradingReport.supervisorGradingReport);
        if (supervisorGradingReport == null) {
            supervisorGradingReport = save(getTemplate(project).createSupervisorReport(project, user));
        }
        return supervisorGradingReport;
    }

    @Override
    public boolean hasFinishedGradingReports(Project project, boolean reviewer) {
        if (reviewer) {
            return getReviewerGradingReport(project).isFinished();
        } else {
            for (GradingReport report : getSupervisorGradingReports(project)) {
                if (!report.isFinished()) {
                    return false;
                }
            }
        }
        return true;
    }

    @Override
    public GradingReportTemplate getTemplate(Project project) {
        QGradingReportTemplate template = QGradingReportTemplate.gradingReportTemplate;
        return from(template)
                .where(template.projectType.eq(project.getProjectType()).and(template.credits.eq(project.getCredits())))
                .uniqueResult(template);
    }

    @Override
    public void copyReport(SupervisorGradingReport to, SupervisorGradingReport from) {
        to.copyProjectAssessmentFrom(from);
        save(to);
    }

    @Override
    public Either<List<GradingCriterion>, GradingReport> submitReport(GradingReport gradingReport) {
        List<GradingCriterion> notAssessed = new ArrayList<>();
        for (GradingCriterion gradingCriterion : gradingReport.getGradingCriteria()) {
            if (!gradingCriterion.isAssessed()) {
                notAssessed.add(gradingCriterion);
            }
        }

        if (!notAssessed.isEmpty()) {
            return Left.apply(notAssessed);
        }

        gradingReport.submit();

        if (gradingReport instanceof ReviewerGradingReport) {
            eventBus.post(new ReviewerGradingReportSubmittedEvent(gradingReport));
        } else if (gradingReport instanceof SupervisorGradingReport) {
            eventBus.post(new SupervisorGradingReportSubmittedEvent((SupervisorGradingReport) gradingReport));
        }

        save(gradingReport);
        return Right.apply(gradingReport);
    }

    @Override
    public Either<List<GradingCriterion>, GradingReport> initialAssessmentDone(final GradingReport gradingReport) {
        List<GradingCriterion> notAssessed = new ArrayList<>();
        for (GradingCriterion gradingCriterion : gradingReport.getGradingCriteria()) {
            if (!gradingCriterion.hasPointsAssigned()) {
                notAssessed.add(gradingCriterion);
            }
        }

        if (!notAssessed.isEmpty()) {
            return Left.apply(notAssessed);
        }

        gradingReport.setState(GradingReport.State.REVIEWING);
        save(gradingReport);

        if (gradingReport instanceof ReviewerGradingReport) {
            eventBus.post(new ReviewerGradingReportInitialAssessmentDoneEvent(gradingReport));
        } else if (gradingReport instanceof SupervisorGradingReport) {
            eventBus.post(new SupervisorGradingReportInitialAssessmentDoneEvent(gradingReport));
        }

        return Right.apply(gradingReport);
    }
}
