package se.su.dsv.scipro.reviewing;

import com.mysema.query.BooleanBuilder;
import com.mysema.query.jpa.JPASubQuery;
import com.mysema.query.jpa.impl.JPAQuery;
import com.mysema.query.types.PathMetadataFactory;
import com.mysema.query.types.Predicate;
import com.mysema.query.types.expr.BooleanExpression;
import com.mysema.query.types.path.StringPath;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.finalseminar.QFinalSeminar;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.project.QProject;
import se.su.dsv.scipro.reusable.SciProUtilities;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.ProjectType;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Collection;
import java.util.Date;

import static com.mysema.query.support.Expressions.anyOf;

public class ProjectFinalSeminarStatisticsServiceImpl extends AbstractServiceImpl<Project, Long> implements ProjectFinalSeminarStatisticsService {

    @Inject
    protected ProjectFinalSeminarStatisticsServiceImpl(Provider<EntityManager> em) {
        super(em, Project.class, QProject.project);
    }

    @Override
    public Long countFinalSeminarStatistics(ProjectFinalSeminarStatisticsService.Filter filter) {
        JPAQuery predicate = from(QProject.project, QFinalSeminar.finalSeminar)
                .where(toPredicate(filter))
                .where(QFinalSeminar.finalSeminar.project.eq(QProject.project));
        return predicate
                .count();
    }

    @Override
    public Long countFinalSeminarStatisticsOkFromReviewer(ProjectFinalSeminarStatisticsService.Filter filter) {
        JPAQuery predicate = from(QProject.project, QFinalSeminar.finalSeminar, QFinalSeminarApproval.finalSeminarApproval)//, QDecision.decision)
                .where(toPredicate(filter))
                .where(QFinalSeminar.finalSeminar.project.eq(QProject.project))
                .where(QFinalSeminarApproval.finalSeminarApproval.project.eq(QProject.project),
                        QFinalSeminarApproval.finalSeminarApproval.instanceOf(FinalSeminarApproval.class),
                        QFinalSeminarApproval.finalSeminarApproval.decisions.any().status.eq(Status.APPROVED));
        return predicate
                .count();
    }

    @Override
    public Iterable<Project> finalSeminarStatisticsNoOkFromReviewer(ProjectFinalSeminarStatisticsService.Filter filter, Pageable pageable) {
        StringPath sort = new StringPath(PathMetadataFactory.forProperty(QProject.project, pageable.getSort().getProperty()));
        return from(QProject.project, QFinalSeminar.finalSeminar)
                .where(toPredicate(filter))
                .where(QFinalSeminar.finalSeminar.project.eq(QProject.project))
                .where(anyOf(QProject.project.notIn(
                                        new JPASubQuery()
                                                .from(QFinalSeminarApproval.finalSeminarApproval)
                                                .where(QFinalSeminarApproval.finalSeminarApproval.decisions.any().status.eq(Status.APPROVED))
                                                .list(QFinalSeminarApproval.finalSeminarApproval.project)
                                ),
                                new JPASubQuery()
                                        .from(QFinalSeminarApproval.finalSeminarApproval)
                                        .where(QFinalSeminarApproval.finalSeminarApproval.project.eq(QProject.project))
                                        .notExists()
                        )
                ).offset(pageable.getOffset())
                .limit(pageable.getLimit())
                .orderBy(pageable.getSort().isAscending() ? sort.asc() : sort.desc())
                .list(QProject.project);
    }

    @Override
    public Long countFinalSeminarStatisticsNoOkFromReviewer(ProjectFinalSeminarStatisticsService.Filter filter) {
        JPAQuery predicate =from(QProject.project, QFinalSeminar.finalSeminar)
                .where(toPredicate(filter))
                .where(QFinalSeminar.finalSeminar.project.eq(QProject.project))
                .where(anyOf(QProject.project.notIn(
                                        new JPASubQuery()
                                                .from(QFinalSeminarApproval.finalSeminarApproval)
                                                .where(QFinalSeminarApproval.finalSeminarApproval.decisions.any().status.eq(Status.APPROVED))
                                                .list(QFinalSeminarApproval.finalSeminarApproval.project)
                                ),
                                new JPASubQuery()
                                        .from(QFinalSeminarApproval.finalSeminarApproval)
                                        .where(QFinalSeminarApproval.finalSeminarApproval.project.eq(QProject.project))
                                        .notExists()
                        )
                );
        return predicate
                .count();
    }

    private static Predicate toPredicate(final Filter filter) {
        BooleanBuilder predicate = new BooleanBuilder();
        if (filter.getProjectTypes() != null) {
            predicate.and(levelFilter(filter.getProjectTypes()));
        }
        if (filter.getCreatedAfter() != null) {
            predicate.and(createdAfter(filter.getCreatedAfter()));
        }
        if (filter.getCreatedBefore() != null) {
            predicate.and(createdBefore(filter.getCreatedBefore()));
        }
        if (filter.getStatuses() != null) {
            predicate.and(hasAnyStatus(filter.getStatuses()));
        }
        predicate.and(QFinalSeminar.finalSeminar.dateCreated.before(filter.getFinalSeminarCreatedBefore()));
        predicate.and(QFinalSeminar.finalSeminar.dateCreated.after(filter.getFinalSeminarCreatedAfter()));
        return predicate;
    }

    private static BooleanBuilder hasAnyStatus(Collection<ProjectStatus> statuses) {
        BooleanBuilder ee = new BooleanBuilder();
        if (statuses != null && !statuses.isEmpty()) {
            for (ProjectStatus status : statuses) {
                ee.or(QProject.project.projectStatus.eq(status));
            }
        } else {
            ee.and(QProject.project.projectStatus.isNull());
        }
        return ee;
    }

    private static BooleanBuilder levelFilter(Collection<ProjectType> levels) {
        BooleanBuilder e = new BooleanBuilder();
        if (levels != null && !levels.isEmpty()) {
            for (ProjectType level : levels) {
                e.or(QProject.project.projectType.eq(level));
            }
        } else {
            e.and(QProject.project.projectType.isNull());
        }
        return e;
    }

    private static BooleanExpression createdAfter(Date after) {
        Date start = SciProUtilities.setTimeOfDayTo0000(after);
        return QProject.project.dateCreated.after(start);
    }

    private static BooleanExpression createdBefore(Date before) {
        Date end = SciProUtilities.setTimeOfDayTo2359(before);
        return QProject.project.dateCreated.before(end);
    }
}
