package se.su.dsv.scipro.firstmeeting

import java.text.SimpleDateFormat
import java.util.{Collections, Date}
import javax.inject.{Inject, Provider}

import org.apache.commons.lang3.time.DateUtils
import se.su.dsv.scipro.`match`.Idea.Status
import se.su.dsv.scipro.`match`.IdeaService.Filter
import se.su.dsv.scipro.`match`.{Idea, IdeaService}
import se.su.dsv.scipro.data.enums.DateStyle
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService
import se.su.dsv.scipro.mail.{EmailRecipient, MailEvent, MailEventService, Recipient}
import se.su.dsv.scipro.misc.DaysService
import se.su.dsv.scipro.system.User
import se.su.dsv.scipro.workerthreads.{AbstractWorker, Scheduler}

import scala.collection.JavaConversions._
import scala.collection.immutable._

class FirstMeetingReminderWorkerSchedule @Inject()(scheduler: Scheduler, firstMeetingReminderWorker: Provider[FirstMeetingReminderWorker]) {
  scheduler.schedule("First meeting reminders").runBy(firstMeetingReminderWorker).dailyAt(11, 30)
}

// Hard to test side-effecting stuff
class FirstMeetingReminderWorker @Inject()(val ideaService: IdeaService, val daysService: DaysService, val mailEventService: MailEventService, val generalSystemSettingsService: GeneralSystemSettingsService)
  extends AbstractWorker {

  import se.su.dsv.scipro.firstmeeting.FirstMeetingReminderWorker._

  override protected def doWork() = {
    val ideas = ideaService.findAll(filter).toList
    val ideasToday = ideaService.findAll(todayFilter).toList

    val today = new Date

    val findNeedReminding = findNeedRemindingMethod(deadline, calculateReminderDaysBasedOnWorkDays(daysService))(today)
    val findNeedRemindingNow = findNeedRemindingMethod(deadline, rightNow)(today)

    val needReminding = findNeedReminding(ideas)
    val needRemindingNow = findNeedRemindingNow(ideasToday)

    val baseURL = generalSystemSettingsService.getGeneralSystemSettingsInstance.getSciproURL

    beginTransaction()
    needReminding groupBy supervisor map(generateReminder(baseURL) _).tupled foreach sendReminder
    needRemindingNow groupBy supervisor map(generateReminderToday _).tupled foreach sendReminder
    commitTransaction()
  }

  private def calculateReminderDaysBasedOnWorkDays(daysService: DaysService): Idea => Set[Date] = { idea =>
    daysAheadToRemind.map(daysService.workDaysAhead(startDate(idea), _))
  }

  def deadline(idea: Idea): Date =
    daysService.workDaysAhead(idea.getCourseStartDate,2)

  private def rightNow: Idea => Set[Date] =
    idea => Set(startDate(idea))

  private def sendReminder(mailEvent: MailEvent) =
    mailEventService.save(mailEvent)

}

// Nice pure testable code
object FirstMeetingReminderWorker {
  val daysAheadToRemind = Set(2, 5)
  val sdf = new SimpleDateFormat(DateStyle.DATE.getFormat)

  val filter: Filter = new Filter
  filter.setNoFirstMeeting(true)
  filter.setStatuses(Collections.singleton(Status.MATCHED))

  val todayFilter: Filter = new Filter
  todayFilter.setNoFirstMeeting(true)
  todayFilter.setStatuses(Collections.singleton(Status.COMPLETED))

  def findNeedRemindingMethod(getDeadline: Idea => Date, reminderDays: Idea => Set[Date]):
  Date =>  Iterable[Idea] => Iterable[(Idea, Date)] =
    date => ideas => for {
      idea <- ideas
      deadline = getDeadline(idea)
      if reminderDays(idea) exists sameDayAs(date)
    } yield (idea, deadline)

  def sameDayAs(date: Date): Date => Boolean = DateUtils.isSameDay(_, date)

  def generateReminder(baseUrl: String)(supervisor: User, ideas: Iterable[(Idea, Date)]): MailEvent = {
    val reminder = new MailEvent
    reminder.getRecipients.add(supervisor)
    reminder.setSubject("Kind reminder to book First meeting(s)")
    val lines = ideas.map{case (idea, deadline) =>
      s"""-Course start: ${sdf.format(idea.getCourseStartDate)}
         |-Deadline: ${sdf.format(deadline)}
         |-Authors: ${idea.getAuthorNames}
         |-Supervisor: ${supervisor.getFullName}
         |-Title: ${idea.getTitle}
         |
         |""".stripMargin
    }

    reminder.setMessageBody(
      s"""
        |According to the information in SciPro, you haven't booked a First meeting with your thesis student(s) yet.
        |
        |To book a meeting, please see the instructions below:
        |
        |First meeting with the student(s)
        | - A meeting is to be held within 3 days after the course start.
        | - The meeting is scheduled by the supervisor before the course starts.
        | - The meeting is scheduled under the “Ideas” tab (an email is automatically sent to the student(s).
        | - To go directly to the "Ideas" tab go to ${baseUrl}supervisor/projectideas
        | - If you have already met the student(s), please create a First meeting after the course start anyway.
        |
        |The following project(s) need a First meeting:
        |${lines.mkString}
        |/Supervisor support
      """.stripMargin)
    reminder
  }

  def generateReminderToday(supervisor: User, ideas: Iterable[(Idea, Date)]): MailEvent = {
    val reminder = new MailEvent
    reminder.getRecipients.add(supervisor)
    val gru = Option(supervisor.getUnit).flatMap(unit => Option(unit.getMatchResponsible)).map(new EmailRecipient(_))
    reminder.addRecipients(gru.toSet[Recipient])
    reminder.setSubject("Kind reminder to book First meeting without further delay")
    val lines = ideas.map{case (idea, deadline) =>
      s"""-Course start: ${sdf.format(idea.getCourseStartDate)}
          |-Authors: ${idea.getAuthorNames}
          |-Supervisor: ${supervisor.getFullName}
          |-Title: ${idea.getTitle}
          |
          |""".stripMargin
    }
    reminder.setMessageBody(
      s"""
         |The thesis course has now started, and it is high time you booked a meeting with your student(s) if you haven't done that.
         |
         |It is recommended that you use the tab “Forum” to interact with the student(s) about when and where to have the "First meeting".
         |
         |Automatic e-mail(s) will then be sent out from SciPro to alert the student(s) about the meeting.
         |
         |The following project(s) need a First meeting:
         |${lines.mkString}
         |/Supervisor support
       """.stripMargin)
    reminder
  }




  private def startDate(idea: Idea): Date = {
    idea.getApplicationPeriod.getCourseStartDate
  }

  private def supervisor(idea: (Idea, Date)) = idea._1.getMatch.getSupervisor
}