package se.su.dsv.scipro.forummail

import javax.inject.Inject

import com.google.common.eventbus.{EventBus, Subscribe}
import se.su.dsv.scipro.file.FileDescription
import se.su.dsv.scipro.forum.dataobjects.ForumThread
import se.su.dsv.scipro.forum.{NewGroupForumReplyEvent, NewProjectForumReplyEvent}
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService
import se.su.dsv.scipro.mail.{MailEvent, MailEventService}
import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.springdata.services.UserProfileService
import se.su.dsv.scipro.system.User

import scala.collection.JavaConverters._
import scala.collection.{immutable, mutable}

class SendForumMail (val forumMailRepository: ForumMailRepository,
                     val mailSettingsService: ForumMailSettingsService,
                     val mailEventService: MailEventService,
                     val formatter: ForumMailFormatter,
                     val generalSystemSettings: GeneralSystemSettingsService,
                     val userProfileService: UserProfileService) {

  @Inject def this(forumMailRepository: ForumMailRepository, mailSettingsService: ForumMailSettingsService, mailEventService: MailEventService, formatter: ForumMailFormatter, generalSystemSettings: GeneralSystemSettingsService, eventBus: EventBus, userProfileService: UserProfileService) = {
    this(forumMailRepository, mailSettingsService, mailEventService, formatter, generalSystemSettings, userProfileService)
    eventBus.register(this)
  }

  @Subscribe def newProjectForumReply(event: NewProjectForumReplyEvent): Unit = {
    if (mailSettingsService.getSettings.isEnabled) {

      val project: Project = event.getProjectThread.getProject

      val recipients: List[User] = (project.getHeadSupervisor :: project.getCoSupervisors.asScala.toList)
        .filterNot(_ == event.getReply.getPostedBy)
        .filter(userProfileService.isReceiveForumMail)

      val attachments = event.getReply.getAttachments.asScala.toSet
      val data = MailData(event.getProjectThread.getForumThread, attachments, event.getProjectThread)

      sendMails(data, recipients.toList, (mailEvent, recipient) => {
        forumMailRepository.save(new ForumMailProject(mailEvent, recipient, event.getProjectThread))
        ()
      })
    }
  }

  @Subscribe def newGroupForumReply(event: NewGroupForumReplyEvent): Unit = {
    if (mailSettingsService.getSettings.isEnabled) {

      val group = event.getGroupThread.getGroup
      val recipients: List[User] = group.getProjects.asScala.flatMap(project => project.getHeadSupervisor :: project.getCoSupervisors.asScala.toList)
        .filterNot(_ == event.getReply.getPostedBy)
        .filter(userProfileService.isReceiveForumMail)
        .toList

      val attachments = event.getReply.getAttachments.asScala.toSet
      val data = MailData(event.getGroupThread.getForumThread, attachments, event.getGroupThread)

      sendMails(data, recipients, (mailEvent, recipient) => {
        forumMailRepository.save(new ForumMailGroup(mailEvent, recipient, event.getGroupThread))
        ()
      })
    }
  }

  private def sendMails[A : Footer : Subject : ThreadURL](data: MailData[A], recipients: immutable.Iterable[User], f: (MailEvent, User) => Unit): Unit = {
    val (subject, body) = generateMailContent(data)

    for (recipient <- recipients) {
      val mailEvent: MailEvent = generateMail(recipient, subject, body)
      f(mailEvent, recipient)
    }
  }

  private case class MailData[A](thread: ForumThread, attachments: immutable.Set[FileDescription], context: A)

  private def generateMailContent[A](data: MailData[A])(implicit $subject: Subject[A], $footer: Footer[A], $url: ThreadURL[A]) = {
    def appendAttachmentInformation(content: StringBuilder): Unit = {
      if (data.attachments.nonEmpty) {
        content.append("\n\n")
        content.append("Attachments: (go to project forum to download)\n")
        data.attachments.foreach(attachment => content.append(attachment.getName).append("\n"))
      }
    }

    val subject = $subject.subject(data.context)
    val content = new mutable.StringBuilder
    content.append(formatter.formatBody(data.thread))
    appendAttachmentInformation(content)
    content.append($footer.footer(data.context)).append("\n")

    val baseUrl = generalSystemSettings.getGeneralSystemSettingsInstance.getSciproURL + ForumMailModule.FORUM_RELATIVE_URL
    content.append("* View forum in SciPro here: ").append($url.url(baseUrl, data.context))

    (subject, content.toString())
  }

  private def generateMail(recipient: User, subject: String, body: String): MailEvent = {
    val mail = new MailEvent
    mail.getRecipients.add(recipient)
    mail.setMessageBody(body)
    mail.setSubject(subject)

    val settings: ForumMailSettings = mailSettingsService.getSettings
    mail.setFromEmail(settings.getReplyAddress)
    mail.setFromName(settings.getAccount)
    mailEventService.save(mail)
  }
}
