package se.su.dsv.scipro.reviewing

import java.text.SimpleDateFormat
import java.util.{Collections, Date}
import javax.inject.{Inject, Provider}

import org.apache.commons.lang3.time.DateUtils
import se.su.dsv.scipro.data.enums.DateStyle
import se.su.dsv.scipro.mail.{EmailRecipient, MailEvent, MailEventService, Recipient}
import se.su.dsv.scipro.misc.DaysService
import se.su.dsv.scipro.reviewing.MyReviewService.Filter
import se.su.dsv.scipro.system.User
import se.su.dsv.scipro.workerthreads.{AbstractWorker, Scheduler}

import scala.collection.JavaConversions._
import scala.collection.immutable.{Iterable, Set}

class  ReviewerDecisionReminderWorkerSchedule @Inject()(scheduler: Scheduler,  reviewerDecisionReminderWorker: Provider[ ReviewerDecisionReminderWorker]) {
  scheduler.schedule("Reviewer decision reminders").runBy(reviewerDecisionReminderWorker).dailyAt(11, 45)
}

class ReviewerDecisionReminderWorker @Inject()(val myReviewService: MyReviewService,
                                               val daysService: DaysService,
                                               val mailEventService: MailEventService) extends AbstractWorker {

  import se.su.dsv.scipro.reviewing.ReviewerDecisionReminderWorker._

  override protected def doWork() = {
    val reviewerRoughDraftApprovals = myReviewService.findAll(roughDraftApprovalFilter).toList
    val reviewerFinalSeminarApprovals = myReviewService.findAll(finalSeminarApprovalFilter).toList

    val today = new Date

    val needRemindingToday = needReminding(deadline, reminderDays)(today)

    val roughDraftApprovalsNeedReminding = needRemindingToday(reviewerRoughDraftApprovals)
    val finalSeminarApprovalsNeedReminding = needRemindingToday(reviewerFinalSeminarApprovals)

    beginTransaction()
    roughDraftApprovalsNeedReminding groupBy reviewer map (generateRoughDraftReminder _).tupled foreach sendReminder
    finalSeminarApprovalsNeedReminding groupBy reviewer map (generateFinalSeminarReminder _).tupled foreach sendReminder
    commitTransaction()
  }

  def deadline(reviewerApproval: ReviewerApproval): Date =
    reviewerApproval.getCurrentDeadline

  def reminderDays(date: Date): Set[Date] =
    daysAfterToRemind.map(daysService.workDaysAfter(date, _))

  private def sendReminder(mailEvent: MailEvent) = {
    mailEventService.save(mailEvent)
  }
}


object ReviewerDecisionReminderWorker {
  val daysAfterToRemind = Set(1)

  val sdf = new SimpleDateFormat(DateStyle.DATE.getFormat)

  val finalSeminarApprovalFilter:Filter = new Filter
  finalSeminarApprovalFilter.setStatuses(Collections.singleton(Status.UNDECIDED))
  finalSeminarApprovalFilter.setStep(ReviewerApproval.Step.FINAL_SEMINAR_APPROVAL)

  val roughDraftApprovalFilter: Filter = new Filter
  roughDraftApprovalFilter.setStatuses(Collections.singleton(Status.UNDECIDED))
  roughDraftApprovalFilter.setStep(ReviewerApproval.Step.ROUGH_DRAFT_APPROVAL)

  def needReminding(getDeadline: ReviewerApproval => Date, getReminderDays: Date => Set[Date]):
  Date => Iterable[ReviewerApproval] => Iterable[(ReviewerApproval, Date)] =
    date => reviewerApprovals => for {
      reviewerApproval <- reviewerApprovals
      deadline = getDeadline(reviewerApproval)
      if getReminderDays(deadline) exists sameDayAs(date)
    } yield (reviewerApproval, deadline)

  def sameDayAs(date: Date): Date => Boolean = DateUtils.isSameDay(_, date)

  def generateRoughDraftReminder(reviewer: User, reviewerApprovals: Iterable[(ReviewerApproval, Date)]): MailEvent = {

    val reminder = new MailEvent
    reminder.getRecipients.add(reviewer)
    reminder.getRecipients.addAll(reviewerApprovals.map(t => t._1.getProject.getHeadSupervisor))
    val gru = Option(reviewer.getUnit).flatMap(unit => Option(unit.getMatchResponsible)).map(new EmailRecipient(_))
    reminder.addRecipients(gru.toSet[Recipient])
    reminder.setSubject("Missed SciPro rough draft approval deadline")
    val lines = reviewerApprovals.map { case (approval, deadline) =>
      s"""-Deadline: ${sdf.format(deadline)}
          |-Supervisor: ${approval.getProject.getHeadSupervisor.getFullName}
          |-Author(s): ${approval.getProject.getAuthorNames}
          |-Title: ${approval.getProject.getTitle}
          |
          |""".stripMargin }

    reminder.setMessageBody(
      s"""
         |According to the information in SciPro,
         |the following rough draft approval(s) that ${reviewer.getFullName } is responsible for are past due:
         |
         |${lines.mkString}/Reviewer support
      """.stripMargin)
    reminder
  }

  def generateFinalSeminarReminder(reviewer: User, reviewerApprovals: Iterable[(ReviewerApproval, Date)]): MailEvent = {
    val reminder = new MailEvent
    reminder.getRecipients.add(reviewer)
    reminder.getRecipients.addAll(reviewerApprovals.map(t => t._1.getProject.getHeadSupervisor))
    val gru = Option(reviewer.getUnit).flatMap(unit => Option(unit.getMatchResponsible)).map(new EmailRecipient(_))
    reminder.addRecipients(gru.toSet[Recipient])
    reminder.setSubject("Missed SciPro final seminar approval deadline")
    val lines = reviewerApprovals.map { case (approval, deadline) =>
      s"""-Deadline: ${sdf.format(deadline)}
        |-Supervisor: ${approval.getProject.getHeadSupervisor.getFullName}
        |-Author(s): ${approval.getProject.getAuthorNames}
        |-Title: ${approval.getProject.getTitle}
        |
        |""".stripMargin }

    reminder.setMessageBody(
      s"""
          |According to the information in SciPro,
          |the following final Seminar approval(s) that ${reviewer.getFullName } is responsible for are past due:
          |
          |${lines.mkString}/Reviewer support
      """.stripMargin)
    reminder
  }

  private def reviewer(reviewerApproval: (ReviewerApproval, Date)) = reviewerApproval._1.getProject.getReviewer


}