package se.su.dsv.scipro.activityplan;

import com.google.common.eventbus.EventBus;
import org.joda.time.MutableDateTime;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.checklist.ChecklistTemplateService;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.*;

import static org.junit.Assert.assertEquals;

public class ActivityPlanFacadeImplIntegrationTest extends IntegrationTest {
    private static final Pageable ALL = new PageRequest(0, Long.MAX_VALUE);

    @Mock
    private EventBus eventBus;
    @Mock
    private ChecklistTemplateService checklistTemplateService;
    @Mock
    private FileDescriptionService fileDescriptionService;
    @Inject
    private ActivityService activityService;
    @Inject
    private ActivityPlanService activityPlanService;
    @Inject
    private DaysService daysService;

    private ActivityPlanFacadeImpl activityPlanFacade;

    private Project project;
    private ActivityPlan activityPlan;
    private Date yesterday;
    private Date today;

    private Date tomorrow;

    @Before
    public void setUp() throws Exception {
        activityPlanFacade = new ActivityPlanFacadeImpl(eventBus, fileDescriptionService, checklistTemplateService, daysService);
        activityPlanFacade.activityService = activityService;
        activityPlanFacade.activityPlanService = activityPlanService;
        project = createProject();
        activityPlan = createActivityPlan(project);
        yesterday = getRelativeDate(-1);
        today = getRelativeDate(0);
        tomorrow = getRelativeDate(1);
    }

    private Date getRelativeDate(int daysFromNow) {
        MutableDateTime dateTime = new MutableDateTime();
        dateTime.addDays(daysFromNow);
        return dateTime.toDate();
    }

    @Test
    public void deleteAll() throws Exception {
        createUser();

        //create events
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.MONTH, 1);
        Activity pse1 = Activity.builder().activityPlan(activityPlan).date(cal.getTime()).name("").description("").build();

        cal.set(Calendar.MONTH, 6);
        Activity pse2 = Activity.builder().activityPlan(activityPlan).date(cal.getTime()).name("").description("").build();

        cal.set(Calendar.MONTH, 12);
        Activity pse3 = Activity.builder().activityPlan(activityPlan).date(cal.getTime()).name("").description("").build();
        pse3.setFileUpload(new FileDescription());

        pse1 = activityService.save(pse1);
        pse2 = activityService.save(pse2);
        pse3 = activityService.save(pse3);

        //add them to schedule
        activityPlan.getActivities().add(pse1);
        activityPlan.getActivities().add(pse2);
        activityPlan.getActivities().add(pse3);

        //save schedule
        activityPlan = activityPlanService.save(activityPlan);

        assertEquals(3, activityPlan.getActivities().size());
        activityPlan = activityPlanFacade.deleteAllEvents(activityPlan);
        assertEquals(0, activityPlan.getActivities().size());
    }


    @Test
    public void findEventsByProjectShortHand() throws Exception {
        List<Activity> expectedList = Collections.singletonList(newActivity());
        assertEquals(expectedList, activityPlanFacade.findEventsByProject(project, ALL));
    }

    @Test
    public void findEventsByProject() throws Exception {
        List<Activity> expectedList = Collections.singletonList(newActivity(today));
        assertEquals(expectedList, activityPlanFacade.findEventsByProject(project, yesterday, tomorrow, ALL));
    }

    @Test
    public void countEventsByProject() throws Exception {
        long expectedCount = 3;
        newActivity(today);
        newActivity(today);
        newActivity(today);
        assertEquals(expectedCount, activityPlanFacade.countEventsByProject(project, yesterday, tomorrow));
    }

    @Test
    public void countEventsByProjectFromIsNull() throws Exception {
        long expectedCount = 3;
        newActivity(yesterday);
        newActivity(today);
        newActivity(today);
        assertEquals(expectedCount, activityPlanFacade.countEventsByProject(project, null, tomorrow));
    }

    @Test
    public void countEventsByProjectToIsNull() throws Exception {
        long expectedCount = 3;
        newActivity(tomorrow);
        newActivity(tomorrow);
        newActivity(tomorrow);
        assertEquals(expectedCount, activityPlanFacade.countEventsByProject(project, yesterday, null));
    }

    @Test
    public void countEventsByBothNull() throws Exception {
        long expectedCount = 3;
        newActivity(yesterday);
        newActivity(today);
        newActivity(tomorrow);
        assertEquals(expectedCount, activityPlanFacade.countEventsByProject(project, null, null));
    }

    private ProjectType createProjectType() {
        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        return save(projectType);
    }

    private Project createProject() {
        Project project = new Project();
        project.setHeadSupervisor(createUser());
        project.setTitle("");
        project.setProjectType(createProjectType());
        project.setStatusMessage("");
        return save(project);
    }

    private ActivityPlan createActivityPlan(Project project){
        ActivityPlan activityPlan = ActivityPlan.builder().project(project).build();
        activityPlan.setProject(project);
        return save(activityPlan);
    }


    private Activity newActivity() {
        return newActivity(today);
    }

    private Activity newActivity(Date date) {
        Activity activity = new Activity();
        activity.setActivityPlan(activityPlan);
        activity.setDate(date);
        return save(activity);
    }

    private User createUser(){
        return save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
    }
}