package se.su.dsv.scipro.activityplan;

import com.google.common.eventbus.EventBus;
import org.hamcrest.CoreMatchers;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.checklist.*;
import se.su.dsv.scipro.file.*;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.IsCollectionContaining.hasItem;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class ActivityPlanFacadeImplTest {

    static final ProjectType PROJECT_TYPE = new ProjectType(DegreeLevel.FIRST_CYCLE, "bach", "bach");
    static final Project SOME_PROJECT = Project.builder().title("title").projectType(PROJECT_TYPE).build();
    static final ChecklistTemplate SOME_TEMPLATE = new ChecklistTemplate("name", new UserBuilder().create());
    static final String SOME_NAME = "someName";
    static final String SOME_TITLE = "someTitle";

    @Mock
    private ChecklistService checklistService;
    @Mock
    private ChecklistTemplateService checklistTemplateService;
    @Mock
    private ChecklistQuestionRepo checklistQuestionRepo;
    @Mock
    private ProjectService projectService;
    @Mock
    private EventBus eventBus;
    @Mock
    private FileDescriptionService fileDescriptionService;
    @Mock
    private ActivityService activityService;
    @Mock
    private DaysService daysService;
    @InjectMocks
    private ActivityPlanFacadeImpl activityPlanFacade;
    private User user;
    private Checklist checklist;
    private ChecklistAnswer answer;
    private ChecklistAnswer wrongAnswer;

    @Before
    public void setUp() throws Exception {
        activityPlanFacade.activityService = activityService;
        activityPlanFacade.projectService = projectService;
                activityPlanFacade.checklistService = checklistService;
                activityPlanFacade.checklistQuestionRepo = checklistQuestionRepo;


        user = new UserBuilder().create();
        User userWithoutAnswers = new UserBuilder().create();

        Project project = new Project();
        project.setTitle(SOME_TITLE);
        checklist = Checklist.builder().name(SOME_NAME).project(project).build();
        ChecklistQuestion question = new ChecklistQuestion("", 1);
        wrongAnswer = new ChecklistAnswer(userWithoutAnswers);
        answer = new ChecklistAnswer(user);
        question.addAnswer(answer);
        question.addAnswer(wrongAnswer);
        checklist.addQuestion(question);
    }

    @Test
    public void uploading_file_to_activity_posts_event() throws Exception {
        final Activity activity = new Activity();
        final ProjectFile fileDescription = new ProjectFile();

        when(fileDescriptionService.storeFile(isA(ProjectFileUpload.class), any(FileSource.class))).thenReturn(fileDescription);

        activityPlanFacade.storeActivityUpload(mock(ProjectFileUpload.class), activity);

        ArgumentCaptor<ActivityFileUploadedEvent> captor = ArgumentCaptor.forClass(ActivityFileUploadedEvent.class);
        verify(eventBus).post(captor.capture());

        ActivityFileUploadedEvent event = captor.getValue();
        assertThat(event.getActivity(), is(activity));
        assertThat(event.getFileDescription(), CoreMatchers.<FileDescription>is(fileDescription));
    }

    @Test
    public void checklist_gets_title_from_template() throws Exception {
        activityPlanFacade.createChecklist(SOME_PROJECT, SOME_TEMPLATE);
        ArgumentCaptor<Checklist> captor = ArgumentCaptor.forClass(Checklist.class);
        verify(checklistService).save(captor.capture());

        assertEquals(captor.getValue().getName(), SOME_TEMPLATE.getName());
    }

    @Test
    public void template_questions_is_attached_to_checklist() throws Exception {
        ChecklistQuestion checklistQuestion = mockChecklistQuestion();

        activityPlanFacade.createChecklist(SOME_PROJECT, SOME_TEMPLATE);
        ArgumentCaptor<Checklist> captor = ArgumentCaptor.forClass(Checklist.class);
        verify(checklistService).save(captor.capture());

        assertThat(captor.getValue().getQuestions(), hasItem(checklistQuestion));
    }

    private ChecklistQuestion mockChecklistQuestion() {
        String question = "q1";
        SOME_TEMPLATE.addQuestion(question);
        ChecklistQuestion checklistQuestion = new ChecklistQuestion(question, 1);
        when(checklistQuestionRepo.save(any(ChecklistQuestion.class))).thenReturn(checklistQuestion);
        return checklistQuestion;
    }

    @Test
    public void isCompletedReturnsTrueWhenChecklistIsCompleted() throws Exception {
        wrongAnswer.setAnswer(ChecklistAnswerEnum.NO_ANSWER);
        answer.setAnswer(ChecklistAnswerEnum.GREEN);
        assertTrue(activityPlanFacade.isCompletedByUser(user, checklist));
    }

    @Test
    public void isCompletedReturnsFalseWhenChecklistIsNotCompleted() throws Exception {
        wrongAnswer.setAnswer(ChecklistAnswerEnum.GREEN);
        answer.setAnswer(ChecklistAnswerEnum.NO_ANSWER);
        assertFalse(activityPlanFacade.isCompletedByUser(user, checklist));
    }
}