package se.su.dsv.scipro.checklist;

import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.reusable.SciProUtilities;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.ArrayList;

import static org.hamcrest.CoreMatchers.hasItem;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;

public class ChecklistTemplateServiceImplTest extends IntegrationTest {

    public static final PageRequest ALL = new PageRequest(0, Long.MAX_VALUE);
    @Inject
    private ChecklistTemplateServiceImpl checklistTemplateService;

    private ChecklistTemplate template;
    private User user;

    @Before
    public void setUp() throws Exception {
        user = save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
        template = newChecklistTemplate(user);
    }

    @Test
    public void find() throws Exception {
        assertThat(checklistTemplateService.findAll(template.getName(), ALL), hasItem(template));
    }

    @Test
    public void count() throws Exception {
        assertEquals(1, checklistTemplateService.count(template.getName()));
    }

    @Test
    public void upChecklistTemplateWithNowhereToMove() throws Exception {
        int number = template.getTemplateNumber();
        checklistTemplateService.upChecklistTemplate(template);
        assertEquals(number, template.getTemplateNumber());
    }

    @Test
    public void upChecklistTemplate() throws Exception {
        ChecklistTemplate template2 = newChecklistTemplate(user);
        int template1Number = template.getTemplateNumber();
        int template2Number = template2.getTemplateNumber();

        checklistTemplateService.upChecklistTemplate(template);

        assertEquals(template1Number, template2.getTemplateNumber());
        assertEquals(template2Number, template.getTemplateNumber());
    }

    @Test
    public void downChecklistTemplateWithNowhereToMove() throws Exception {
        int number = template.getTemplateNumber();
        checklistTemplateService.downChecklistTemplate(template);
        assertEquals(number, template.getTemplateNumber());
    }

    @Test
    public void downChecklistTemplate() throws Exception {
        ChecklistTemplate template2 = newChecklistTemplate(user);
        int template1Number = template.getTemplateNumber();
        int template2Number = template2.getTemplateNumber();

        checklistTemplateService.downChecklistTemplate(template2);

        assertEquals(template1Number, template2.getTemplateNumber());
        assertEquals(template2Number, template.getTemplateNumber());
    }

    @Test
    public void prepareNewTemplate() throws Exception {
        ChecklistTemplate checklistTemplate = newChecklistTemplate(user);
        assertEquals(user, checklistTemplate.getCreator());
        assertEquals(checklistTemplateService.count() - 1, checklistTemplate.getTemplateNumber());
    }

    @Test
    public void find_peer_request_checklists() throws Exception {
        ProjectType master = save(new ProjectType(DegreeLevel.SECOND_CYCLE, "Master", "Master"));
        Project project = save(Project.builder().title("Title").projectType(master).headSupervisor(user).build());
        template.getCategories().add(save(new ChecklistCategory(ChecklistTemplateServiceImpl.PEER)));
        template.getProjectTypes().add(master);

        assertThat(checklistTemplateService.findPeerRequestChecklists(project), hasItem(template));
    }

    @Test
    public void safeDeleteChecklistTemplate() throws Exception {
        ChecklistTemplate template = newChecklistTemplate(user);
        ChecklistTemplate anotherTemplate = newChecklistTemplate(user);
        template.setTemplateNumber(1);
        anotherTemplate.setTemplateNumber(2);
        checklistTemplateService.safeDeleteChecklistTemplate(template);
        assertEquals(template.getTemplateNumber(), anotherTemplate.getTemplateNumber());
    }

    private ChecklistTemplate newChecklistTemplate(User user) {
        ChecklistTemplate clt = new ChecklistTemplate("New checklist template (change this to real name)", user);
        clt.setCategories(new ArrayList<ChecklistCategory>());
        ArrayList<String> questionList = new ArrayList<>();
        questionList.add("Example question (remove this and add relevant questions)");
        clt.setQuestions(questionList);
        clt.setTemplateNumber(SciProUtilities.safeLongToInt(checklistTemplateService.count()));
        return save(clt);
    }
}
