package se.su.dsv.scipro.finalseminar;

import com.google.common.eventbus.EventBus;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.ArgumentCaptor;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.plagiarism.TurnItInCheck;
import se.su.dsv.scipro.plagiarism.TurnItInCheckRepo;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.GradingReportTemplate;
import se.su.dsv.scipro.report.OppositionReport;
import se.su.dsv.scipro.report.OppositionReportService;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.InstanceProvider;

import javax.persistence.EntityManager;
import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class FinalSeminarServiceImplTest {

    private FinalSeminarServiceImpl seminarService;
    private DaysService daysService;
    Project dummyProject;
    private FinalSeminar finalSeminar;
    private FinalSeminarSettingsService finalSeminarSettingsService;
    private TurnItInCheckRepo turnItInCheckRepo;

    @Mock(answer = Answers.RETURNS_MOCKS)
    private EntityManager em;

    @Mock
    FinalSeminarService finalSeminarService;
    @Mock
    FileDescriptionService fileDescriptionService;
    FinalSeminarOppositionService finalSeminarOppositionService;
    @Mock
    OppositionReportService oppositionReportService;
    private EventBus eventBus;

    @Before
    public void setup() {
        fileDescriptionService = mock(FileDescriptionService.class);
        finalSeminarSettingsService = mock(FinalSeminarSettingsService.class);
        daysService = mock(DaysService.class);
        turnItInCheckRepo = mock(TurnItInCheckRepo.class);
        eventBus = mock(EventBus.class);

        seminarService = new ExtendedFinalSeminarServiceImpl(new InstanceProvider<>(em), eventBus);
        seminarService.finalSeminarSettingsService = finalSeminarSettingsService;
        seminarService.daysService = daysService;
        seminarService.fileDescriptionService = fileDescriptionService;
        seminarService.turnItInCheckRepo = turnItInCheckRepo;

        seminarService.fileDescriptionService = fileDescriptionService;
        seminarService.turnItInCheckRepo = turnItInCheckRepo;
        seminarService.oppositionReportService = oppositionReportService;

        finalSeminarOppositionService = mock(FinalSeminarOppositionService.class);
        seminarService.finalSeminarOppositionService = finalSeminarOppositionService;

        dummyProject = create_project(1L);

        finalSeminar = new FinalSeminar();
    }

    @Test
    public void hasDeadlinePassed() throws Exception {
        finalSeminar.setStartDate(new DateTime().minusDays(2).toDate());
        Date date = new DateTime().minusDays(1).toDate();
        FinalSeminarSettings finalSeminarSettings = new FinalSeminarSettings();
        finalSeminarSettings.setDaysAheadToUploadThesis(5);

        when(finalSeminarSettingsService.getInstance()).thenReturn(finalSeminarSettings);
        when(daysService.workDaysAhead(finalSeminar.getStartDate(), finalSeminarSettings.getDaysAheadToUploadThesis())).thenReturn(date);


        assertTrue(seminarService.hasDeadlinePassed(finalSeminar));
    }

    @Test
    public void hasThesis() throws Exception {
        FileDescription document = mock(FileDescription.class);
        finalSeminar.setDocument(document);
        when(document.isDataAvailable()).thenReturn(true);

        assertTrue(seminarService.hasThesis(finalSeminar));
    }

    @Test
    public void hasNoThesisDocumentIsNull() throws Exception {
        assertFalse(seminarService.hasThesis(finalSeminar));
    }

    @Test
    public void hasNoThesisNoFilePath() throws Exception {
        FileDescription document = mock(FileDescription.class);
        finalSeminar.setDocument(document);
        when(document.isDataAvailable()).thenReturn(false);

        assertFalse(seminarService.hasThesis(finalSeminar));
    }

    @Test
    public void deleteThesisDocumentIsNull() throws Exception {
        assertEquals(finalSeminar, seminarService.deleteThesis(finalSeminar));
    }

    @Test
    public void deletes() throws Exception {
        FinalSeminar finalSeminar = new FinalSeminar();

        FinalSeminarOpposition finalSeminarOpposition = new FinalSeminarOpposition();
        DomainObjects.injectId(finalSeminarOpposition, 1L);
        finalSeminarOpposition.setOppositionReport(new OppositionReport(new GradingReportTemplate(new ProjectType(DegreeLevel.FIRST_CYCLE, "bachelor", "bachelor"), 1), finalSeminarOpposition));
        finalSeminar.setOppositions(Collections.singletonList(finalSeminarOpposition));

        FileDescription document = new FileDescription();
        document.setId(1L);
        finalSeminar.setDocument(document);

        TurnItInCheck check = new TurnItInCheck();
        DomainObjects.injectId(check, 1L);
        when(turnItInCheckRepo.findByFileDescription(isA(FileDescription.class))).thenReturn(check);
        when(fileDescriptionService.findOne(isA(Long.class))).thenReturn(document);

        seminarService.delete(finalSeminar);

        verify(oppositionReportService).deleteOppositionReport(isA(FinalSeminarOpposition.class));
        verify(finalSeminarOppositionService).delete(isA(Long.class));
        verify(turnItInCheckRepo).delete(isA(Long.class));

        ArgumentCaptor<FinalSeminarDeletedEvent> captor = ArgumentCaptor.forClass(FinalSeminarDeletedEvent.class);
        verify(eventBus).post(captor.capture());

        FinalSeminarDeletedEvent event = captor.getValue();
        assertThat(event.getFinalSeminar(), is(finalSeminar));

    }

    @Test
    public void deleteThesis() throws Exception {
        finalSeminar.setDocument(new FileDescription());
        finalSeminar.setDocumentUploadDate(new Date());

        when(turnItInCheckRepo.findOne(isA(Long.class))).thenReturn(new TurnItInCheck());

        assertEquals(finalSeminar, seminarService.deleteThesis(finalSeminar));
        assertNull(finalSeminar.getDocument());
        assertNull(finalSeminar.getDocumentUploadDate());
    }

    private <T extends DomainObject> T get_with_id(T dom, Long id) {
        try {
            Class<? extends DomainObject> doClass = dom.getClass();
            Field idField = doClass.getDeclaredField("id");
            idField.setAccessible(true);
            idField.set(dom, id);
        } catch (Exception ex) {
            dom = Mockito.spy(dom);
            Mockito.when(dom.getId()).thenReturn(id);
            System.out.println("could not inject id");
            ex.printStackTrace();
        }
        return dom;
    }

    private Project create_project(Long id) {
        Project p = get_with_id(new Project(), id);
        User supervisor = get_with_id(new User(), 10L);
        p.setHeadSupervisor(supervisor);
        return p;
    }

}
