package se.su.dsv.scipro.forum;

import com.google.common.eventbus.EventBus;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.runners.MockitoJUnitRunner;
import org.mockito.stubbing.Answer;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.forum.dataobjects.ForumPostReadState;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forummail.ForumMailSettings;
import se.su.dsv.scipro.forummail.ForumMailSettingsService;
import se.su.dsv.scipro.mail.MailEvent;
import se.su.dsv.scipro.mail.MailEventService;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ForumBuilder;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class BasicForumServiceImplTest {

    @Mock
    private AbstractThreadRepository threadRepository;
    @Mock
    private ForumPostReadStateRepository readStateRepository;
    @Mock
    private ForumPostRepository postRepository;
    @Mock
    private MailEventService mailEventService;
    @Mock
    private ForumMailSettingsService mailSettingsService;
    @Mock
    private EventBus eventBus;
    @InjectMocks
    private BasicForumServiceImpl basicForumService;

    @Before
    public void setUp() throws Exception {
        when(readStateRepository.find(any(User.class), any(ForumPost.class))).thenReturn(new ForumPostReadState());
        when(readStateRepository.save(isA(ForumPostReadState.class))).thenAnswer(new Answer<Object>() {
            @Override
            public Object answer(final InvocationOnMock invocation) throws Throwable {
                return invocation.getArguments()[0];
            }
        });
        when(mailEventService.save(isA(MailEvent.class))).thenReturn(new MailEvent());
        when(mailSettingsService.getSettings()).thenReturn(new ForumMailSettings());
        when(threadRepository.save(any(ForumThread.class))).thenAnswer(new Answer<Object>() {
            @Override
            public Object answer(final InvocationOnMock invocation) throws Throwable {
                return invocation.getArgumentAt(0, ForumThread.class);
            }
        });
        when(postRepository.save(any(ForumPost.class))).thenAnswer(new Answer<Object>() {
            @Override
            public Object answer(final InvocationOnMock invocation) throws Throwable {
                return invocation.getArgumentAt(0, ForumPost.class);
            }
        });
    }

    @Test
    public void testGetPostPageByForumThread() throws Exception {
        List<ForumPost> posts = Collections.singletonList(new ForumBuilder().createPost());
        when(postRepository.findByThread(isA(ForumThread.class))).thenReturn(posts);

        List<ForumPost> servicePage = basicForumService.getPosts(mock(ForumThread.class));

        assertEquals(posts, servicePage);
    }

    @Test
    public void testMarkRead() throws Exception {
        User user = new User();
        ForumPost post = new ForumPost();

        boolean read = basicForumService.setRead(user, post, true);

        assertTrue("Did not return proper read state", read);

        ArgumentCaptor<ForumPostReadState> captor = ArgumentCaptor.forClass(ForumPostReadState.class);
        verify(readStateRepository, times(1)).save(captor.capture());

        assertTrue("Did not save correct read state", captor.getValue().isRead());
    }

    @Test
    public void testMarkUnread() throws Exception {
        // when
        User user = new User();
        ForumPost post = new ForumPost();

        // when
        boolean read = basicForumService.setRead(user, post, false);

        // then
        assertFalse("Did not return proper read state", read);

        ArgumentCaptor<ForumPostReadState> captor = ArgumentCaptor.forClass(ForumPostReadState.class);
        verify(readStateRepository, times(1)).save(captor.capture());

        assertFalse("Did not save correct read state", captor.getValue().isRead());
    }

    @Test
    public void testMarkThreadRead() throws Exception {
        User user = new User();
        ForumPost post = new ForumPost();
        ForumPost post2 = new ForumPost();
        ForumThread forumThread = new ForumThread();
        forumThread.addPost(post);
        forumThread.addPost(post2);

        basicForumService.setThreadRead(user, forumThread, true);

        ArgumentCaptor<ForumPostReadState> captor = ArgumentCaptor.forClass(ForumPostReadState.class);
        verify(readStateRepository, times(2)).save(captor.capture());

        assertTrue("Did not save correct read state", captor.getValue().isRead());
    }

    @Test
    public void testIsThreadRead() throws Exception {
        User goodUser = new UserBuilder().setFirstName("Reads").setLastName("Forum").create();
        User badUser = new UserBuilder().setFirstName("Does not read").setLastName("Forum").create();

        ForumPost post = new ForumPost();
        ForumPostReadState readState = new ForumPostReadState(goodUser, post);
        readState.setRead(true);

        ForumPostReadState notReadState = new ForumPostReadState(badUser, post);
        notReadState.setRead(false);

        ForumThread forumThread = new ForumThread();
        forumThread.addPost(post);

        when(readStateRepository.find(eq(goodUser), isA(ForumPost.class))).thenReturn(readState);
        when(readStateRepository.find(eq(badUser), isA(ForumPost.class))).thenReturn(notReadState);

        boolean goodUserState = basicForumService.isThreadRead(goodUser, forumThread);
        boolean badUserState = basicForumService.isThreadRead(badUser, forumThread);

        assertTrue("Good user has not read all thread posts", goodUserState);
        assertFalse("Bad user has read all thread posts", badUserState);
    }

    @Test
    public void create_forum_thread() throws Exception {
        final String subject = "Subject";

        ForumThread thread = basicForumService.createThread(subject);

        assertThat(thread.getSubject(), is(subject));
    }

    @Test
    public void reply_to_thread() throws Exception {
        final ForumThread forumThread = new ForumThread();
        final User poster = User.builder().firstName("Bob").lastName("Example").emailAddress("bob@example.com").build();
        final String content = "content";

        ForumPost forumPost = basicForumService.createReply(forumThread, poster, content, Collections.<FileDescription>emptySet());

        assertThat(forumPost.getContent(), is(content));
        assertThat(forumPost.getPostedBy(), is(poster));
        assertThat(forumPost.getForumThread(), is(forumThread));
    }

    @Test
    public void mark_post_read_posts_event() throws Exception {
        final ForumPost post = new ForumPost();
        post.setId(235235L);
        final User user = new User();
        user.setId(2378924L);

        basicForumService.setRead(user, post, true);

        ArgumentCaptor<ForumPostReadEvent> captor = ArgumentCaptor.forClass(ForumPostReadEvent.class);
        verify(eventBus).post(captor.capture());

        ForumPostReadEvent event = captor.getValue();
        assertEquals(event.getPost(), post);
        assertEquals(event.getUser(), user);
    }

    @Test
    public void mark_post_unread_does_not_post_read_event() throws Exception {
        final ForumPost post = new ForumPost();
        post.setId(235235L);
        final User user = new User();
        user.setId(2378924L);

        basicForumService.setRead(user, post, false);

        verify(eventBus, never()).post(isA(ForumPostReadEvent.class));
    }
}