package se.su.dsv.scipro.forummail;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.system.User;

import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMultipart;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import static org.hamcrest.CoreMatchers.containsString;
import static org.hamcrest.CoreMatchers.startsWith;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class ForumMailFormatterImplTest {

    ForumMailFormatterImpl parser;

    @Before
    public void setUp() throws Exception {
        parser = new ForumMailFormatterImpl();
    }

    @Test
    public void message_without_delimiter_dont_substring() throws Exception {
        String noDelimiter = "string without delimiter";
        Part message = createMessage(noDelimiter, "text/plain");
        assertEquals(noDelimiter, parser.format(message));
    }

    @Test
    public void substring_on_delimiter() throws Exception {
        String body = stringWithDelimiter("This is the reply", "This is the old message");
        Part message = createMessage(body, "text/plain");
        assertEquals("This is the reply", parser.format(message));
    }

    @Test
    public void unknown_mime_type_returns_null() throws Exception {
        Part message = createMessage("message body", "foo/bar");
        assertEquals("", parser.format(message));
    }

    @Test
    public void multipart_return_text_plain_if_available() throws Exception {
        String plainText = "plain text";
        Part message = createMultipartMessage(plainText, 2);
        assertEquals(plainText, parser.format(message));
    }

    @Test
    public void empty_string_when_no_text_plain_is_available() throws Exception {
        assertEquals("", parser.format(createMultipartMessage("text", 1)));
    }

    @Test
    public void empty_string_on_messaging_exception() throws Exception {
        assertEquals("", parser.format(createInvalidMessage()));
    }

    @Test
    public void composed_content_starts_with_delimiter() throws Exception {
        assertThat(parser.formatBody(newThread()), startsWith(ForumMailFormatterImpl.DELIMITER));
    }

    @Test
    public void includes_the_content_of_every_post_in_the_thread() throws Exception {
        ForumThread forumThread = newThread();

        String content = parser.formatBody(forumThread);

        for (ForumPost post : forumThread.getPosts()) {
            assertThat(content, containsString(post.getContent()));
        }
    }

    @Test
    public void includes_the_poster_of_every_post_in_the_thread() throws Exception {
        ForumThread forumThread = newThread();

        String content = parser.formatBody(forumThread);

        for (ForumPost forumPost : forumThread.getPosts()) {
            assertThat(content, containsString(forumPost.getPostedBy().getFullName()));
        }
    }

    private ForumThread newThread() {
        ForumThread forumThread = mock(ForumThread.class);
        List<ForumPost> forumPosts = Arrays.asList(createPost("My first post"), createPost("My second post"));
        when(forumThread.getPosts()).thenReturn(forumPosts);
        when(forumThread.getPostCount()).thenReturn(forumPosts.size());
        return forumThread;
    }

    private ForumPost createPost(String content) {
        ForumPost forumPost = new ForumPost();
        forumPost.setContent(content);
        forumPost.setPostedBy(User.builder().firstName("Bob").lastName("Tester").emailAddress("bob@example.com").build());
        return forumPost;
    }

    private String stringWithDelimiter(String first, String second) {
        return first + ForumMailFormatterImpl.DELIMITER + second;
    }

    private Part createMessage(String body, String type) throws MessagingException {
        Part part = new MimeBodyPart();
        part.setContent(body, type);
        part.setHeader("Content-Type", type);
        return part;
    }

    private Part createMultipartMessage(String plain, int parts) throws MessagingException, IOException {
        Part part = new MimeBodyPart();
        Multipart mp = mock(Multipart.class);
        when(mp.getCount()).thenReturn(parts);
        MimeBodyPart textPlainPart = createBodyPart(plain, "text/plain");
        MimeBodyPart multi = createBodyPart("multi", "multipart/mixed");
        when(mp.getBodyPart(0)).thenReturn(multi);
        when(mp.getBodyPart(1)).thenReturn(textPlainPart);
        part.setContent(mp, "multipart/alternative");
        part.setHeader("Content-Type", "multipart/alternative");
        return part;
    }

    private Part createInvalidMessage() throws MessagingException {
        MimeBodyPart part = new MimeBodyPart();
        MimeMultipart mp = new MimeMultipart();
        mp.addBodyPart(new MimeBodyPart());
        part.setContent(mp);
        part.setHeader("Content-Type", "multipart/alternative");
        return part;
    }

    private MimeBodyPart createBodyPart(String plain, String contentType) throws MessagingException {
        MimeBodyPart textPlainPart = new MimeBodyPart();
        textPlainPart.setText(plain);
        textPlainPart.setHeader("Content-Type", contentType);
        return textPlainPart;
    }

}
