package se.su.dsv.scipro.report;

import org.junit.Assert;
import org.junit.Test;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

public abstract class GradeCalculatorTest {
    @Test
    public void a_report_whose_answers_does_not_meet_minimum_requirements_fails() throws Exception {
        assessAllCriteria(0);
        Assert.assertEquals(GradingReport.Grade.F, getGradingReport().getGrade(getGradeCalculator()));
    }

    @Test
    public void a_report_whose_answers_are_all_max_points_has_A() throws Exception {
        assessAllCriteriaTheirMaxPoints();
        Assert.assertEquals(GradingReport.Grade.A, getGradingReport().getGrade(getGradeCalculator()));
    }

    @Test
    public void a_report_without_assessed_criteria_has_null_grade() throws Exception {
        assertNull(getGradeCalculator().getGrade(getGradingReport()));
    }

    @Test
    public void a_report_that_answered_all_criteria_has_a_grade() throws Exception {
        assessAllCriteria();
        assertNotNull(getGradeCalculator().getGrade(getGradingReport()));
    }

    @Test
    public void calculating_total_points_for_unfinished_report() throws Exception {
        int points = 5;
        getGradingReport().getGradingCriteria().get(0).setPoints(points);

        Assert.assertEquals(points, getGradingReport().getPoints(getGradeCalculator()));
    }

    protected void assessAllCriteria() {
        assessAllCriteria(0);
    }

    protected void assessAllCriteria(int points) {
        for (GradingCriterion gradingCriterion : getGradingReport().getGradingCriteria()) {
            assess(gradingCriterion, points);
        }
    }

    protected void assessAllCriteriaWithMinimumAndTotal(int remaining) {
        for (GradingCriterion gradingCriterion : getGradingReport().getGradingCriteria()) {
            int toAssign = Math.min(remaining, gradingCriterion.getPointsRequiredToPass());
            int leftAfterAssignment = assignAtMost(toAssign, gradingCriterion);
            remaining -= (toAssign - leftAfterAssignment);
        }

        for (GradingCriterion gradingCriterion : getGradingReport().getGradingCriteria()) {
            remaining = assignAtMost(remaining, gradingCriterion);
        }
    }

    protected int assignAtMost(int max, GradingCriterion gradingCriterion) {
        int current = gradingCriterion.getPoints() != null ? gradingCriterion.getPoints() : 0;
        int canAssign = gradingCriterion.getMaxPoints() - current;
        int shouldAssign = Math.min(max, canAssign);
        assess(gradingCriterion, current + shouldAssign);
        return max - shouldAssign;
    }

    private void assessAllCriteriaTheirMaxPoints() {
        for (GradingCriterion gradingCriterion : getGradingReport().getGradingCriteria()) {
            assess(gradingCriterion, gradingCriterion.getMaxPoints());
        }
    }

    protected void assignPointsToProjectCriteriaWithFxFlag(int numberOfCriteriaToAssign) {
        for (GradingCriterion gradingCriterion : getGradingReport().getGradingCriteria()) {
            if (gradingCriterion.isFx() && gradingCriterion.isProjectCriterion() && numberOfCriteriaToAssign > 0) {
                assess(gradingCriterion, gradingCriterion.getMaxPoints());
                numberOfCriteriaToAssign--;
            }
        }
    }

    protected void assignPointsToTwoIndividualCriteriaWithFxFlag(int numberOfCriteriaToAssign) {
        for (GradingCriterion gradingCriterion : getGradingReport().getGradingCriteria()) {
            if (gradingCriterion.isFx() && gradingCriterion.isIndividualCriterion() && numberOfCriteriaToAssign > 0) {
                assess(gradingCriterion, gradingCriterion.getMaxPoints());
                numberOfCriteriaToAssign--;
            }
        }
    }

    protected abstract GradingReport getGradingReport();

    protected abstract GradeCalculator getGradeCalculator();

    protected abstract GradingReportTemplate prepareTemplate();

    protected void assess(GradingCriterion gradingCriterion, int points) {
        gradingCriterion.setPoints(points);
        gradingCriterion.setFeedback("Some random feedback");
    }
}
