package se.su.dsv.scipro.report;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import scala.util.Either;
import se.su.dsv.scipro.finalseminar.FinalSeminar;
import se.su.dsv.scipro.finalseminar.FinalSeminarOpposition;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.*;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.*;
import static se.su.dsv.scipro.report.GradingReportServiceImpl.OPPOSITION_ENGLISH;
import static se.su.dsv.scipro.report.GradingReportServiceImpl.OPPOSITION_SWEDISH;

public class GradingReportServiceImplIntegrationTest extends IntegrationTest {

    public static final String FEEDBACK_ON_OPPOSITION = "Feedback on opposition";
    public static final int OPPOSITION_CRITERION_POINTS = 2;
    public static final String FEEDBACK = "feedback";
    @Inject
    private GradingReportServiceImpl gradingReportService;

    private ProjectType projectType;
    private GradingReportTemplate gradingReportTemplate;
    private Project project;
    private SupervisorGradingReport gradingReport;
    private User student;

    @Before
    public void setUp() throws Exception {
        projectType = createProjectType(DegreeLevel.FIRST_CYCLE);
        gradingReportTemplate = createGradingReportTemplate(projectType);
        student = createStudent();
        project = createProject(projectType, 30);
        gradingReportTemplate = createProjectGradingCriterion(gradingReportTemplate, 2);
        gradingReportTemplate = createIndividualGradingCriterion(gradingReportTemplate, 2);
        gradingReport = createGradingReport(project, student);
    }

    @Test
    public void has_as_many_reports_as_students_in_project() throws Exception {
        User student1 = createStudent();
        project.addProjectParticipant(student1);

        createGradingReport(project, student1);

        assertEquals(project.getProjectParticipants().size(), gradingReportService.getSupervisorGradingReports(project).size());
    }

    @Test
    public void creates_grading_reports_if_not_created() throws Exception {
        assertEquals(1, gradingReportService.getSupervisorGradingReports(project).size());
    }

    @Test
    public void copy_report() throws Exception {
        Project project = createProject(projectType, 30);
        SupervisorGradingReport filledInReport = createFinishedGradingReport(project);
        SupervisorGradingReport notFilledInReport = createGradingReport(project, createStudent());

        gradingReportService.copyReport(notFilledInReport, filledInReport);

        Integer toPoints = notFilledInReport.getGradingCriteria().get(0).getPoints();
        Integer fromPoints = filledInReport.getGradingCriteria().get(0).getPoints();
        assertEquals(toPoints, fromPoints);
    }

    @Test
    public void submit_supervisor_grading_report_flags_report_as_submitted() throws Exception {
        assessAllCriteria(gradingReport);
        Either<List<GradingCriterion>, GradingReport> result = gradingReportService.submitReport(gradingReport);
        assertTrue(result.isRight());
        assertThat(result.right().get().getState(), is(GradingReport.State.FINALIZED));
    }

    @Test
    public void submitting_supervisor_report_throws_exception_if_report_is_not_finished() throws Exception {
        Either<List<GradingCriterion>, GradingReport> result = gradingReportService.submitReport(gradingReport);
        assertFalse(result.isRight());
    }

    @Test
    public void copying_a_report_also_copies_plagiarism_feedback() throws Exception {
        String plagiarismFeedback = "feedback";
        gradingReport.setPlagiarismFeedback(plagiarismFeedback);
        save(gradingReport);

        SupervisorGradingReport copyTarget = gradingReportTemplate.createSupervisorReport(project, createStudent());

        gradingReportService.copyReport(copyTarget, gradingReport);

        Assert.assertThat(copyTarget.getPlagiarismFeedback(), is(plagiarismFeedback));
    }

    @Test
    public void update_opposition_criterion() throws Exception {
        addOppositionCriterion();
        boolean updated = updateOppositionCriterion();

        GradingCriterion oppositionCriterion = findOppositionCriterion(OPPOSITION_SWEDISH);
        assert oppositionCriterion != null;
        assertEquals(FEEDBACK_ON_OPPOSITION, oppositionCriterion.getFeedback());
        assertEquals((Integer) OPPOSITION_CRITERION_POINTS, oppositionCriterion.getPoints());
        assertTrue(updated);
    }

    @Test
    public void update_opposition_if_title_matches_english_title() throws Exception {
        addEnglishOppositionCriterion();
        boolean updated = updateOppositionCriterion();

        GradingCriterion oppositionCriterion = findOppositionCriterion(OPPOSITION_ENGLISH);
        assert oppositionCriterion != null;
        assertEquals(FEEDBACK_ON_OPPOSITION, oppositionCriterion.getFeedback());
        assertEquals((Integer) OPPOSITION_CRITERION_POINTS, oppositionCriterion.getPoints());
        assertTrue(updated);
    }

    @Test
    public void updating_opposition_criterion_does_nothing_if_criterion_already_has_values() throws Exception {
        addOppositionCriterion();
        assessAllCriteria(gradingReport);
        boolean updated = updateOppositionCriterion();

        GradingCriterion oppositionCriterion = findOppositionCriterion(OPPOSITION_SWEDISH);
        assert oppositionCriterion != null;
        assertEquals(FEEDBACK, oppositionCriterion.getFeedback());
        assertEquals((Integer) oppositionCriterion.getMaxPoints(), oppositionCriterion.getPoints());
        assertFalse(updated);
    }

    @Test
    public void updating_opposition_criterion_does_nothing_if_criterion_already_has_feedback() throws Exception {
        addOppositionCriterion();
        setFeedbackOnAllCriteria(gradingReport);
        updateOppositionCriterion();

        GradingCriterion oppositionCriterion = findOppositionCriterion(OPPOSITION_SWEDISH);
        assert oppositionCriterion != null;
        assertEquals(FEEDBACK, oppositionCriterion.getFeedback());
        assertEquals(null, oppositionCriterion.getPoints());
    }

    @Test
    public void updating_opposition_criterion_does_nothing_if_criterion_already_has_points() throws Exception {
        addOppositionCriterion();
        int points = 1;
        setPointsOnAllCriteria(gradingReport, points);
        updateOppositionCriterion();

        GradingCriterion oppositionCriterion = findOppositionCriterion(OPPOSITION_SWEDISH);
        assert oppositionCriterion != null;
        assertEquals((Integer) points, oppositionCriterion.getPoints());
        assertEquals(null, oppositionCriterion.getFeedback());
    }

    @Test
    public void must_assess_all_criteria_before_finishing_initial_assessment() throws Exception {
        Either<List<GradingCriterion>, GradingReport> result = gradingReportService.initialAssessmentDone(gradingReport);
        assertFalse(result.isRight());
    }

    @Test
    public void finishing_initial_assessment_is_possible_after_doing_complete_assessment() throws Exception {
        assessAllCriteria(gradingReport);
        Either<List<GradingCriterion>, GradingReport> result = gradingReportService.initialAssessmentDone(gradingReport);
        assertTrue(result.isRight());
        assertThat(result.right().get().getState(), is(GradingReport.State.REVIEWING));
    }

    private void addOppositionCriterion() {
        gradingReportTemplate = createSwedishOppositionCriteria(gradingReportTemplate, 2);
        gradingReport = createGradingReport(project, student);
    }

    private void addEnglishOppositionCriterion() {
        gradingReportTemplate = createEnglishOppositionCriteria(gradingReportTemplate, 2);
        gradingReport = createGradingReport(project, student);
    }

    private boolean updateOppositionCriterion() {
        FinalSeminarOpposition opposition = createFinalSeminarOpposition();
        return gradingReportService.updateOppositionCriteria(gradingReport, opposition);
    }

    private GradingCriterion findOppositionCriterion(String title) {
        for (GradingCriterion gradingCriterion : gradingReport.getIndividualCriteria()) {
            if (gradingCriterion.getTitle().equals(title)) {
                return gradingCriterion;
            }
        }
        return null;
    }

    private FinalSeminarOpposition createFinalSeminarOpposition() {
        FinalSeminarOpposition finalSeminarOpposition = new FinalSeminarOpposition();
        finalSeminarOpposition.setProject(createProject(projectType, 30));
        finalSeminarOpposition.setUser(createStudent());
        finalSeminarOpposition.setFinalSeminar(createFinalSeminar());

        finalSeminarOpposition.setFeedback(FEEDBACK_ON_OPPOSITION);
        finalSeminarOpposition.setPoints(OPPOSITION_CRITERION_POINTS);

        return save(finalSeminarOpposition);
    }

    private FinalSeminar createFinalSeminar() {
        Language english = new Language();
        english.setName("English");
        save(english);
        FinalSeminar finalSeminar = new FinalSeminar(2, 2);
        finalSeminar.setProject(project);
        finalSeminar.setStartDate(new Date());
        finalSeminar.setRoom("101");
        finalSeminar.setPresentationLanguage(english);
        finalSeminar.setReportLanguage(english);
        return save(finalSeminar);
    }

    private void assessAllCriteria(SupervisorGradingReport gradingReport) {
        for (GradingCriterion criterion : gradingReport.getGradingCriteria()) {
            criterion.setPoints(criterion.getMaxPoints());
            criterion.setFeedback(FEEDBACK);
        }
        gradingReport.setPlagiarismFeedback("plagiarism feedback");
    }

    private void setPointsOnAllCriteria(SupervisorGradingReport gradingReport, int points) {
        for (GradingCriterion criterion : gradingReport.getGradingCriteria()) {
            criterion.setPoints(points);
        }
    }

    private void setFeedbackOnAllCriteria(SupervisorGradingReport gradingReport) {
        for (GradingCriterion criterion : gradingReport.getGradingCriteria()) {
            criterion.setFeedback(FEEDBACK);
        }
    }

    private SupervisorGradingReport createFinishedGradingReport(Project project) {
        SupervisorGradingReport finished = createGradingReport(project, student);
        for (GradingCriterion gradingCriterion : finished.getGradingCriteria()) {
            gradingCriterion.setPoints(gradingCriterion.getMaxPoints());
        }
        return save(finished);
    }

    private SupervisorGradingReport createGradingReport(Project project, User student) {
        SupervisorGradingReport gradingReport = gradingReportTemplate.createSupervisorReport(project, student);
        return save(gradingReport);
    }

    private GradingReportTemplate createProjectGradingCriterion(GradingReportTemplate gradingReportTemplate, int maxPoints) {
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = getPointTemplates(maxPoints, "criterion description");
        gradingReportTemplate.addProjectCriterion("Project Criterion", 0, gradingCriterionPointTemplates);
        return save(gradingReportTemplate);
    }

    private List<GradingCriterionPointTemplate> getPointTemplates(int maxPoints, String desc) {
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();
        gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(0).description(desc).build());
        for (int i = 1; i <= maxPoints; i++) {
            gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(i).description(desc).build());
        }
        return gradingCriterionPointTemplates;
    }

    private GradingReportTemplate createIndividualGradingCriterion(GradingReportTemplate gradingReportTemplate, int maxPoints) {
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = getPointTemplates(maxPoints, "criterion description");
        gradingReportTemplate.addIndividualCriterion("Individual Criterion", 0, gradingCriterionPointTemplates);
        return save(gradingReportTemplate);
    }

    private GradingReportTemplate createSwedishOppositionCriteria(GradingReportTemplate gradingReportTemplate, int maxPoints) {
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = getPointTemplates(maxPoints, "Opposition");
        gradingReportTemplate.addIndividualCriterion(OPPOSITION_SWEDISH, 0, gradingCriterionPointTemplates);
        return save(gradingReportTemplate);
    }

    private GradingReportTemplate createEnglishOppositionCriteria(GradingReportTemplate gradingReportTemplate, int maxPoints) {
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = getPointTemplates(maxPoints, "Opposition");
        gradingReportTemplate.addIndividualCriterion(OPPOSITION_ENGLISH, 0, gradingCriterionPointTemplates);
        return save(gradingReportTemplate);
    }

    private GradingReportTemplate createGradingReportTemplate(ProjectType projectType) {
        GradingReportTemplate gradingReportTemplate = new GradingReportTemplate(projectType, 30);
        return save(gradingReportTemplate);
    }

    private Project createProject(ProjectType projectType, int credits) {
        TreeSet<User> projectParticipants = new TreeSet<>(new User.ByNameComparator());
        projectParticipants.addAll(Collections.singletonList(student));
        Project project = Project.builder().title("some title").projectType(projectType).projectParticipants(projectParticipants).headSupervisor(createSupervisor()).build();
        project.setCredits(credits);
        return save(project);
    }

    private ProjectType createProjectType(DegreeLevel degreeLevel) {
        ProjectType projectType = new ProjectType(degreeLevel, "Some name", "Some description");
        return save(projectType);
    }

    private User createStudent() {
        return save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
    }

    private User createSupervisor() {
        User user = save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
        user.addRole(Roles.SUPERVISOR);
        return save(user);
    }

}
