package se.su.dsv.scipro.report;

import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionRepo;
import se.su.dsv.scipro.finalseminar.FinalSeminar;
import se.su.dsv.scipro.finalseminar.FinalSeminarOpposition;
import se.su.dsv.scipro.finalseminar.FinalSeminarOppositionRepo;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserRepo;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.ObjectMother;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyInt;

@RunWith(MockitoJUnitRunner.class)
public class OppositionReportServiceImplTest {

    public static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, "bachelor", "bachelor");

    @Mock
    OppositionReportRepo oppositionReportRepo;
    @Mock
    GradingReportTemplateRepo gradingReportTemplateRepo;
    @Mock
    UserRepo userRepo;
    @InjectMocks
    OppositionReportServiceImpl oppositionReportService;
    private OppositionReport oppositionReport;
    private FinalSeminarOpposition finalSeminarOpposition;
    @Mock
    FileDescriptionRepo fileDescriptionRepo;
    @Mock
    FinalSeminarOppositionRepo finalSeminarOppositionRepo;

    @Before
    public void setUp() throws Exception {
        GradingReportTemplate template = new GradingReportTemplate(BACHELOR, 30);
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();
        gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(1).description("Description").build());
        template.addProjectCriterion("title", 1, gradingCriterionPointTemplates);
        finalSeminarOpposition = createFinalSeminarOpposition();
        DomainObjects.injectId(finalSeminarOpposition, 1L);

        oppositionReport = template.createOppositionReport(finalSeminarOpposition);
    }

    @Test
    public void save_report() throws Exception {
        oppositionReportService.save(oppositionReport);
        Mockito.verify(oppositionReportRepo).save(oppositionReport);
    }

    @Test
    public void create_report_if_not_existing() throws Exception {
        GradingReportTemplate template = Mockito.mock(GradingReportTemplate.class);
        Mockito.when(template.createOppositionReport(any(FinalSeminarOpposition.class))).thenReturn(oppositionReport);
        Mockito.when(gradingReportTemplateRepo.findByProjectTypeAndCredits(any(ProjectType.class), anyInt())).thenReturn(template);
        Mockito.when(oppositionReportRepo.save(oppositionReport)).thenReturn(oppositionReport);
        assertEquals(oppositionReport, oppositionReportService.findOrCreateReport(createFinalSeminarOpposition()));
    }

    @Test
    public void get_existing_opposition_report() throws Exception {
        Mockito.when(oppositionReportRepo.findByFinalSeminarOpposition(any(FinalSeminarOpposition.class))).thenReturn(oppositionReport);
        assertEquals(oppositionReport, oppositionReportService.findOrCreateReport(createFinalSeminarOpposition()));
    }

    @Test
    public void delete_opposition_report() throws Exception {
        finalSeminarOpposition.setOppositionReport(oppositionReport);
        oppositionReportService.deleteOppositionReport(finalSeminarOpposition);

        assertEquals(null, finalSeminarOpposition.getOppositionReport());
    }

    @Test
    public void delete_uploaded_opponent_report() throws Exception {
        finalSeminarOpposition.setOpponentReport(new FileDescription());
        oppositionReportService.deleteOpponentReport(finalSeminarOpposition);

        assertEquals(null, finalSeminarOpposition.getOpponentReport());
    }

    private FinalSeminarOpposition createFinalSeminarOpposition() {
        FinalSeminarOpposition finalSeminarOpposition = new FinalSeminarOpposition();
        finalSeminarOpposition.setFinalSeminar(createFinalSeminar());
        finalSeminarOpposition.setProject(ObjectMother.SOME_PROJECT);
        finalSeminarOpposition.setUser(createUser());
        return finalSeminarOpposition;
    }

    private User createUser() {
        User studentUser = new UserBuilder().create();
        studentUser = userRepo.save(studentUser);
        return studentUser;
    }

    private FinalSeminar createFinalSeminar() {
        FinalSeminar finalSeminar = new FinalSeminar();
        finalSeminar.setProject(ObjectMother.SOME_PROJECT);
        finalSeminar.setStartDate(new DateTime().minusDays(1).toDate());
        finalSeminar.setRoom("room");
        return finalSeminar;
    }
}
