package se.su.dsv.scipro.report;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.*;

public class SupervisorGradingReportTest {

    private SupervisorGradingReport gradingReport;
    private GradingReportTemplate gradingReportTemplate;
    private Project project;
    public static final String FEEDBACK = "Feedback";

    @Before
    public void prepareTemplate() throws Exception {
        ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "bachelor", "bachelor");
        project = Project.builder().title("Foo").projectType(bachelor).build();

        gradingReportTemplate = new GradingReportTemplate(bachelor, 30);
        addCriteria(gradingReportTemplate);

        gradingReport = gradingReportTemplate.createSupervisorReport(project, new User());
    }

    @Test
    public void a_report_without_any_assessment_is_not_finished() throws Exception {
        assertFalse(gradingReport.isFinished());
    }

    @Test
    public void a_report_with_assessment_for_all_criteria_and_plagiarism_feedback_is_finished() throws Exception {
        assessAllCriteria();
        gradingReport.setPlagiarismFeedback("This is such a copy");
        assertTrue(gradingReport.isFinished());
    }

    @Test
    public void a_report_is_only_finished_if_plagiarism_feedback_has_been_assigned() throws Exception {
        assessAllCriteria();
        assertFalse(gradingReport.isFinished());
    }

    @Test
    public void copy_from_project_assessment_copies_points() throws Exception {
        User user = User.builder().firstName("first").lastName("last").emailAddress("mail").build();
        SupervisorGradingReport gradingReport2 = gradingReportTemplate.createSupervisorReport(project, user);

        Integer points = 1;

        assessAllCriteria(points);

        gradingReport2.copyProjectAssessmentFrom(gradingReport);
        for (GradingCriterion criterion : gradingReport2.getProjectCriteria()) {
            assertEquals(points, criterion.getPoints());
        }
    }

    @Test
    public void copy_from_project_assessment_copies_feedback() throws Exception {
        User user = User.builder().firstName("first").lastName("last").emailAddress("mail").build();
        SupervisorGradingReport gradingReport2 = gradingReportTemplate.createSupervisorReport(project, user);

        assessAllCriteria();

        gradingReport2.copyProjectAssessmentFrom(gradingReport);
        for (GradingCriterion criterion : gradingReport2.getProjectCriteria()) {
            assertEquals(FEEDBACK, criterion.getFeedback());
        }
    }

    private void assessAllCriteria() {
        assessAllCriteria(0);
    }

    private void assessAllCriteria(int points) {
        for (GradingCriterion gradingCriterion : gradingReport.getGradingCriteria()) {
            assess(gradingCriterion, points);
        }
    }

    private void assess(GradingCriterion gradingCriterion, int points) {
        gradingCriterion.setPoints(points);
        gradingCriterion.setFeedback(FEEDBACK);
    }

    private void addCriteria(GradingReportTemplate gradingReportTemplate) {

        gradingReportTemplate.addProjectCriterion("U1 Sammanfattning", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U2 Introduktion", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U3 Problem", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U4 Frågeställning", 1, getPointTemplates(2));
        gradingReportTemplate.addProjectCriterion("U5 Vetenskaplig förankring", 1, getPointTemplates(2));
        gradingReportTemplate.addProjectCriterion("U6 Metodval", 1, getPointTemplates(2));
        gradingReportTemplate.addProjectCriterion("U7 Metodtillämpning", 1, getPointTemplates(2));
        gradingReportTemplate.addProjectCriterion("U8 Resultat", 1, getPointTemplates(2));
        gradingReportTemplate.addProjectCriterion("U9 Slutsatser och diskussion", 1, getPointTemplates(2)).setFx(false);
        gradingReportTemplate.addProjectCriterion("U10 Form, struktur och språk", 1, getPointTemplates(1)).setFx(false);
        gradingReportTemplate.addProjectCriterion("U11 Argumentation", 1, getPointTemplates(1)).setFx(false);
        gradingReportTemplate.addProjectCriterion("U12 Källhänvisningar och dokumentation", 1, getPointTemplates(1)).setFx(false);
        gradingReportTemplate.addProjectCriterion("U13 Originalitet och signifikans", 0, getPointTemplates(3)).setFx(false);

        gradingReportTemplate.addIndividualCriterion("Ö1 Oppositionsrapport", 1, getPointTemplates(2));
        gradingReportTemplate.addIndividualCriterion("Ö2 Presentationer", 1, getPointTemplates(1));
        gradingReportTemplate.addIndividualCriterion("Ö3 Aktivitet vid seminarier och möten", 1, getPointTemplates(1));
        gradingReportTemplate.addIndividualCriterion("Ö4 Deadlines", 0, getPointTemplates(1)).setFx(false);
        gradingReportTemplate.addIndividualCriterion("Ö5 Revision efter slutseminarium", 0, getPointTemplates(1)).setFx(false);
    }

    private List<GradingCriterionPointTemplate> getPointTemplates(int maxPoint) {
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();
        for (int i = 0; i <= maxPoint; i++) {
            gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(i).description("Description").build());
        }
        return gradingCriterionPointTemplates;
    }
}
