package se.su.dsv.scipro.springdata.serviceimpls;

import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.springdata.services.SupervisorService;
import se.su.dsv.scipro.system.ResearchArea;
import se.su.dsv.scipro.system.Unit;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.Username;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import static java.util.Arrays.asList;
import static org.junit.Assert.assertEquals;
import static se.su.dsv.scipro.springdata.services.SupervisorService.FilterParams;

public class SupervisorServiceImplTest extends IntegrationTest {
    @Inject
    private SupervisorService supervisorService;

    private User employee;
    private List<User> emptyPage, employeePage;
    private PageRequest pageRequest;

    @Before
    public void setUp() throws Exception {
        employee = createUser();
        pageRequest = new PageRequest(0, 5);
        emptyPage = new ArrayList<>();
        employeePage = asList(employee);
    }

    @Test
    public void findByResearchArea() throws Exception {
        ResearchArea researchArea = createResearchArea();
        employee.setResearchAreas(new HashSet<>(asList(researchArea)));

        assertEquals(employeePage, supervisorService.findByResearchArea(researchArea, pageRequest));
    }

    @Test
    public void findByResearchAreaAndAreaIsNullShouldFindAll() throws Exception {
        assertEquals(employeePage, supervisorService.findByResearchArea(null, pageRequest));
    }

    @Test
    public void countByResearchArea() throws Exception {
        ResearchArea researchArea = createResearchArea();
        employee.setResearchAreas(new HashSet<>(asList(researchArea)));

        Long expected = 1L;

        assertEquals(expected, supervisorService.countByResearchArea(researchArea));
    }

    @Test
    public void findAllWithEmptyParams() throws Exception {
        FilterParams params = new FilterParams();

        assertEquals(employeePage, supervisorService.findAll(params, pageRequest));
    }

    @Test
    public void findAllWithUnitParams() throws Exception {
        FilterParams params = new FilterParams();
        Unit unit = createUnit();
        employee.setUnit(unit);
        params.setUnit(unit);

        assertEquals(employeePage, supervisorService.findAll(params, pageRequest));
    }

    @Test
    public void findAllWithUnitParamsWhereSupervisorDoesNotHaveSameUnit() throws Exception {
        FilterParams params = new FilterParams();
        Unit unit = createUnit();
        params.setUnit(unit);

        assertEquals(emptyPage, supervisorService.findAll(params, pageRequest));
    }

    @Test
    public void findAllWithSupervisorParams() throws Exception {
        FilterParams params = new FilterParams();
        params.setSupervisor(employee);

        assertEquals(employeePage, supervisorService.findAll(params, pageRequest));
    }

    @Test
    public void findAllWithNoResearchAreaParams() throws Exception {
        FilterParams params = new FilterParams();

        assertEquals(employeePage, supervisorService.findAll(params, pageRequest));
    }

    @Test
    public void findAllWithNoUnitParams() throws Exception {
        FilterParams params = new FilterParams();

        assertEquals(employeePage, supervisorService.findAll(params, pageRequest));
    }

    private User createUser() {
        User user = new User();
        user.setFirstName("Rolf");
        user.setLastName("Svensson");
        user.setEmailAddress("emailAddress@dsv.su.se");
        user.addRole(Roles.SUPERVISOR);
        Username username = new Username();
        username.setUsername("Userrosv1532@dsv.su.se");
        save(user);
        username.setUser(user);
        save(username);

        return user;
    }

    private ResearchArea createResearchArea() {
        ResearchArea researchArea = new ResearchArea();
        researchArea.setTitle("Some title");
        return save(researchArea);
    }

    private Unit createUnit() {
        Unit unit = new Unit();
        unit.setIdentifier(2L);
        unit.setTitle("unit title");
        return save(unit);
    }
}
