package se.su.dsv.scipro.system;

import org.hamcrest.CoreMatchers;
import org.junit.Assert;
import org.junit.Test;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.List;

import static java.util.Arrays.asList;
import static org.junit.Assert.*;

public class ProjectTypeServiceImplIntegrationTest extends IntegrationTest {

    @Inject
    private ProjectTypeService projectTypeService;

    @Test
    public void find_by_specific_degree_levels() throws Exception {
        ProjectType bachelor = createProjectType(DegreeLevel.FIRST_CYCLE, "bachelor");
        createProjectType(DegreeLevel.SECOND_CYCLE, "master");
        createProjectType(DegreeLevel.NONE, "ICT4D");

        List<ProjectType> projectTypes = asList(bachelor);
        assertEquals(projectTypes, projectTypeService.findByDegreeLevels(asList(DegreeLevel.FIRST_CYCLE)));
    }

    @Test
    public void find_with_module_activated() throws Exception {
        ProjectType withModule = createProjectType(DegreeLevel.NONE, "bar");
        ProjectType withoutModule = createProjectType(DegreeLevel.SECOND_CYCLE, "foo");
        withoutModule.removeModule(ProjectModule.MATCH);
        save(withoutModule);

        List<ProjectType> projectTypes = projectTypeService.findWithModule(ProjectModule.MATCH);

        Assert.assertThat(projectTypes.size(), CoreMatchers.is(1));
        Assert.assertThat(projectTypes, CoreMatchers.hasItem(withModule));
    }

    @Test
    public void find_projects_types_from_supervisors_active_projects() throws Exception {
        User supervisor = save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
        ProjectType bachelor = createProjectType(DegreeLevel.FIRST_CYCLE, "bachelor");
        createProjectType(DegreeLevel.SECOND_CYCLE, "master");

        save(Project.builder().title("bachelor").projectType(bachelor).headSupervisor(supervisor).build());
        save(Project.builder().title("another bachelor").projectType(bachelor).headSupervisor(supervisor).build());

        List<ProjectType> projectTypes = projectTypeService.findBySupervisorProjects(supervisor);

        Assert.assertThat(projectTypes.size(), CoreMatchers.is(1));
        Assert.assertThat(projectTypes, CoreMatchers.hasItem(bachelor));
    }

    @Test
    public void find_projects_types_from_supervisors_active_projects_count_following_projects_as_well() throws Exception {
        User supervisor = save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
        ProjectType bachelor = createProjectType(DegreeLevel.FIRST_CYCLE, "bachelor");
        ProjectType master = createProjectType(DegreeLevel.SECOND_CYCLE, "master");

        save(Project.builder().title("bachelor").projectType(bachelor).headSupervisor(supervisor).build());
        Project reviewingProject = Project.builder().title("another bachelor").projectType(master).headSupervisor(save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build())).build();
        reviewingProject.addReviewer(supervisor);
        save(reviewingProject);

        List<ProjectType> projectTypes = projectTypeService.findBySupervisorProjects(supervisor);

        Assert.assertThat(projectTypes.size(), CoreMatchers.is(2));
        Assert.assertThat(projectTypes, CoreMatchers.hasItems(bachelor, master));
    }

    @Test
    public void specified_module_exists_on_at_least_one_project_type() throws Exception {
        createProjectType(DegreeLevel.FIRST_CYCLE, "bachelor");
        assertTrue(projectTypeService.projectTypeWithModuleExists(ProjectModule.MATCH));
    }

    @Test
    public void specified_module_does_not_exist() throws Exception {
        ProjectType bachelor = createProjectType(DegreeLevel.FIRST_CYCLE, "bachelor");
        bachelor.removeModule(ProjectModule.MATCH);
        assertFalse(projectTypeService.projectTypeWithModuleExists(ProjectModule.MATCH));
    }

    private ProjectType createProjectType(DegreeLevel degreeLevel, String name) {
        ProjectType projectType = new ProjectType(degreeLevel, name, "description");
        return save(projectType);
    }
}
