package se.su.dsv.scipro.system;

import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.Collections;
import java.util.List;

import static org.hamcrest.CoreMatchers.hasItem;
import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;

public class UserServiceImplIntegrationTest extends IntegrationTest {

    @Inject
    private UserService userService;

    private User billGates;
    private User steveJobs;
    private UserService.Filter filter;

    @Before
    public void setUp() throws Exception {
        billGates = save(newUser("Bill", "Gates", "ceo@microsoft.com"));
        makeSysAdmin(billGates);
        makeExternal(billGates);

        steveJobs = save(newUser("Steve", "Jobs", "ceo@apple.com"));
        makeAdmin(steveJobs);

        createUserNames();

        filter = new UserService.Filter();
        filter.setRoles(Roles.SYSADMIN, Roles.SUPERVISOR, Roles.ADMIN, Roles.EXAMINER, Roles.AUTHOR);
    }

    @Test
    public void filter_by_name() throws Exception {
        filter.setName(billGates.getFirstName());

        Iterable<User> users = findUsers();
        assertThat(users, hasItem(billGates));
        assertThat(countUsers(), is(1L));
    }

    @Test
    public void filter_by_email_address() throws Exception {
        filter.setName(billGates.getEmailAddress());

        Iterable<User> users = findUsers();
        assertThat(users, hasItem(billGates));
        assertThat(countUsers(), is(1L));
    }

    @Test
    public void filter_by_role() throws Exception {
        filter.setRoles(Roles.SYSADMIN);

        Iterable<User> users = findUsers();
        assertThat(users, hasItem(billGates));
        assertThat(countUsers(), is(1L));
    }

    @Test
    public void empty_filter_returns_everyone() throws Exception {
        Iterable<User> users = findUsers();
        assertThat(users, hasItem(billGates));
        assertThat(users, hasItem(steveJobs));
        assertThat(countUsers(), is(2L));
    }

    @Test
    public void filter_by_internal_users_only() throws Exception {
        filter.setInternalUsersOnly(true);

        Iterable<User> users = findUsers();
        assertThat(users, hasItem(steveJobs));
        assertThat(countUsers(), is(1L));
    }

    @Test
    public void filter_with_no_roles_should_result_in_no_users() throws Exception {
        filter.setRoles(Collections.<Roles>emptySet());

        assertThat(countUsers(), is(0L));
    }

    @Test
    public void auto_complete_by_first_name() throws Exception {
        List<User> foundUsers = userService.getAutoCompleteUsers("Bill", 3);
        assertEquals(1, foundUsers.size());
        assertEquals(billGates, foundUsers.get(0));
    }

    @Test
    public void auto_complete_by_email() throws Exception {
        List<User> foundUsers = userService.getAutoCompleteUsers("@apple.com", 3);
        assertEquals(1, foundUsers.size());
        assertEquals(steveJobs, foundUsers.get(0));
    }

    @Test
    public void auto_complete_by_username() throws Exception {
        List<User> foundUsers = userService.getAutoCompleteUsers("ceo", 3);
        assertEquals(2, foundUsers.size());
    }

    @Test
    public void find_by_external_identifier() {
        assertEquals(billGates, userService.findByExternalIdentifier(billGates.getIdentifier()));
    }

    @Test
    public void find_by_email_address() {
        assertEquals(steveJobs, userService.findByEmailAddress(steveJobs.getEmailAddress()));
    }

    @Test
    public void find_by_username_and_realm() {
        assertEquals(billGates, userService.findByUsername("ceo@microsoft.com"));
    }

    private void createUserNames() {
        createUsername(steveJobs);
        createUsername(billGates);
    }

    private void createUsername(User user) {
        Username username = new Username();
        username.setUser(user);
        username.setUsername(user.getEmailAddress());
        save(username);
    }

    private long countUsers() {
        return userService.count(filter);
    }

    private Iterable<User> findUsers() {
        return userService.findAll(filter, new PageRequest(0, Integer.MAX_VALUE));
    }

    private User newUser(String firstName, String lastName, String emailAddress) {
        return User.builder().firstName(firstName).lastName(lastName).emailAddress(emailAddress).build();
    }

    private void makeSysAdmin(User user) {
        user.addRole(Roles.SYSADMIN);
    }

    private void makeAdmin(User user) {
        user.addRole(Roles.ADMIN);
    }

    private void makeExternal(User user) {
        user.setIdentifier(42L);
        save(user);
    }
}
