package se.su.dsv.scipro.finalseminar

import java.util.Date

import org.hamcrest.CoreMatchers.is
import org.hamcrest.MatcherAssert.assertThat
import org.junit.runner.RunWith
import org.junit.{Before, Test}
import org.mockito.Mock
import org.mockito.Mockito.when
import org.mockito.runners.MockitoJUnitRunner
import se.su.dsv.scipro.file.FileDescription
import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.reviewing.{FinalSeminarApproval, FinalSeminarApprovalService}
import se.su.dsv.scipro.system.{DegreeLevel, ProjectType, User}

@RunWith(classOf[MockitoJUnitRunner])
class FinalSeminarCreationServiceTest {
  @Mock private var finalSeminarApprovalService: FinalSeminarApprovalService = _

  private var finalSeminarCreationService: FinalSeminarCreationService = _
  private var project: Project = _

  @Before def setUp(): Unit = {
    finalSeminarCreationService = new FinalSeminarCreationServiceImpl(finalSeminarApprovalService)
    project = Project.builder.title("Title").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor")).build
    ()
  }

  @Test def creation_is_allowed_when_there_is_no_reviewer() = {
    noReviewerOnProject()
    creationAllowedIs(NoReviewer)
  }

  @Test def requires_approval_when_reviewer_exists_and_not_asked() = {
    reviewerOnProject()
    notAskedForApproval()
    creationAllowedIs(RequestNeeded)
  }

  @Test def awaiting_approval_when_reviewer_asked_but_no_response() = {
    reviewerOnProject()
    val finalSeminarApproval = awaitingApproval
    creationAllowedIs(new AwaitingDecision(finalSeminarApproval))
  }

  @Test def granted_by_timeout_if_no_response_before_deadline() = {
    reviewerOnProject()
    val finalSeminarApproval = timedOutApproval
    creationAllowedIs(GrantedByTimeout(finalSeminarApproval))
  }

  @Test def denied_approval_results_in_denied_creation_status() = {
    reviewerOnProject()
    val finalSeminarApproval = deniedApproval
    creationAllowedIs(Denied(finalSeminarApproval))
  }

  @Test def granted_approval_results_in_granted_creation_status() = {
    reviewerOnProject()
    grantedApproval
    creationAllowedIs(Granted)
  }

  def grantedApproval = {
    val finalSeminarApproval = awaitingApproval
    finalSeminarApproval.approve("Ok", None, None)
    finalSeminarApproval
  }

  def deniedApproval = {
    val finalSeminarApproval = awaitingApproval
    finalSeminarApproval.reject("Very bad discussion", None, None)
    finalSeminarApproval
  }

  private def timedOutApproval = {
    val finalSeminarApproval: FinalSeminarApproval = new FinalSeminarApproval(project, new FileDescription, "test", new Date(System.currentTimeMillis() - 3600000L))
    when(finalSeminarApprovalService.findBy(project)).thenReturn(Some(finalSeminarApproval))
    finalSeminarApproval  }

  private def awaitingApproval = {
    val finalSeminarApproval: FinalSeminarApproval = new FinalSeminarApproval(project, new FileDescription, "test", new Date(System.currentTimeMillis() + 3600000L))
    when(finalSeminarApprovalService.findBy(project)).thenReturn(Some(finalSeminarApproval))
    finalSeminarApproval
  }

  private def notAskedForApproval() = {
    when(finalSeminarApprovalService.findBy(project)).thenReturn(None)
  }

  private def reviewerOnProject() = {
    val reviewer: User = User.builder.firstName("Bob").lastName("Reviewer").emailAddress("bob@example.com").build
    reviewer.setId(1L)
    project.addReviewer(reviewer)
  }

  private def noReviewerOnProject() = {
    import scala.collection.JavaConversions.collectionAsScalaIterable
    project.getReviewers.foreach(project.removeReviewer)
  }

  private def creationAllowedIs(approval: FinalSeminarCreationReviewerApproval) = {
    assertThat(finalSeminarCreationService.isCreationAllowed(project), is(approval))
  }
}