package se.su.dsv.scipro.io.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.io.ExternalExporter;
import se.su.dsv.scipro.io.dto.*;
import se.su.dsv.scipro.io.exceptions.ExternalExportException;
import se.su.dsv.scipro.io.http.DaisyAPI;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.Unit;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.ws.rs.core.Response;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

public class ExternalExporterDaisyImpl implements ExternalExporter {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExternalExporterDaisyImpl.class);

    static final int MAX_TITLE_LENGTH = 255;
    private static Map<DegreeLevel, Level> classMap = new HashMap<DegreeLevel, Level>() {
        {
            put(ProjectType.UNKNOWN, Level.UNKNOWN);
            put(ProjectType.UNKNOWN, Level.PRE_UNIVERSITY);
            put(ProjectType.BACHELOR, Level.FIRST_CYCLE);
            put(ProjectType.MASTER, Level.SECOND_CYCLE);
            put(ProjectType.UNKNOWN, Level.THIRD_CYCLE);
        }
    };

    private final DaisyAPI api;

    @Inject
    public ExternalExporterDaisyImpl(DaisyAPI api) {
        this.api = api;
    }

    @Override
    public void createProject(final Project project, final Unit unit)
            throws ExternalExportException {
        Level level = toDaisyLevel(project);

        se.su.dsv.scipro.io.dto.Unit unitDTO = new se.su.dsv.scipro.io.dto.Unit();
        unitDTO.setId(unit.getIdentifier());

        Thesis exportProjectDTO = new Thesis();
        exportProjectDTO.setTitle(truncate(project.getTitle()));
        Calendar startDate = Calendar.getInstance();
        startDate.setTime(project.getDateCreated());
        exportProjectDTO.setStartDate(startDate);
        exportProjectDTO.setLevel(level);
        exportProjectDTO.setUnit(unitDTO);

        Response response = api.createProject(exportProjectDTO);

        expect(response, Response.Status.CREATED, "Failed to create project [project={}]", project);

        try {
            String path = response.getLocation().getPath();
            int index = path.lastIndexOf('/');
            long identifier = Long.parseLong(path.substring(index + 1));
            project.setIdentifier(identifier);
        } catch (Exception ex) {
            LOGGER.error("Could not extract external id from location header [project={}, location={}]", project, response.getLocation());
            throw new ExternalExportException("Failed to extract external identifier from response", ex);
        }
    }

    private static String truncate(String projectTitle) {
        return projectTitle.length() > MAX_TITLE_LENGTH ? projectTitle.substring(0, MAX_TITLE_LENGTH) : projectTitle;
    }

    private Level toDaisyLevel(Project project) {
        return classMap.get(project.getProjectTypeDegreeLevel());
    }

    @Override
    public void addStudentToProject(Project project, User user)
            throws ExternalExportException {

        Person personDTO = new Person();
        personDTO.setId(user.getIdentifier());

        Course course = new Course();
        course.setLevel(toDaisyLevel(project));

        StudentProjectParticipant studentDTO = new StudentProjectParticipant();
        studentDTO.setCourse(course);
        studentDTO.setPerson(personDTO);

        Response response = api.addAuthor(project.getIdentifier(), studentDTO);

        expect(response, Response.Status.CREATED, "Could not add student to project [student=\"{}\", project=\"{}\"]", user, project);
    }

    @Override
    public void addContributorToProject(Project project, User user, Role role)
            throws ExternalExportException {
        Person personDTO = new Person();
        personDTO.setId(user.getIdentifier());

        ProjectParticipant contributorDTO = new ProjectParticipant();

        contributorDTO.setPerson(personDTO);
        contributorDTO.setRole(role);

        Response response = api.addContributor(project.getIdentifier(), contributorDTO);

        expect(response, Response.Status.CREATED, "Could not add contributor to project [project=\"{}\", user=\"{}\", role=\"{}\"]", project, user, role);
    }

    @Override
    public void deleteProject(Project project) throws ExternalExportException {
        Response response = api.deleteProject(project.getIdentifier());

        expect(response, Response.Status.NO_CONTENT, "Project deletion failed [id={}, title={}]", project.getId(), project.getTitle());
    }

    private void expect(final Response response, final Response.Status status, final String errorMessage, final Object... parameters)
            throws ExternalExportException {
        if (status.getStatusCode() == response.getStatus()) {
            return;
        }

        String errorHeader = parseError(response);

        LOGGER.info(errorMessage, parameters);
        LOGGER.info("Client response [status={}, error={}]", response.getStatus(), errorHeader);

        throw new ExternalExportException(errorHeader);
    }

    private String parseError(Response response) {
        if (!response.getHeaders().containsKey("X-Error-Message")) {
            return "Unknown";
        }

        return String.valueOf(response.getHeaders().getFirst("X-Error-Message"));
    }
}
