package se.su.dsv.scipro.io.impl;

import com.google.inject.persist.Transactional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.io.dto.*;
import se.su.dsv.scipro.io.dto.ResearchArea;
import se.su.dsv.scipro.io.http.DaisyAPI;
import se.su.dsv.scipro.match.ProgramService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.*;
import se.su.dsv.scipro.system.Program;

import javax.inject.Inject;
import javax.inject.Named;
import javax.ws.rs.core.Response;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Named
public class ImporterTransactionsImpl implements ImporterTransactions {

    private static final Logger LOGGER = LoggerFactory.getLogger(ImporterTransactionsImpl.class);

    private final UserService userService;
    private final ResearchAreaService researchAreaService;
    private final ProjectService projectService;
    private final UserNameService userNameService;
    private final DaisyAPI client;
    private final ProgramService programService;
    private final LanguageService languageService;

    @Inject
    public ImporterTransactionsImpl(
            UserService userService,
            ResearchAreaService researchAreaService,
            ProjectService projectService,
            UserNameService userNameService,
            DaisyAPI client,
            ProgramService programService,
            LanguageService languageService) {
        this.userService = userService;
        this.researchAreaService = researchAreaService;
        this.projectService = projectService;
        this.userNameService = userNameService;
        this.client = client;
        this.programService = programService;
        this.languageService = languageService;
    }

    @Override
    @Transactional
    public User importPerson(Person person) {
        LOGGER.info("Importing person: {} {} ({})", person.getFirstName(), person.getLastName(), person.getId());
        User user = updatePersonData(person.getId(), person);
        importUsernames(person, user);
        return user;
    }

    @Override
    public User importStudent(Person person) {
        User user = importPerson(person);
        importProgramsAndMakeStudent(person, user);
        return user;
    }

    @Transactional
    @Override
    public User importSupervisor(Person person) {
        User user = importPerson(person);
        importResearchAreasAndMakeSupervisor(person, user);
        return user;
    }

    private void importProgramsAndMakeStudent(Person person, User user) {
        Response response = client.getStudent(person.getId());
        if (response.getStatus() == Response.Status.OK.getStatusCode()) {
            se.su.dsv.scipro.io.dto.Student student = response.readEntity(se.su.dsv.scipro.io.dto.Student.class);
            for (Admission admission : student.getAdmissions()) {
                Program program = importProgram(admission.getId());
                user.addProgram(program);
            }
            userService.save(user);
        }
    }

    private Program importProgram(Long id) {
        se.su.dsv.scipro.io.dto.Program program = client.getProgram(id);
        return programService.updateExternalProgram(program.getId(), program.getName(), program.getNameEn(), program.getCode());
    }

    private void importUsernames(Person person, User local) {
        Set<UserName> daisyUsernames = client.getUsernames(person.getId());
        for (UserName daisyUsername : daisyUsernames) {
            String completeUsername = String.format("%s@%s", daisyUsername.getUsername(), daisyUsername.getRealm());
            Username existingUsername = userNameService.findByUsername(completeUsername);
            if (existingUsername == null) {
                Username username = new Username();
                username.setUsername(completeUsername);
                username.setUser(local);
                userNameService.save(username);
            }
        }
    }

    @Transactional
    private void importResearchAreasAndMakeSupervisor(Person person, User employee) {
        List<ResearchArea> researchAreas = client.getResearchAreas(person.getId());
        if (!researchAreas.isEmpty()) {
            for (ResearchArea daisyResearchArea : researchAreas) {
                se.su.dsv.scipro.system.ResearchArea researchArea =
                        researchAreaService.updateExternalResearchArea(daisyResearchArea.getId(), daisyResearchArea.getName());
                employee.addResearchArea(researchArea);
            }
        }
        employee.addRole(Roles.SUPERVISOR);
        if (employee.getLanguages().isEmpty()){
            employee.setLanguages(new HashSet<>(languageService.findAll()));
        }
        userService.save(employee);
    }

    private User updatePersonData(Long externalIdentifier, Person person) {
        User local = userService.findByExternalIdentifier(externalIdentifier);
        if (local == null) {
            local = new User();
        }
        local = overwritePersonData(local, person);
        return local;
    }

    private User overwritePersonData(User local, Person person) {
        local.setFirstName(person.getFirstName());
        local.setLastName(person.getLastName());
        local.setEmailAddress(person.getEmail());
        local.setIdentifier(person.getId());
        return userService.save(local);
    }

    @Override
    @Transactional
    public void importProject(Thesis thesis) {
        if (thesis.getLevel().equals(Level.FIRST_CYCLE) || thesis.getLevel().equals(Level.SECOND_CYCLE)){
            LOGGER.info("Importing project: {} ({})", thesis.getTitle(), thesis.getId());
            Set<StudentProjectParticipant> authors = client.getAuthors(thesis.getId());
            Project project = convertProjectAndSave(thesis);
            if (project == null) {
                return;
            }
            addCreditsToProject(authors, project);
        }
    }

    private Project convertProjectAndSave(Thesis thesis) {
        return projectService.findByExternalIdentifier(thesis.getId());
    }

    private void addCreditsToProject(Set<StudentProjectParticipant> authors, Project project) {
        for (StudentProjectParticipant contributor : authors) {
            Course course = contributor.getCourse();
            project.setCredits(course.getCredits());
        }
        projectService.save(project);
    }

}
