package se.su.dsv.scipro.integration.daisy.workers;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import scala.Option;
import se.su.dsv.scipro.io.ExternalImporter;
import se.su.dsv.scipro.io.dto.Person;
import se.su.dsv.scipro.io.dto.Thesis;
import se.su.dsv.scipro.io.http.DaisyAPI;
import se.su.dsv.scipro.io.impl.ImporterTransactions;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;

import static java.util.Arrays.asList;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class UserImportWorkerTest {

    @Mock
    private ExternalImporter externalImporter;
    @Mock
    private ImporterTransactions importerTransactions;
    @Mock
    private DaisyAPI daisyAPI;
    @Mock
    private UserService userService;

    private UserImportWorker worker;

    @Before
    public void setUp() throws Exception {
        when(userService.findAll()).thenReturn(Collections.<User>emptyList());
        worker = new UserImportWorker(daisyAPI, importerTransactions, externalImporter, userService);
    }

    @Test
    public void imports_all_supervisors_projects() throws Exception {
        Person supervisor1 = createSupervisor(8L);
        Person supervisor2 = createSupervisor(17L);

        Thesis thesis1 = new Thesis();
        Thesis thesis2 = new Thesis();

        mockSupervisors(supervisor1, supervisor2);
        mockSupervisorProjects(supervisor1, thesis1);
        mockSupervisorProjects(supervisor2, thesis2);

        worker.doWork();

        verify(importerTransactions).importProject(thesis1);
        verify(importerTransactions).importProject(thesis2);
    }

    @Test
    public void if_one_supervisor_fails_the_rest_should_still_run() throws Exception {
        Person crashing = createSupervisor(1L);
        Person working = createSupervisor(2L);

        mockSupervisors(crashing, working);
        mockSupervisorProjects(working, new Thesis());
        doThrow(new RuntimeException()).when(daisyAPI).getProjects(crashing.getId());

        worker.doWork();

        verify(importerTransactions).importProject(any(Thesis.class));
    }

    @Test
    public void if_one_project_fails_the_rest_should_still_run() throws Exception {
        Thesis first = new Thesis();
        Thesis second = new Thesis();

        mockSupervisors(createSupervisor(1L));
        mockSupervisorProjects(anyLong(), first, second);
        doThrow(new RuntimeException()).when(importerTransactions).importProject(first);

        worker.doWork();

        verify(importerTransactions).importProject(second);
    }

    @Test
    public void imports_every_person_with_a_daisy_id() throws Exception {
        Person person = createSupervisor(8L);
        person.setEmail("kalle@example.com");
        when(userService.findAll()).thenReturn(Arrays.asList(sciproUser(), daisyUser()));
        when(daisyAPI.findPersonById(anyLong())).thenReturn(Option.apply(person));

        worker.doWork();

        verify(importerTransactions).importPerson(person);
    }

    @Test
    public void skips_import_of_users_without_email() throws Exception {
        Person person = createSupervisor(8L);
        when(userService.findAll()).thenReturn(Arrays.asList(sciproUser(), daisyUser()));
        when(daisyAPI.findPersonById(anyLong())).thenReturn(Option.apply(person));

        worker.doWork();

        verify(importerTransactions, never()).importPerson(person);
    }

    @Test
    public void if_one_import_fails_still_tries_every_user() throws Exception {
        when(userService.findAll()).thenReturn(Arrays.asList(daisyUser(), daisyUser()));
        when(daisyAPI.findPersonById(anyLong())).thenThrow(new RuntimeException());

        worker.doWork();

        verify(daisyAPI, times(2)).findPersonById(anyLong());
    }

    private User sciproUser() {
        return User.builder().firstName("SciPro").lastName("User").emailAddress("scipro@dsv.su.se").build();
    }

    private User daisyUser() {
        return User.builder().firstName("Daisy").lastName("User").emailAddress("daisy@dsv.su.se").identifier(8L).build();
    }

    private void mockSupervisorProjects(Person supervisor, Thesis... theses) {
        mockSupervisorProjects(supervisor.getId(), theses);
    }

    private void mockSupervisorProjects(long id, Thesis... theses) {
        when(daisyAPI.getProjects(id)).thenReturn(new HashSet<>(asList(theses)));
    }

    private void mockSupervisors(Person... supervisors) {
        when(daisyAPI.getSupervisors(anyLong())).thenReturn(asList(supervisors));
    }

    private Person createSupervisor(long value) {
        Person supervisor = new Person();
        supervisor.setId(value);
        return supervisor;
    }
}
