package se.su.dsv.scipro.activityplan;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.util.ListModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.checklist.Checklist;
import se.su.dsv.scipro.checklist.ChecklistTemplate;
import se.su.dsv.scipro.checklist.ChecklistTemplateService;
import se.su.dsv.scipro.components.AjaxDropDownChoice;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.components.BootstrapDateTimeComponent;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@ProjectModuleComponent(ProjectModule.ACTIVITY_PLAN)
public class EditActivityPanel extends Panel {

    private IModel<Project> project;

    public EditActivityPanel(String id, IModel<Project> project, IModel<Activity> activity) {
        super(id, project);
        this.project = project;
        add(new ActivityForm("form", activity));
    }


    @Inject
    private ChecklistTemplateService checklistTemplateService;
    @Inject
    private ActivityPlanFacade activityPlanFacade;
    @Inject
    private NotificationController notificationController;

    private class ActivityForm extends Form<Activity> {

        private ChecklistTemplate checklistTemplate;
        private final WebMarkupContainer wmc;
        private final AutoHidingListView<String> questions;

        public ActivityForm(String id, IModel<Activity> model) {
            super(id, model);

            add(new FeedbackPanel("feedback"));

            wmc = new WebMarkupContainer("wmc");
            wmc.setOutputMarkupId(true);
            add(wmc);

            if (model.getObject().getChecklist()!=null){
                checklistTemplate = checklistTemplateService.findByName(model.getObject().getChecklist().getName());
            }

            TextField<String> title = new TextField<>("title", new PropertyModel<String>(model, "title"));
            title.setRequired(true);
            add(title);

            TextArea<String> description = new TextArea<>("description", new PropertyModel<String>(model, "description"));
            add(description);

            FormComponent<Date> dateTime = new BootstrapDateTimeComponent("dateTime", new PropertyModel<Date>(model, "date"));
            dateTime.setRequired(true);
            add(dateTime);

            DropDownChoice<Action> action = new DropDownChoice<>("action", new PropertyModel<Action>(model, "action"), Arrays.asList(Action.values()), new EnumChoiceRenderer<Action>(this));
            action.setRequired(true);
            add(action);

            final List<ChecklistTemplate> checklistTemplates = checklistTemplateService.findByProject(project.getObject());
            IChoiceRenderer<ChecklistTemplate> iChoiceRenderer = new IChoiceRenderer<ChecklistTemplate>() {
                @Override
                public Object getDisplayValue(ChecklistTemplate object) {
                    return object.getName();
                }

                @Override
                public String getIdValue(ChecklistTemplate object, int index) {
                    return String.valueOf(object.getId());
                }
            };

            AjaxDropDownChoice<ChecklistTemplate> checklist = new AjaxDropDownChoice<ChecklistTemplate>("checklist", new PropertyModel<ChecklistTemplate>(ActivityForm.this, "checklistTemplate"), new ListModel<>(checklistTemplates), iChoiceRenderer) {
                @Override
                public void onNewSelection(AjaxRequestTarget target, ChecklistTemplate objectSelected) {
                    target.add(wmc);
                }
            };
            checklist .setNullValid(true);
            add(checklist );

            questions = new AutoHidingListView<String>("questions", getSome()) {
                @Override
                protected void populateItem(ListItem<String> item) {
                    item.add(new Label("question", item.getModel()));
                }
            };
            wmc.add(questions);
        }

        @Override
        protected void onSubmit() {
            boolean isNew = getModelObject().getId() == null;
            getModelObject().setActivityPlan(activityPlanFacade.retrieveActivityPlan(project.getObject()));

            if (checklistTemplate!=null){
                ChecklistTemplate loadedTemplate = getLoadedChecklistTemplate(checklistTemplate).getObject();
                Checklist checklist = activityPlanFacade.createChecklist(project.getObject(), loadedTemplate);
                getModelObject().setChecklist(checklist);
            } else {
                getModelObject().setChecklist(null);
            }

            Activity activity = activityPlanFacade.saveActivity(getModelObject());
            setModelObject(activity);
            if (isNew) {
                notifyCreatedActivityPlan(activity.getTitle());
            } else {
                notifyUpdatedActivityPlan(activity.getTitle());
            }
            success("Saved activity");
        }

        private LoadableDetachableModel<List<String>> getSome(){
            return new LoadableDetachableModel<List<String>>() {
                @Override
                protected List<String> load() {
                    if (checklistTemplate!=null){
                        return checklistTemplateService.findOne(checklistTemplate.getId()).getQuestions();
                    }
                    return new ArrayList<>();
                }
            };
        }

        private LoadableDetachableModel<ChecklistTemplate> getLoadedChecklistTemplate(final ChecklistTemplate checklistTemplate){
            return new LoadableDetachableModel<ChecklistTemplate>() {
                @Override
                protected ChecklistTemplate load() {
                    return checklistTemplateService.findOne(checklistTemplate.getId());
                }
            };
        }

        private void notifyCreatedActivityPlan(String name) {
            notifyChangeActivityPlan(name, ProjectEvent.Event.ACTIVITY_ADDED);
        }

        private void notifyUpdatedActivityPlan(String name) {
            notifyChangeActivityPlan(name, ProjectEvent.Event.ACTIVITY_EDITED);
        }

        private void notifyChangeActivityPlan(String name, ProjectEvent.Event event) {
            PageParameters pp = new PageParameters();

            Project project = EditActivityPanel.this.project.getObject();
            pp.add(PageParameterKeys.MAP.get(Project.class), String.valueOf(project.getId()));

            NotificationSource source = new NotificationSource();
            source.setMessage(name);

            notificationController.notifyProject(project, event, source);
        }
    }
}
