package se.su.dsv.scipro.admin.pages;

import com.google.common.eventbus.EventBus;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.util.SetModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.AddUsersPanel;
import se.su.dsv.scipro.components.SupervisorAutoComplete;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAdminProjectManagement;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.mail.MailEvent;
import se.su.dsv.scipro.mail.MailEventService;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.project.ReviewerAssignedEvent;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserRepo;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.*;

public class AdminEditProjectPage extends AbstractAdminProjectPage implements MenuHighlightAdminProjectManagement {

    public static final String FEEDBACK = "feedback";
    public static final String TITLE_FIELD = "titleField";
    public static final String PROJECT_TYPE_SELECT = "projectTypeSelect";
    public static final String HEAD_SUPERVISOR_SELECT = "headSupervisorSelect";
    public static final String FORM = "form";
    public static final String AUTHORS = "authors";
    public static final String REVIEWERS = "reviewers";
    public static final String CO_SUPERVISORS = "coSupervisors";

    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private ProjectService projectService;
    @Inject
    private NotificationController notificationController;
    @Inject
    private GeneralSystemSettingsService generalSystemSettingsService;
    @Inject
    private MailEventService mailEventService;
    @Inject
    private UserRepo userRepo;
    @Inject
    private EventBus eventBus;

    private boolean addMore = false;

    public AdminEditProjectPage(PageParameters pp) {
        final long id = pp.get(PageParameterKeys.MAP.get(Project.class)).toLong(0);
        final Project project = projectService.findOne(id);
        add(new ProjectForm(FORM, new DetachableServiceModel<>(projectService, project != null ? project : new Project())));
    }

    private class ProjectForm extends Form<Project> {

        private final AddUsersPanel authors;
        private final AddUsersPanel coSupervisors;
        private final AddUsersPanel reviewers;

        private Collection<ProjectEvent.Event> events = EnumSet.noneOf(ProjectEvent.Event.class);

        public ProjectForm(String id, final IModel<Project> model) {
            super(id, model);
            add(new FeedbackPanel(FEEDBACK));

            add(new RequiredTextField<>(TITLE_FIELD, new PropertyModel<>(model, "title")));

            List<ProjectType> types = projectTypeService.findAllActive();
            DropDownChoice<ProjectType> projectTypeSelect = new DropDownChoice<>(PROJECT_TYPE_SELECT,
                    new PropertyModel<ProjectType>(model, "projectType"), types, new ChoiceRenderer<>("name", "id"));
            projectTypeSelect.setRequired(true);
            if (types.size() == 1) {
                projectTypeSelect.setModelObject(types.get(0));
            }
            add(projectTypeSelect);

            SupervisorAutoComplete headSupervisor = new SupervisorAutoComplete(HEAD_SUPERVISOR_SELECT, new PropertyModel<User>(model, "headSupervisor")) {
                @Override
                protected void onModelChanged() {
                    events.add(ProjectEvent.Event.HEAD_SUPERVISOR_CHANGED);
                }
            };
            headSupervisor.setRequired(true);
            add(headSupervisor);
            authors = new AddUsersPanel(AUTHORS, new SetModel<>(new TreeSet<>(getModelObject().getProjectParticipants())), AddUsersPanel.LookFor.AUTHORS);
            add(authors);
            reviewers = new AddUsersPanel(REVIEWERS, new SetModel<>(new TreeSet<>(getModelObject().getReviewers())), AddUsersPanel.LookFor.EMPLOYEES);
            add(reviewers);
            coSupervisors = new AddUsersPanel(CO_SUPERVISORS, new SetModel<>(new TreeSet<>(getModelObject().getCoSupervisors())), AddUsersPanel.LookFor.EMPLOYEES);
            add(coSupervisors);
            add(new Button("saveAndAddMore") {
                @Override
                public void onSubmit() {
                    addMore = true;
                }
            });

        }

        @Override
        protected void onSubmit() {
            if (getModelObject().getId() == null) {
                if (hasAuthorsAnyActiveProjects()) {
                    setResponsePage(new AdminCreateProjectConfirmationPage(addMore, getModel()));
                } else {
                    projectService.save(getModelObject());
                    info(getString("saved", getModel()));
                    notificationController.notifyProject(getModelObject(), ProjectEvent.Event.CREATED, new NotificationSource());
                }
            } else {
                projectService.save(getModelObject());
                info(getString("saved", getModel()));

                for (ProjectEvent.Event event : events) {
                    notificationController.notifyProject(getModelObject(), event, new NotificationSource());
                    if (event.equals(ProjectEvent.Event.HEAD_SUPERVISOR_CHANGED)) {
                        createSupervisorChangeMail();
                    }
                }
            }
            events.clear();
        }

        private boolean hasAuthorsAnyActiveProjects() {
            for (User student : authors.getModelObject()) {
                if (projectService.findByUserAndStatus(student, ProjectStatus.ACTIVE).size() > 0) {
                    return true;
                }
            }
            return false;
        }

        private void createSupervisorChangeMail() {
            GeneralSystemSettings settings = generalSystemSettingsService.getGeneralSystemSettingsInstance();
            MailEvent mailEvent = new MailEvent();
            mailEvent.setFromEmail(settings.getSystemFromMail());
            mailEvent.setFromName(settings.getMailFromName());
            mailEvent.getNonUserRecipients().addAll(settings.getSupervisorChangeMails());
            mailEvent.setSubject(getString("supervisor.change.subject"));
            mailEvent.setMessageBody(getString("supervisor.change.body", getModel()));
            mailEventService.save(mailEvent);
        }

        @Override
        protected void onValidate() {
            super.onValidate();
            validateAuthors();
        }

        @Override
        protected void onValidateModelObjects() {
            validateSameUser(authors.getModelObject(), reviewers.getModelObject(), "reviewerAuthorSame");
            validateSameUser(authors.getModelObject(), coSupervisors.getModelObject(), "authorCoSame");
            validateSameUser(reviewers.getModelObject(), coSupervisors.getModelObject(), "reviewerCoSame");
            validateHeadSupervisorSameAs(coSupervisors.getModelObject(), "headCoSame");
            validateHeadSupervisorSameAs(reviewers.getModelObject(), "headReviewerSame");
            validateHeadSupervisorSameAs(authors.getModelObject(), "headAuthorSame");
            validateReviewerHasReviewerRole(reviewers.getModelObject(), "reviewerNotReviewer");
            notifyAuthorsChanged();
            notifyReviewersChanged();
            notifyCoSupervisorsChanged();
            TreeSet<User> projectParticipants = new TreeSet<>(new User.ByNameComparator());
            projectParticipants.addAll(authors.getModelObject());
            getModelObject().setProjectParticipants(projectParticipants);
            TreeSet<User> coSupervisors = new TreeSet<>(new User.ByNameComparator());
            coSupervisors.addAll(this.coSupervisors.getModelObject());
            getModelObject().setCoSupervisors(coSupervisors);
            TreeSet<User> reviewers = new TreeSet<>(new User.ByNameComparator());
            reviewers.addAll(this.reviewers.getModelObject());
            getModelObject().setReviewers(reviewers);
            for (User reviewer : this.reviewers.getModelObject()) {
                eventBus.post(new ReviewerAssignedEvent(getModelObject(), reviewer));
            }
        }

        private void notifyCoSupervisorsChanged() {
            for (User user : coSupervisors.getModelObject()) {
                if (!getModelObject().isCoSupervisor(user)) {
                    events.add(ProjectEvent.Event.CO_SUPERVISOR_CHANGED);
                }
            }
        }

        private void notifyReviewersChanged() {
            for (User user : reviewers.getModelObject()) {
                if (!getModelObject().isReviewer(user)) {
                    events.add(ProjectEvent.Event.REVIEWERS_CHANGED);
                }
            }
        }

        private void notifyAuthorsChanged() {
            for (User author : authors.getModelObject()) {
                if (!getModelObject().isParticipant(author)) {
                    events.add(ProjectEvent.Event.AUTHORS_CHANGED);
                }
            }
        }

        private void validateSameUser(Set<User> users, Set<User> otherUsers, String errorKey) {
            for (User user : users) {
                if (otherUsers.contains(user)) {
                    error(getString(errorKey));
                }
            }
        }

        private void validateHeadSupervisorSameAs(Set<User> users, String errorKey) {
            for (User user : users) {
                if (user.equals(getModelObject().getHeadSupervisor())) {
                    error(getString(errorKey));
                }
            }
        }

        private void validateReviewerHasReviewerRole(Set<User> users, String errorKey) {
            for (User user : users) {
                if (!userRepo.findOne(user.getId()).getRoles().contains(Roles.REVIEWER)) {
                    error(getString(errorKey));
                }
            }
        }

        private void validateAuthors() {
            if (authors.getModelObject().isEmpty()) {
                error(getString("authorRequired"));
            }
        }
    }
}
