package se.su.dsv.scipro.admin.pages;

import com.google.common.collect.Lists;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.AjaxSelfUpdatingTimerBehavior;
import org.apache.wicket.ajax.markup.html.AjaxFallbackLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.time.Duration;
import se.su.dsv.scipro.basepanels.SystemNoticePanel;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.components.OppositeVisibility;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.workerthreads.Scheduler;
import se.su.dsv.scipro.workerthreads.Task;
import se.su.dsv.scipro.workerthreads.WorkerData;
import se.su.dsv.scipro.workerthreads.WorkerDataService;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

@Authorization(authorizedRoles = {Roles.SYSADMIN})
public class SystemMaintenancePage extends AbstractAdminSystemPage {

    private static final int REFRESH_TIMER = 120;

    @Inject
    private WorkerDataService workerDataService;
    @Inject
    private Scheduler scheduler;

    private final WebMarkupContainer runningContainer = new WebMarkupContainer("runningContainer");
    private final WebMarkupContainer scheduledContainer = new WebMarkupContainer("scheduledContainer");

    public SystemMaintenancePage() {
        add(new SystemNoticeForm("systemNoticeForm"));

        runningContainer.setOutputMarkupId(true);
        scheduledContainer.setOutputMarkupId(true);

        runningContainer.add(new AjaxSelfUpdatingTimerBehavior(Duration.seconds(REFRESH_TIMER)) {

            @Override
            protected void onPostProcessTarget(AjaxRequestTarget target) {
                if (target != null) {
                    target.add(runningContainer);
                }
                super.onPostProcessTarget(target);
            }
        });
        add(runningContainer);
        add(scheduledContainer);

        updateRunning();
        updateScheduled();
    }

    private static String getElapsedTimeHoursMinutesSecondsString(final long timeMs) {
        final String format = String.format("%%0%dd", 2);
        final long time = timeMs / 1000;
        String seconds = String.format(format, (time % 60));
        String minutes = String.format(format, (time % (60 * 60)) / 60);
        String hours = String.format(format, (time % (60 * 60 * 24)) / 3600);
        String days = String.format(format, (time / (60 * 60 * 24)));
        return days + "d " + hours + "h " + minutes + "m " + seconds + "s";
    }

    private void updateRunning() {
        IModel<List<Task>> runningList = new LoadableDetachableModel<List<Task>>() {
            @Override
            protected List<Task> load() {
                return Lists.newArrayList(scheduler.getRunningWorkers());
            }
        };
        ListView<Task> runningListView = new AutoHidingListView<Task>("runningList", runningList) {
            @Override
            protected void populateItem(ListItem<Task> item) {
                item.add(new Label("runningThread", new PropertyModel<>(item.getModel(), "description")));
            }
        };
        runningContainer.add(new Label("noneLabel", "None").add(new OppositeVisibility(runningListView)));

        runningContainer.add(runningListView);
    }

    private void updateScheduled() {
        scheduledContainer.removeAll();
        IModel<List<Task>> scheduleList = new ListAdapterModel<>(new LoadableDetachableModel<Iterable<? extends Task>>() {
            @Override
            protected Iterable<? extends Task> load() {
                return scheduler.getTasks();
            }
        });
        ListView<Task> scheduleListView = new ListView<Task>("scheduleList", scheduleList) {
            @Override
            protected void populateItem(ListItem<Task> item) {
                final Task ws = item.getModelObject();
                WorkerData wd = workerDataService.getWorkerDataByName(ws.getWorkerName());
                item.add(new DateLabel("lastRun", new PropertyModel<Date>(wd, "lastRun"), DateStyle.DATETIME));
                item.add(new DateLabel("lastSuccessfulRun", new PropertyModel<Date>(wd, "lastSuccessfulRun"), DateStyle.DATETIME));
                item.add(new Label("period", getElapsedTimeHoursMinutesSecondsString(ws.getPeriod())));
                item.add(new Label("descr", ws.getDescription()));
                item.add(new AjaxFallbackLink<Task>("startLink", item.getModel()) {
                    @Override
                    public void onClick(AjaxRequestTarget target) {
                        scheduler.execute(getModelObject());
                        if (target != null) {
                            target.add(runningContainer);
                        }
                    }
                });
            }
        };
        scheduledContainer.add(scheduleListView);
    }

    private static class SystemNoticeForm extends Form<Void> {

        private IModel<String> noticeModel = Model.of(getApplication().getMetaData(SystemNoticePanel.SYSTEM_NOTICE));

        public SystemNoticeForm(String id) {
            super(id);
            add(new TextField<>("noticeField", noticeModel));
        }

        @Override
        protected void onSubmit() {
            getApplication().setMetaData(SystemNoticePanel.SYSTEM_NOTICE, noticeModel.getObject());
        }
    }
}