package se.su.dsv.scipro.admin.panels;

import com.google.common.collect.Lists;
import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormChoiceComponentUpdatingBehavior;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.markup.html.form.palette.Palette;
import org.apache.wicket.extensions.markup.html.form.palette.component.Recorder;
import org.apache.wicket.feedback.ComponentFeedbackMessageFilter;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.EnclosureContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.panel.ComponentFeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.validation.validator.StringValidator;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.components.AjaxCheckBoxMultipleChoice;
import se.su.dsv.scipro.components.IdeaMailChoice;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.components.ProjectMailChoiceNew;
import se.su.dsv.scipro.data.enums.MailChoice;
import se.su.dsv.scipro.data.facade.MailFacade;
import se.su.dsv.scipro.date.DatePickerPanel;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.mail.*;
import se.su.dsv.scipro.reusable.SciProUtilities;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.CopyTextPanel;

import javax.inject.Inject;
import java.util.*;

public class AdminMailPanel extends Panel {

    public static final int MONTHS = 6;
    public static final int BODY_MIN_LENGTH = 5;

    @Inject
    private MailEventService mailEventService;
    @Inject
    private GeneralSystemSettingsService generalSystemSettingsService;
    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private MailFacade mailFacade;

    private List<Recipient> userList;
    private static final Logger LOGGER = LoggerFactory.getLogger(AdminMailPanel.class);
    private static final int RECIPIENTS_PER_MAIL = 50;
    private static final String PROJECTS = "Projects";
    private Set<ProjectType> projectTypeList;
    private IdeaMailChoice ideaChoice;
    private ProjectMailChoiceNew projectChoice;
    private RadioChoice<String> choice;
    private WebMarkupContainer choiceContainer, labelContainer;
    private MailChoice selectedRecipientSet;
    private DatePickerPanel dpp;
    private Label projectPickerLabel, ideaPickerLabel;
    private WebMarkupContainer filterContainer;
    private ModalWindowPlus modalWindowPlus;
    private Palette<Recipient> palette;
    private Label total;

    private IModel<String> presetSubject;
    private IModel<String> presetContent;

    public AdminMailPanel(String id) {
        super(id);
        presetSubject = Model.of("");
        presetContent = Model.of("");
        setOutputMarkupId(true);
        filterContainer = new WebMarkupContainer("filterContainer");
        add(filterContainer);
        selectedRecipientSet = MailChoice.THESIS_SUPPORT;
        setUpProjectTypeSelection();
        setUpDatePanel();
        setUpRadioButtons();
        setUpMailForm();
    }

    public AdminMailPanel(String id, Set<User> userSetIn) {
        this(id, userSetIn, Model.of(""), Model.of(""));
    }

    public AdminMailPanel(String id, Set<User> userSetIn, IModel<String> presetSubject, IModel<String> presetContent) {
        super(id);
        this.presetSubject = presetSubject;
        this.presetContent = presetContent;
        userList = new ArrayList<>(userSetIn.size());
        for (User user : userSetIn) {
            userList.add(new UserRecipient(user));
        }
        filterContainer = new WebMarkupContainer("filterContainer");
        add(filterContainer);
        filterContainer.setVisible(false);
        setUpMailForm();
    }

    private void setUpProjectTypeSelection() {
        AjaxCheckBoxMultipleChoice<ProjectType> projectType = new AjaxCheckBoxMultipleChoice<ProjectType>("projectType",
                projectTypeService.findByDegreeLevels(Arrays.asList(DegreeLevel.FIRST_CYCLE, DegreeLevel.SECOND_CYCLE)),
                projectTypeService.findAllActive(),
                new ChoiceRenderer<ProjectType>("name", "id")) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                projectTypeList.clear();
                if (!getModelObject().isEmpty()) {
                    projectTypeList.addAll(getModelObject());
                }
                updateUserSet(target);
            }
        };
        projectTypeList = new HashSet<>(projectType.getModelObject());
        filterContainer.add(projectType);
    }

    private void setUpDatePanel() {
        labelContainer = new WebMarkupContainer("labelContainer");
        labelContainer.setOutputMarkupId(true);
        projectPickerLabel = new Label("pickerLabel", "Started between:");
        ideaPickerLabel = new Label("ideaPickerLabel", "Submitted between:");
        ideaPickerLabel.setVisible(false);
        labelContainer.add(projectPickerLabel);
        labelContainer.add(ideaPickerLabel);
        filterContainer.add(labelContainer);

        DateTime start = new DateTime();
        start = start.minusMonths(MONTHS);
        dpp = new DatePickerPanel("datePanel", start.toDate(), null) {
            @Override
            public void updateStartDate(AjaxRequestTarget target) {
                updateUserSet(target);
            }

            @Override
            public void updateEndDate(AjaxRequestTarget target) {
                updateUserSet(target);
            }
        };
        filterContainer.add(dpp);
    }

    private void setUpMailForm() {
        MailForm mailForm = new MailForm("mailForm");
        add(mailForm);
        add(new FencedFeedbackPanel("feedbackPanel", new ComponentFeedbackMessageFilter(mailForm)));
    }

    private void setUpRadioButtons() {
        String projectIdeas = "Ideas";
        List<String> options = Arrays.asList(PROJECTS, projectIdeas);
        choice = new RadioChoice<>("mainChoice", Model.of(PROJECTS), options);
        choice.setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER);
        choiceContainer = new WebMarkupContainer("choiceContainer");
        choiceContainer.setOutputMarkupId(true);
        ideaChoice = new IdeaMailChoice("ideaChoices");
        ideaChoice.setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER);
        projectChoice = new ProjectMailChoiceNew("projectChoices");
        projectChoice.setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER);
        EnclosureContainer ideaEnclosure = new EnclosureContainer("ideaEnclosure", ideaChoice);
        EnclosureContainer projectEnclosure = new EnclosureContainer("projectEnclosure", projectChoice);
        userList = new ArrayList<>(getRecipients(selectedRecipientSet, projectTypeList));
        ideaChoice.setVisible(false);
        filterContainer.add(choice);
        ideaEnclosure.add(ideaChoice);
        projectEnclosure.add(projectChoice);
        choiceContainer.add(ideaEnclosure);
        choiceContainer.add(projectEnclosure);
        filterContainer.add(choiceContainer);
        addChoiceBehaviors();
    }

    private void addChoiceBehaviors() {
        AjaxFormChoiceComponentUpdatingBehavior projectChoiceBehavior = new AjaxFormChoiceComponentUpdatingBehavior() {

            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                selectedRecipientSet = projectChoice.getModelObject();
                updateUserSet(target);
            }
        };
        AjaxFormChoiceComponentUpdatingBehavior ideaChoiceBehavior = new AjaxFormChoiceComponentUpdatingBehavior() {

            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                selectedRecipientSet = ideaChoice.getModelObject();
                updateUserSet(target);
            }

        };
        AjaxFormChoiceComponentUpdatingBehavior mainChoiceBehavior = new AjaxFormChoiceComponentUpdatingBehavior() {

            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                if (choice.getModelObject().equals(PROJECTS)) {
                    selectedRecipientSet = projectChoice.getModelObject();
                    showIdeaAndProject(false, true);
                } else {
                    selectedRecipientSet = ideaChoice.getModelObject();
                    showIdeaAndProject(true, false);
                }
                target.add(choiceContainer);
                target.add(labelContainer);
                updateUserSet(target);
            }
        };
        ideaChoice.add(ideaChoiceBehavior);
        projectChoice.add(projectChoiceBehavior);
        choice.add(mainChoiceBehavior);
    }

    private void showIdeaAndProject(boolean showIdea, boolean showProject) {
        ideaChoice.setVisible(showIdea);
        ideaPickerLabel.setVisible(showIdea);
        projectChoice.setVisible(showProject);
        projectPickerLabel.setVisible(showProject);
    }

    private void updateUserSet(AjaxRequestTarget target) {
        userList = new ArrayList<>(getRecipients(selectedRecipientSet, projectTypeList));
        palette.setModelObject(new ArrayList<>(userList));
        target.add(palette);
        target.add(total);
    }

    private Set<Recipient> getRecipients(MailChoice choice, Set<ProjectType> projectTypes) {
        if (projectTypes.isEmpty()) {
            return new HashSet<>();
        }
        return mailFacade.getRecipients(choice, projectTypes, dpp.getStartDate(), dpp.getEndDate());
    }

    public class MailForm extends Form<Void> {
        private TextField<String> mailSubjectField;
        private TextArea<String> mailBodyField;
        private String fromName;
        private String fromEmail;

        public MailForm(String id) {
            super(id);
            GeneralSystemSettings gsettings = generalSystemSettingsService.getGeneralSystemSettingsInstance();
            fromName = gsettings.getMailFromName();
            fromEmail = gsettings.getSystemFromMail();

            Label disabledLabel = new Label("disabledLabel",
                    "Sending of e-mail through SciPro is inactivated. Please activate it under Admin -> System Settings, if you want to use this feature.");
            disabledLabel.setVisible(!gsettings.isMailNotifications());
            add(disabledLabel);

            Button submitButton = new Button("submitButton");
            submitButton.setEnabled(gsettings.isMailNotifications());
            submitButton.add(AttributeModifier.replace(
                    "onclick",
                    "if (!confirm('Are you sure you want to send this e-mail to the selected recipients?')){ return false; }"));
            add(submitButton);
            mailSubjectField = new TextField<>("mailSubjectField", presetSubject);
            mailSubjectField.setRequired(true);
            mailBodyField = new TextArea<>("mailBodyField", presetContent);
            mailBodyField.setRequired(true);
            mailBodyField.add(StringValidator.minimumLength(BODY_MIN_LENGTH));

            add(new ComponentFeedbackPanel("subjectErrors", mailSubjectField));
            add(new ComponentFeedbackPanel("messageErrors", mailBodyField));
            add(mailSubjectField);
            add(mailBodyField);

            IChoiceRenderer<Recipient> icr = new IChoiceRenderer<Recipient>() {
                @Override
                public Object getDisplayValue(Recipient object) {
                    return object.accept(new DisplayingVisitor());
                }

                @Override
                public String getIdValue(Recipient object, int index) {
                    // Palette sends 0 as index for every iteration, yay Wicket
                    return Integer.toString(userList.indexOf(object));
                }
            };

            palette = new Palette<Recipient>("palette", Model.ofList(new ArrayList<UserRecipient>()), getUserList(), icr, 5, false, false){
                @Override
                protected Recorder<Recipient> newRecorderComponent() {
                    Recorder<Recipient> recorder = super.newRecorderComponent();
                    recorder.add(new AjaxFormComponentUpdatingBehavior("change") {
                        @Override
                        protected void onUpdate(AjaxRequestTarget target) {
                            target.add(total);
                        }
                    });
                    return recorder;
                }

            };
            palette.setOutputMarkupId(true);
            add(palette);

            total = new Label("totalRecipients", new LoadableDetachableModel<Integer>() {
                @Override
                protected Integer load() {
                    return palette.getModelCollection().size();
                }
            });
            total.setOutputMarkupId(true);
            add(total);

            modalWindowPlus = new ModalWindowPlus("dialog");
            add(modalWindowPlus);

            add(new AjaxLink<Void>("copy") {
                @Override
                public void onClick(AjaxRequestTarget target) {
                    String separator = System.getProperty("line.separator");
                    StringBuilder str = new StringBuilder();
                    Collection<Recipient> selectedChoices = palette.getModelCollection();
                    for (Recipient user : selectedChoices) {
                        str.append(user.accept(new DisplayingVisitor())).append(",").append(separator);
                    }
                    modalWindowPlus.setContent(new CopyTextPanel(modalWindowPlus.getContentId(), Model.of(str.toString())));
                    modalWindowPlus.show(target);
                }
            });
        }

        @Override
        protected void onSubmit() {
            try {
                List<Recipient> selected = SciProUtilities.copyIterator(palette.getSelectedChoices());
                if (!selected.isEmpty()) {
                    for (List<Recipient> list : Lists.partition(selected, RECIPIENTS_PER_MAIL)) {
                        MailEvent me = new MailEvent(mailSubjectField.getInput(), mailBodyField.getInput(), Collections.<User>emptySet(), fromName, fromEmail);
                        me.addRecipients(list);
                        mailEventService.save(me);
                    }
                    info("E-mail sent successfully to the selected recipients");
                } else {
                    error("You need to specify recipients for your e-mail.");
                }
            } catch (Exception e) {
                LOGGER.error("Failed to send e-mail", e);
            }
        }

        private LoadableDetachableModel<List<Recipient>> getUserList() {
            return new LoadableDetachableModel<List<Recipient>>() {
                @Override
                protected List<Recipient> load() {
                    return userList;
                }
            };
        }
    }

    private static class DisplayingVisitor implements RecipientVisitor<String> {
        @Override
        public String visit(final EmailRecipient emailRecipient) {
            return emailRecipient.getEmail();
        }

        @Override
        public String visit(final UserRecipient userRecipient) {
            final User user = userRecipient.getUser();
            return String.format("%s <%s>", user.getFullName(), user.getEmailAddress());
        }
    }
}
