package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.validation.IValidatable;
import org.apache.wicket.validation.IValidator;
import org.apache.wicket.validation.ValidationError;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAdminApplicationPeriod;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.match.AbstractAdminMatchPage;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.ApplicationPeriodExemption;
import se.su.dsv.scipro.match.ApplicationPeriodService;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;
import java.util.Set;

public class AdminEditApplicationPeriodExemptionsPage extends AbstractAdminMatchPage implements MenuHighlightAdminApplicationPeriod {
    public static final String EXEMPTIONS = "exemptions";
    public static final String ADD_FORM = "addForm";
    public static final String AUTHOR = "author";
    public static final String START = "start";
    public static final String UNTIL = "until";
    public static final String COMMENT = "comment";
    public static final String ADD_EXEMPTION = "addExemption";
    public static final String DELETE = "delete";
    public static final String EMPTY = "empty";

    @Inject
    private ApplicationPeriodService applicationPeriodService;

    public AdminEditApplicationPeriodExemptionsPage(PageParameters pp) {
        final IModel<ApplicationPeriod> model =
                new DetachableServiceModel<>(applicationPeriodService, pp.get(PageParameterKeys.MAP.get(ApplicationPeriod.class)).toLong());
        final IModel<List<ApplicationPeriodExemption>> exemptionsModel =
                new ListAdapterModel<>(new PropertyModel<Set<ApplicationPeriodExemption>>(model, "applicationPeriodExemptions"));

        add(new AddExemptionForm(ADD_FORM, model));

        add(new PropertyListView<ApplicationPeriodExemption>(EXEMPTIONS, exemptionsModel) {
            @Override
            protected void populateItem(final ListItem<ApplicationPeriodExemption> item) {
                item.add(new UserLinkPanel("user"));
                item.add(new DateLabel("start", DateStyle.DATETIME));
                item.add(new DateLabel("until", DateStyle.DATETIME));
                item.add(new MultiLineLabel("comment"));
                item.add(new UserLinkPanel("grantedBy"));
                item.add(new DateLabel("grantedOn", DateStyle.DATETIME));

                item.add(new Link<ApplicationPeriodExemption>(DELETE, item.getModel()) {
                    @Override
                    public void onClick() {
                        model.getObject().removeApplicationPeriodExemption(item.getModelObject());
                        applicationPeriodService.save(model.getObject());
                        getSession().info(getString("deleted"));
                        setResponsePage(getPageClass(), getPageParameters());
                    }
                });
            }
        });

        add(new Label(EMPTY, new ResourceModel("empty")) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(!model.getObject().hasApplicationPeriodExemptions());
            }
        });
    }

    private class AddExemptionForm extends Form<ApplicationPeriod> {
        @Inject
        private UserService userService;

        public AddExemptionForm(String id, final IModel<ApplicationPeriod> model) {
            super(id, model);

            add(new FeedbackPanel("feedback"));

            final Model<ApplicationPeriodExemption> exemptionModel = Model.of(new ApplicationPeriodExemption());

            final AuthorAutoComplete author = new AuthorAutoComplete(AUTHOR) {
                @Override
                protected void action(AjaxRequestTarget target, User object) {
                    exemptionModel.getObject().setUser(object);
                }
            };
            author.setRequired(true);
            author.add(new UniqueValidator());
            author.setModel(new DetachableServiceModel<>(userService));
            add(author);

            BootstrapDateField start = new BootstrapDateField(START, new PropertyModel<Date>(exemptionModel, "start"));
            start.setRequired(true);
            start.add(new BootstrapDatePicker());
            add(start);

            BootstrapDateField until = new BootstrapDateField(UNTIL, new PropertyModel<Date>(exemptionModel, "until"));
            until.setRequired(true);
            until.add(new BootstrapDatePicker());
            add(until);

            TextArea<String> comment = new TextArea<>(COMMENT, new PropertyModel<String>(exemptionModel, "comment"));
            comment.setRequired(true);
            add(comment);

            add(new InfoPanel("infoPanel", new ResourceModel("info.text")));

            add(new Button(ADD_EXEMPTION) {
                @Override
                public void onSubmit() {
                    super.onSubmit();
                    if (exemptionModel.getObject().getStart().after(exemptionModel.getObject().getUntil())) {
                        error(getString("start.date.after.end.date"));
                    } else {
                        ApplicationPeriod applicationPeriodModel = model.getObject();
                        exemptionModel.getObject().setApplicationPeriod(applicationPeriodModel);
                        exemptionModel.getObject().setGrantedBy(SciProSession.get().getUser());
                        exemptionModel.getObject().setGrantedOn(new Date());
                        model.getObject().addApplicationPeriodExemption(exemptionModel.getObject());
                        applicationPeriodService.save(model.getObject());
                        exemptionModel.setObject(new ApplicationPeriodExemption());
                        info(getString("added"));
                    }
                }
            });
        }

        private class UniqueValidator implements IValidator<User> {
            @Override
            public void validate(IValidatable<User> validation) {
                for (ApplicationPeriodExemption ape : getModelObject().getApplicationPeriodExemptions()){
                    if (ape.getUser().equals(validation.getValue())){
                        validation.error(new ValidationError(this));
                    }
                }
            }
        }

    }
}