package se.su.dsv.scipro.components;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.upload.FileUpload;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.exceptions.InvalidFileFormatException;

/**
 * Reusable upload component, supports straight uploads as well as linking of already existing resources.
 * The former is handled by implementing onNewUpload(), the latter is handled by implementing getSelectionElements() and beforeOnNewSelectionChecker().
 * This API is quite new and still subject to frequent change.
 */
public abstract class AbstractUploadForm extends Form<Void> {

    private static final Logger LOGGER = LoggerFactory.getLogger(AbstractUploadForm.class);

    /**
     * Constructor when you need to override the id (ie: when not using the default panel wrapper for whatever reason).
     * Usage of this constructor should be rare and usually only called from subclass implementations, normally you will want to use the default idiom:
     * <code>add(new AbstractUploadForm(){...implementation...}.createDefaultWrapperPanel("componentId"));</code>
     */
    public AbstractUploadForm(final String id) {
        super(id);
    }

    /**
     * Default constructor, uses a fixed id to support the panel wrapper.
     */
    public AbstractUploadForm() {
        this("uploadForm");
    }

    /**
     * Retrieves a default panel wrapper, this is the recommended way to use this class whenever you just want a quick upload form.
     */
    public UploadFormPanel createDefaultWrapperPanel(final String panelId) {
        return new UploadFormPanel(panelId, this);
    }

    /**
     * Init-after-construction pattern, to support implementations having instance data assigned via construction without affecting setup of the component hierarchy.
     */
    @Override
    public void onInitialize() {
        super.onInitialize();
        setup();
    }

    private void setup() {
        setMultiPart(true);
        final FileUploadField fileUploadField = new FileUploadField("fileUpload");
        fileUploadField.setRequired(true);
        add(fileUploadField);
        Button submitButton = new AjaxButton("submitButton", this) {

            @Override
            public void onSubmit(AjaxRequestTarget target, Form<?> form) {
                try {
                    if (fileUploadField.getFileUpload() != null) {
                        FileUpload fileUpload = fileUploadField.getFileUpload();
                        onNewUpload(fileUpload);
                    }
                } catch (final InvalidFileFormatException iffe) {
                    String message = "The upload is in a unacceptable file format: " + iffe.getMessage();
                    error(message);
                    LOGGER.info(message, iffe);
                } catch (final Exception ioe) {//Use explicit error handling for uploading causing IOExceptions
                    String message = "An error occurred while uploading the resource, please try again";
                    error(message);
                    LOGGER.info(message, ioe);
                }
                if (!hasError()) {
                    onFinished(target);
                } else {
                    onError(target, form);
                }
            }

            @Override
            protected void onError(AjaxRequestTarget target, Form<?> form) {
                setResponsePage(getPage());
            }
        };
        add(submitButton);
    }

    /**
     * Implement this method in case you want something special to happen once the upload/selection has been processed.
     * A prime example is to redirect the user to another page, etc.
     * Note: this method is only called if no Checked Exceptions are thrown by beforeOnNewSelectionChecker or onNewUpload
     *
     */
    protected void onFinished(AjaxRequestTarget target) {
    }

    /**
     * Implement to handle a newly uploaded file, default does nothing.
     * Implementations are allowed to throw IOException (the idea is that taking care of a new file involves potential IO failure) and clients
     * are expected to take this into consideration.
     * Called whenever a file is uploaded and the form is posted.
     */
    protected void onNewUpload(final FileUpload fileUpload) {
    }
}
