package se.su.dsv.scipro.finalseminar;

import lombok.Getter;
import lombok.Setter;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.admin.pages.AbstractAdminProjectPage;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAdminFinalSeminars;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.datatables.AjaxCheckboxWrapper;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;

import static java.util.Arrays.asList;

@Authorization(authorizedRoles = {Roles.ADMIN})
public class AdminFinalSeminarExemptionPage extends AbstractAdminProjectPage implements MenuHighlightAdminFinalSeminars {
    public static final String DP = "dp";
    public static final String FORM = "form";
    public static final String FEEDBACK = "feedback";
    public static final int MIN_CHARS = 3;

    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private ProjectService projectService;
    @Inject
    private FinalSeminarService finalSeminarService;

    private ExportableDataPanel<Project, String> exportableDataPanel;
    private ProjectService.Filter filter;
    private FencedFeedbackPanel feedbackPanel;

    public AdminFinalSeminarExemptionPage() {

        feedbackPanel = new FencedFeedbackPanel(FEEDBACK, this);
        feedbackPanel.setOutputMarkupId(true);
        add(feedbackPanel);

        add(new FilterForm(FORM));
        addDP();

    }

    private void addDP() {
        SortableDataProvider<Project, String> provider = new FilteredDataProvider<>(projectService, Model.of(filter));
        provider.setSort("dateCreated", SortOrder.DESCENDING);
        exportableDataPanel = new ExportableDataPanel<>(DP, createColumns(), provider);
        add(exportableDataPanel);
    }

    private List<IColumn<Project, String>> createColumns() {
        List<IColumn<Project, String>> columns = new ArrayList<>();
        columns.add(new PropertyColumn<Project, String>(Model.of("Title"), "title", "title"));
        columns.add(new AbstractExportableColumn<Project, String, String>(Model.of(FinalSeminar.U_SINGULAR)) {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> item, String s, IModel<Project> iModel) {
                if (finalSeminarService.findByProject(iModel.getObject()) != null) {
                    item.add(new DateLabel(s, new Model<>(finalSeminarService.findByProject(iModel.getObject()).getStartDate()), DateStyle.DATETIME));
                } else {
                    item.add(new Label(s, new ResourceModel("noSeminar")));
                }
            }

            @Override
            public IModel<String> getDataModel(IModel<Project> rowModel) {
                if (finalSeminarService.findByProject(rowModel.getObject()) != null) {
                    return Model.of(finalSeminarService.findByProject(rowModel.getObject()).getStartDate().toString());
                } else {
                    return Model.of("noSeminar");
                }
            }
        });

        columns.add(new AbstractExportableColumn<Project, String, Boolean>(Model.of("Exempted from final seminar rules")) {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> item, String componentId, final IModel<Project> iModel) {
                item.add(new AjaxCheckboxWrapper(componentId, new PropertyModel<Boolean>(iModel, "finalSeminarRuleExempted")) {
                    @Override
                    public void onChange(AjaxRequestTarget target, boolean selected) {
                        iModel.getObject().setFinalSeminarRuleExempted(selected);
                        iModel.setObject(projectService.save(iModel.getObject()));
                        if (selected) {
                            feedbackPanel.info(getString("exemptionGranted"));
                        } else {
                            feedbackPanel.info(getString("exemptionRevoked"));
                        }
                        target.add(exportableDataPanel);
                        target.add(feedbackPanel);
                    }

                    @Override
                    protected void onConfigure() {
                        setEnabled(!(iModel.getObject().isFinalSeminarRuleExempted() && finalSeminarService.findByProject(iModel.getObject()) != null));
                    }
                });
            }

            @Override
            public IModel<Boolean> getDataModel(IModel<Project> rowModel) {
                return Model.of(rowModel.getObject().isFinalSeminarRuleExempted());
            }
        });
        return columns;
    }

    public class FilterForm extends Form<Void> {
        @Getter
        @Setter
        @SuppressWarnings("UnusedDeclaration")
        private String titleString;

        public FilterForm(String id) {
            super(id);
            filter = new ProjectService.Filter();

            addSupervisorFilter();
            addProjectTitleFilter();
            addAuthorFilter();
            addProjectTypeFilter();
            addProjectStatusFilter();
        }

        private void addSupervisorFilter() {
            add(new EmployeeAutoComplete("supervisorFilter") {
                @Override
                protected void action(AjaxRequestTarget pTarget, User newSelection) {
                    filter.setSupervisor(newSelection);
                    pTarget.add(exportableDataPanel);
                }
            });
        }

        private void addProjectTitleFilter() {
            TextField<String> titleField = new TextField<>("titleFilter", new PropertyModel<String>(this, "titleString"));
            titleField.add(new OnChangeAjaxBehavior() {

                @Override
                protected void onUpdate(AjaxRequestTarget target) {
                    if (titleString != null && titleString.length() >= MIN_CHARS) {
                        filter.setTitleContains(titleString);
                        target.add(exportableDataPanel);
                    } else {
                        filter.setTitleContains(null);
                        target.add(exportableDataPanel);
                    }

                }
            });
            add(titleField);
        }

        private void addAuthorFilter() {
            add(new AuthorAutoComplete("authorFilter") {
                @Override
                protected void action(AjaxRequestTarget pTarget, User newSelection) {
                    filter.setAuthorUser(newSelection);
                    pTarget.add(exportableDataPanel);
                }
            });
        }

        private void addProjectStatusFilter() {
            AjaxCheckBoxMultipleChoice<ProjectStatus> projectStatus = new AjaxCheckBoxMultipleChoice<ProjectStatus>("statusFilter",
                    new ArrayList<>(Collections.singletonList(ProjectStatus.ACTIVE)),
                    asList(ProjectStatus.values()),
                    new EnumChoiceRenderer<ProjectStatus>(this)) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    filter.setStatuses(new HashSet<>(getModelObject()));
                    target.add(exportableDataPanel);
                }
            };

            add(projectStatus);
            filter.setStatuses(new HashSet<>(projectStatus.getModelObject()));
        }

        private void addProjectTypeFilter() {
            AjaxCheckBoxMultipleChoice<ProjectType> projectType = new AjaxCheckBoxMultipleChoice<ProjectType>("projectTypeFilter",
                    projectTypeService.findWithModule(ProjectModule.FINAL_SEMINAR), projectTypeService.findWithModule(ProjectModule.FINAL_SEMINAR),
                    new ChoiceRenderer<ProjectType>("name", "id")) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    if (!getModelObject().isEmpty()) {
                        filter.setProjectTypes(getModelObject());
                    }
                    target.add(exportableDataPanel);
                }
            };
            filter.setProjectTypes(projectType.getModelObject());
            add(projectType);
        }
    }
}