package se.su.dsv.scipro.finalseminar;

import com.google.common.eventbus.EventBus;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.components.AjaxRadioChoice;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.JavascriptEventConfirmation;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Set;

public class SeminarActiveParticipantsPanel extends GenericPanel<FinalSeminar> {

    public static final String PARTICIPANTS = "activeParticipations";
    public static final String USER = "user";
    public static final String REMOVE_PARTICIPANT = "removeParticipant";
    public static final String RADIOS = "radios";
    public static final String FEEDBACK = "feedback";
    private final FeedbackPanel feedbackPanel;

    @Inject
    private EventBus eventBus;
    @Inject
    private FinalSeminarActiveParticipationService finalSeminarActiveParticipationService;
    @Inject
    private FinalSeminarService finalSeminarService;

    public SeminarActiveParticipantsPanel(String id, final IModel<FinalSeminar> seminar) {
        super(id, seminar);

        feedbackPanel = new FencedFeedbackPanel(FEEDBACK, this);
        feedbackPanel.setOutputMarkupId(true);
        add(feedbackPanel);

        final IModel<List<FinalSeminarActiveParticipation>> participants = new ListAdapterModel<>(new PropertyModel<Set<FinalSeminarActiveParticipation>>(seminar, "activeParticipations"));
        add(new Label("noParticipants", new ResourceModel("no.participants")) {
            @Override
            protected void onConfigure() {
                setVisible(participants.getObject().isEmpty());
            }
        });
        add(new ListView<FinalSeminarActiveParticipation>(PARTICIPANTS, participants) {
            @Override
            protected void populateItem(final ListItem<FinalSeminarActiveParticipation> item) {
                item.add(new UserLinkPanel(USER, new PropertyModel<User>(item.getModel(), "user")));

                item.add(new Link<FinalSeminarActiveParticipation>(REMOVE_PARTICIPANT, item.getModel()) {
                    {
                        add(new JavascriptEventConfirmation("onclick", SeminarActiveParticipantsPanel.this.getString("confirm")));
                    }

                    @Override
                    public void onClick() {
                        SeminarActiveParticipantsPanel.this.success(getString("removed", item.getModel()));
                        FinalSeminar finalSeminar = getModelObject().getFinalSeminar();
                        finalSeminar.removeActiveParticipant(getModelObject());
                        finalSeminarService.save(finalSeminar);
                        // Detach the (cached) ListAdapterModel and remove the already added children
                        // or the ListView will not reflect the change in the underlying list
                        getParent().getParent().detach();
                        getParent().getParent().removeAll();
                    }

                    @Override
                    protected void onConfigure() {
                        super.onConfigure();
                        setVisibilityAllowed(allowDeleteParticipation());
                    }
                });

                item.add(new ActiveParticipationRadioChoice(RADIOS, item.getModel()));
            }
        });
    }

    private class ActiveParticipationRadioChoice extends AjaxRadioChoice<FinalSeminarGrade> {

        private IModel<FinalSeminarActiveParticipation> participant;

        public ActiveParticipationRadioChoice(String id, IModel<FinalSeminarActiveParticipation> participant) {
            super(id, new PropertyModel<FinalSeminarGrade>(participant, "grade"), Arrays.asList(FinalSeminarGrade.values()),
                    new EnumChoiceRenderer<FinalSeminarGrade>(SeminarActiveParticipantsPanel.this),
                    SeminarActiveParticipantsPanel.this.getString("confirmation", participant));
            this.participant = participant;
            setSuffix(" ");
        }

        @Override
        protected void update(AjaxRequestTarget target) {
            finalSeminarActiveParticipationService.save(participant.getObject());
            success(getString("feedback.active.participant", participant));

            if (FinalSeminarGrade.APPROVED.equals(getModelObject())) {
                eventBus.post(new ParticipationApprovedEvent(participant.getObject()));
            } else if (FinalSeminarGrade.NOT_APPROVED.equals(getModelObject())) {
                eventBus.post(new ParticipationFailedEvent(participant.getObject()));
            }

            target.add(feedbackPanel);
        }

        @Override
        protected void onConfigure() {
            super.onConfigure();
            setVisibilityAllowed(isHeadSupervisor() && startDateHasPassed());
        }
    }

    private boolean allowDeleteParticipation() {
        return isHeadSupervisor() || isAdmin();
    }

    private boolean isHeadSupervisor() {
        return SciProSession.get().getUser().equals(getModelObject().getProject().getHeadSupervisor());
    }

    private boolean isAdmin() {
        return SciProSession.get().authorizedForRole(Roles.SYSADMIN) && SciProSession.get().hasActualRole(Roles.SYSADMIN);
    }

    private boolean startDateHasPassed() {
        return getModelObject().getStartDate().before(new Date());
    }

}
