package se.su.dsv.scipro.finalthesis;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.finalseminar.FinalSeminarService;
import se.su.dsv.scipro.grading.ThesisApprovedPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.util.AjaxConfirmationLink;

import javax.inject.Inject;
import java.util.Date;

import static se.su.dsv.scipro.finalthesis.FinalThesis.Status.*;

@ProjectModuleComponent(ProjectModule.GRADING)
public class ApproveFinalThesisPanel extends GenericPanel<Project> {
    public static final String APPROVE = "approve";
    public static final String REJECT = "reject";
    public static final String REJECT_PANEL = "rejectPanel";
    public static final String REJECT_CONTAINER = "rejectContainer";
    public static final String THESIS_CONTAINER = "thesisContainer";
    public static final String WAITING_FOR_AUTHOR_CONTAINER = "waitingForAuthorContainer";
    public static final String FILE_PANEL = "filePanel";
    public static final String DATE_LABEL = "dateLabel";
    public static final String THESIS_DECISION_CONTAINER = "thesisDecisionContainer";
    public static final String APPROVED_THESIS_CONTAINER = "approvedThesisContainer";
    public static final String THESIS_APPROVED_PANEL = "thesisApprovedPanel";
    public static final String UPLOAD_FORM_PANEL = "uploadFormPanel";
    public static final String REJECTED_DATE_LABEL = "rejectedDateLabel";
    public static final String STATUS_REJECTED_LABEL = "statusRejectedLabel";
    public static final String STATUS_NO_THESIS_LABEL = "statusNoThesisLabel";
    public static final String REJECTED_INFO_LABEL = "rejectedInfoLabel";


    private final ThesisContainer thesisContainer;
    private final WebMarkupContainer waitingForAuthorContainer;
    private WebMarkupContainer thesisApprovedContainer;
    private IModel<Project> projectIModel;

    @Inject
    private FinalThesisService finalThesisService;
    @Inject
    private FinalSeminarService finalSeminarService;

    @Override
    protected void onConfigure() {
        setVisibilityAllowed(visibilityAllowed());
        thesisContainer.setVisibilityAllowed(finalThesisService.hasFinalThesis(getModelObject()) && getFinalThesis().getObject().getStatus().equals(NO_DECISION));
        waitingForAuthorContainer.setVisibilityAllowed(!finalThesisService.hasFinalThesis(getModelObject()) || thesisHasStatus(REJECTED));
        thesisApprovedContainer.setVisibilityAllowed(getFinalThesis().getObject() != null && getFinalThesis().getObject().getStatus().equals(APPROVED));
    }

    private boolean visibilityAllowed() {
        if (finalSeminarService.hasHadFinalSeminar(getModelObject()) && !projectIModel.getObject().isReviewer(SciProSession.get().getUser())) {
            return true;
        } else if (thesisHasStatus(APPROVED)) {
            return true;
        }
        return false;
    }

    public ApproveFinalThesisPanel(String id, final IModel<Project> projectIModel) {
        super(id, projectIModel);
        this.projectIModel = projectIModel;

        thesisContainer = new ThesisContainer(THESIS_CONTAINER);
        add(thesisContainer);

        waitingForAuthorContainer = new WaitingForAuthorContainer(WAITING_FOR_AUTHOR_CONTAINER);
        add(waitingForAuthorContainer);

        thesisApprovedContainer = new WebMarkupContainer(APPROVED_THESIS_CONTAINER);
        thesisApprovedContainer.setOutputMarkupId(true);
        add(thesisApprovedContainer);
        thesisApprovedContainer.add(new ThesisApprovedPanel(THESIS_APPROVED_PANEL, projectIModel));

    }

    private class WaitingForAuthorContainer extends WebMarkupContainer {
        public WaitingForAuthorContainer(String id) {
            super(id);

            add(new Label(STATUS_NO_THESIS_LABEL, new ResourceModel("no.thesis.status.label")) {
                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(!finalThesisService.hasFinalThesis(getModelObject()));
                }
            });
            add(new Label(STATUS_REJECTED_LABEL, new ResourceModel("thesis.rejected.status.label")) {
                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(thesisHasStatus(REJECTED));
                }
            });


            add(new DateLabel(REJECTED_DATE_LABEL, getRejectedDate()) {
                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(thesisHasStatus(REJECTED));
                }
            });

            add(new Label(REJECTED_INFO_LABEL, new ResourceModel("thesis.rejected.info.label")) {
                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(thesisHasStatus(REJECTED));
                }
            });
            add(new FinalThesisUploadFormPanel(UPLOAD_FORM_PANEL, getModel()));
        }
    }

    private class ThesisContainer extends WebMarkupContainer {
        private RejectFinalThesisPanel rejectFinalThesisPanel;

        private WebMarkupContainer decisionContainer;

        public ThesisContainer(String id) {
            super(id);

            decisionContainer = new WebMarkupContainer(THESIS_DECISION_CONTAINER);
            decisionContainer.setOutputMarkupId(true);
            add(decisionContainer);

            LoadableDetachableModel<FileDescription> fileDescriptionModel = getFileDescription();
            decisionContainer.add(new ViewAttachmentPanel(FILE_PANEL, fileDescriptionModel));

            decisionContainer.add(new DateLabel(DATE_LABEL, new PropertyModel<Date>(getFinalThesis(), "dateCreated"), DateStyle.DATETIME));

            rejectFinalThesisPanel = new RejectFinalThesisPanel(REJECT_PANEL,
                    ApproveFinalThesisPanel.this.getModel()) {
                @Override
                protected void cancel() {
                    showRejectionPanel(false);
                }
            };
            rejectFinalThesisPanel.setVisibilityAllowed(false);
            final WebMarkupContainer rejectContainer = new WebMarkupContainer(REJECT_CONTAINER);
            rejectContainer.add(rejectFinalThesisPanel);
            rejectContainer.setOutputMarkupId(true);
            add(rejectContainer);

            decisionContainer.add(new AjaxConfirmationLink<Void>(APPROVE, "Are you sure that you want to approve this final thesis?") {
                @Override
                public void onClick(AjaxRequestTarget target) {
                    Project project = ApproveFinalThesisPanel.this.getModelObject();
                    finalThesisService.approve(project);
                    setResponsePage(getPage());
                }
            });

            decisionContainer.add(new AjaxLink(REJECT) {
                @Override
                public void onClick(AjaxRequestTarget target) {
                    showRejectionPanel(true);
                    target.add(rejectContainer);
                    target.add(decisionContainer);
                }
            });
        }


        private void showRejectionPanel(boolean show) {
            rejectFinalThesisPanel.setVisibilityAllowed(show);
            decisionContainer.setVisibilityAllowed(!show);
        }
    }

    private LoadableDetachableModel<FinalThesis> getFinalThesis() {
        return new LoadableDetachableModel<FinalThesis>() {
            @Override
            protected FinalThesis load() {
                return finalThesisService.findByProject(getModelObject());
            }
        };
    }

    private LoadableDetachableModel<FileDescription> getFileDescription() {
        return new LoadableDetachableModel<FileDescription>() {
            @Override
            protected FileDescription load() {
                return finalThesisService.getFinalThesisFileDescription(getModelObject());
            }
        };
    }

    private LoadableDetachableModel<Date> getRejectedDate() {
        return new LoadableDetachableModel<Date>() {
            @Override
            protected Date load() {
                return finalThesisService.getRejectedDate(getModelObject());
            }
        };
    }

    private boolean thesisHasStatus(FinalThesis.Status status) {
        FinalThesis finalThesis = getFinalThesis().getObject();
        return finalThesis != null && finalThesis.getStatus().equals(status);
    }
}
