package se.su.dsv.scipro.grading;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.markup.html.link.ResourceLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.ComponentFeedbackPanel;
import org.apache.wicket.markup.html.panel.EmptyPanel;
import org.apache.wicket.markup.html.panel.Fragment;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.ResourceModel;
import scala.util.Either;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.*;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import javax.inject.Inject;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

public class ReviewerFillOutGradingReportPanel extends GenericPanel<Project> {

    public static final String CRITERIA_PANEL = "criteriaPanel";
    public static final String FORM = "form";
    public static final String INDIVIDUAL_PARTS = "individualParts";
    public static final String POINTS_PANEL = "pointsPanel";
    public static final String INDIVIDUAL_CRITERIA = "individualCriteria";
    public static final String INDIVIDUAL_CRITERIA_POINTS_LABEL = "points";
    public static final String INDIVIDUAL_CRITERIA_TITLE_LABEL = "title";
    public static final String PDF_DOWNLOAD = "pdfDownload";
    public static final String SAVE = "save";
    public static final String NAME_LABEL = "user.fullName";
    public static final String INFO_LABEL = "infoLabel";
    public static final String FORM_FEEDBACK = "formFeedback";

    @Inject
    private GradingReportService gradingReportService;

    public ReviewerFillOutGradingReportPanel(String id, final IModel<Project> projectModel) {
        super(id, projectModel);

        IModel<ReviewerGradingReport> gradingReport = getGradingReport();
        final ReviewerReportForm form = new ReviewerReportForm(FORM, gradingReport);
        add(form);
        add(new ScrollingSaveButtonPanel(SAVE, form));
        addIndividualParts();
        add(new ExternalLink("daisy", "https://daisy.dsv.su.se/anstalld/uppsats/visaBetyg.jspa?uppsatstillfID=" + projectModel.getObject().getIdentifier(), "Grading in Daisy") {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(projectModel.getObject().getIdentifier()!=null);
            }
        });
    }

    private void addIndividualParts() {
        add(new PropertyListView<SupervisorGradingReport>(INDIVIDUAL_PARTS, getSupervisorReports()) {
            @Override
            protected void populateItem(ListItem<SupervisorGradingReport> item) {
                final IModel<SupervisorGradingReport> gradingReport = item.getModel();
                item.add(new Label(NAME_LABEL));

                final Label infoLabel = new Label(INFO_LABEL, new ResourceModel("info")) {
                    @Override
                    protected void onConfigure() {
                        setVisibilityAllowed(!gradingReport.getObject().isSubmitted());
                    }
                };
                item.add(infoLabel);

                item.add(new PropertyListView<GradingCriterion>(INDIVIDUAL_CRITERIA) {
                    @Override
                    protected void populateItem(ListItem<GradingCriterion> item) {
                        item.add(new Label(INDIVIDUAL_CRITERIA_TITLE_LABEL));
                        item.add(new Label(INDIVIDUAL_CRITERIA_POINTS_LABEL));
                    }
                });
                item.add(new GradingReportPointsPanel(POINTS_PANEL, getGradingReport(), getGradeCalculator(gradingReport.getObject())));
                item.add(new ResourceLink<GradingReport>(PDF_DOWNLOAD,
                        new GradingReportPdfResource(
                                new ReviewerGradingReportPdf(getGradingReport(), gradingReport, getGradeCalculator(gradingReport.getObject())))));
            }
        });
    }

    private GradeCalculator getGradeCalculator(SupervisorGradingReport supervisorGradingReport) {
        DegreeLevel degreeLevel = getModelObject().getProjectType().getDegreeLevel();
        if (degreeLevel.equals(DegreeLevel.FIRST_CYCLE)) {
            return new ReviewerBachelorGradeCalculator(supervisorGradingReport);
        } else if (degreeLevel.equals(DegreeLevel.SECOND_CYCLE)) {
            return getMasterGradeCalculator(supervisorGradingReport);
        }
        throw new IllegalStateException("No grade calculation implemented for project type");
    }

    private GradeCalculator getMasterGradeCalculator(SupervisorGradingReport supervisorGradingReport) {
        int credits = getModelObject().getCredits();
        if (credits == ProjectType.HP_15) {
            return new ReviewerMaster15GradeCalculator(supervisorGradingReport);
        } else if (credits == ProjectType.HP_30 || credits == 0) {
            return new ReviewerMaster30GradeCalculator(supervisorGradingReport);
        }
        throw new IllegalStateException("No grade calculation implemented for master project with " + credits + " credits");
    }

    private class ReviewerReportForm extends Form<ReviewerGradingReport> {

        public ReviewerReportForm(String id, IModel<ReviewerGradingReport> reportModel) {
            super(id, reportModel);

            ComponentFeedbackPanel formFeedback = new ComponentFeedbackPanel(FORM_FEEDBACK, this);
            formFeedback.setOutputMarkupId(true);
            add(formFeedback);

            add(new AutoSavingCriteriaPanel(CRITERIA_PANEL, getModel()));
            addNextStep(reportModel);
            add(new TemplatePanel("calculation", ReviewerFillOutGradingReportPanel.this.getModel()));
        }

        private void addNextStep(final IModel<ReviewerGradingReport> gradingReportIModel) {
            switch (gradingReportIModel.getObject().getState()) {
                case INITIAL:
                    addOrReplace(new InitialFragment("nextStep", gradingReportIModel));
                    break;
                case REVIEWING:
                    addOrReplace(new ReviewingFragment("nextStep", gradingReportIModel));
                    break;
                case FINALIZED:
                    addOrReplace(new EmptyPanel("nextStep"));
                    break;
            }
        }

        private class InitialFragment extends Fragment {
            public InitialFragment(final String id, final IModel<ReviewerGradingReport> reportModel) {
                super(id, "initialStep", ReviewerFillOutGradingReportPanel.this, reportModel);

                add(new Button("initialAssessmentDone") {
                    @Override
                    public void onAfterSubmit() {
                        Either<List<GradingCriterion>, GradingReport> result
                                = gradingReportService.initialAssessmentDone(reportModel.getObject());
                        if (result.isRight()) {
                            getForm().success("Initial assessment done");
                            addNextStep(reportModel);
                        }
                        else {
                            getForm().error(getString("need.to.assess.all.criteria"));
                            List<GradingCriterion> notAssessed = result.left().get();
                            for (GradingCriterion gradingCriterion : notAssessed) {
                                getForm().error(getString("criteria.not.assessed", Model.of(gradingCriterion),
                                        "The criteria \"" + gradingCriterion.getTitle() + "\" has not been assessed."));
                            }
                        }
                    }
                });
            }
        }

        private class ReviewingFragment extends Fragment {
            public ReviewingFragment(final String id, final IModel<ReviewerGradingReport> reportModel) {
                super(id, "reviewingStep", ReviewerFillOutGradingReportPanel.this, reportModel);

                add(new Button("finalize") {
                    @Override
                    public void onAfterSubmit() {
                        Either<List<GradingCriterion>, GradingReport> result
                                = gradingReportService.submitReport(reportModel.getObject());
                        if (result.isRight()) {
                            getForm().success("Finalized grading report");
                            addNextStep(reportModel);
                        }
                        else {
                            getForm().error(getString("not.finished"));
                            List<GradingCriterion> notAssessed = result.left().get();
                            for (GradingCriterion gradingCriterion : notAssessed) {
                                getForm().error(getString("criteria.not.assessed", Model.of(gradingCriterion),
                                        "The criteria \"" + gradingCriterion.getTitle() + "\" has not been assessed."));
                            }
                        }
                    }
                });
            }
        }

        @Override
        protected void onConfigure() {
            super.onConfigure();
            setEnabled(!getModelObject().isSubmitted());
        }

        @Override
        protected void onSubmit() {
            super.onSubmit();
            gradingReportService.save(getModelObject());
            String dateString = new SimpleDateFormat("HH:mm:ss").format(new Date());
            getSession().success(getString("saved") + " " + dateString);
        }
    }

    private IModel<ReviewerGradingReport> getGradingReport() {
        return new LoadableDetachableModel<ReviewerGradingReport>() {
            @Override
            protected ReviewerGradingReport load() {
                return gradingReportService.getReviewerGradingReport(getModelObject());
            }
        };
    }

    private IModel<List<SupervisorGradingReport>> getSupervisorReports() {
        return new LoadableDetachableModel<List<SupervisorGradingReport>>() {
            @Override
            protected List<SupervisorGradingReport> load() {
                return gradingReportService.getSupervisorGradingReports(getModelObject());
            }
        };
    }
}
