package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.EnclosureContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.model.util.SetModel;
import se.su.dsv.scipro.components.AddUsersPanel;
import se.su.dsv.scipro.components.EmployeeAutoCompleteDivPanel;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.date.DateService;
import se.su.dsv.scipro.match.Idea.Type;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.AjaxConfirmationLink;
import se.su.dsv.scipro.util.Pair;

import javax.inject.Inject;
import java.util.*;

public abstract class AdminIdeaDetailsCrudPanel extends Panel {

    public static final String PROJECT_TYPE = "projectType";
    public static final String NONE = "none";

    @Inject
    private IdeaService ideaService;
    @Inject
    private TargetService targetService;
    @Inject
    private DateService dateService;

    private AddUsersPanel authorsPanel;

    private Set<User> originalAuthors = new HashSet<>();

    public AdminIdeaDetailsCrudPanel(String id, IModel<Idea> model) {
        super(id, model);
        FeedbackPanel feedback = new FeedbackPanel("feedback");
        feedback.setOutputMarkupId(true);
        add(feedback);
        add(new IdeaDetailsForm("form", model, feedback));

        for (IdeaParticipation ip : model.getObject().getIdeaParticipations()){
            originalAuthors.add(ip.getUser());
        }
    }

    public abstract void updateTarget(AjaxRequestTarget target);

    public abstract void updateAndClose(AjaxRequestTarget target);

    private class IdeaDetailsForm extends Form<Idea> {

        public IdeaDetailsForm(String id, final IModel<Idea> model, FeedbackPanel feedback) {
            super(id, model);
            Idea idea = model.getObject();
            WebMarkupContainer container = new WebMarkupContainer("container");
            container.setOutputMarkupId(true);
            String periodString = idea.getApplicationPeriod() != null ? idea.getApplicationPeriod().getName() + " "
                    + format(idea.getApplicationPeriod().getStartDate(), null) + " - " + format(idea.getApplicationPeriod().getEndDate(), null)
                    + " Course start: " + format(idea.getApplicationPeriod().getCourseStartDate(), null) : "None";
            String meetingString = idea.getFirstMeeting() != null ? format(idea.getFirstMeeting().getFirstMeetingDate(), DateStyle.DATETIME) + ", " + idea.getFirstMeeting().getRoom() : "None";
            String areaString = idea.getResearchArea() != null ? idea.getResearchArea().getTitle() : "No research area selected";

            container.add(new Label(PROJECT_TYPE, idea.getProjectType().getName()));
            container.add(new PropertyListView<Language>("languages", new ListAdapterModel<>(new PropertyModel<Set<Language>>(model, "languages"))) {
                @Override
                protected void populateItem(final ListItem<Language> item) {
                    item.add(new Label("name"));
                }
            });
            container.add(new Label("status", ideaService.getStatusString(idea)));
            container.add(new Label("period", periodString));
            container.add(new Label("firstMeeting", meetingString));
            container.add(new Label("area", areaString));
            addDescAndPrereqs(container, idea);
            addKeywords(container, model);
            container.add(new IdeaWatsonInfoPanel("watson", model));
            addSupervisorSelection(container, model, feedback);
            addAuthorSelection(container, idea, feedback);
            add(container);
        }

        private void addDescAndPrereqs(WebMarkupContainer container, Idea idea) {
            Label prereqs = new Label("reqs", idea.getPrerequisites() != null ? Model.of(idea.getPrerequisites()) : new ResourceModel(NONE));

            prereqs.setVisible(idea.getType().equals(Type.SUPERVISOR));

            EnclosureContainer enclosureContainer = new EnclosureContainer("enclosure", prereqs);
            enclosureContainer.add(prereqs);
            enclosureContainer.add(new Label("description", idea.getDescription()));
            container.add(enclosureContainer);

        }

        private void addKeywords(WebMarkupContainer container, IModel<Idea> idea) {
            container.add(new Label("noKeysLabel", "No keywords available").setVisible(idea.getObject().getKeywords().isEmpty()));
            container.add(new Label("keywords", new KeywordsModel(idea)));
        }

        private void addSupervisorSelection(final WebMarkupContainer container, final IModel<Idea> ideaModel, final FeedbackPanel feedback) {
            container.add(new EmployeeAutoCompleteDivPanel("supervisorSelection", new PropertyModel<User>(ideaModel, "match.supervisor")) {
                @Override
                public void onNewEmployeeSelection(AjaxRequestTarget target,
                                                   User newSelection) {
                    Idea idea = ideaModel.getObject();
                    if (targetService.hasTargetsLeft(idea.getApplicationPeriod(), newSelection, idea.getProjectType())) {
                        ideaService.changeSupervisor(idea, newSelection, SciProSession.get().getUser());
                        info("Supervisor changed to: " + newSelection.getFullName());
                    } else {
                        error("Supervisor target numbers are reached. Unable to change supervisor.");
                    }
                    target.add(feedback);
                    target.add(container);
                    updateTarget(target);
                }

                @Override
                protected void onConfigure() {
                    super.onConfigure();
                    Idea idea = ideaModel.getObject();
                    setEnabled(idea.getApplicationPeriod() != null && idea.getProjectType() != null);
                }
            });
            addUnmatchButton(container, ideaModel);
        }

        private void addUnmatchButton(final WebMarkupContainer container, final IModel<Idea> idea) {
            AjaxConfirmationLink<Idea> unmatchButton = new AjaxConfirmationLink<Idea>("unmatch", idea, "Are you sure you want to unmatch this idea?") {
                @Override
                public void onClick(AjaxRequestTarget target) {
                    ideaService.adminUnmatchIdea(getModelObject(), SciProSession.get().getUser());
                    info("Unmatched idea: " + getModelObject().getTitle());
                    updateAndClose(target);
                }

                @Override
                protected void onConfigure() {
                    super.onConfigure();
                    Idea idea = getModelObject();
                    setVisible(idea.getType().equals(Type.STUDENT) && idea.getMatchStatus().equals(Idea.Status.MATCHED));
                }
            };
            container.add(unmatchButton);
        }

        private void addAuthorSelection(final WebMarkupContainer container, final Idea idea, final FeedbackPanel feedback) {
            SortedSet<User> authors = new TreeSet<>(new User.ByNameComparator());
            for (IdeaParticipation ip : idea.getIdeaParticipations()) {
                authors.add(ip.getUser());
            }
            authorsPanel = new AddUsersPanel("authorsPanel", new SetModel<>(authors), AddUsersPanel.LookFor.AUTHORS);
            container.add(authorsPanel);
            authorsPanel.setEnabled(idea.getApplicationPeriod() != null);

            AjaxLink<Void> saveAuthors = new AjaxLink<Void>("saveAuthorButton") {

                @Override
                public void onClick(AjaxRequestTarget target) {
                    Pair<Boolean, String> acceptance = ideaService.validateAdminAddAuthors(idea, authorsPanel.getModelObject());
                    if (acceptance.getHead()) {
                        ideaService.adminSetAuthors(idea, authorsPanel.getModelObject(), SciProSession.get().getUser());
                        if (authorsPanel.getModelObject().isEmpty()) {
                            info("Authors " + getAuthorsString(originalAuthors) + " removed from the idea " + idea.getTitle());
                        } else {
                            info("Authors changed on the idea " + idea.getTitle());
                        }
                        setResponsePage(getPage());
                    } else {
                        error(acceptance.getTail());
                        target.add(feedback);
                    }
                }
            };
            saveAuthors.setEnabled(idea.getApplicationPeriod() != null);
            container.add(saveAuthors);
        }
    }

    private String format(Date date, DateStyle dateStyle) {
        if (dateStyle != null) {
            return dateService.format(date, dateStyle);
        } else {
            return dateService.format(date, DateStyle.DATE);
        }
    }

    private String getAuthorsString(Collection<User> students){
        boolean first = true;
        StringBuilder authors = new StringBuilder("");
        for (User user : students) {
            if (first) {
                first = false;
                authors.append(user.getFirstName()).append(" ").append(user.getLastName());
            } else {
                authors.append(", ").append(user.getFirstName()).append(" ").append(user.getLastName());
            }
        }
        return authors.toString();
    }
}


