package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.*;
import se.su.dsv.scipro.components.AjaxCheckBoxMultipleChoice;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.dataproviders.GenericDataProvider;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.statistics.AbstractAdminStatisticsPage;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

import static java.util.Arrays.asList;

@Authorization(authorizedRoles = {Roles.ADMIN})
public class AdminIdeaStatisticsPage extends AbstractAdminStatisticsPage {

    public static final String TABLE = "table";
    public static final String STATUS_FILTER = "statusFilter";
    @Inject
    ApplicationPeriodService applicationPeriodService;
    @Inject
    IdeaService ideaService;
    @Inject
    ProjectTypeService projectTypeService;

    private ExportableDataPanel<ApplicationPeriod, String> dataTable;
    private IdeaService.Filter params = new IdeaService.Filter();

    public AdminIdeaStatisticsPage() {
        addStatusFilter();
        addDataTable();
    }

    private void addStatusFilter() {
        add(new AjaxCheckBoxMultipleChoice<Idea.Status>(STATUS_FILTER, getStatusModel(),
                getStatusChoices(), new EnumChoiceRenderer<Idea.Status>(this)) {

            @Override
            public void onUpdate(AjaxRequestTarget target) {
                target.add(dataTable);
            }
        });
    }

    private PropertyModel<List<Idea.Status>> getStatusModel() {
        List<Idea.Status> defaultSelection = new ArrayList<>();
        defaultSelection.addAll(getStatusChoices());
        params.setStatuses(defaultSelection);
        return new PropertyModel<>(params, "statuses");
    }

    private List<Idea.Status> getStatusChoices() {
        return asList(Idea.Status.UNMATCHED, Idea.Status.MATCHED, Idea.Status.COMPLETED, Idea.Status.INACTIVE);
    }

    private void addDataTable() {
        SortableDataProvider<ApplicationPeriod, String> provider = new GenericDataProvider<>(applicationPeriodService);
        provider.setSort("startDate", SortOrder.DESCENDING);
        dataTable = new ExportableDataPanel<>(TABLE, createColumns(), provider);
        add(dataTable);
    }

    private List<IColumn<ApplicationPeriod, String>> createColumns() {
        List<IColumn<ApplicationPeriod, String>> columns = new ArrayList<>();

        columns.add(new PropertyColumn<ApplicationPeriod, String>(new ResourceModel("applicationPeriod").wrapOnAssignment(this), "name", "name"));
        columns.add(totalIdeasColumn());
        for (ProjectType projectType : projectTypeService.findWithModule(ProjectModule.MATCH)) {
            columns.add(ideasByProjectTypeColumn(new StringResourceModel("projectTypeHeader", Model.of(projectType)).wrapOnAssignment(this), projectType));
        }
        columns.add(ideasByIdeaTypeColumn(new ResourceModel("supervisorIdeas").wrapOnAssignment(this), Idea.Type.SUPERVISOR));
        columns.add(ideasByIdeaTypeColumn(new ResourceModel("studentIdeas").wrapOnAssignment(this), Idea.Type.STUDENT));
        columns.add(numberOfStudentsColumn());

        return columns;
    }

    private AbstractExportableColumn<ApplicationPeriod, String, String> totalIdeasColumn() {
        return new AbstractExportableColumn<ApplicationPeriod, String, String>(new ResourceModel("totalIdeas").wrapOnAssignment(this)) {
            @Override
            public void populateItem(Item<ICellPopulator<ApplicationPeriod>> cellItem, String componentId, IModel<ApplicationPeriod> rowModel) {
                cellItem.add(new Label(componentId, String.valueOf(ideaService.countByApplicationPeriod(rowModel.getObject(), params))));
            }

            @Override
            public IModel<String> getDataModel(IModel<ApplicationPeriod> rowModel) {
                return Model.of(String.valueOf(ideaService.countByApplicationPeriod(rowModel.getObject(), params)));
            }
        };
    }

    private IColumn<ApplicationPeriod, String> ideasByProjectTypeColumn(IModel<String> headerModel, final ProjectType projectType) {
        return new AbstractExportableColumn<ApplicationPeriod, String, String>(headerModel) {
            @Override
            public void populateItem(Item<ICellPopulator<ApplicationPeriod>> cellItem, String id, IModel<ApplicationPeriod> model) {
                cellItem.add(new Label(id, String.valueOf(ideaService.countByApplicationPeriodAndProjectType(model.getObject(), projectType, params))));
            }

            @Override
            public IModel<String> getDataModel(IModel<ApplicationPeriod> rowModel) {
                return Model.of(String.valueOf(ideaService.countByApplicationPeriodAndProjectType(rowModel.getObject(), projectType, params)));
            }
        };
    }

    private IColumn<ApplicationPeriod, String> ideasByIdeaTypeColumn(IModel<String> headerModel, final Idea.Type type) {
        return new AbstractExportableColumn<ApplicationPeriod, String, String>(headerModel) {

            @Override
            public void populateItem(Item<ICellPopulator<ApplicationPeriod>> cellItem, String id, IModel<ApplicationPeriod> model) {
                cellItem.add(new Label(id, String.valueOf(ideaService.countByApplicationPeriodAndType(model.getObject(), type, params))));
            }

            @Override
            public IModel<String> getDataModel(IModel<ApplicationPeriod> rowModel) {
                return Model.of(String.valueOf(ideaService.countByApplicationPeriodAndType(rowModel.getObject(), type, params)));
            }
        };
    }

    private AbstractColumn<ApplicationPeriod, String> numberOfStudentsColumn() {
        return new AbstractExportableColumn<ApplicationPeriod, String, String>(new ResourceModel("numberOfStudents").wrapOnAssignment(this)) {
            @Override
            public void populateItem(Item<ICellPopulator<ApplicationPeriod>> item, String id, IModel<ApplicationPeriod> model) {
                item.add(new Label(id, String.valueOf(ideaService.countAuthorsByApplicationPeriod(model.getObject(), params))));
            }

            @Override
            public IModel<String> getDataModel(IModel<ApplicationPeriod> rowModel) {
                return Model.of(String.valueOf(ideaService.countAuthorsByApplicationPeriod(rowModel.getObject(), params)));
            }
        };
    }

}
