package se.su.dsv.scipro.match;

import org.apache.wicket.WicketRuntimeException;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.components.AjaxIconLinkPanel;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.GenericDataProvider;
import se.su.dsv.scipro.datatables.ClickableIconColumn;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.springdata.services.UnitService;
import se.su.dsv.scipro.system.Unit;

import javax.inject.Inject;
import javax.validation.ConstraintViolationException;
import java.util.ArrayList;
import java.util.List;

@Authorization(authorizedRoles = {Roles.SYSADMIN, Roles.ADMIN})
public class AdminUnitPage extends AbstractAdminMatchPage {

    private static final Logger LOGGER = LoggerFactory.getLogger(AdminUnitPage.class);
    protected final ExportableDataPanel<Unit, String> dp;
    private final FeedbackPanel feedbackPanel;
    private ModalWindowPlus modalWindowPlus;

    @Inject
    private UnitService unitService;

    public AdminUnitPage() {
        feedbackPanel = new FeedbackPanel("feedback");
        feedbackPanel.setOutputMarkupId(true);
        add(feedbackPanel);

        SortableDataProvider<Unit, String> provider = new GenericDataProvider<>(unitService);

        dp = new ExportableDataPanel<>("dp", createColumns(), provider);
        dp.setOutputMarkupId(true);
        add(dp);

        addDialog();
        addCreateNewLink();
	}

    private void addDialog() {
        modalWindowPlus = new ModalWindowPlus("dialog");
        add(modalWindowPlus);
    }

    private void addCreateNewLink() {
        AjaxLink<Void> createNewLink = new AjaxLink<Void>("createNewLink") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                modalWindowPlus.setTitle("Create new unit");
                Unit unit = new Unit();
                modalWindowPlus.setContent(new EditUnitPanel(modalWindowPlus.getContentId(), new DetachableServiceModel<>(unitService, unit)) {
                    @Override
                    public void onSave(AjaxRequestTarget target) {
                        target.add(dp);
                    }
                });
                modalWindowPlus.show(target);
            }
        };
        add(createNewLink);
    }

    private List<IColumn<Unit, String>> createColumns() {
        List<IColumn<Unit, String>> columns = new ArrayList<>();
        columns.add(new PropertyColumn<Unit, String>(Model.of("Title"), "title", "title"));
        columns.add(new AbstractColumn<Unit, String>(Model.of("Has external system identifier")) {
            @Override
            public void populateItem(Item<ICellPopulator<Unit>> cellItem, String componentId, IModel<Unit> rowModel) {
                String hasExternalIdentifier = (rowModel.getObject().getIdentifier() == null) ? "No" : "Yes";
                cellItem.add(new Label(componentId, Model.of(hasExternalIdentifier)));
            }
        });
        columns.add(new PropertyColumn<Unit, String>(Model.of("Match responsible"), "matchResponsible", "matchResponsible"));
        columns.add(new ClickableIconColumn<Unit>(Model.of("Edit unit"), null, ClickableIconColumn.EDIT) {
            @Override
            protected void onClick(IModel<Unit> clicked, AjaxRequestTarget target) {
                modalWindowPlus.setTitle("Edit unit: " + clicked.getObject().getTitle());
                modalWindowPlus.setContent(new EditUnitPanel(modalWindowPlus.getContentId(), clicked) {
                    @Override
                    public void onSave(AjaxRequestTarget target) {
                        target.add(dp);
                    }
                });
                modalWindowPlus.show(target);
            }
        });
        columns.add(new AbstractColumn<Unit, String>(Model.of("Delete")) {
            @Override
            public void populateItem(Item<ICellPopulator<Unit>> cellItem, final String componentId, final IModel<Unit> rowModel) {
                AjaxIconLinkPanel<Unit> delete = new AjaxIconLinkPanel<Unit>(componentId, rowModel, Model.of(AjaxIconLinkPanel.DELETE), "Are you sure?") {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<Unit> model) {
                        try {
                            unitService.delete(model.getObject());
                            info("Deleted unit");
                        } catch (ConstraintViolationException cve) {
                            String message = "Could not delete unit because it is being used";
                            error(message);
                            LOGGER.info(message, cve);
                        } catch (WicketRuntimeException wre) {
                            String message = "Could not delete unit";
                            error(message);
                            LOGGER.info(message, wre);
                        }
                        target.add(feedbackPanel);
                        target.add(dp);
                    }
                };
                cellItem.add(delete);
            }
        });
        return columns;
    }
}
