package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormChoiceComponentUpdatingBehavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.ComponentFeedbackPanel;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.*;
import org.apache.wicket.model.util.ListModel;
import org.apache.wicket.model.util.SetModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.validation.validator.StringValidator;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.projectpartner.ProjectPartnerPage;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.*;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class ProjectIdeaSubmissionPanel extends GenericPanel<Idea> {

    public static final String PROJECT_TYPE_INFO = "projectTypeInfo";
    public static final String PROJECT_TYPE = "projectType";
    public static final String CO_AUTHOR_CHOICE = "coAuthorChoice";
    public static final String IDEA_FORM = "ideaForm";
    public static final String PROGRAM = "program";
    public static final String PROGRAM_DROP_DOWN = "programDropDown";
    @Inject
    private IdeaService ideaService;
    @Inject
    private ResearchAreaService researchAreaService;
    @Inject
    private KeywordService keywordService;
    @Inject
    private ApplicationPeriodService applicationPeriodService;
    @Inject
    private UserService userService;
    @Inject
    private LanguageService languageService;

    private PageParameters pp;

    private Set<Keyword> keywords = new HashSet<>();

    public ProjectIdeaSubmissionPanel(String id, IModel<Idea> model, boolean isNewIdea) {
        super(id, model);

        pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(ApplicationPeriod.class), model.getObject().getApplicationPeriod().getId());

        IdeaForm form = new IdeaForm(IDEA_FORM, model, isNewIdea);
        add(form);
        add(new BookmarkablePageLink<Void>("sitop", ProjectNewAllSupervisorIdeasPage.class, pp));
        addFeedback(form);
        addDialog();
    }

    private void addDialog() {
        ModalWindowPlus modalWindowPlus = new ModalWindowPlus("dialog");
        add(modalWindowPlus);
    }

    private void addFeedback(IdeaForm form) {
        FeedbackPanel feedback = new ComponentFeedbackPanel("feedback", form);
        feedback.setOutputMarkupId(true);
        add(feedback);
    }

    private LoadableDetachableModel<User> getLoadedStudent(){
        return new LoadableDetachableModel<User>() {
            @Override
            protected User load() {
                return userService.findOne(SciProSession.get().getUser().getId());
            }
        };
    }

    private class IdeaForm extends Form<Idea> {

        public static final int MIN_TITLE_LENGTH = 5;
        private static final int MAX_KEYWORDS_SIZE = 5;

        private final boolean isNewIdea;
        private KeywordFilterPanel keywordsPanel;
        private DropDownChoice<Program> programDropDownChoice;
        private User creator;
        private CoAuthorChoice coAuthorChoice;

        public IdeaForm(String id, IModel<Idea> model, final boolean isNewIdea) {
            super(id, model);
            this.isNewIdea = isNewIdea;
            this.creator = SciProSession.get().getUser();
            addProjectTypeSelection();
            addLanguageSelection();
            addCoAuthorSelection();
            addProgramSelection();
            addTitleField();
            addWatsonFields();
            addResearchAreaAndKeywordsSelection();
            addContract();
        }

        private void addProjectTypeSelection() {
            final RadioChoice<ProjectType> projectTypeChoice = new RadioChoice<>(PROJECT_TYPE, new PropertyModel<ProjectType>(getModel(), "projectType"),
                    new ArrayList<>(getModelObject().getApplicationPeriod().getProjectTypes()), new ChoiceRenderer<ProjectType>("name", "id"));
            projectTypeChoice.setRequired(true);
            projectTypeChoice.setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER);
            projectTypeChoice.add(new AjaxFormChoiceComponentUpdatingBehavior() {

                @Override
                protected void onUpdate(AjaxRequestTarget target) {
                    coAuthorChoice.setRequired(coAuthorIsRequired(projectTypeChoice));
                }
            });
            add(projectTypeChoice);
            add(new ComponentFeedbackPanel("projectTypeFeedback", projectTypeChoice));
            add(new InfoPanel(PROJECT_TYPE_INFO, new ResourceModel("projectTypeInfo")));
        }

        private boolean coAuthorIsRequired(RadioChoice<ProjectType> projectType) {
            return projectType.getModelObject().getMinAuthors() > 1 && !applicationPeriodService.hasExemption(creator, getModelObject().getApplicationPeriod());
        }

        private void addCoAuthorSelection() {
            coAuthorChoice = new CoAuthorChoice(CO_AUTHOR_CHOICE, creator, getModel());
            coAuthorChoice.setRequired(true);
            add(coAuthorChoice);
            add(new ComponentFeedbackPanel("coAuthorFeedback", coAuthorChoice));
            addCoAuthorInfoLink();
        }

        private void addProgramSelection() {
            final User student = getLoadedStudent().getObject();
            WebMarkupContainer program = new WebMarkupContainer(PROGRAM) {
                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(student.hasPrograms());
                }
            };
            add(program);
            programDropDownChoice = new DropDownChoice<>(PROGRAM_DROP_DOWN, new Model<Program>(),
                    new ArrayList<>(student.getPrograms()), new ChoiceRenderer<Program>("name", "id"));
            programDropDownChoice.setNullValid(true);
            program.add(programDropDownChoice);
            program.add(new InfoPanel("programInfo", new ResourceModel("programInfo")));
        }


        private void addLanguageSelection() {
            CheckBoxMultipleChoice<Language> language = new CheckBoxMultipleChoice<>("languages", new PropertyModel<Set<Language>>(getModel(), "languages"),
                    languageService.findAll(), new ChoiceRenderer<>("name", "id"));
            language.setRequired(true);
            language.setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER);
            add(new ComponentFeedbackPanel("languageFeedback", language));
            add(language);
            add(new InfoPanel("languageInfo", new ResourceModel("languageInfo")));
        }

        private void addCoAuthorInfoLink() {
            add(new Link<Void>("partnerLink") {
                @Override
                public void onClick() {
                    setResponsePage(ProjectPartnerPage.class);
                }
            });
        }

        private void addTitleField() {
            TextField<String> title = new TextField<>("title", new PropertyModel<String>(getModel(), "title"));
            title.setRequired(true);
            title.add(StringValidator.minimumLength(MIN_TITLE_LENGTH));
            add(new ComponentFeedbackPanel("titleFeedback", title));
            add(title);
        }

        private void addWatsonFields() {
            CounterTextArea what = new CounterTextArea("what", new PropertyModel<String>(getModel(), "watson.what"), Watson.MAX_CHARS);
            what.setRequired(true);
            add(what);

            CounterTextArea why = new CounterTextArea("why", new PropertyModel<String>(getModel(), "watson.why"), Watson.MAX_CHARS);
            why.setRequired(true);
            add(why);

            CounterTextArea theory = new CounterTextArea("theory", new PropertyModel<String>(getModel(), "watson.theoryHow"), Watson.MAX_CHARS);
            theory.setRequired(true);
            add(theory);

            CounterTextArea practical = new CounterTextArea("practical", new PropertyModel<String>(getModel(), "watson.practicalHow"), Watson.MAX_CHARS);
            practical.setRequired(true);
            add(practical);
        }

        private void addResearchAreaAndKeywordsSelection() {
            final WebMarkupContainer areaContainer = new WebMarkupContainer("areaContainer");
            areaContainer.setOutputMarkupId(true);

            areaContainer.add(new InfoPanel("researchAreaInfo", new ResourceModel("researchAreaInfo")));

            List<ResearchArea> areas = researchAreaService.findNotDeleted();

            AjaxDropDownChoice<ResearchArea> areaDropDown = new AjaxDropDownChoice<ResearchArea>("area", new PropertyModel<ResearchArea>(getModel(), "researchArea"),
                    new ListModel<>(areas), new ChoiceRenderer<ResearchArea>("title", "id")) {
                @Override
                public void onNewSelection(AjaxRequestTarget target, ResearchArea selected) {
                    Set<Keyword> keywordHashSet = new HashSet<>(keywordService.findByResearchArea(selected));
                    keywordsPanel.setModelObject(keywordHashSet);
                    keywordsPanel.setVisible(!keywordHashSet.isEmpty());
                    target.add(areaContainer);
                    target.add(keywordsPanel);
                }
            };
            areaDropDown.setRequired(true);
            areaContainer.add(areaDropDown);
            areaContainer.add(new ComponentFeedbackPanel("researchAreaFeedback", areaDropDown));

            keywordsPanel = new KeywordFilterPanel("keywords", new SetModel<>(new HashSet<Keyword>()), new ListModel<>(new ArrayList<>(getModelObject().getKeywords()))) {
                @Override
                public void onKeywordsSelected(AjaxRequestTarget target, Set<Keyword> selected) {
                    keywords = selected;
                }
            };
            keywordsPanel.setOutputMarkupPlaceholderTag(true);
            keywordsPanel.setVisible(false);
            areaContainer.add(keywordsPanel);
            areaContainer.add(new ComponentFeedbackPanel("keywordsFeedback", keywordsPanel));
            add(areaContainer);
        }

        private void addContract() {
            CheckBox contract = new CheckBox("contract", new Model<Boolean>());
            contract.add(new CheckBoxRequiredValidator());
            add(contract);
            add(new ComponentFeedbackPanel("contractFeedback", contract));
            add(new BookmarkablePageLink<Void>("sibottom", ProjectNewAllSupervisorIdeasPage.class, pp));
        }

        @Override
        protected void onSubmit() {
            ideaService.saveStudentIdea(getModelObject(), creator, programDropDownChoice.getModelObject(), coAuthorChoice.getModelObject(),
                    new ArrayList<>(keywords), isNewIdea);
            setResponsePage(ProjectIdeaStartPage.class);
        }


        @Override
        protected void onError() {
            error(getString("submissionFailed"));
        }

        @Override
        protected void onValidate() {
            super.onValidate();
            if (keywordNotWithinLimits()) {
                keywordsPanel.error(getString("keywordError"));
            }
        }

        private boolean keywordNotWithinLimits() {
            return keywords.isEmpty() || keywords.size() > MAX_KEYWORDS_SIZE;
        }
    }
}
