package se.su.dsv.scipro.match;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.util.ListModel;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.match.Idea.Type;
import se.su.dsv.scipro.profile.StudentProfileProgramPanel;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.*;
import se.su.dsv.scipro.util.Pair;

import javax.inject.Inject;
import java.util.*;

public class SupervisorAllStudentIdeasPanel extends GenericPanel<ApplicationPeriod> {

    @Inject
    private IdeaService ideaService;
    @Inject
    private KeywordService keywordService;
    @Inject
    private ResearchAreaService researchAreaService;

    private ModalWindowPlus modalWindowPlus;
    private final IdeaService.Filter filter = new IdeaService.Filter();
    private ExportableDataPanel<Idea, String> dataTable;
    private FeedbackPanel feedback;

    public SupervisorAllStudentIdeasPanel(String id, final IModel<ApplicationPeriod> applicationPeriod) {
        super(id, applicationPeriod);
    }

    @Override
    protected void onInitialize() {
        super.onInitialize();
        addDialog();
        addFeedback();
        addFiltering();
        addDataTable();
        prepareFilter(filter);
    }

    private void addFeedback() {
        feedback = new FeedbackPanel("feedback");
        feedback.setOutputMarkupId(true);
        add(feedback);
    }

    protected void prepareFilter(IdeaService.Filter filter) {
        filter.setTypes(Collections.singleton(Type.STUDENT));
        filter.setApplicationPeriod(getModelObject());
    }

    @Override
    protected void onConfigure() {
        super.onConfigure();
        setVisibilityAllowed(getModelObject() != null && getModelObject().isOpen());
    }

    private void addDialog() {
        modalWindowPlus = new ModalWindowPlus("dialog");
        add(modalWindowPlus);
    }

    private void addDataTable() {
        SortableDataProvider<Idea, String> provider = new FilteredDataProvider<>(ideaService, Model.of(filter));
        provider.setSort("dateCreated", SortOrder.DESCENDING);
        dataTable = new ExportableDataPanel<>("dataTable", createColumns(), provider);
        add(dataTable);
    }

    private List<IColumn<Idea, String>> createColumns() {
        List<IColumn<Idea, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Idea>(Model.of("Created"), "dateCreated", "dateCreated"));
        columns.add(new PropertyColumn<Idea, String>(Model.of("Type"), "projectType.name", "projectType.name"));

        columns.add(new AbstractColumn<Idea, String>(Model.of("Languages")) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> cellItem, String componentId, IModel<Idea> rowModel) {
                cellItem.add(new DisplayMultiplesPanel<Language>(componentId, new ListAdapterModel<>(new PropertyModel<Iterable<Language>>(rowModel, "languages"))) {
                    @Override
                    public Component getComponent(String componentId, IModel<Language> t) {
                        return new Label(componentId, new PropertyModel<Language>(t, "name"));
                    }
                });
            }
        });

        columns.add(new AbstractColumn<Idea, String>(Model.of("Title"), "title") {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> components, String s, final IModel<Idea> model) {
                components.add(new AjaxLinkPanel<Idea>(s, model, new PropertyModel(model, "title")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<Idea> model) {
                        modalWindowPlus.setContent(new SupervisorAcceptIdeaDialogPanel(modalWindowPlus.getContentId(), model));
                        modalWindowPlus.setTitle(model.getObject().getTitle());
                        modalWindowPlus.show(target);
                    }
                });
            }
        });

        columns.add(new AbstractColumn<Idea, String>(Model.of("Authors")) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> cellItem, String componentId, IModel<Idea> rowModel) {
                cellItem.add(new DisplayMultiplesPanel<IdeaParticipation>(componentId, new ListAdapterModel<>(new PropertyModel<Set<IdeaParticipation>>(rowModel, "ideaParticipations"))) {
                    @Override
                    public Component getComponent(String componentId, IModel<IdeaParticipation> t) {
                        return new StudentProfileProgramPanel(componentId, t, false);
                    }
                });
            }
        });
        columns.add(new PropertyColumn<Idea, String>(Model.of("Research Area"), "researchArea.title", "researchArea.title"));
        columns.add(new KeywordsColumn<String>(Model.of("Keywords")));
        columns.add(new AbstractColumn<Idea, String>(Model.of("Supervisor"), "match.supervisor") {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> cellItem, String componentId, IModel<Idea> rowModel) {
                Idea idea = rowModel.getObject();
                if (idea.hasSupervisor()) {
                    cellItem.add(new UserLinkPanel(componentId, new PropertyModel<User>(rowModel, "match.supervisor")));
                } else {
                    cellItem.add(new AjaxConfirmingLinkPanel<Idea>(componentId, rowModel, Model.of("Select")) {
                        @Override
                        public void onClick(AjaxRequestTarget target, IModel<Idea> model) {
                            User supervisor = getCurrentSupervisor();
                            Pair<Boolean, String> acceptance = ideaService.validateSupervisorAcceptance(model.getObject(), supervisor);
                            if (acceptance.getHead()) {
                                ideaService.confirmSupervision(model.getObject(), supervisor);
                                success(acceptance.getTail());
                            } else {
                                error(acceptance.getTail());
                            }
                            target.add(feedback);
                            target.add(dataTable);
                        }

                        private User getCurrentSupervisor() {
                            return SciProSession.get().getUser();
                        }
                    });
                }
            }
        });
        return columns;
    }

    private void addFiltering() {

        AjaxCheckBoxMultipleChoice<ProjectType> projectType = new AjaxCheckBoxMultipleChoice<ProjectType>("projectTypeFilter",
                getProjectTypeModel(), getChoices(), new ChoiceRenderer<ProjectType>("name", "id")) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                target.add(dataTable);
            }
        };
        add(projectType);

        final KeywordFilterPanel keywordFilter = new KeywordFilterPanel("keywordFilter", getKeywords()) {
            @Override
            public void onKeywordsSelected(AjaxRequestTarget target, Set<Keyword> keywords) {
                filter.setKeywords(keywords);
                target.add(dataTable);
            }
        };
        keywordFilter.setOutputMarkupPlaceholderTag(true);
        keywordFilter.setVisible(false);
        add(keywordFilter);

        researchAreaFilter(keywordFilter);
    }

    private void researchAreaFilter(final KeywordFilterPanel keywordFilter) {
        add(new AjaxDropDownChoice<ResearchArea>("areaFilter", new Model<ResearchArea>(),
                new ListModel<>(researchAreaService.findNotDeleted()),
                new ChoiceRenderer<ResearchArea>("title", "id")) {
            @Override
            public void onNewSelection(AjaxRequestTarget target, ResearchArea objectSelected) {
                filter.setResearchArea(getModelObject());
                target.add(dataTable);
                keywordFilter.setVisible(getModelObject() != null && !keywordService.findByResearchArea(getModelObject()).isEmpty());
                target.add(keywordFilter);
            }
        });
    }

    private PropertyModel<List<ProjectType>> getProjectTypeModel() {
        filter.setLevels(getChoices());
        return new PropertyModel<>(filter, "levels");
    }

    protected List<ProjectType> getChoices() {
        return new ArrayList<>(getModelObject().getProjectTypes());
    }

    private IModel<Set<Keyword>> getKeywords() {
        return new LoadableDetachableModel<Set<Keyword>>() {
            @Override
            protected Set<Keyword> load() {
                if (filter.getResearchArea() != null) {
                    return new HashSet<>(keywordService.findByResearchArea(filter.getResearchArea()));
                }
                return new HashSet<>();
            }
        };
    }
}
