package se.su.dsv.scipro.match;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.panel.ComponentFeedbackPanel;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.*;
import org.apache.wicket.model.util.ListModel;
import org.apache.wicket.model.util.SetModel;
import org.apache.wicket.util.visit.IVisit;
import org.apache.wicket.util.visit.IVisitor;
import org.apache.wicket.validation.validator.StringValidator;
import se.su.dsv.scipro.components.AjaxDropDownChoice;
import se.su.dsv.scipro.components.CounterTextArea;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.*;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import static java.util.Arrays.asList;

public class SupervisorEditIdeaPanel extends Panel {

    public static final String PROJECT_TYPE = "projectType";
    public static final String IDEA_FORM = "ideaForm";

    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private IdeaService ideaService;
    @Inject
    private KeywordService keywordService;
    @Inject
    private UserService userService;
    @Inject
    private LanguageService languageService;

    private Set<Keyword> selectedKeywords;
    private Set<Keyword> selectableKeywords;

    public SupervisorEditIdeaPanel(String id, IModel<Idea> model, boolean isNewIdea) {
        super(id, model);

        if (model.getObject().getKeywords() != null) {
            selectedKeywords = new HashSet<>(model.getObject().getKeywords());
        } else {
            selectedKeywords = new HashSet<>();
        }

        if (model.getObject().getResearchArea() != null) {
            selectableKeywords = new HashSet<>(keywordService.findByResearchArea(model.getObject().getResearchArea()));
        } else {
            selectableKeywords = new HashSet<>();
        }

        IdeaForm ideaForm = new IdeaForm(IDEA_FORM, model, isNewIdea);
        add(ideaForm);
        addFeedback(ideaForm);
    }

    private void addFeedback(IdeaForm ideaForm) {
        FeedbackPanel feedback = new ComponentFeedbackPanel("feedback", ideaForm);
        feedback.setOutputMarkupId(true);
        add(feedback);
    }

    private class IdeaForm extends Form<Idea> {

        public static final int MIN_TITLE_LENGTH = 5;
        public static final int DESCRIPTION_MAX_CHARS = 4000;
        public static final int PREREQS_MAX_CHARS = 4000;
        public static final int MAX_KEYWORDS_SIZE = 5;
        public static final int MAX_TITLE_LENGTH = 250;
        private KeywordFilterPanel keywordsPanel;

        public IdeaForm(String id, IModel<Idea> model, final boolean isNewIdea) {
            super(id, model);
            addProjectTypeSelection();
            addLanguageSelection();
            addTitleField();
            addDescriptionField();
            addPrerequisitesField();
            addResearchAreaAndKeywordsSelection();
            addPublishedSelection();
            addSaveButton(model, isNewIdea);
        }

        private void addProjectTypeSelection() {
            RadioChoice<ProjectType> projectType = new RadioChoice<>(PROJECT_TYPE, new PropertyModel<ProjectType>(getModel(), "projectType"),
                    projectTypeService.findWithModule(ProjectModule.MATCH),
                    new ChoiceRenderer<ProjectType>("name", "id"));
            projectType.setRequired(true);
            projectType.setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER);
            add(projectType);
            addComponentFeedback(this, "projectTypeFeedback", projectType);
        }

        private void addLanguageSelection() {
            CheckBoxMultipleChoice<Language> language = new CheckBoxMultipleChoice<>("language", new PropertyModel<Set<Language>>(getModel(), "languages"),
                    languageService.findAll(), new ChoiceRenderer<>("name", "id"));
            language.setRequired(true);
            language.setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER);
            addComponentFeedback(this, "languageFeedback", language);
            add(language);
        }

        private void addTitleField() {
            TextField<String> title = new TextField<>("title", new PropertyModel<String>(getModel(), "title"));
            title.setRequired(true);
            title.add(StringValidator.lengthBetween(MIN_TITLE_LENGTH, MAX_TITLE_LENGTH));
            addComponentFeedback(this, "titleFeedback", title);
            add(title);
            add(new Label("titleSmallText", new StringResourceModel("titleDesc", null, MIN_TITLE_LENGTH, MAX_TITLE_LENGTH)));
            add(new InfoPanel("titleInfo", new ResourceModel("titleInfo")));
        }

        private void addDescriptionField() {
            CounterTextArea description = new CounterTextArea("description", new PropertyModel<String>(getModel(), "description"), DESCRIPTION_MAX_CHARS);
            description.setRequired(true);
            add(description);
            add(new InfoPanel("descriptionInfo", new ResourceModel("descriptionInfo")));
        }

        private void addPrerequisitesField() {
            CounterTextArea prerequisites = new CounterTextArea("prerequisites", new PropertyModel<String>(getModel(), "prerequisites"), PREREQS_MAX_CHARS);
            prerequisites.setRequired(true);
            add(prerequisites);
            add(new InfoPanel("prereqsInfo", new ResourceModel("prereqsInfo")));
        }

        private void addResearchAreaAndKeywordsSelection() {
            final WebMarkupContainer areaContainer = new WebMarkupContainer("areaContainer");
            areaContainer.setOutputMarkupId(true);

            AjaxDropDownChoice<ResearchArea> areaDropDown = getResearchAreaDropDown();
            areaContainer.add(areaDropDown);
            addComponentFeedback(areaContainer, "researchAreaFeedback", areaDropDown);


            keywordsPanel = getKeywordsPanel();
            areaContainer.add(getKeywordsPanel());
            addComponentFeedback(areaContainer, "keywordsFeedback", keywordsPanel);

            add(areaContainer);

            areaContainer.add(new InfoPanel("researchAreaInfo", new ResourceModel("researchAreaInfo")));
        }

        private AjaxDropDownChoice<ResearchArea> getResearchAreaDropDown() {
            AjaxDropDownChoice<ResearchArea> areaDropDown = new AjaxDropDownChoice<ResearchArea>("area",
                    new PropertyModel<ResearchArea>(getModel(), "researchArea"),
                    new ListAdapterModel<>(new PropertyModel<Set<ResearchArea>>(getUser(), "activeResearchAreas")),
                    new ChoiceRenderer<ResearchArea>("title", "id")) {
                @Override
                public void onNewSelection(AjaxRequestTarget target, ResearchArea selected) {
                    Set<Keyword> keywordsModelObject = new HashSet<>(keywordService.findByResearchArea(selected));
                    keywordsPanel.setModelObject(keywordsModelObject);
                    if (!keywordsModelObject.isEmpty()) {
                        keywordsPanel.setVisible(true);
                    }
                    target.add(keywordsPanel);
                }
            };
            areaDropDown.setRequired(true);
            return areaDropDown;
        }

        private KeywordFilterPanel getKeywordsPanel() {
            keywordsPanel = new KeywordFilterPanel("keywords", new SetModel<>(selectableKeywords), new ListModel<>(new ArrayList<>(selectedKeywords))) {
                @Override
                public void onKeywordsSelected(AjaxRequestTarget target, Set<Keyword> selected) {
                    selectedKeywords = selected;
                }
            };
            keywordsPanel.setOutputMarkupPlaceholderTag(true);
            keywordsPanel.setVisible(!selectableKeywords.isEmpty());
            return keywordsPanel;
        }

        private void addPublishedSelection() {
            RadioChoice<Boolean> published = new RadioChoice<>("published", new PropertyModel<Boolean>(getModel(), "published"),
                    asList(new Boolean[]{Boolean.TRUE, Boolean.FALSE}), getPublishedChoiceRenderer());
            published.setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER);
            add(published);
            add(new InfoPanel("publishInfo", new ResourceModel("publishInfo")));
        }

        private void addSaveButton(final IModel<Idea> model, final boolean isNewIdea) {
            add(new AjaxButton("save") {
                @Override
                protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                    ideaService.saveSupervisorIdea(model.getObject(), SciProSession.get().getUser(), new ArrayList<>(selectedKeywords), isNewIdea);
                    setResponsePage(SupervisorIdeaStartPage.class);
                }

                @Override
                protected void onError(AjaxRequestTarget target, Form<?> form) {
                    getForm().error(getString("defaultError"));
                    updateFeedbackPanels(target);
                    target.appendJavaScript("window.scrollTo(0,0);");
                }
            });
        }

        private void addComponentFeedback(WebMarkupContainer container, String id, Component component) {
            ComponentFeedbackPanel feedbackPanel = new ComponentFeedbackPanel(id, component);
            feedbackPanel.setOutputMarkupId(true);
            container.add(feedbackPanel);
        }

        @Override
        protected void onValidate() {
            super.onValidate();
            if (keywordsNotWithinLimits()) {
                keywordsPanel.error(getString("keywordError"));
            }
        }

        private boolean keywordsNotWithinLimits() {
            return selectedKeywords == null || selectedKeywords.isEmpty() || selectedKeywords.size() > MAX_KEYWORDS_SIZE;
        }

        private IChoiceRenderer<Boolean> getPublishedChoiceRenderer() {
            return new IChoiceRenderer<Boolean>() {
                @Override
                public Object getDisplayValue(Boolean b) {
                    String displayValue = "Not ready to publish";
                    if (b != null && b) {
                        displayValue = "Ready to publish (Available to students if target > 0)";
                    }
                    return displayValue;
                }

                @Override
                public String getIdValue(Boolean object, int index) {
                    return object.toString();
                }
            };
        }
    }

    private IModel<User> getUser() {
        return new LoadableDetachableModel<User>() {
            @Override
            protected User load() {
                return userService.findOne(SciProSession.get().getUser().getId());
            }
        };
    }

    private void updateFeedbackPanels(final AjaxRequestTarget target) {
        target.getPage().visitChildren(FeedbackPanel.class, new IVisitor<FeedbackPanel, Void>() {
            @Override
            public void component(final FeedbackPanel feedback, IVisit visit) {
                target.add(feedback);
            }
        });
    }

}
