/**
 *
 */
package se.su.dsv.scipro.peer;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.joda.time.DateTime;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.date.DatePickerPanel;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.*;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

import static java.util.Arrays.asList;

public class PeerRequestListPanel extends Panel {

    public static final int YEAR = 12;
    @Inject
    private PeerRequestService peerRequestService;
    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private LanguageService languageService;

    private PeerRequestService.Filter filter;
    private ExportableDataPanel<PeerRequest, String> table;

    public PeerRequestListPanel(String id) {
        super(id);
        this.setOutputMarkupId(true);
        add(new FilterForm("form"));
        setupDataTable();
    }

    private void setupDataTable() {
        SortableDataProvider<PeerRequest, String> dataProvider =
                new FilteredDataProvider<>(peerRequestService, Model.of(filter), "dateCreated", SortOrder.DESCENDING);
        table = new ExportableDataPanel<>("dataTable", createColumns(), dataProvider);
        add(table);
    }

    private List<IColumn<PeerRequest, String>> createColumns() {
        List<IColumn<PeerRequest, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<PeerRequest>(new ResourceModel("dateHeader"), "dateCreated", "dateCreated", DateStyle.DATETIME));

        columns.add(new AbstractColumn<PeerRequest, String>(new ResourceModel("authorHeader"), "requester.lastName") {
            @Override
            public void populateItem(Item<ICellPopulator<PeerRequest>> cellItem, String componentId, IModel<PeerRequest> rowModel) {
                cellItem.add(new UserLinkPanel(componentId, new PropertyModel<User>(rowModel, "requester")));
            }
        });

        columns.add(new PropertyColumn<PeerRequest, String>(new ResourceModel("projectHeader"), "project.title", "project.title"));
        columns.add(new EnumPropertyColumn<PeerRequest, RequestStatus>(new ResourceModel("statusHeader"), "status", "status"));

        return columns;
    }

    private class FilterForm extends Form<Void> {

        public FilterForm(String id) {
            super(id);
            filter = new PeerRequestService.Filter();
            final AuthorAutoComplete authorField = new AuthorAutoComplete("filterAuthor") {

                @Override
                protected void action(AjaxRequestTarget pTarget, User newSelection) {
                    filter.setAuthor(newSelection);
                    pTarget.add(table);
                }
            };
            add(authorField);

            java.util.List<Language> languages = languageService.findAll();
            final PropertyModel<Language> languagePropertyModel = new PropertyModel<>(filter,"language");
            final DropDownChoice<Language> languageFilter = new DropDownChoice<>("filterLanguage", languagePropertyModel, languages, new ChoiceRenderer<>("name", "id"));
            languageFilter.add(new AjaxFormComponentUpdatingBehavior("onchange") {

                @Override
                protected void onUpdate(AjaxRequestTarget target) {
                    filter.setLanguage(languagePropertyModel.getObject());
                    target.add(table);
                }
            });

            filter.setLanguage(languagePropertyModel.getObject());
            add(languageFilter);

            addProjectTypesFilter();

            AjaxCheckBoxMultipleChoice<RequestStatus> requestStatus = new AjaxCheckBoxMultipleChoice<RequestStatus>("statusFilter",
                    getRequestStatusModel(), asList(RequestStatus.values()), new EnumChoiceRenderer<RequestStatus>(this)) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    target.add(table);
                }
            };
            add(requestStatus);

            DateTime start = new DateTime();
            start = start.minusMonths(YEAR);
            final DatePickerPanel datePanel = new DatePickerPanel("datePanel", start.toDate()) {

                @Override
                public void updateStartDate(AjaxRequestTarget target) {
                    filter.setCreatedAfter(getStartDate());
                    target.add(table);
                }

                @Override
                public void updateEndDate(AjaxRequestTarget target) {
                    filter.setCreatedBefore(getEndDate());
                    target.add(table);
                }
            };
            filter.setCreatedAfter(start.toDate());
            filter.setCreatedBefore(datePanel.getEndDate());
            add(datePanel);
            final AjaxCheckBox supervisorProjects = new AjaxCheckBox("supervisorCheckBox", Model.of(false)) {

                @Override
                protected void onUpdate(AjaxRequestTarget target) {
                    if (this.getModelObject()) {
                        User supervisor = SciProSession.get().getUser();
                        filter.setSupervisor(supervisor);
                    } else {
                        filter.setSupervisor(null);
                    }
                    target.add(table);
                }

            };
            add(supervisorProjects);
        }

        private void addProjectTypesFilter() {
            AjaxCheckBoxMultipleChoice<ProjectType> projectTypeFilter = new AjaxCheckBoxMultipleChoice<ProjectType>("projectTypeFilter",
                    projectTypeService.findWithModule(ProjectModule.PEER),
                    projectTypeService.findWithModule(ProjectModule.PEER),
                    new ChoiceRenderer<ProjectType>("name", "id")) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    filter.setProjectTypes(getModelObject());
                    target.add(table);
                }
            };
            filter.setProjectTypes(projectTypeFilter.getModelObject());
            add(projectTypeFilter);
        }

        private PropertyModel<List<RequestStatus>> getRequestStatusModel() {
            filter.setStatuses(new ArrayList<>(asList(RequestStatus.values())));
            return new PropertyModel<>(filter, "statuses");
        }
    }
}
