package se.su.dsv.scipro.phd;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.*;
import se.su.dsv.scipro.components.BootstrapDateField;
import se.su.dsv.scipro.components.BootstrapDatePicker;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.DatesValidator;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.JavascriptEventConfirmation;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

@ProjectModuleComponent(ProjectModule.DEPARTMENT_SERVICE)
public class DepartmentServiceByUserPanel extends GenericPanel<User> {

    public static final String FEEDBACK = "Feedback";

    @Inject
    private DepartmentServiceService departmentServiceService;

    private Model<Boolean> allContainerVisible;

    public DepartmentServiceByUserPanel(String id, IModel<User> model) {
        super(id, model);
        add(new Label("user", new PropertyModel<>(model, "fullName")));
        addListWithCurrentAndFutureDepartmentServices();
        addListWithAllDepartmentServices();
        addSubmissionForm();
    }

    private void addListWithCurrentAndFutureDepartmentServices() {
        add(new DepartmentServiceList("current", getCurrentDepartmentServices()));
        add(new Label("noCurrent", new ResourceModel("no.current")) {
            @Override
            protected void onConfigure() {
                setVisible(getCurrentDepartmentServices().getObject().isEmpty());
            }
        });
    }

    private void addListWithAllDepartmentServices() {

        allContainerVisible = new Model<>(false);

        final WebMarkupContainer previous = new WebMarkupContainer("previous"){
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(allContainerVisible.getObject());
            }
        };
        previous.setOutputMarkupPlaceholderTag(true);

        add(previous);
        previous.add(new DepartmentServiceList("previousList", getPreviousDepartmentServices()));

        AjaxLink<Void> show = new AjaxLink<Void>("show") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                allContainerVisible.setObject(!allContainerVisible.getObject());
                target.add(previous);
                target.add(this);
            }

            @Override
            protected void onConfigure() {
                setVisible(!getPreviousDepartmentServices().getObject().isEmpty());
            }

            @Override
            public IModel<?> getBody() {
                if (allContainerVisible.getObject()){
                    return Model.of("Hide all previous services");
                }else {
                    return Model.of("Show all previous services");
                }
            }
        };
        show.setOutputMarkupId(true);
        add(show);
    }

    private void addSubmissionForm() {
        CreateForm form = new CreateForm("form");
        add(form);
        Button button = new Button("addButton");
        button.add(AttributeAppender.replace("onclick", "$(" + form.getMarkupId() + ").slideToggle();"));
        add(button);
    }

    private LoadableDetachableModel<List<DepartmentService>> getCurrentDepartmentServices() {
        return new LoadableDetachableModel<List<DepartmentService>>() {
            @Override
            protected List<DepartmentService> load() {
                return departmentServiceService.getCurrent(getModelObject());
            }
        };
    }

    private LoadableDetachableModel<List<DepartmentService>> getPreviousDepartmentServices() {
        return new LoadableDetachableModel<List<DepartmentService>>() {
            @Override
            protected List<DepartmentService> load() {
                return departmentServiceService.getPrevious(getModelObject());
            }
        };
    }

    private LoadableDetachableModel<DepartmentService> createDepartmentService() {
        return new LoadableDetachableModel<DepartmentService>() {
            @Override
            protected DepartmentService load() {
                return new DepartmentService(getModelObject());
            }
        };
    }

    private class CreateForm extends Form<DepartmentService> {

        public CreateForm(String id) {
            super(id, createDepartmentService());
            add(new RequiredTextField<>("hours", new PropertyModel<Integer>(getModel(), "hours"), Integer.class));
            add(new TextArea<>("description", new PropertyModel<>(getModel(), "description")));
            FormComponent<Date> dateStarted = addDateField("dateStarted");
            FormComponent<Date> dateEnded = addDateField("dateEnded");

            add(new DatesValidator(dateStarted, dateEnded));
        }

        private FormComponent<Date> addDateField(String id) {
            FormComponent<Date> date = new BootstrapDateField(id, new PropertyModel<Date>(getModel(), id));
            date.add(new BootstrapDatePicker());
            date.setRequired(true);
            add(new FencedFeedbackPanel(id + FEEDBACK, date));
            add(date);
            return date;
        }

        @Override
        protected void onSubmit() {
            departmentServiceService.save(getModelObject());
            this.detach();
            success(getString("saved"));
        }

        @Override
        protected void onComponentTag(ComponentTag tag) {
            super.onComponentTag(tag);
            if(!hasError()) {
                tag.append("style", "display: none;", ";");
            }
        }
    }

    private class DepartmentServiceList extends ListView<DepartmentService> {

        public DepartmentServiceList(String id, IModel<List<DepartmentService>> model) {
            super(id, model);
        }

        @Override
        protected void populateItem(ListItem<DepartmentService> item) {
            item.add(new DateLabel("dateStarted", new PropertyModel<Date>(item.getModel(), "dateStarted")));
            item.add(new DateLabel("dateEnded", new PropertyModel<Date>(item.getModel(), "dateEnded")));
            item.add(new Label("hours", new PropertyModel<>(item.getModel(), "hours")));
            item.add(new Label("description", new PropertyModel<>(item.getModel(), "description")));
            Link<DepartmentService> delete = new Link<DepartmentService>("delete", item.getModel()) {
                @Override
                public void onClick() {
                    departmentServiceService.delete(getModelObject());
                    DepartmentServiceByUserPanel.this.detach();
                    DepartmentServiceByUserPanel.this.success(getString("deleted"));
                }
            };
            item.add(delete.add(new JavascriptEventConfirmation("onclick", new ResourceModel("confirmDelete"))));
        }
    }
}
