package se.su.dsv.scipro.statistics;


import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.joda.time.DateTime;
import se.su.dsv.scipro.components.AjaxCheckBoxMultipleChoice;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.dataproviders.GenericDataProvider;
import se.su.dsv.scipro.date.DatePickerPanel;
import se.su.dsv.scipro.match.Idea;
import se.su.dsv.scipro.match.IdeaService;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.springdata.services.SupervisorService;
import se.su.dsv.scipro.springdata.services.UnitService;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.system.Unit;

import javax.inject.Inject;
import java.util.*;

public class AdminUnitStatisticsPanel extends Panel {

    public static final int MONTHS = 12;
    @Inject
    UnitService unitService;
    @Inject
    SupervisorService supervisorService;
    @Inject
    ProjectService projectService;
    @Inject
    IdeaService ideaService;
    @Inject
    ProjectTypeService projectTypeService;

    private ProjectService.Filter projectParams = new ProjectService.Filter();
    private final IdeaService.Filter ideaParams = new IdeaService.Filter();
    private ExportableDataPanel<Unit, String> table;

	public AdminUnitStatisticsPanel(String id) {
		super(id);
        addProjectTypesFilter();
        addDateFilter();
        addDataTable();
    }

    private void addDataTable() {
        SortableDataProvider<Unit, String> provider = new GenericDataProvider<>(unitService);
        provider.setSort("title", SortOrder.DESCENDING);
        table = new ExportableDataPanel<>(TABLE, createColumns(), provider);
        add(table);
    }

    private List<IColumn<Unit, String>> createColumns() {
        List<IColumn<Unit, String>> columns = new ArrayList<>();
        columns.add(new PropertyColumn<Unit, String>(Model.of("Unit"), "title", "title"));
        columns.add(newSupervisorCountColumn());
        columns.add(newActiveProjectsColumn());
        columns.add(newMatchedIdeasColumn());
        return columns;
    }

    private IColumn<Unit, String> newSupervisorCountColumn() {
        return new AbstractColumn<Unit, String>(Model.of("Supervisors")) {
            @Override
            public void populateItem(Item<ICellPopulator<Unit>> cellItem, String componentId, IModel<Unit> model) {
                SupervisorService.FilterParams params = new SupervisorService.FilterParams();
                params.setUnit(model.getObject());
                cellItem.add(new Label(componentId, String.valueOf(supervisorService.count(params))));
            }
        };
    }

    private IColumn<Unit, String> newActiveProjectsColumn() {
        return new AbstractColumn<Unit, String>(Model.of("Active projects")) {
            @Override
            public void populateItem(Item<ICellPopulator<Unit>> cellItem, String componentId, IModel<Unit> model) {
                projectParams.setUnit(model.getObject());
                cellItem.add(new Label(componentId, String.valueOf(projectService.count(projectParams))));
            }
        };
    }

    private IColumn<Unit, String> newMatchedIdeasColumn() {
        return new AbstractColumn<Unit, String>(Model.of("Matched ideas without projects")) {
            @Override
            public void populateItem(Item<ICellPopulator<Unit>> cellItem, String componentId, IModel<Unit> model) {
                ideaParams.setUnit(model.getObject());
                cellItem.add(new Label(componentId, String.valueOf(ideaService.count(ideaParams))));
            }
        };
    }

    private void addDateFilter() {
        Date countFrom = new DateTime().minusMonths(MONTHS).toDate();
        initParams(countFrom);
        add(new DatePickerPanel("dateFilter", countFrom) {
            @Override
            public void updateStartDate(AjaxRequestTarget target) {
                projectParams.setCreatedAfter(getStartDate());
                ideaParams.setCreatedAfter(getStartDate());
                target.add(table);
            }

            @Override
            public void updateEndDate(AjaxRequestTarget target) {
                projectParams.setCreatedBefore(getEndDate());
                ideaParams.setCreatedBefore(getEndDate());
                target.add(table);
            }
        });
    }

    private void addProjectTypesFilter() {
        AjaxCheckBoxMultipleChoice<ProjectType> projectTypeFilter = new AjaxCheckBoxMultipleChoice<ProjectType>("projectTypeFilter",
                        projectTypeService.findByDegreeLevels(Arrays.asList(DegreeLevel.FIRST_CYCLE, DegreeLevel.SECOND_CYCLE)),
                        projectTypeService.findAllActive(),
                        new ChoiceRenderer<ProjectType>("name", "id")) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                if(!getModelObject().isEmpty()) {
                    projectParams.setProjectTypes(getModelObject());
                    ideaParams.setLevels(getModelObject());
                }
                target.add(table);
            }
        };
        projectParams.setProjectTypes(projectTypeFilter.getModelObject());
        ideaParams.setLevels(projectTypeFilter.getModelObject());
        add(projectTypeFilter);
    }

    private void initParams(Date countFrom) {
        projectParams =  new ProjectService.Filter();
        projectParams.setStatuses(new HashSet<>(Collections.singletonList(ProjectStatus.ACTIVE)));
        projectParams.setCreatedAfter(countFrom);
        projectParams.setCreatedBefore(new Date());
        projectParams.setProjectTypes(projectTypeService.findAllActive());
        ideaParams.setStatuses(Collections.singletonList(Idea.Status.MATCHED));
        ideaParams.setCreatedAfter(countFrom);
        ideaParams.setCreatedBefore(new Date());
        ideaParams.setLevels(projectTypeService.findAllActive());
    }

    static final String TABLE = "dataTable";
}
