package se.su.dsv.scipro.supervisor.panels;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.AjaxCheckBoxMultipleChoice;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.components.datatables.MultipleUsersColumn;
import se.su.dsv.scipro.components.datatables.UserColumn;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.datatables.project.ProjectStateColumn;
import se.su.dsv.scipro.datatables.project.ProjectTitleColumn;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.project.ProjectTeamMemberRoles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.settings.dataobjects.UserProfile;
import se.su.dsv.scipro.springdata.services.UserProfileService;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

import static java.util.Arrays.asList;

public class SupervisorMyProjectsPanel extends Panel {

    public static final String FILTER_FORM = "filterForm";
    public static final String ROLE_FILTER = "roles";
    public static final String DATA_PANEL = "dp";
    public static final String STATUS_FILTER = "statuses";
    public static final String TYPE_FILTER = "projectTypes";
    public static final String SUPERVISOR = "filterSupervisor";

    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private ProjectService projectService;
    @Inject
    private UserProfileService profileService;

    private ExportableDataPanel dataPanel;
    private ProjectService.Filter filter = new ProjectService.Filter();

    public SupervisorMyProjectsPanel(String id) {
        super(id);
        dataViewSetup();
        prepareFilter();
        addFilterForm();
    }

    private void dataViewSetup() {
        SortableDataProvider<Project, String> provider = new FilteredDataProvider<>(projectService, Model.of(this.filter));
        provider.setSort("dateCreated", SortOrder.DESCENDING);
        dataPanel = new ExportableDataPanel<>(DATA_PANEL, createColumns(), provider, 100L);
        add(dataPanel);
    }

    private List<IColumn<Project, String>> createColumns() {
        List<IColumn<Project, String>> columns = new ArrayList<>();
        columns.add(new ProjectStateColumn(Model.of("State"), "stateOfMind"));
        columns.add(new DateColumn<Project>(Model.of("Started"), "dateCreated", "dateCreated"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Type"), "projectType.name", "projectType.name"));
        columns.add(new ProjectTitleColumn(Model.of("Title"), "title"));
        columns.add(new MultipleUsersColumn<Project, String>(Model.of("Authors")) {
            @Override
            public ListAdapterModel<User> getUsers(final IModel<Project> rowModel) {
                return new ListAdapterModel<>(new PropertyModel<Iterable<User>>(rowModel, "ProjectParticipants"));
            }
        });
        columns.add(new UserColumn<Project, String>(Model.of("Head supervisor")) {
            @Override
            protected IModel<User> getUser(final IModel<Project> rowModel) {
                return new PropertyModel<>(rowModel, "headSupervisor");
            }
        });
        return columns;
    }

    private void addFilterForm() {
        FilterForm filterForm = new FilterForm(FILTER_FORM, Model.of(filter));
        filterForm.setOutputMarkupPlaceholderTag(true);
        add(filterForm);
    }

    private void prepareFilter() {
        filter.setSupervisor(currentUser());

        loadDefaultFilter();
    }

    private void loadDefaultFilter() {
        UserProfile userProfile = currentProfile();
        filter.setStatuses(userProfile.getDefaultProjectStatusFilter());
        filter.setRoles(userProfile.getDefaultProjectTeamMemberRolesFilter());
        filter.setFilterSupervisor(userProfile.isDefaultSupervisorFilter());
        filter.setProjectTypes(userProfile.getDefaultProjectTypeFilter());
    }

    private User currentUser() {
        return SciProSession.get().getUser();
    }

    private UserProfile currentProfile() {
        return profileService.findByUser(currentUser());
    }

    private class FilterForm extends Form<ProjectService.Filter> {

        public FilterForm(String id, IModel<ProjectService.Filter> filterParams) {
            super(id, new CompoundPropertyModel<>(filterParams));

            add(new AjaxCheckBox(SUPERVISOR) {
                @Override
                protected void onUpdate(AjaxRequestTarget target) {
                    target.add(dataPanel);
                    updateProfileWithCurrentFilter();
                }
            });
            add(new AjaxCheckBoxMultipleChoice<ProjectTeamMemberRoles>(ROLE_FILTER, asList(ProjectTeamMemberRoles.values()), new EnumChoiceRenderer<ProjectTeamMemberRoles>(this)) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    target.add(dataPanel);
                    updateProfileWithCurrentFilter();
                }
            });
            add(new AjaxCheckBoxMultipleChoice<ProjectStatus>(STATUS_FILTER, asList(ProjectStatus.values()), new EnumChoiceRenderer<ProjectStatus>(this)) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    target.add(dataPanel);
                    updateProfileWithCurrentFilter();
                }
            });
            add(new AjaxCheckBoxMultipleChoice<ProjectType>(TYPE_FILTER, projectTypeService.findBySupervisorProjects(SciProSession.get().getUser()), new ChoiceRenderer<ProjectType>("name", "id")) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    target.add(dataPanel);
                    updateProfileWithCurrentFilter();
                }
            });
        }

        private void updateProfileWithCurrentFilter() {
            UserProfile userProfile = currentProfile();
            userProfile.setDefaultProjectStatusFilter(filter.getStatuses());
            userProfile.setDefaultProjectTeamMemberRolesFilter(filter.getRoles());
            userProfile.setDefaultSupervisorFilter(filter.isFilterSupervisor());
            userProfile.setDefaultProjectTypeFilter(filter.getProjectTypes());
            profileService.save(userProfile);
        }
    }
}
