package se.su.dsv.scipro.user;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.AjaxRequestAttributes;
import org.apache.wicket.ajax.attributes.ThrottlingSettings;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.time.Duration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.components.AjaxCheckBoxMultipleChoice;
import se.su.dsv.scipro.components.AjaxLinkPanel;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.datatables.ClickableIconColumn;
import se.su.dsv.scipro.notifications.NotificationService;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.HashService;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

@Authorization(authorizedRoles = Roles.ADMIN)
public class AdminInternalUsersPage extends AbstractAdminUsersPage {

    private static final Logger LOGGER = LoggerFactory.getLogger(AdminInternalUsersPage.class);

    public static final String REGISTRATION_LINK = "registrationLink";
    public static final String FEEDBACK = "feedback";
    public static final long DURATION = 500L;

    @Inject
    private UserService userService;
    @Inject
    private NotificationService notificationService;
    @Inject
    private HashService hashService;

    private final FeedbackPanel feedback;
    private final ExportableDataPanel<User, String> dp;
    private final UserService.Filter filter;

    public AdminInternalUsersPage() {
        add(new BookmarkablePageLink<Void>(REGISTRATION_LINK, AdminEditUserPage.class));
        add(new BookmarkablePageLink<Void>("importUsers", AdminImportUserPage.class));

        feedback = new FeedbackPanel(FEEDBACK);
        feedback.setOutputMarkupId(true);
        add(feedback);

        filter = new UserService.Filter();
        filter.setInternalUsersOnly(true);
        SortableDataProvider<User, String> provider = new FilteredDataProvider<>(userService, Model.of(filter));
	    provider.setSort("dateCreated", SortOrder.DESCENDING);

        addNameFilter();
        addRoleFilter();

        dp = new ExportableDataPanel<>("dp", createColumns(), provider);
        dp.setOutputMarkupId(true);
        add(dp);
    }

    private List<IColumn<User, String>> createColumns() {
        List<IColumn<User, String>> columns = new ArrayList<>();
        columns.add(new PropertyColumn<User, String>(Model.of("Last name"), "lastName", "lastName" ));
        columns.add(new PropertyColumn<User, String>(Model.of("First name"), "firstName", "firstName"));
        columns.add(new PropertyColumn<User, String>(Model.of("E-mail"), "emailAddress", "emailAddress"));
        columns.add(new DateColumn<User>(Model.of("Created"), "dateCreated", "dateCreated"));
        columns.add(new ClickableIconColumn<User>(Model.of("Edit"), null, ClickableIconColumn.EDIT) {
            @Override
            protected void onClick(IModel<User> clicked, AjaxRequestTarget target) {
                PageParameters parameters = new PageParameters();
                parameters.set(PageParameterKeys.MAP.get(User.class), clicked.getObject().getId());
                setResponsePage(AdminEditUserPage.class, parameters);
            }
        });

        columns.add(new AbstractColumn<User, String>(Model.of("Change password")) {
            @Override
            public void populateItem(Item<ICellPopulator<User>> cellItem, String componentId, IModel<User> rowModel) {
                cellItem.add(new AjaxLinkPanel<User>(componentId, rowModel, Model.of("Change password")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<User> model) {
                        PageParameters parameters = new PageParameters();
                        parameters.set(PageParameterKeys.MAP.get(User.class), model.getObject().getId());
                        setResponsePage(AdminChangePasswordPage.class, parameters);
                    }
                });
            }
        });

        columns.add(new ClickableIconColumn<User>(Model.of("Delete"), null, ClickableIconColumn.DELETE, "Are you sure?") {
            @Override
            protected void onClick(IModel<User> clicked, AjaxRequestTarget target) {
                try {
                    LOGGER.info("{} is deleting user: {}", SciProSession.get().getUser(), clicked.getObject());
                    notificationService.deleteAllForUser(clicked.getObject());
                    hashService.deleteAllHashesForUser(clicked.getObject());
                    userService.delete(clicked.getObject());
                    success(getString("deleteSuccess", clicked));
                } catch (Exception e) {
                    String deleteFailed = getString("deleteFailed");
                    error(deleteFailed);
                    LOGGER.info(deleteFailed, e);
                }
                target.add(feedback);
                target.add(dp);
            }
        });
        return columns;
    }

    private void addNameFilter() {
        TextField<String> searchField = new TextField<>("name", new PropertyModel<String>(filter, "name"));

        OnChangeAjaxBehavior userSearchTextBehaviour = new OnChangeAjaxBehavior() {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                target.add(dp);
            }

            @Override
            protected void updateAjaxAttributes(AjaxRequestAttributes attributes) {
                super.updateAjaxAttributes(attributes);
                attributes.setThrottlingSettings(new ThrottlingSettings(getId(), Duration.milliseconds(DURATION), true));
            }
        };

        searchField.add(userSearchTextBehaviour);
        add(searchField);
    }

    private void addRoleFilter() {
        add(new AjaxCheckBoxMultipleChoice<Roles>("role", new PropertyModel<Collection<Roles>>(filter, "roles"), Arrays.asList(Roles.values()), new EnumChoiceRenderer<Roles>(this)) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                target.add(dp);
            }
        });
    }
}
