package se.su.dsv.scipro.reviewer

import java.util
import javax.inject.Inject

import org.apache.wicket.AttributeModifier
import org.apache.wicket.markup.html.basic.Label
import org.apache.wicket.markup.html.link.ExternalLink
import org.apache.wicket.markup.html.list.{ListItem, ListView}
import org.apache.wicket.model._
import org.apache.wicket.request.mapper.parameter.PageParameters
import se.su.dsv.scipro.components.{InfoPanel, OppositeVisibility}
import se.su.dsv.scipro.data.DetachableServiceModel
import se.su.dsv.scipro.file.{FileDescription, FileDescriptionService}
import se.su.dsv.scipro.finalthesis.{FinalThesis, FinalThesisService}
import se.su.dsv.scipro.forum.panels.threaded.SubmitForumReplyPanel
import se.su.dsv.scipro.grading.{CriteriaPanel, ReviewerFillOutGradingReportPanel}
import se.su.dsv.scipro.project.panels.ProjectHeaderPanel
import se.su.dsv.scipro.project.{Project, ProjectService}
import se.su.dsv.scipro.report.GradingReport.State
import se.su.dsv.scipro.report.{GradingReportService, SupervisorGradingReport}
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel
import se.su.dsv.scipro.reviewer.timeline.TimelinePanel
import se.su.dsv.scipro.reviewing.ReviewerInteractionService
import se.su.dsv.scipro.session.SciProSession
import se.su.dsv.scipro.system.User
import se.su.dsv.scipro.util.PageParameterKeys

import scala.util.Try

class ReviewerGradingPage(pp: PageParameters) extends ReviewerPage(pp) {
  @Inject private var projectService: ProjectService = _
  @Inject private var reviewerInteractionService: ReviewerInteractionService = _
  @Inject private var finalThesisService: FinalThesisService = _
  @Inject private var gradingReportService: GradingReportService = _
  @Inject private var fileDescriptionService: FileDescriptionService = _

  val maybeProjectModel = for {
    id <- Try(pp.get(PageParameterKeys.MAP.get(classOf[Project])).toLong).toOption
    project <- Option(projectService.findOne(id))
    if project.isReviewer(SciProSession.get.getUser)
  } yield new DetachableServiceModel(projectService, project)

  maybeProjectModel.map(initPage).getOrElse(setResponsePage(getApplication.getHomePage))

  def initPage(projectModel: IModel[Project]): Unit = {
    add(new Label("projectTitle", new PropertyModel[String](projectModel, "title")))
    add(new ProjectHeaderPanel("projectHeader", projectModel))
    add(new ViewAttachmentPanel("thesisFile", new PropertyModel[FileDescription](getFinalThesis(projectModel), "fileDescription")))
    add(new ReviewerFillOutGradingReportPanel("fillOutPanel", projectModel))
    add(new InfoPanel("info", new ResourceModel("info")))
    add(new SubmitForumReplyPanel[Project]("communication", projectModel, new ReviewerInteractionForumThread(reviewerInteractionService, fileDescriptionService)))
    add(new TimelinePanel("timeline", projectModel))
    addSupervisorReports(projectModel)
    ()
  }

  def addSupervisorReports(projectModel: IModel[Project]): Unit = {
    val supervisorReportsModel = new LoadableDetachableModel[util.List[SupervisorGradingReport]] {
      override def load = gradingReportService.getSupervisorGradingReports(projectModel.getObject)
    }
    add(new ListView[SupervisorGradingReport]("supervisorReportTabs", supervisorReportsModel) {
      override def populateItem(item: ListItem[SupervisorGradingReport]): Unit = {
        val author = new PropertyModel[User](item.getModel, "user")
        val tabTitle = new StringResourceModel("supervisor.grading.report", author)
        item.add(new ExternalLink("tab", Model.of("#author" + item.getIndex), tabTitle))
      }
    })
    add(new ListView[SupervisorGradingReport]("supervisorReports", supervisorReportsModel) {
      override def populateItem(item: ListItem[SupervisorGradingReport]): Unit = {
        item.add(AttributeModifier.replace("id", "author" + item.getIndex))

        val criteriaPanel = new CriteriaPanel("supervisorReport", item.getModel) {
          override def onConfigure() = setVisible(reviewerHasDoneInitialAssessment && supervisorHasDoneInitialAssessment)

          private def reviewerHasDoneInitialAssessment =
            gradingReportService.getReviewerGradingReport(projectModel.getObject).getState != State.INITIAL

          private def supervisorHasDoneInitialAssessment = item.getModelObject.getState != State.INITIAL
        }
        item.add(criteriaPanel)
        item.add(new Label("initialAssessmentNotDone", new ResourceModel("initial.assessment.not.done"))
          .add(new OppositeVisibility(criteriaPanel)))
      }
    })
    ()
  }

  def getFinalThesis(projectModel: IModel[Project]): IModel[FinalThesis] = {
    new LoadableDetachableModel[FinalThesis] {
      override protected def load: FinalThesis = finalThesisService.findByProject(projectModel.getObject)
    }
  }
}
