package se.su.dsv.scipro.activityplan;

import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.checklist.Checklist;
import se.su.dsv.scipro.checklists.ProjectViewChecklistPage;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Date;

public class ActivityPlanPanelTest extends SciProTest {

    private Project project;
    private ActivityPlanPanel panel;

    @Before
    public void setUp() throws Exception {
        project = Project.builder().title("My project").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor")).headSupervisor(new User()).build();
        project.setId(20934L);
    }

    @Test
    public void renders() throws Exception {
        startPanel();
    }

    @Test
    public void renders_with_activity_plans() throws Exception {
        mockExistingEvents();

        startPanel();
    }

    @Test
    public void works_for_all_action_types() throws Exception {
        for (Action action : Action.values()) {
            Activity activity = createActivity();
            activity.setAction(action);

            mockExistingEvents(activity);

            startPanel();
        }
    }

    @Test
    public void clicking_delete_deletes_the_event() throws Exception {
        Activity activity = createActivity();

        mockExistingEvents(activity);

        startPanel();

        deleteSchedule(1);

        Mockito.verify(activityPlanFacade).deleteActivity(activity);
    }

    @Test
    public void deleting_event_refreshes_table() throws Exception {
        mockExistingEvents();

        startPanel();

        deleteSchedule(1);

        tester.assertComponentOnAjaxResponse(path(panel, "table"));
    }

    @Test
    public void if_event_has_no_checklist_link_should_be_disabled() throws Exception {
        mockExistingEvents();

        startPanel();

        tester.assertDisabled(checklistLink());
    }

    @Test
    public void checklist_link() throws Exception {
        Checklist checklist = Checklist.builder().name("Waynes checklist").project(project).build();
        checklist.setId(7856L);
        Activity activity = createActivity();
        activity.setChecklist(checklist);
        mockExistingEvents(activity);
        startPanel();
        PageParameters pageParameters = new PageParameters();
        pageParameters.set(PageParameterKeys.MAP.get(Checklist.class), checklist.getId());
        tester.assertBookmarkablePageLink(checklistLink(), ProjectViewChecklistPage.class, pageParameters );
    }

    private String checklistLink() {
        return path(panel, "table", "body", "rows", 1, "cells", 4, "cell", "checklistLink");
    }

    private void deleteSchedule(int row) {
        tester.clickLink(path(panel, "table", "body", "rows", row, "cells", 6, "cell", "link"));
    }

    private void mockExistingEvents() {
        mockExistingEvents(createActivity());
    }

    private void mockExistingEvents(Activity... events) {
        Mockito.when(activityPlanFacade.findEventsByProject(Matchers.eq(project), Matchers.any(Pageable.class)))
                .thenReturn(Arrays.asList(events));
        Mockito.when(activityPlanFacade.countEventsByProject(Matchers.eq(project), Matchers.any(Date.class), Matchers.any(Date.class)))
                .thenReturn((long) events.length);
    }

    private Activity createActivity() {
        ActivityPlan activityPlan = new ActivityPlan();
        activityPlan.setProject(project);
        Activity activity = new Activity();
        activity.setActivityPlan(activityPlan);
        return activity;
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new ActivityPlanPanel("id", Model.of(project)));
    }
}
