package se.su.dsv.scipro.admin.pages;

import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.behavior.AbstractAjaxBehavior;
import org.apache.wicket.behavior.Behavior;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.tester.FormTester;
import org.apache.wicket.util.tester.WicketTesterHelper;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.components.AddUsersPanel;
import se.su.dsv.scipro.components.SupervisorAutoComplete;
import se.su.dsv.scipro.mail.MailEvent;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.admin.pages.AdminEditProjectPage.*;

public class AdminEditProjectPageTest extends SciProTest {

    public static final long AUTHOR_ID = 1L;
    public static final long SUPERVISOR_ID = 2L;
    public static final long SOME_EMPLOYEE_ID = 666L;
    public static final boolean PROJECT_TYPE = true;
    public static final boolean NO_PROJECT_TYPE = false;
    private static final Long ANOTHER_AUTHOR_ID = 8L;
    public static final long EXISTING_CO_SUPERVISOR_ID = 11L;
    public static final long EXISTING_REVIEWER_ID = 12L;
    public static final long EXISTING_PARTICIPANT_ID = 13L;

    private User supervisor;
    private User someEmployee;
    private User author;
    private Project project;
    private User existingCoSupervisor;
    private User existingReviewer;
    private User existingParticipant;

    @Before
    public void setUp() throws Exception {
        when(projectTypeService.findAllActive()).thenReturn(Collections.singletonList(ObjectMother.BACHELOR));
        author = createUser("author", "user", "author@example.com", AUTHOR_ID);
        author.addRole(Roles.AUTHOR);
        someEmployee = createUser("some", "employee", "some@employee.com", SOME_EMPLOYEE_ID);
        someEmployee.addRole(Roles.SUPERVISOR);
        someEmployee.addRole(Roles.REVIEWER);
        supervisor = createUser("supervisor", "user", "author@example.com", SUPERVISOR_ID);
        supervisor.addRole(Roles.SUPERVISOR);
        supervisor.addRole(Roles.REVIEWER);
        User anotherAuthor = createUser("another", "author", "another@example.com", ANOTHER_AUTHOR_ID);
        anotherAuthor.addRole(Roles.AUTHOR);

        existingCoSupervisor = createUser("pelle", "svensson", "pelle@example.com", EXISTING_CO_SUPERVISOR_ID);
        existingCoSupervisor.addRole(Roles.SUPERVISOR);

        existingReviewer = createUser("stina", "svensson", "stina@example.com", EXISTING_REVIEWER_ID);
        existingReviewer.addRole(Roles.REVIEWER);
        existingReviewer.addRole(Roles.SUPERVISOR);

        existingParticipant = createUser("kalle", "svensson", "kalle@example.com", EXISTING_PARTICIPANT_ID);
        existingParticipant.addRole(Roles.AUTHOR);

        when(userRepo.findOne(AUTHOR_ID)).thenReturn(author);
        when(userRepo.findOne(SUPERVISOR_ID)).thenReturn(supervisor);
        when(userRepo.findOne(SOME_EMPLOYEE_ID)).thenReturn(someEmployee);
        when(userRepo.findOne(ANOTHER_AUTHOR_ID)).thenReturn(anotherAuthor);

        when(userRepo.findOne(EXISTING_REVIEWER_ID)).thenReturn(existingReviewer);
        when(userRepo.findOne(EXISTING_CO_SUPERVISOR_ID)).thenReturn(existingCoSupervisor);
        when(userRepo.findOne(EXISTING_PARTICIPANT_ID)).thenReturn(existingParticipant);

        project = new Project();
        project.addProjectParticipant(author);
        project.setId(20987234L);
        startPage(project);
    }

    @Test
    public void components() throws Exception {
        startPage(mockExistingProject());
        tester.assertComponent(path(FORM, FEEDBACK), FeedbackPanel.class);
        tester.assertComponent(path(FORM, TITLE_FIELD), RequiredTextField.class);
        tester.assertComponent(path(FORM, PROJECT_TYPE_SELECT), DropDownChoice.class);
        tester.assertComponent(path(FORM, HEAD_SUPERVISOR_SELECT), SupervisorAutoComplete.class);

        tester.assertComponent(path(FORM, AUTHORS), AddUsersPanel.class);
        tester.assertComponent(path(FORM, REVIEWERS), AddUsersPanel.class);
        tester.assertComponent(path(FORM, CO_SUPERVISORS), AddUsersPanel.class);
    }

    @Test
    public void title_required() throws Exception {
        submitForm("", SUPERVISOR_ID, PROJECT_TYPE, AUTHOR_ID, null, null);
        tester.assertErrorMessages(tester.getLastRenderedPage().getString("titleField.Required"));
    }

    @Test
    public void project_type_required() throws Exception {
        submitForm("title", SUPERVISOR_ID, NO_PROJECT_TYPE, AUTHOR_ID, null, null);
        tester.assertErrorMessages(tester.getLastRenderedPage().getString("projectTypeSelect.Required"));
    }

    @Test
    public void head_supervisor_required() throws Exception {
        submitForm("title", null, PROJECT_TYPE, AUTHOR_ID, null, null);
        tester.assertErrorMessages(tester.getLastRenderedPage().getString("headSupervisorSelect.Required"));
    }

    @Test
    public void author_required() throws Exception {
        startPage(new Project());
        submitForm("title", SUPERVISOR_ID, PROJECT_TYPE, null, null, null);
        tester.assertErrorMessages(tester.getLastRenderedPage().getString("authorRequired"));
    }

    @Test
    public void not_same_user_head_reviewer() throws Exception {
        project.addReviewer(someEmployee);
        startPage(project);
        submitForm("title", SOME_EMPLOYEE_ID, PROJECT_TYPE, AUTHOR_ID, null, null);
        tester.assertErrorMessages(tester.getLastRenderedPage().getString("headReviewerSame"));
    }

    @Test
    public void not_same_user_head_co() throws Exception {
        project.addCoSupervisor(someEmployee);
        startPage(project);
        submitForm("title", SOME_EMPLOYEE_ID, PROJECT_TYPE, AUTHOR_ID, null, null);
        tester.assertErrorMessages(tester.getLastRenderedPage().getString("headCoSame"));
    }

    @Test
    public void not_same_user_head_author() throws Exception {
        submitForm("title", AUTHOR_ID, PROJECT_TYPE, AUTHOR_ID, null, null);
        tester.assertErrorMessages(tester.getLastRenderedPage().getString("headAuthorSame"));
    }

    @Test
    public void not_same_user_reviewer_co_supervisor() throws Exception {
        project.addCoSupervisor(someEmployee);
        project.addReviewer(someEmployee);
        startPage(project);
        submitForm("title", SUPERVISOR_ID, PROJECT_TYPE, AUTHOR_ID, null, null);
        tester.assertErrorMessages(tester.getLastRenderedPage().getString("reviewerCoSame"));
    }

    @Test
    public void not_same_user_reviewer_author() throws Exception {
        author.addRole(Roles.REVIEWER);
        project.addReviewer(author);
        startPage(project);
        submitForm("title", SUPERVISOR_ID, PROJECT_TYPE, AUTHOR_ID, null, null);
        tester.assertErrorMessages(tester.getLastRenderedPage().getString("reviewerAuthorSame"));
    }

    @Test
    public void not_same_user_co_author() throws Exception {
        project.addCoSupervisor(author);
        startPage(project);
        submitForm("title", SUPERVISOR_ID, PROJECT_TYPE, AUTHOR_ID, null, null);
        tester.assertErrorMessages(tester.getLastRenderedPage().getString("authorCoSame"));
    }

    @Test
    public void saves() throws Exception {
        String title = "cool title";
        submitForm(title, SUPERVISOR_ID, PROJECT_TYPE, AUTHOR_ID, null, null);
        ArgumentCaptor<Project> captor = ArgumentCaptor.forClass(Project.class);
        verify(projectService).save(captor.capture());

        tester.assertNoErrorMessage();
        assertEquals(title, captor.getValue().getTitle());
    }

    @Test
    public void sends_created_notification_on_new_projects() throws Exception {
        startPage(new Project());
        submitForm("title", SUPERVISOR_ID, PROJECT_TYPE, AUTHOR_ID, null, null);

        verifyNotification(ProjectEvent.Event.CREATED);
    }

    @Test
    public void sends_notification_on_head_supervisor_change() throws Exception {
        startPage(mockExistingProject());
        submitForm("title", SOME_EMPLOYEE_ID, PROJECT_TYPE, null, null, null);
        verifyNotification(ProjectEvent.Event.HEAD_SUPERVISOR_CHANGED);
        verify(mailEventService).save(any(MailEvent.class));
    }

    @Test
    public void sends_notification_when_authors_changed() throws Exception {
        startPage(mockExistingProject());
        submitForm("title", SUPERVISOR_ID, PROJECT_TYPE, ANOTHER_AUTHOR_ID, null, null);
        verifyNotification(ProjectEvent.Event.AUTHORS_CHANGED);
    }

    @Test
    public void sends_notification_when_reviewers_change() throws Exception {
        startPage(mockExistingProject());
        submitForm("title", SUPERVISOR_ID, PROJECT_TYPE, ANOTHER_AUTHOR_ID, SOME_EMPLOYEE_ID, null);
        verifyNotification(ProjectEvent.Event.REVIEWERS_CHANGED);
    }

    @Test
    public void sends_notification_when_co_supervisors_change() throws Exception {
        startPage(mockExistingProject());
        submitForm("title", SUPERVISOR_ID, PROJECT_TYPE, ANOTHER_AUTHOR_ID, null, SOME_EMPLOYEE_ID);
        verifyNotification(ProjectEvent.Event.CO_SUPERVISOR_CHANGED);
    }

    private void verifyNotification(ProjectEvent.Event event) {
        ArgumentCaptor<Project> argumentCaptor = ArgumentCaptor.forClass(Project.class);
        verify(projectService).save(argumentCaptor.capture());
        verify(notificationController).notifyProject(eq(argumentCaptor.getValue()), eq(event), any(NotificationSource.class));
    }

    private Project mockExistingProject() {
        Project project = Project.builder().title("a nice title").projectType(ObjectMother.BACHELOR).build();

        project.addCoSupervisor(existingCoSupervisor);

        project.addReviewer(existingReviewer);
        project.setHeadSupervisor(supervisor);

        project.addProjectParticipant(existingParticipant);
        project.setId(777L);



        return project;
    }

    private void submitForm(String title, Long supervisorId, boolean submitProjectType, Long authorId, Long reviewerId, Long coSupervisorId) {
        FormTester formTester = tester.newFormTester(path(FORM));
        if (authorId != null) {
            addAuthor(formTester, authorId);
        }
        if(reviewerId != null) {
            addFollower(formTester, reviewerId, REVIEWERS);
        }
        if(coSupervisorId != null) {
            addFollower(formTester, coSupervisorId, CO_SUPERVISORS);
        }
        if (supervisorId != null) {
            formTester.setValue(HEAD_SUPERVISOR_SELECT, String.valueOf(supervisorId));
        }
        if (submitProjectType) {
            formTester.select(PROJECT_TYPE_SELECT, 0);
        }
        formTester.setValue(TITLE_FIELD, title);
        formTester.submit();
    }

    private void addFollower(FormTester formTester, Long reviewerId, String list) {
        formTester.setValue(path(list, "wmc", "add"), String.valueOf(reviewerId));
        Behavior behavior = WicketTesterHelper.findBehavior(tester.getLastRenderedPage().get(path(FORM, list, "wmc", "add")), AjaxFormComponentUpdatingBehavior.class);
        tester.executeBehavior((AbstractAjaxBehavior) behavior);
    }

    private void addAuthor(FormTester formTester, long authorId) {
        formTester.setValue(path(AUTHORS, "wmc", "add"), String.valueOf(authorId));
        Behavior behavior = WicketTesterHelper.findBehavior(tester.getLastRenderedPage().get(path(FORM, AUTHORS, "wmc", "add")), AjaxFormComponentUpdatingBehavior.class);
        tester.executeBehavior((AbstractAjaxBehavior) behavior);
    }

    private User createUser(String firstName, String lastName, String emailAddress, long id) {
        User user = User.builder().firstName(firstName).lastName(lastName).emailAddress(emailAddress).build();
        user.setId(id);
        return user;
    }

    private void startPage(Project project) {
        when(projectService.findOne(project.getId())).thenReturn(project);
        PageParameters pp = new PageParameters();
        pp.set(PageParameterKeys.MAP.get(Project.class), project.getId());
        tester.startPage(AdminEditProjectPage.class, pp);
    }
}
