package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.border.Border;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.tester.FormTester;
import org.joda.time.DateTime;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Matchers;
import org.mockito.Mock;
import org.mockito.Mockito;
import scala.Option;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.grading.FillOutReportPanel;
import se.su.dsv.scipro.grading.Grade;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.pages.ProjectDetailsPage;
import se.su.dsv.scipro.report.GradingCriterionPointTemplate;
import se.su.dsv.scipro.report.GradingReportTemplate;
import se.su.dsv.scipro.report.OppositionReport;
import se.su.dsv.scipro.report.ReportService;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static se.su.dsv.scipro.finalseminar.OppositionReportPage.FILL_OUT_REPORT;
import static se.su.dsv.scipro.finalseminar.OppositionReportPage.THESIS_SUMMARY;

public class OppositionReportPageTest extends SciProTest {

    public static final String CRITERION_DESCRIPTION = "For 1 point: Be nice to your supervisor";
    public static final String CRITERTION_TITLE = "U1 Sammanfattning";

    @Mock
    private ReportService reportService;

    private FinalSeminarOpposition finalSeminarOpposition;
    private User user;

    @Before
    public void prepareOppositionReport() throws Exception {
        ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "bachelor", "bachelor");
        Project opponentsProject = Project.builder().title("Foo").projectType(bachelor).headSupervisor(User.builder().firstName("Bob").lastName("Kelso").emailAddress("bob@example.com").build()).build();
        Project projectToOppose = Project.builder().title("Foo").projectType(bachelor).headSupervisor(User.builder().firstName("Bob").lastName("Kelso").emailAddress("bob@example.com").build()).build();
        DomainObjects.injectId(opponentsProject, 666L);
        DomainObjects.injectId(projectToOppose, 777L);
        GradingReportTemplate reportTemplate = createTemplate(bachelor);
        user = new UserBuilder().create();
        FinalSeminar finalSeminar = createFinalSeminar(projectToOppose);
        finalSeminarOpposition = createOpposition(opponentsProject, finalSeminar);

        OppositionReport oppositionReport = reportTemplate.createOppositionReport(finalSeminarOpposition);
        Mockito.when(oppositionReportService.findOrCreateReport(finalSeminarOpposition)).thenReturn(oppositionReport);
        Mockito.when(projectService.findOne(Matchers.anyLong())).thenReturn(opponentsProject);
        Mockito.when(projectService.isPartOfProject(eq(opponentsProject), Matchers.any(User.class))).thenReturn(true);
        Mockito.when(finalSeminarService.findByProject(opponentsProject)).thenReturn(finalSeminar);
        Mockito.when(daysService.workDaysAfter(Matchers.any(Date.class), Matchers.anyInt())).thenReturn(new Date());
        Mockito.when(gradeService.getGrade(any(Project.class))).thenReturn(Option.<Grade>empty());
    }

    @Test
    public void renders() throws Exception {
        startPage();
    }

    @Test
    public void redirect_if_no_opposition_id() throws Exception {
        startPage();
        tester.assertRenderedPage(ProjectDetailsPage.class);
    }

    @Test
    public void correct_page_renders_with_existing_opposition() throws Exception {
        startOppositionPage();
        tester.assertRenderedPage(OppositionReportPage.class);
    }

    @Test
    public void disable_form_if_opposition_does_not_belong_to_logged_in_user() throws Exception {
        long oppositionId = 4L;
        Mockito.when(finalSeminarOppositionRepo.findOne(oppositionId)).thenReturn(finalSeminarOpposition);
        startPage(oppositionId);
        tester.assertDisabled(FILL_OUT_REPORT);
    }

    @Test
    public void redirect_if_no_opposition_is_found_from_id() throws Exception {
        Mockito.when(finalSeminarOppositionRepo.findOne(Matchers.anyLong())).thenReturn(null);
        startPage(1L);
        tester.assertRenderedPage(ProjectDetailsPage.class);
    }

    @Test
    public void opposition_report_has_thesis_summary_text_area() throws Exception {
        startOppositionPage();
        tester.assertComponent(path(FILL_OUT_REPORT, FillOutReportPanel.FORM, FILL_OUT_REPORT + "_" + Border.BODY, THESIS_SUMMARY), TextArea.class);
    }

    @Test
    public void submitting_form_saves_thesis_summary() throws Exception {
        startOppositionPage();

        String summary = "my summary";

        FormTester formTester = tester.newFormTester(path(FILL_OUT_REPORT, FillOutReportPanel.FORM));
        formTester.setValue(path(FILL_OUT_REPORT + "_" + Border.BODY, THESIS_SUMMARY), summary);
        formTester.submit();

        ArgumentCaptor<OppositionReport> captor = ArgumentCaptor.forClass(OppositionReport.class);
        Mockito.verify(reportService).save(captor.capture(), eq(Option.<FileUpload>empty()));

        Assert.assertEquals(summary, captor.getValue().getThesisSummary());
    }

    private void startPage(long oppositionId) {
        PageParameters pp = getParameters();
        pp.add("oid", oppositionId);
        tester.startPage(OppositionReportPage.class, pp);
    }

    private void startPage() {
        tester.startPage(OppositionReportPage.class, getParameters());
    }

    private void startOppositionPage() {
        long oppositionId = 4L;
        setLoggedInAs(user);
        Mockito.when(finalSeminarOppositionRepo.findOne(oppositionId)).thenReturn(finalSeminarOpposition);
        startPage(oppositionId);
    }

    private PageParameters getParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.set(PageParameterKeys.MAP.get(Project.class), 1L);
        return pageParameters;
    }

    private GradingReportTemplate createTemplate(ProjectType bachelor) {
        GradingReportTemplate reportTemplate = new GradingReportTemplate(bachelor, 30);
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();
        gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(1).description(CRITERION_DESCRIPTION).build());
        reportTemplate.addProjectCriterion(CRITERTION_TITLE, 1, gradingCriterionPointTemplates);
        return reportTemplate;
    }

    private FinalSeminarOpposition createOpposition(Project project, FinalSeminar finalSeminar) {
        FinalSeminarOpposition finalSeminarOpposition = new FinalSeminarOpposition();
        finalSeminarOpposition.setUser(user);
        finalSeminarOpposition.setFinalSeminar(finalSeminar);
        finalSeminarOpposition.setProject(project);
        return finalSeminarOpposition;
    }

    private FinalSeminar createFinalSeminar(Project project) {
        FinalSeminar finalSeminar = new FinalSeminar();
        finalSeminar.setProject(project);
        finalSeminar.setDocument(createFileDescription());
        finalSeminar.setStartDate(new DateTime().plusDays(1).toDate());
        return finalSeminar;
    }

    private FileDescription createFileDescription() {
        FileDescription fileDescription = new FileDescription();
        fileDescription.setId(1L);
        fileDescription.setName("title");
        return fileDescription;
    }
}
