package se.su.dsv.scipro.grading;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.RadioChoice;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.model.LoadableDetachableModel;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.*;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.ArrayList;
import java.util.List;

import static se.su.dsv.scipro.grading.CriteriaPanel.*;

public class CriteriaPanelTest extends SciProTest {

    public static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
    public static final Project SOME_PROJECT = Project.builder().title("Grading report project").projectType(BACHELOR).build();
    public static final User SOME_USER = new UserBuilder().setFirstName("Author").setLastName("One").create();
    public static final User SOME_USER_2 = new UserBuilder().setFirstName("Author").setLastName("two").create();

    private CriteriaPanel panel;
    private SupervisorGradingReport gradingReport;

    static {
        setUpProject();
    }

    @Before
    public void setUp() throws Exception {
        prepareReports();
    }

    @Test
    public void lists_all_grading_criteria() throws Exception {
        startPanel();

        tester.assertListView(path(CRITERIA), gradingReport.getGradingCriteria());
    }

    @Test
    public void displays_max_points_for_all_criteria() throws Exception {
        startPanel();

        tester.assertVisible(path(CRITERIA, 0, MAX_POINTS));
        tester.assertComponent(path(CRITERIA, 0, MAX_POINTS), Label.class);
    }

    @Test
    public void displays_required_points_for_all_criteria() throws Exception {
        startPanel();

        tester.assertLabel(path(CRITERIA, 0, MAX_POINTS), "(*1* is required to pass)");
        tester.assertLabel(path(CRITERIA, 1, MAX_POINTS), "(*0* is required to pass)");
    }

    @Test
    public void displays_title_for_all_criteria() throws Exception {
        startPanel();

        tester.assertLabel(path(CRITERIA, 0, TITLE), gradingReport.getGradingCriteria().get(0).getTitle());
    }

    @Test
    public void has_number_text_field_for_all_criteria() throws Exception {
        startPanel();

        tester.assertComponent(path(CRITERIA, 0, POINTS), RadioChoice.class);
    }

    @Test
    public void has_assessment_text_area_for_all_criteria() throws Exception {
        startPanel();

        tester.assertComponent(path(CRITERIA, 0, FEEDBACK), TextArea.class);
    }

    @Test
    public void shows_existing_comment() throws Exception {
        String comment = "comment";

        for (GradingCriterion gradingCriterion : gradingReport.getGradingCriteria()) {
            gradingCriterion.setFeedback(comment);
        }

        startPanel();

        tester.assertModelValue(path(CRITERIA, 0, FEEDBACK), comment);
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new CriteriaPanel("id", new LoadableDetachableModel<GradingReport>() {
            @Override
            protected GradingReport load() {
                return gradingReport;
            }
        }));
    }

    @Override
    protected String path(Object... parts) {
        return super.path(panel, super.path(parts));
    }

    private static void setUpProject() {
        SOME_PROJECT.setHeadSupervisor(new UserBuilder().setFirstName("Head").setLastName("Supervisor").create());
        SOME_PROJECT.addProjectParticipant(SOME_USER);
        SOME_PROJECT.addProjectParticipant(SOME_USER_2);
        SOME_PROJECT.addReviewer(new UserBuilder().create());
    }

    private void prepareReports() {
        GradingReportTemplate gradingReportTemplate = new GradingReportTemplate(BACHELOR, 30);
        gradingReportTemplate.addProjectCriterion("project criterion", 1, getPointTemplates(2));
        gradingReportTemplate.addIndividualCriterion("individual criterion", 0, getPointTemplates(2));
        gradingReport = gradingReportTemplate.createSupervisorReport(SOME_PROJECT, SOME_USER);
        for (GradingCriterion gradingCriterion : gradingReport.getGradingCriteria()) {
            gradingCriterion.setPoints(1);
            gradingCriterion.setFeedback("Feedback");
        }
        gradingReport.setPlagiarismFeedback("feedback");
    }

    private List<GradingCriterionPointTemplate> getPointTemplates(int maxPoint) {
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();
        for (int i = 0; i <= maxPoint; i++) {
            gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(i).description("Description").build());
        }
        return gradingCriterionPointTemplates;
    }
}
