package se.su.dsv.scipro.grading;

import org.apache.wicket.markup.html.form.SubmitLink;
import org.apache.wicket.model.Model;
import org.apache.wicket.protocol.http.mock.MockHttpServletRequest;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import scala.Option;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.finalseminar.FinalSeminar;
import se.su.dsv.scipro.finalseminar.FinalSeminarOpposition;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.*;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.grading.FillOutReportPanel.*;

public class FillOutReportPanelTest extends SciProTest {

    private OppositionReport oppositionReport;
    private FillOutReportPanel panel;

    @Mock
    private ReportService reportService;

    @Before
    public void setUp() throws Exception {
        FinalSeminar finalSeminar = new FinalSeminar();
        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING);
        User headSupervisorUser = new UserBuilder().create();
        Project project = Project.builder().title("title").projectType(projectType).headSupervisor(headSupervisorUser).build();
        finalSeminar.setProject(project);
        GradingReportTemplate template = new GradingReportTemplate(projectType, 30);
        template.addProjectCriterion("U1", 1, getPointTemplates(1));
        FinalSeminarOpposition finalSeminarOpposition = new FinalSeminarOpposition();
        finalSeminarOpposition.setFinalSeminar(finalSeminar);
        finalSeminarOpposition.setProject(project);
        oppositionReport = template.createOppositionReport(finalSeminarOpposition);

        startPanel();
    }

    private List<GradingCriterionPointTemplate> getPointTemplates(int maxPoint) {
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();
        for (int i = 0; i <= maxPoint; i++) {
            gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(i).description("Description").build());
        }
        return gradingCriterionPointTemplates;
    }

    @Test
    public void has_criteria_list() throws Exception {
        tester.assertListView(path(panel, FORM, GRADING_CRITERIA), oppositionReport.getCriteria());
    }

    @Test
    public void displays_criteria_titles() throws Exception {
        tester.assertModelValue(path(panel, FORM, GRADING_CRITERIA, 0, TITLE), oppositionReport.getCriteria().get(0).getTitle());
    }

    @Test
    public void displays_criteria_descriptions() throws Exception {
        tester.assertModelValue(path(panel, FORM, GRADING_CRITERIA, 0, DESCRIPTION), oppositionReport.getCriteria().get(0).getDescription());
    }

    @Test
    public void has_text_area_for_feedback() throws Exception {
        tester.assertModelValue(path(panel, FORM, GRADING_CRITERIA, 0, FEEDBACK), oppositionReport.getCriteria().get(0).getFeedback());
    }

    @Test
    public void has_scrolling_save_button() throws Exception {
        tester.assertComponent(path(panel, SAVE), ScrollingSaveButtonPanel.class);
    }

    @Test
    public void submitting_form_saves_report() throws Exception {
        String feedback = "new feedback";
        submitForm(feedback);

        verify(reportService).save(oppositionReport, Option.<FileUpload>empty());
        assertEquals(feedback, oppositionReport.getCriteria().get(0).getFeedback());
    }

    @Test
    public void has_submit_button() throws Exception {
        tester.assertComponent(path(panel, FORM, FillOutReportPanel.SUBMIT), SubmitLink.class);
    }

    @Test
    public void submit_button_enabled_after_filling_out_whole_report() throws Exception {
        submitForm("feedback");
        tester.assertEnabled(path(panel, FORM, FillOutReportPanel.SUBMIT));
    }

    @Test
    public void submit_button_submits_report() throws Exception {
        submitForm("feedback");
        MockHttpServletRequest request = tester.getRequest();
        request.setUseMultiPartContentType(true);

        tester.clickLink(path(panel, FORM, SUBMIT));

        verify(reportService).submit(oppositionReport);
    }

    @Test
    public void error_feedback_when_trying_to_submit_not_finished_report() throws Exception {
        when(reportService.submit(oppositionReport)).thenThrow(new IllegalStateException());
        startPanel();

        submitForm("feedback");
        MockHttpServletRequest request = tester.getRequest();
        request.setUseMultiPartContentType(true);

        tester.clickLink(path(panel, FORM, SUBMIT));

        tester.assertErrorMessages(panel.getString("notFilledOut"));
    }

    @Test
    public void all_fields_are_disabled_if_report_is_submitted() throws Exception {
        oppositionReport.setThesisSummary("something");
        for (Criterion criterion : oppositionReport.getCriteria()) {
            criterion.setFeedback("feedback");
        }
        oppositionReport.submit();
        startPanel();
        tester.assertDisabled(path(panel, FORM, GRADING_CRITERIA, 0, FEEDBACK));
    }

    @Test
    public void file_upload_field__visible_if_no_attachment_exists() throws Exception {
        tester.assertVisible(path(panel, FORM, "uploadAttachment"));
    }

    @Test
    public void file_upload_field_invisible_if_attachment_exists() throws Exception {
        startPanelWithAttachmentToReport();
        tester.assertInvisible(path(panel, FORM, "uploadAttachment"));
    }

    @Test
    public void delete_attachment_link_calls_service() throws Exception {
        startPanelWithAttachmentToReport();
        tester.clickLink(path(panel, FORM, "deleteAttachment"));
        verify(reportService).deleteAttachment(oppositionReport);
    }

    private void startPanelWithAttachmentToReport() {
        FileDescription attachment = new FileDescription();
        attachment.setId(6666L);
        oppositionReport.setAttachment(attachment);
        startPanel();
    }

    private void submitForm(String feedback) {
        FormTester formTester = tester.newFormTester(path(panel, FORM));
        formTester.setValue(path(GRADING_CRITERIA, 0, FEEDBACK), feedback);
        formTester.submit();
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new FillOutReportPanel<>("id", Model.of(oppositionReport)));
    }

}
