package se.su.dsv.scipro.grading;

import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Mockito;
import scala.util.Either;
import scala.util.Left;
import scala.util.Right;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.finalthesis.FinalThesis;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.*;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.io.Serializable;
import java.util.*;

import static org.hamcrest.CoreMatchers.hasItem;
import static org.junit.Assert.assertThat;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.grading.ReviewerFillOutGradingReportPanel.*;

public class ReviewerFillOutGradingReportPanelTest extends SciProTest {

    public static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, "bachelor", "bachelor");
    private static final User SOME_STUDENT = new UserBuilder().create();
    public static final User SOME_STUDENT2 = new UserBuilder().create();
    public Project project;
    private ReviewerFillOutGradingReportPanel panel;
    private ReviewerGradingReport reviewerGradingReport;
    private SupervisorGradingReport supervisorReport;
    private SupervisorGradingReport supervisorReport2;

    @Before
    public void setUp() throws Exception {
        prepareReport();

        Mockito.when(gradingReportService.getReviewerGradingReport(project)).thenReturn(reviewerGradingReport);
        Mockito.when(gradingReportService.getSupervisorGradingReports(project)).thenReturn(Arrays.asList(supervisorReport, supervisorReport2));
    }

    @Test
    public void has_criteria_panel() throws Exception {
        startPanel();

        tester.assertComponent(path(FORM, CRITERIA_PANEL), AutoSavingCriteriaPanel.class);
    }

    @Test
    public void has_save_button() throws Exception {
        startPanel();

        tester.assertComponent(path(SAVE), ScrollingSaveButtonPanel.class);
    }

    @Test
    public void has_list_of_grading_reports() throws Exception {
        startPanel();

        tester.assertListView(path(INDIVIDUAL_PARTS), Arrays.asList(supervisorReport, supervisorReport2));
    }

    @Test
    public void has_points_panel_for_each_grading_report() throws Exception {
        finishAndSubmit(supervisorReport);
        finishAndSubmit(supervisorReport2);

        startPanel();

        tester.assertComponent(path(INDIVIDUAL_PARTS, 0, POINTS_PANEL), GradingReportPointsPanel.class);
        tester.assertComponent(path(INDIVIDUAL_PARTS, 1, POINTS_PANEL), GradingReportPointsPanel.class);
    }

    @Test
    public void lists_individual_criteria() throws Exception {
        finishAndSubmit(supervisorReport);

        startPanel();

        tester.assertListView(path(INDIVIDUAL_PARTS, 0, INDIVIDUAL_CRITERIA), supervisorReport.getIndividualCriteria());
    }

    @Test
    public void shows_points_for_individual_criteria() throws Exception {
        finishAndSubmit(supervisorReport);

        startPanel();

        tester.assertModelValue(path(INDIVIDUAL_PARTS, 0, INDIVIDUAL_CRITERIA, 0, INDIVIDUAL_CRITERIA_POINTS_LABEL), supervisorReport.getIndividualCriteria().get(0).getPoints());
    }

    @Test
    public void shows_title_of_individual_criteria() throws Exception {
        finishAndSubmit(supervisorReport);

        startPanel();

        tester.assertModelValue(path(INDIVIDUAL_PARTS, 0, INDIVIDUAL_CRITERIA, 0, INDIVIDUAL_CRITERIA_TITLE_LABEL), supervisorReport.getIndividualCriteria().get(0).getTitle());
    }

    @Test
    public void click_download_writes_to_pdf() throws Exception {
        finishAndSubmit(reviewerGradingReport);
        finishAndSubmit(supervisorReport);

        startPanel();

        tester.clickLink(path(INDIVIDUAL_PARTS, 0, PDF_DOWNLOAD));

        Assert.assertEquals("application/pdf", tester.getContentTypeFromResponseHeader());
    }

    @Test
    public void download_link_is_always_enabled() throws Exception {
        startPanel();

        tester.assertEnabled(path(INDIVIDUAL_PARTS, 0, PDF_DOWNLOAD));
    }

    @Test
    public void saves_project_assessment_on_submit() throws Exception {
        startPanel();
        FormTester formTester = tester.newFormTester(path(FORM));
        Integer points = 1;
        String feedback = "new feedback";
        for (int i = 0; i < reviewerGradingReport.getGradingCriteria().size(); i++) {
            formTester.setValue(super.path(CRITERIA_PANEL, CriteriaPanel.CRITERIA, i, CriteriaPanel.POINTS), String.valueOf(points));
            formTester.setValue(super.path(CRITERIA_PANEL, CriteriaPanel.CRITERIA, i, CriteriaPanel.FEEDBACK), feedback);
        }
        formTester.submit();

        ArgumentCaptor<GradingReport> captor = ArgumentCaptor.forClass(GradingReport.class);
        Mockito.verify(gradingReportService, Mockito.atLeastOnce()).save(captor.capture());
        for (GradingCriterion criterion : captor.getValue().getGradingCriteria()) {
            Assert.assertEquals(points, criterion.getPoints());
            Assert.assertEquals(feedback, criterion.getFeedback());
        }
    }

    @Test
    public void displays_name_of_author() throws Exception {
        startPanel();

        tester.assertModelValue(path(INDIVIDUAL_PARTS, 0, NAME_LABEL), supervisorReport.getUser().getFullName());
    }

    @Test
    public void shows_info_about_waiting_for_supervisor_if_supervisor_has_not_submitted() throws Exception {
        startPanel();

        tester.assertVisible(path(INDIVIDUAL_PARTS, 0, INFO_LABEL));
    }

    @Test
    public void hides_info_about_waiting_for_supervisor_if_supervisor_has_submitted() throws Exception {
        finishAndSubmit(supervisorReport);

        startPanel();

        tester.assertInvisible(path(INDIVIDUAL_PARTS, 0, INFO_LABEL));
    }

    @Test
    public void disables_form_if_report_is_submitted() throws Exception {
        finishAndSubmit(reviewerGradingReport);

        startPanel();

        tester.assertDisabled(path(FORM));
    }

    @Test
    public void hides_scrolling_save_button_panel_if_report_is_submitted() throws Exception {
        finishAndSubmit(reviewerGradingReport);

        startPanel();

        tester.assertInvisible(path(SAVE));
    }

    @Test
    public void trying_to_submit_unfinished_report_should_show_error() throws Exception {
        Either<List<GradingCriterion>, GradingReport> result = Left.apply(reviewerGradingReport.getGradingCriteria());
        when(gradingReportService.submitReport(reviewerGradingReport)).thenReturn(result);
        reviewerGradingReport.setState(GradingReport.State.REVIEWING);

        startPanel();

        FormTester formTester = tester.newFormTester(path(FORM));
        formTester.submit(super.path("nextStep", "finalize"));

        List<Serializable> messages = tester.getMessages(FeedbackMessage.ERROR);
        assertThat(messages, hasItem(panel.getString("not.finished")));
    }

    @Test
    public void finish_initial_assessment() throws Exception {
        Either<List<GradingCriterion>, GradingReport> result = Right.<List<GradingCriterion>, GradingReport>apply(reviewerGradingReport);
        when(gradingReportService.initialAssessmentDone(reviewerGradingReport)).thenReturn(result);
        startPanel();
        FormTester formTester = tester.newFormTester(path(FORM));
        formTester.submit(super.path("nextStep", "initialAssessmentDone"));
        verify(gradingReportService).initialAssessmentDone(reviewerGradingReport);
    }

    @Test
    public void finalize_grading_report() throws Exception {
        Either<List<GradingCriterion>, GradingReport> result = Right.<List<GradingCriterion>, GradingReport>apply(reviewerGradingReport);
        when(gradingReportService.submitReport(reviewerGradingReport)).thenReturn(result);
        assessAllCriteria(reviewerGradingReport);
        reviewerGradingReport.setState(GradingReport.State.REVIEWING);
        startPanel();
        FormTester formTester = tester.newFormTester(path(FORM));
        formTester.submit(super.path("nextStep", "finalize"));
        verify(gradingReportService).submitReport(reviewerGradingReport);
    }

    @Override
    protected String path(Object... parts) {
        return super.path(panel, super.path(parts));
    }

    private void startPanel() {
        createFinalThesis();
        panel = tester.startComponentInPage(new ReviewerFillOutGradingReportPanel("id", Model.of(project)));
    }

    private void createFinalThesis() {
        FinalThesis finalThesis = new FinalThesis();
        finalThesis.setId(1L);
        FileDescription fileDescription = new FileDescription();
        fileDescription.setId(1L);
        finalThesis.setFileDescription(fileDescription);
        when(finalThesisService.findByProject(project)).thenReturn(finalThesis);
    }

    private void prepareReport() {
        TreeSet<User> projectParticipants = new TreeSet<>(new User.ByNameComparator());
        projectParticipants.addAll(Arrays.asList(SOME_STUDENT, SOME_STUDENT2));
        project = Project.builder()
                .title("some title")
                .projectType(BACHELOR)
                .projectParticipants(projectParticipants)
                .headSupervisor(new UserBuilder().setFirstName("Super").setLastName("Visor").create())
                .build();
        GradingReportTemplate gradingReportTemplate = new GradingReportTemplate(BACHELOR, 30);
        gradingReportTemplate.addProjectCriterion("project criterion", 0, getPointTemplates(2));
        gradingReportTemplate.addIndividualCriterion("individual criterion", 0, getPointTemplates(2));
        reviewerGradingReport = gradingReportTemplate.createReviewerReport(project);
        for (GradingCriterion gradingCriterion : reviewerGradingReport.getGradingCriteria()) {
            gradingCriterion.setPoints(1);
            gradingCriterion.setFeedback("Feedback");
        }

        supervisorReport = gradingReportTemplate.createSupervisorReport(project, SOME_STUDENT);
        supervisorReport2 = gradingReportTemplate.createSupervisorReport(project, SOME_STUDENT2);
    }

    private List<GradingCriterionPointTemplate> getPointTemplates(int maxPoint) {
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();
        for (int i = 0; i <= maxPoint; i++) {
            gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(i).description("Description").build());
        }
        return gradingCriterionPointTemplates;
    }

    private void finishAndSubmit(SupervisorGradingReport supervisorReport) {
        assessAllCriteria(supervisorReport);
        supervisorReport.setPlagiarismFeedback("plagiarism feedback");
        supervisorReport.submit();
    }

    private void finishAndSubmit(ReviewerGradingReport reviewerReport) {
        assessAllCriteria(reviewerReport);
        reviewerReport.submit();
    }

    private void assessAllCriteria(GradingReport gradingReport) {
        for (GradingCriterion criterion : gradingReport.getGradingCriteria()) {
            criterion.setPoints(criterion.getMaxPoints());
            criterion.setFeedback("criterion feedback");
        }
    }
}
