package se.su.dsv.scipro.loginlogout.pages;

import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.Hash;
import se.su.dsv.scipro.system.Password;
import se.su.dsv.scipro.system.User;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class SetPasswordPageTest extends SciProTest {

    @Test(expected= ClassCastException.class)
    public void empty_pp_fails() throws Exception {
        getPage(new PageParameters());
    }

    @Test(expected= ClassCastException.class)
    public void fails_when_nonexisting_hash() throws Exception {
        PageParameters pp = new PageParameters();
        pp.add("hash", "anicehash");
        getPage(pp);
    }

    @Test
    public void renders_when_existing_hash() throws Exception {
        startPage();
        tester.assertRenderedPage(SetPasswordPage.class);
    }

    private void startPage() {
        User user = User.builder().firstName("John").lastName("Rambo").emailAddress("rambo@us.gov").build();
        Hash hash = Hash.builder().hash("anicehash").user(user).build();
        when(hashService.findByHash(isA(String.class))).thenReturn(hash);
        PageParameters pp = new PageParameters();
        pp.add("hash", "anicehash");
        getPage(pp);
    }

    @Test
    public void passwords_required() throws Exception {
        startPage();
        FormTester formTester = tester.newFormTester("form");
        formTester.submit();
        tester.assertErrorMessages("Field 'password' is required", "Field 'repeatPassword' is required");
    }

    @Test
    public void matching_passwords_required() throws Exception {
        startPage();
        FormTester formTester = tester.newFormTester("form");
        formTester.setValue("password", "123456Ab");
        formTester.setValue("repeatPassword", "q");
        formTester.submit();
        tester.assertErrorMessages("password and repeatPassword must be equal.");
    }

    @Test
    public void lower_case_required() throws Exception {
        startPage();
        FormTester formTester = tester.newFormTester("form");
        formTester.setValue("password", "123456AB");
        formTester.setValue("repeatPassword", "123456AB");
        formTester.submit();
        tester.assertErrorMessages("Password must be more than 8 characters and requires at least 1 upper case letter, 1 lower case letter and 1 number.");
    }

    @Test
    public void upper_case_required() throws Exception {
        startPage();
        FormTester formTester = tester.newFormTester("form");
        formTester.setValue("password", "123456ab");
        formTester.setValue("repeatPassword", "123456ab");
        formTester.submit();
        tester.assertErrorMessages("Password must be more than 8 characters and requires at least 1 upper case letter, 1 lower case letter and 1 number.");
    }

    @Test
    public void number_required() throws Exception {
        startPage();
        FormTester formTester = tester.newFormTester("form");
        formTester.setValue("password", "AAAABBBB");
        formTester.setValue("repeatPassword", "AAAABBBB");
        formTester.submit();
        tester.assertErrorMessages("Password must be more than 8 characters and requires at least 1 upper case letter, 1 lower case letter and 1 number.");
    }

    @Test
    public void disallow_non_az_AZ_09() throws Exception {
        startPage();
        FormTester formTester = tester.newFormTester("form");
        formTester.setValue("password", "123456Ab#");
        formTester.setValue("repeatPassword", "123456Ab#");
        formTester.submit();
        tester.assertErrorMessages("Password must be more than 8 characters and requires at least 1 upper case letter, 1 lower case letter and 1 number.");
    }

    @Test
    public void too_short() throws Exception {
        startPage();
        FormTester formTester = tester.newFormTester("form");
        formTester.setValue("password", "1Ab");
        formTester.setValue("repeatPassword", "1Ab");
        formTester.submit();
        tester.assertErrorMessages("Password must be more than 8 characters and requires at least 1 upper case letter, 1 lower case letter and 1 number.");
    }

    @Test
    public void does_stuff_first_password_version() throws Exception {
        startPage();
        FormTester formTester = tester.newFormTester("form");
        formTester.setValue("password", "123456Ab");
        formTester.setValue("repeatPassword", "123456Ab");
        formTester.submit();
        tester.assertNoErrorMessage();
        verify(passwordFacade).setPassword(isA(User.class), isA(String.class));
    }

    @Test
    public void does_stuff_update_existing_password_version() throws Exception {
        User user = User.builder().firstName("John").lastName("Rambo").emailAddress("rambo@us.gov").build();
        Hash hash = Hash.builder().hash("anicehash").user(user).build();

        when(hashService.findByHash(isA(String.class))).thenReturn(hash);
        when(passwordService.findByUser(isA(User.class))).thenReturn(new Password());

        PageParameters pp = new PageParameters();
        pp.add("hash", "anicehash");
        getPage(pp);
        FormTester formTester = tester.newFormTester("form");
        formTester.setValue("password", "123456Ab");
        formTester.setValue("repeatPassword", "123456Ab");
        formTester.submit();
        tester.assertNoErrorMessage();
        verify(passwordFacade).setPassword(isA(User.class), isA(String.class));
    }

    private SetPasswordPage getPage(PageParameters pp){
        return tester.startPage(SetPasswordPage.class, pp);
    }
}