package se.su.dsv.scipro.panels;

import org.apache.commons.lang3.StringUtils;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.peer.Comment;
import se.su.dsv.scipro.peer.CommentThread;
import se.su.dsv.scipro.peer.PeerReview;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.*;
import static org.mockito.internal.verification.VerificationModeFactory.times;
import static se.su.dsv.scipro.panels.CommentThreadPanel.*;

public class CommentThreadPanelTest extends SciProTest {

    private CommentThreadPanel panel;
    private PeerReview review;
    private List<Comment> commentList;
    private User creatorUser;

    @Before
    public void setUp() throws Exception {
        when(dateService.getFormat(isA(DateStyle.class))).thenReturn("yyyy-mm-dd");
        Project project = Project.builder().title("someP").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "x", "s")).build();
        review = new PeerReview();
        review.setProject(project);
        review.setComment(StringUtils.repeat("a;", PeerReview.MINIMUM_COMMENT_LENGTH_IF_NO_ANSWERS));
        review.submit();
        creatorUser = new UserBuilder().create();
        commentList = setUpList();
    }

    @Test
    public void testRenders() throws Exception {
        startPanel();
        assertNotNull(panel);
    }

    @Test
    public void testStartWithoutACommentThreadCreatesANewOne() throws Exception {
        when(commentThreadService.getCommentThread(review)).thenReturn(null);
        startPanel();
        verify(commentThreadService, times(1)).save(any(CommentThread.class));
    }

    @Test
    public void testStartWithACommentThreadDoesNotCreateANewOne() throws Exception {
        when(commentThreadService.getCommentThread(review)).thenReturn(new CommentThread(new PeerReview()));
        startPanel();
        verify(commentThreadService, never()).save(any(CommentThread.class));

    }

    @Test
    public void testSubmitEmptyFormRendersErrorFeedback() throws Exception {
        startPanel();
        FormTester formTester = tester.newFormTester(path(panel.getId(), COMMENT_FORM));
        formTester.submit();
        tester.assertErrorMessages(panel.getString("commentTextArea.Required"));
    }

    @Test
    public void testSubmitFormWithCorrectInput() throws Exception {
        startPanel();
        FormTester formTester = tester.newFormTester(path(panel.getId(), COMMENT_FORM));
        formTester.setValue(COMMENT_TEXT_AREA, "comment");
        formTester.submit();
        tester.assertNoErrorMessage();
        verify(commentService).save((Comment) formTester.getForm().getModelObject());
    }

    @Test
    public void testCommentListView() throws Exception {
        startPanel();
        tester.assertListView(path(panel.getId(), COMMENTS), commentList);
    }

    @Test
    public void testDeleteLinkIsVisibleForCreator() throws Exception {
        setLoggedInAs(creatorUser);
        startPanel();

        boolean isVisible = panel.get(path(COMMENTS, 0, DELETE_LINK)).isVisibleInHierarchy();
        assertTrue(isVisible);
    }


    @Test
    public void testDeleteLinkIsInvisibleForNonCreators() throws Exception {
        setLoggedInAs(new UserBuilder().create());
        startPanel();

        tester.assertInvisible(path(panel, COMMENTS, 0, DELETE_LINK));
    }

    @Test
    public void testClickDelete() throws Exception {
        setLoggedInAs(creatorUser);
        startPanel();
        tester.clickLink(path(panel.getId(), COMMENTS, 0, DELETE_LINK));
        verify(commentService).delete(any(Comment.class));
    }

    private List<Comment> setUpList() {
        Comment comment = new Comment(creatorUser, null);
        comment.setCreator(creatorUser);
        comment.setComment("comment");
        List<Comment> commentList = Collections.singletonList(comment);
        when(commentThreadService.getCommentThread(review)).thenReturn(new CommentThread(new PeerReview()));
        when(commentService.findByCommentThread(isA(CommentThread.class))).thenReturn(commentList);
        return commentList;
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new CommentThreadPanel("panel", Model.of(review)));
    }
}
