package se.su.dsv.scipro.peer;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.PeerEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.peer.PeerRequestSelectionPanel.*;
import static se.su.dsv.scipro.test.ObjectMother.BACHELOR;
import static se.su.dsv.scipro.test.ObjectMother.SOME_PROJECT;

public class PeerRequestSelectionPanelTest extends SciProTest {

    public static final String PANEL = "id";
    private Project requestProject;
    private PeerRequestSelectionPanel panel;

    @Before
    public void setUp() throws Exception {
        requestProject = Project.builder().title("Project").projectType(BACHELOR).build();
        startPanel(2L, 1L, new ArrayList<PeerReview>());
    }

    @Test
    public void panel_is_visible_if_user_is_allowed_to_select_request() throws Exception {
        tester.assertVisible(PANEL);
    }

    @Test
    public void panel_is_not_visible_if_user_is_disallowed() throws Exception {
        startPanel(0L, 0L, new ArrayList<PeerReview>());
        tester.assertInvisible(PANEL);
    }

    @Test
    public void panel_is_not_visible_if_reviews_in_progress_exists() throws Exception {
        startPanel(2L, 1L, Collections.singletonList(new PeerReview()));
        tester.assertInvisible(PANEL);
    }

    @Test
    public void request_list_shows_request_project_title() throws Exception {
        tester.assertLabel(path(PANEL, AVAILABLE_REQUESTS, 1, TITLE), requestProject.getTitle());
    }

    @Test
    public void trying_to_accept_review_with_own_projects_request() throws Exception {
        expectErrorMessage(new PartOfProjectException(), "partOfProjectException");
    }

    @Test
    public void trying_to_accept_recently_reviewed_projects_request() throws Exception {
        expectErrorMessage(new RecentlyReviewedException(), "recentlyReviewedException");
    }

    @Test
    public void trying_to_accept_already_taken_request() throws Exception {
        expectErrorMessage(new RequestTakenException(), "requestTakenException");
    }

    @Test
    public void trying_to_accept_with_unknown_error() throws Exception {
        expectErrorMessage(new Exception(), "unknownError");
    }

    @Test
    public void accept_review_notifies() throws Exception {
        tester.clickLink(path(PANEL, AVAILABLE_REQUESTS, 1, ACCEPT));
        verify(notificationController).notifyPeer(any(PeerReview.class), eq(PeerEvent.Event.REVIEW_ACCEPTED), any(NotificationSource.class));
    }

    private void expectErrorMessage(Exception exception, String expected) throws Exception {
        when(peerPortal.acceptReview(any(PeerRequest.class), any(User.class), any(Project.class))).thenThrow(exception);
        startPanel(2L, 1L, new ArrayList<PeerReview>());
        tester.clickLink(path(PANEL, AVAILABLE_REQUESTS, 1, ACCEPT));
        tester.assertErrorMessages(panel.getString(expected));
    }

    private void startPanel(long requests, long reviews, List<PeerReview> existingReviews) {
        when(peerRequestService.findAvailableRequests(eq(SciProSession.get().getUser()), eq(RequestStatus.WAITING), eq(BACHELOR), any(Pageable.class))).thenReturn(Collections.singletonList(createRequest()));
        when(peerRequestService.countAvailableRequests(SciProSession.get().getUser(), RequestStatus.WAITING, BACHELOR)).thenReturn(1L);
        when(peerRequestService.count(any(PeerRequestService.Filter.class))).thenReturn(requests);
        when(peerReviewService.count(any(PeerReviewService.Filter.class))).thenReturn(reviews);
        when(peerReviewService.findInProgressReviews(SciProSession.get().getUser(), SOME_PROJECT)).thenReturn(existingReviews);
        panel = tester.startComponentInPage(new PeerRequestSelectionPanel(PANEL, Model.of(SOME_PROJECT)));
    }

    private PeerRequest createRequest() {
        PeerRequest peerRequest = new PeerRequest();
        peerRequest.setProject(requestProject);
        return peerRequest;
    }

}
