package se.su.dsv.scipro.peer;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.basepages.errorpages.AccessDeniedPage;
import se.su.dsv.scipro.basepages.errorpages.NotFoundPage;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.util.PageParameterKeys;

import static org.mockito.Matchers.*;
import static org.mockito.Mockito.when;

public class ViewPeerReviewPageTest extends PageTest {

    private PeerReview peerReview;
    private GeneralSystemSettings generalSystemSettings;

    @Before
    public void setUp() throws Exception {
        User reviewer = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        DomainObjects.injectId(reviewer, 234L);

        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        Project project = Project.builder().title("Title").projectType(projectType).build();
        project.setHeadSupervisor(reviewer);
        DomainObjects.injectId(project, 234234L);

        PeerRequest peerRequest = new PeerRequest();
        peerRequest.setProject(project);

        peerReview = new PeerReview();
        peerReview.setStatus(PeerReview.ReviewStatus.COMPLETED);
        peerReview.setPeerRequest(peerRequest);
        peerReview.setReviewer(reviewer);
        peerReview.setProject(project);
        when(peerReviewService.findOne(anyLong())).thenReturn(peerReview);
        when(projectService.findOne(anyLong())).thenReturn(project);
        when(projectService.isPartOfProject(eq(project), any(User.class))).thenReturn(true);
        generalSystemSettings  = new GeneralSystemSettings();
        when(generalSystemSettingsService.getGeneralSystemSettingsInstance()).thenReturn(generalSystemSettings);
        setLoggedInAs(reviewer);
    }

    @Test
    public void has_a_display_review_panel() throws Exception {
        startPage();

        tester.assertComponent("review", DisplayReviewPanel.class);
        tester.assertModelValue("review", peerReview);
    }

    @Test
    public void shows_request_details() throws Exception {
        startPage();

        tester.assertComponent("request", ReviewPageRequestDetailsPanel.class);
        tester.assertModelValue("request", peerReview.getPeerRequest());
    }

    @Test
    public void reviewer_is_allowed_to_see_the_review() throws Exception {
        generalSystemSettings.setPublicReviewsActivated(false);

        startPage();

        tester.assertRenderedPage(getPage());
    }

    @Test
    public void random_user_can_not_see() throws Exception {
        setLoggedInAs(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
        generalSystemSettings.setPublicReviewsActivated(false);

        startPage();

        tester.assertRenderedPage(AccessDeniedPage.class);
    }

    @Test
    public void if_reviews_are_public_random_user_can_see() throws Exception {
        setLoggedInAs(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
        generalSystemSettings.setPublicReviewsActivated(true);

        startPage();

        tester.assertRenderedPage(getPage());
    }

    @Test
    public void handles_invalid_ids() throws Exception {
        final PageParameters invalid = getPageParameters();
        invalid.set(PageParameterKeys.MAP.get(PeerReview.class), "abc");

        tester.startPage(getPage(), invalid);

        tester.assertRenderedPage(NotFoundPage.class);
    }

    @Test
    public void handles_missing_review() throws Exception {
        when(peerReviewService.findOne(anyLong())).thenReturn(null);

        startPage();

        tester.assertRenderedPage(NotFoundPage.class);
    }

    @Override
    protected Class<? extends Page> getPage() {
        return ViewPeerReviewPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.set(PageParameterKeys.MAP.get(PeerReview.class), 1L);
        pageParameters.set(PageParameterKeys.MAP.get(Project.class), 1L);
        return pageParameters;
    }
}
