package se.su.dsv.scipro.phd;

import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.file.File;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import java.net.URI;
import java.net.URISyntaxException;

import static org.junit.Assert.assertNotNull;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.phd.StudyPlanPanel.*;

public class StudyPlanPanelTest extends SciProTest {
    public static final ProjectType PROJECT_TYPE = new ProjectType(DegreeLevel.THIRD_CYCLE, "PhD", "PhD");

    private StudyPlanPanel panel;
    private StudyPlan studyPlan;
    private Project project;

    @Before
    public void setUp() throws Exception {
        project = Project.builder().title("PhD project").projectType(PROJECT_TYPE).build();
        addStudyPlan();
    }

    @Test
    public void testRenders() throws Exception {
        startPanel();
        assertNotNull("Panel does not render", panel);
    }

    @Test
    public void visible_if_study_plan_exists() throws Exception {
        startPanel();
        tester.assertVisible(path(panel));
    }

    @Test
    public void form_visible_if_no_basic_contract() throws Exception {
        startPanel();
        tester.assertVisible(path(panel, BASIC_CONTRACT_FORM));
    }

    @Test
    public void form_invisible_if_basic_contract() throws Exception {
        addBasicContract();
        startPanel();
        tester.assertInvisible(path(panel, BASIC_CONTRACT_FORM));
    }

    @Test
    public void form_requires_file() throws Exception {
        startPanel();
        FormTester formTester = tester.newFormTester(path(panel, BASIC_CONTRACT_FORM));
        formTester.submit();
        tester.assertErrorMessages("Select a basic contract file");
    }

    @Test
    public void form_stores_file() throws Exception {
        startPanel();
        FormTester formTester = tester.newFormTester(path(panel, BASIC_CONTRACT_FORM));

        File file = getTestFile();
        String contentType = "text/plain";

        formTester.setFile(BASIC_CONTRACT_UPLOAD, file, contentType);
        formTester.submit();
        tester.assertNoErrorMessage();
        verify(studyPlanService).addBasicContract(isA(FileUpload.class), isA(StudyPlan.class));
    }

    @Test
    public void annual_review_form_visible_if_basic_contract() throws Exception {
        addBasicContract();
        startPanel();
        tester.assertComponent(path(panel, StudyPlanPanel.CONTAINER, ANNUAL_REVIEW_FORM), Form.class);
        tester.assertComponent(path(panel, StudyPlanPanel.CONTAINER, ANNUAL_REVIEW_FORM, ANNUAL_REVIEW_UPLOAD), FileUploadField.class);
    }

    @Test
    public void annual_review_form_invisible_if_no_basic_contract() throws Exception {
        startPanel();
        tester.assertInvisible(path(panel, StudyPlanPanel.CONTAINER, ANNUAL_REVIEW_FORM));
        tester.assertInvisible(path(panel, StudyPlanPanel.CONTAINER, ANNUAL_REVIEW_FORM, ANNUAL_REVIEW_UPLOAD));
    }

    @Test
    public void annual_review_form_requires_file_and_year() throws Exception {
        addBasicContract();
        startPanel();
        FormTester formTester = tester.newFormTester(path(panel, StudyPlanPanel.CONTAINER, ANNUAL_REVIEW_FORM));
        formTester.submit();
        tester.assertErrorMessages("Select an annual review file");
    }

    @Test
    public void annual_review_form_saves() throws Exception {
        addBasicContract();
        startPanel();
        FormTester formTester = tester.newFormTester(path(panel, StudyPlanPanel.CONTAINER, ANNUAL_REVIEW_FORM));

        File file = getTestFile();
        String contentType = "text/plain";
        formTester.setFile(ANNUAL_REVIEW_UPLOAD, file, contentType);
        formTester.submit();
        tester.assertNoErrorMessage();
        verify(studyPlanService).addAnnualReview(isA(FileUpload.class), isA(StudyPlan.class));
    }

    @Test
    public void annual_review_list_visible_if_basic_contract() throws Exception {
        addBasicContract();
        startPanel();
        tester.assertVisible(path(panel, ANNUAL_REVIEWS));
    }

    @Test
    public void annual_review_list_invisible_if_no_basic_contract() throws Exception {
        startPanel();
        tester.assertInvisible(path(panel, ANNUAL_REVIEWS));
    }

    @Test
    public void list_components() throws Exception {
        addBasicContract();
        addAnnualReviews();
        startPanel();

        tester.assertComponent(path(panel, ANNUAL_REVIEWS, 0, ANNUAL_REVIEW), ViewAttachmentPanel.class);
    }

    @Test
    public void annual_review_delete() throws Exception {
        addBasicContract();
        addAnnualReviews();
        startPanel();

        tester.clickLink(path(panel, ANNUAL_REVIEWS, 0, DELETE));
        verify(studyPlanService).removeAnnualReview(isA(StudyPlan.class), isA(AnnualReview.class));
    }

    private void addStudyPlan(){
        studyPlan = new StudyPlan(project);
        when(studyPlanService.findByProject(isA(Project.class))).thenReturn(studyPlan);
    }

    private void addBasicContract(){
        FileDescription basicContract = new FileDescription();
        basicContract.setName("basic contract");
        basicContract.setIdentifier("1");
        basicContract.setId(1L);
        studyPlan.setBasicContract(basicContract);
    }

    private void addAnnualReviews(){
        FileDescription annualReviewFile = new FileDescription();
        annualReviewFile.setName("basic contract");
        annualReviewFile.setIdentifier("1");
        annualReviewFile.setId(1L);

        studyPlan.addAnnualReview(annualReviewFile);
    }

    private File getTestFile() throws URISyntaxException {
        URI resource = Thread.currentThread().getContextClassLoader().getResource("final_thesis.txt").toURI();
        return new File(resource);
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new StudyPlanPanel("panel", Model.of(project)));
    }
}