package se.su.dsv.scipro.project.panels;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.Model;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.*;

import static org.junit.Assert.assertNotNull;
import static org.mockito.Mockito.*;
import static se.su.dsv.scipro.project.panels.ProjectHeaderPanel.CREDITS;

public class ProjectHeaderPanelTest extends SciProTest {

    ProjectHeaderPanel panel;
    Project project;

    @Before
    public void setUp() throws Exception {
        project = createProject();
        startPanel();
    }

    @Test
    public void testRenders() throws Exception {
        assertNotNull("panel does not render", panel);
    }

    @Test
    public void testShowProjectTitle() throws Exception {
        tester.assertComponent(panel(ProjectHeaderPanel.TITLE), Label.class);
        tester.assertLabel(panel(ProjectHeaderPanel.TITLE), project.getTitle());
    }

    @Test
    public void testShowProjectStatus() throws Exception {
        tester.assertComponent(panel(ProjectHeaderPanel.STATUS), Label.class);
        tester.assertLabel(panel(ProjectHeaderPanel.STATUS), project.getProjectStatus().toString());
    }

    @Test
    public void testShowProjectLevel() throws Exception {
        tester.assertComponent(panel(ProjectHeaderPanel.PROJECT_TYPE), Label.class);
        tester.assertLabel(panel(ProjectHeaderPanel.PROJECT_TYPE), project.getProjectType().getName());
    }

    @Test
    public void testShowHeadSupervisor() throws Exception {
        tester.assertComponent(panel(ProjectHeaderPanel.SUPERVISOR), UserLinkPanel.class);
    }

    @Test
    public void testShowAuthors() throws Exception {
        tester.assertComponent(panel(ProjectHeaderPanel.AUTHOR_LIST), ListView.class);
        List<User> projectAuthors = new ArrayList<>(project.getProjectParticipants());
        tester.assertListView(panel(ProjectHeaderPanel.AUTHOR_LIST), projectAuthors);
    }

    @Test
    public void testHideCoSupervisorsIfEmpty() throws Exception {
        tester.assertInvisible(panel(ProjectHeaderPanel.CO_SUPERVISOR_LIST));
    }

    @Test
    public void testShowCoSupervisorsIfAvailable() throws Exception {
        Project project = createProject();
        User coSupervisor = new UserBuilder().create();
        project.addCoSupervisor(coSupervisor);
        startPanel(project);

        tester.assertVisible(panel(ProjectHeaderPanel.CO_SUPERVISOR_LIST));
        tester.assertListView(panel(ProjectHeaderPanel.CO_SUPERVISOR_LIST), Collections.singletonList(coSupervisor));
    }

    @Test
    public void testHideReviewerIfEmpty() throws Exception {
        tester.assertInvisible(panel(ProjectHeaderPanel.REVIEWER_LIST));
    }

    @Test
    public void testShowReviewerIfAvailable() throws Exception {
        Project project = createProject();
        User reviewer = new UserBuilder().create();
        project.addReviewer(reviewer);
        startPanel(project);

        tester.assertVisible(panel(ProjectHeaderPanel.REVIEWER_LIST));
        tester.assertListView(panel(ProjectHeaderPanel.REVIEWER_LIST), Collections.singletonList(reviewer));
    }

    @Test
    public void testShowDateLabel() throws Exception {
        tester.assertComponent(panel(ProjectHeaderPanel.START_DATE), Label.class);
    }

    @Test
    public void testShowCorrectDateIfAvailable() throws Exception {
        Project project = createProject();
        Date startDate = new DateTime(2013, 5, 21, 13, 0, 1, 1).toDate();
        project.setDateCreated(startDate);
        String dateString = "2013-05-21";
        when(dateService.format(isA(Date.class))).thenReturn(dateString);
        startPanel(project);

        assertNotNull(project.getDateCreated());
        verify(dateService, atLeast(1)).format(isA(Date.class));
        tester.assertLabel(panel(ProjectHeaderPanel.START_DATE), dateString);
    }

    @Test
    public void shows_project_points() throws Exception {
        int credits = 30;
        project.setCredits(credits);
        startPanel();

        tester.assertLabel(panel(CREDITS), Integer.toString(credits));
    }

    @Test
    public void hides_points_if_zero() throws Exception {
        tester.assertInvisible(panel(CREDITS));
    }

	@Test
	public void only_head_supervisor_can_see_edit_link() throws Exception {
		User user = new UserBuilder().setId(2L).setFirstName("any").setLastName("any").create();
		setLoggedInAs(user);
		tester.assertInvisible(panel(ProjectHeaderPanel.EDIT_LINK));
	}

	private void startPanel() {
        startPanel(project);
    }

    private void startPanel(final Project p) {
        panel = tester.startComponentInPage(new ProjectHeaderPanel("panel", Model.of(p)));
    }

    private Project createProject() {
        User user = new UserBuilder().setId(1L).setFirstName("author").setLastName("1").create();
        Project project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(user)
                .build();
        DomainObjects.injectId(project, 1L);

        return project;
    }

    private String panel(Object... parts) {
        return super.path(panel.getId(), super.path(parts));
    }
}
