package se.su.dsv.scipro.supervisor.pages;

import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.project.ProjectTeamMemberRoles;
import se.su.dsv.scipro.settings.dataobjects.UserProfile;
import se.su.dsv.scipro.supervisor.panels.SupervisorMyProjectsPanel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;

import java.util.Arrays;
import java.util.Collection;
import java.util.EnumSet;
import java.util.List;

import static org.hamcrest.CoreMatchers.hasItem;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;
import static org.mockito.Mockito.*;
import static se.su.dsv.scipro.supervisor.pages.SupervisorStartPage.PROJECTS_OVERVIEW_PANEL;

public class SupervisorStartPageTest extends SciProTest {

    private UserProfile userProfile;

    @Before
    public void setUp() throws Exception {
        mockProjectsInDatabase();
        createDefaultUserProfile();
        startPage();
    }

    @Test
    public void renders() throws Exception {
        tester.assertRenderedPage(SupervisorStartPage.class);
    }

    @Test
    public void select_supervisor_only_filter() throws Exception {

        filterForm().setValue(SupervisorMyProjectsPanel.SUPERVISOR, false).submit();

        assertEquals(false, getFilterParams().isFilterSupervisor());
    }

    @Test
    public void selecting_role_updates_filter() throws Exception {
        ProjectTeamMemberRoles role = ProjectTeamMemberRoles.REVIEWER;

        filterForm().select(SupervisorMyProjectsPanel.ROLE_FILTER, role.ordinal()).submit();

        assertThat(getFilterParams().getRoles(), hasItem(role));
    }

    @Test
    public void selecting_status_updates_filter() throws Exception {
        ProjectStatus projectStatus = ProjectStatus.INACTIVE;

        filterForm().select(SupervisorMyProjectsPanel.STATUS_FILTER, projectStatus.ordinal()).submit();

        assertThat(getFilterParams().getStatuses(), hasItem(projectStatus));
    }

    @Test
    public void loads_default_project_status_filter_from_user_profile() throws Exception {
        assertEquals(userProfile.getDefaultProjectStatusFilter(), getFilterParams().getStatuses());
    }

    @Test
    public void updates_profile_when_filter_changes() throws Exception {
        Collection<ProjectStatus> statuses = EnumSet.allOf(ProjectStatus.class);

        for (ProjectStatus status : statuses) {
            filterForm().select(SupervisorMyProjectsPanel.STATUS_FILTER, status.ordinal());
        }
        ajaxClickFilter(SupervisorMyProjectsPanel.STATUS_FILTER);

        ArgumentCaptor<UserProfile> captor = ArgumentCaptor.forClass(UserProfile.class);
        verify(this.userProfileService).save(captor.capture());
        assertEquals(statuses, captor.getValue().getDefaultProjectStatusFilter());
    }

    @Test
    public void ajax_clicking_project_status_updates_table() throws Exception {
        ajaxClickFilter(SupervisorMyProjectsPanel.STATUS_FILTER);
        tester.assertComponentOnAjaxResponse(path(PROJECTS_OVERVIEW_PANEL, SupervisorMyProjectsPanel.DATA_PANEL));
    }

    @Test
    public void ajax_clicking_project_status_updates_default_selection_in_profile() throws Exception {
        ajaxClickFilter(SupervisorMyProjectsPanel.STATUS_FILTER);
        verify(this.userProfileService).save(any(UserProfile.class));
    }

    private void ajaxClickFilter(String filterComponentId) {
        tester.executeAjaxEvent(filterPath(filterComponentId), "click");
    }

    private void mockProjectsInDatabase() {
        ProjectType bachelor = new ProjectType(ProjectType.BACHELOR, "Bachelor", "Bachelor");
        Project project = Project.builder().title("Some project").projectType(bachelor).build();
        project.setHeadSupervisor(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
        DomainObjects.injectId(project, 1L);

        when(projectService.findAll(any(ProjectService.Filter.class), any(Pageable.class)))
                .thenReturn(pageContaining(project));
        when(projectService.count(any(ProjectService.Filter.class))).thenReturn(1L);
    }

    private void createDefaultUserProfile() {
        userProfile = new UserProfile();
        when(userProfileService.findByUser(any(User.class))).thenReturn(userProfile);
    }

    private static List<Project> pageContaining(Project... project) {
        return Arrays.asList(project);
    }

    private void startPage() {
        tester.startPage(SupervisorStartPage.class);
        verify(projectService, atLeastOnce()).count(any(ProjectService.Filter.class));
    }

    private FormTester filterForm() {
        return tester.newFormTester(path(PROJECTS_OVERVIEW_PANEL, SupervisorMyProjectsPanel.FILTER_FORM));
    }

    private String filterPath(String filterComponentId) {
        return path(PROJECTS_OVERVIEW_PANEL, SupervisorMyProjectsPanel.FILTER_FORM, filterComponentId);
    }

    private ProjectService.Filter getFilterParams() {
        ArgumentCaptor<ProjectService.Filter> captor = ArgumentCaptor.forClass(ProjectService.Filter.class);
        verify(projectService, atLeastOnce()).count(captor.capture());
        return captor.getValue();
    }
}
